
use v5.10; # at least for Perl 5.10
no warnings 'experimental::smartmatch'; #Avoid known warnings with given-when
package Logger;

use strict;
use File::Basename;
use File::Path;
use IO::File;
use Carp qw(cluck);

require Exporter;

my $fh;
my $filename;
my $masks;

use constant {
	ERR  => 0,
	CRIT => 1,
	WARN => 2,
	INFO => 3,
	DEBG => 4,
	NOTI => 5,
	MESG => 6,
	MESGNONEWLN => 7,
};

use constant {
	LOG_TIMESTAMP => 1,
	LOG_CONSOLE   => 2,
	LOG_FILE      => 4,
};


BEGIN {

	$fh = new IO::File;
	our @ISA = qw(Exporter);
	our @EXPORT = qw(ERR CRIT WARN INFO DEBG NOTI MESG MESGNONEWLN Log close_log LOG_TIMESTAMP LOG_CONSOLE set_filename);
	$masks = LOG_TIMESTAMP + LOG_CONSOLE + LOG_FILE; #FIXME: Perl does not suppoer the real bit patterns
}

sub set_mask {
	my $var = shift;
	$masks |= $var;
}

sub unset_mask {
	my $var = shift;
	$masks ^= $var; #But this works.
}

#Get the log file name and also creates the base directory.
#If no log file is mentioned ndsdcinit.log is assumed
sub set_filename {
	$filename = shift;
	if (not defined $filename or $filename eq "") {
		$filename = "/var/opt/novell/xad/log/ndsdcinit.log";
	}
	my $base = (fileparse $filename) [1];
	my $fname = (fileparse $filename) [0];

	#Passed argument is a directory.
	if (not defined $fname or $fname eq '') {
		print STDERR "A valid log file name must be passed, not directory name\n";
		return -1;
	}

	#Creates the base directory if required.
	if (! -d $base) {
		mkpath $base or warn("Could not create the directory $base");
	}
}

sub print_stderr {

	if ($masks & LOG_TIMESTAMP) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,
			$yday,$isdst)=localtime(time);
		printf STDERR "%4d-%02d-%02d %02d:%02d:%02d ",
		$year+1900,$mon+1,$mday,$hour,$min,$sec;
	}
	print STDERR "@_";
}

#Prints on the stdout
sub print_screen {

	if ($masks & LOG_TIMESTAMP) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,
			$yday,$isdst)=localtime(time);
		printf STDOUT "%4d-%02d-%02d %02d:%02d:%02d ",
		$year+1900,$mon+1,$mday,$hour,$min,$sec;
	}
	print STDOUT "@_";
}

sub print_file {
	#Check if file opened is valid, otherwise open it now.
	if (! $fh->opened) {
		if (not defined $filename or $filename eq "") {
			set_filename();
		}
		$fh->open(">>$filename") or warn("could not open the log file $filename");
	}

	if ($masks & LOG_TIMESTAMP) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,
			$yday,$isdst)=localtime(time);
		printf $fh "%4d-%02d-%02d %02d:%02d:%02d ",
		$year+1900,$mon+1,$mday,$hour,$min,$sec;
	}
	print $fh "@_";
}

#Error message must be printed on the screen.
sub _err{
#	my $exitreason = "\nerr val:$!";
	print_file @_; #, $exitreason;
	Carp::cluck(@_);
	exit 1;
}
#Crtitical essage must be printed on the screen.
sub _crit {
	print_stderr @_;
	print_file @_;
}
sub _warn {
	if ($masks & LOG_CONSOLE) {
		print_screen @_;
	}
	if ($masks & LOG_FILE) {
		print_file @_;
	}
}
sub _info {
	if ($masks & LOG_CONSOLE) {
		print_screen @_;
	}
	if ($masks & LOG_FILE) {
		print_file @_;
	}
}
sub _debg {
	if ($masks & LOG_CONSOLE) {
		print_screen @_;
	}
	if ($masks & LOG_FILE) {
		print_file @_;
	}
}
#Notofication messages are to be printed on the screen.
sub _noti {
	print_screen @_;
	print_file @_;
}

sub _mesg{
	#Message are generally greetings like saying starting the process.
	#Hence no timestamp required.
	unset_mask(LOG_TIMESTAMP);

	if ($masks & LOG_CONSOLE) {
		print_screen @_;
	}
	if ($masks & LOG_FILE) {
		print_file  @_;
	}
	#Reverting back for other.
	set_mask(LOG_TIMESTAMP);

}
sub _log{
	if ($masks & LOG_CONSOLE) {
		print_screen @_;
	}
	if ($masks & LOG_FILE) {
		print_file @_;
	}
}

sub Log {
	my $type;
	my $msg;
	my $header;

	#Finds the caller's filename
	$header = (caller 0)[1];

	$header .= ':';

	#Finds the line number of the caller;
	$header .= (caller 0) [2];

	#Check if type is passed;
	if (@_ == 2) {
		$type = shift;
	}
	$msg = shift;

	#added for password mask
        $msg =~ s/ -w\s*\S+/ -w *****/;

	if ($type != MESGNONEWLN) {
		$msg .= "\n";
	}

#	print LOG_TIMESTAMP;
	given($type) {
		when (undef)	{ _log ($msg); } #No type mentioned, just a log
		when (ERR)	{ _err ($msg); }
		when (CRIT)	{ _crit($header, $msg); }
		when (WARN)	{ _warn($header, $msg); }
		when (INFO)	{ _info($msg); }
		when (DEBG)	{ _debg($header, $msg); }
		when (NOTI)	{ _noti($header, $msg); }
		when (MESG)	{ _mesg($msg); }
		when (MESGNONEWLN)	{ _mesg($msg); }
		default		{ print STDERR "Wrong arguments passed from $header and passed message is \"$msg\"\n"; } #Something really Wrong here.
	}
}

#Close the file handle.
sub close_log {
	 $fh->close;
}

1;
