#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         dns.pm
#
#   FACILITY:
#         Provides all the dns related work
#
#   ABSTRACT:
#
#
#   REVISION: $Id$
#
#################################################################################

package dns;


use strict;
use warnings;
use XAD::registry;
use Logger qw(:DEFAULT);
use NDS::import;
use XAD::dnsrecords;
use File::Glob ':glob';
use File::Basename;
use File::Path qw(mkpath);
use File::Temp;
use util;

# variables
my $debug = 0;
my $retry_count = 10;
my $LDAP_NO_SUCH_OBJECT =  32;      # Net LDAP Error code for NO Such Object
my $LDAP_SERVER_NOMAPPING_OID = "2.16.840.1.113719.1.513.4.7";
my $port_dsfw = 636;

sub is_sites_SRV_record_present() {
	my $is_replica = registry::getReg("IS_REPLICA");
	my $site_of_DC = registry::getReg("Site");
	my $domain_name = `/bin/hostname -d`;
	chomp($domain_name);
	my $srv_record = "_gc._tcp.$site_of_DC._sites.$domain_name";

	if ($is_replica eq "TRUE") {
		my $result = `/usr/bin/dig -t SRV $srv_record  +short`;
		my $dig_err = $?>>8;
		chomp($result);
		if (not defined($result) or $result eq "") {
			Log(INFO, "[$srv_record] for the $site_of_DC is absent. [$result]");
			return (0);
		}
	}

	return (1);
}

#Generates RRs LDIF file;
sub Process_DNS_Files {
	my ($Filename, $DNSZONE_CONTEXT, $Is_ADC, $Action) = @_;
	my ($ReadLine, $TempLine, $InstallMachineName, $IPAddress) = "";
	my $IPFlg = 0;
	my $SRV_record_present = is_sites_SRV_record_present();
	open(Filename, $Filename) || die "Cannot open $Filename";
	print "Reading from $Filename to process DNS records" if $debug;
	$InstallMachineName = lc(registry::getReg("InstallMachineName"));
	chomp $InstallMachineName;
	$IPAddress = registry::getReg("IP Address");
	chomp $IPAddress;
	my $File_IP = basename($Filename, ".zone");
	if($File_IP eq $IPAddress)
	{
		# set the IPFlg, used for reverse lookup zone record creation
		$IPFlg = 1;
	}
	print "\n IP flag : $IPFlg\n" if $debug;

	while($ReadLine = <Filename>)
	{
		chomp $ReadLine ;
		# leave all the comments starting with ;
		if(!($ReadLine =~ /^\;/))
		{
			$TempLine = $ReadLine;
			$TempLine =~ s/\s+//g;
			if($ReadLine =~ /^_/)
			{
				# a SRV record found, process it;
				if ($Is_ADC and  !$SRV_record_present and ($ReadLine =~ /\._sites/ig)) {

					#Special case for ADC, when it is configured under a new SITE
					#checking for config-site.ldif file at location
					#/var/opt/novell/xad/ds/config/
					#This file will be present only if the ADC is configured under
					#a new site and site does not exists in the tree before.
					#For any site 6 addtional SRV records needs to be created
					#These records are getting created for ADC here.

					Log (INFO, "Processing [$ReadLine] for ADC");
					Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, "FALSE", $Action);
				} else {
					Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
				}
			}
			if($TempLine =~ /^$InstallMachineName/)
			{
				# A record found,
				# process it
				Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
			}
			if(($ReadLine =~ /IN A/) && (!($TempLine =~ /^$InstallMachineName/)))
			{
				# A record found
				# process it
				Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
			}
			if($ReadLine =~ /IN CNAME/)
			{
				# CNAME record found, process it
				Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
			}
			if($ReadLine =~ /IN NS/)
			{
				# NS record found, process it
				Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
			}
			if($ReadLine =~ /\d*\.*\d*\.*\d*\s*IN\s*PTR/)
			{
				# PTR record found, process it
				Process_RR($ReadLine, $IPFlg, $DNSZONE_CONTEXT, $Is_ADC, $Action);
			}
				# other records, not to be processed
		}
	}
	close(Filename);

}

#check whether reverse zone is present in dns server object or not
sub is_reverse_zone_present {
	my $rev_zone_to_find = shift;
	my $rev_zone_context = shift;
	my $frd_provider = shift;
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $zone_present = 0;
	my $ldap_out_file = mktemp("/tmp/ldap_zone_list.XXXXXXXX");
	my $server_object = registry::getReg("DNSSERVER");

	Log(INFO, "Executing ... ".
		" /usr/bin/ldapsearch -h $frd_provider -p 1389 -D '$ldapadminname' -w * -s base -b '$server_object'".
		" -LLL -x -Z> $ldap_out_file");
	my @output = `/usr/bin/ldapsearch -h $frd_provider -p 1389 -D '$ldapadminname' \\
		-w '$ENV{'ADM_PASSWD'}' -s base \\
		-b '$server_object' -LLL -x -Z > $ldap_out_file`;
	my $return_status = $? >> 8;
	if ($return_status != 0) {
		unlink($ldap_out_file);
		Log(ERR, "ldapsearch failed with status [$return_status] [@output]");
	}

	my $rev_zone = `cat $ldap_out_file \\
			| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '\\
			| grep -i '$rev_zone_to_find,$rev_zone_context'`;
	if ($? == 0 and $rev_zone ne "") {
		$zone_present = 1;
	}
	unlink($ldap_out_file);

	return $zone_present;
}

#
# Make FRD the Zone server for any zone that is passed. This enables
# FRD to be aware of the zone if delegate zone is not created.
# This function is used primarily for Reverse Zone
#
sub dns_make_frd_zoneserver {

    # Get passed arguments
	my ($frd_provider, $zone_obj, $ldif_file) = @_;
	my ($ret, $count);
	my $forest_root = registry::getReg("FORESTROOT");
	my $frd_dns_server = "";
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $forestnc = registry::getReg("Forest NC");
	my $dns_server = registry::getReg("DNSSERVER");

	# Locating the FRD DNS Server
	Log ("Locating the Forest Root Domain DNS server [$frd_provider]");
	eval {
		# Checking if the obtained hostname is valid.
		my $frd_hostname = `echo $frd_provider | awk -F \".\" '{print \$1}'`;
		chomp($frd_hostname);

		Log ("Executing .. /usr/bin/ldapsearch  -o ldif-wrap=no  -h $frd_provider".
			" -E $LDAP_SERVER_NOMAPPING_OID -D '$ldapadminname' -w * -s sub -b  $forestnc".
			" (&(cn=DNS_$frd_hostname)(objectclass=dNIPDNSServer)) dn -Z -LLL -x");
		do {
			my @output = `/usr/bin/ldapsearch  -o ldif-wrap=no  -h $frd_provider\\
				-E $LDAP_SERVER_NOMAPPING_OID -D '$ldapadminname' \\
				-w '$ENV{'ADM_PASSWD'}' -s sub \\
				-b '$forestnc' "(&(cn=DNS_$frd_hostname)(objectclass=dNIPDNSServer))"\\
				dn -Z -LLL -x 2>&1 `;
			$ret = $? >> 8;
			$count ++;
			chomp(@output);
			if ($ret != 0) {
				Log ("Check for Forest Root Domain DNS Server Failed: [@output: $ret]".
					", rechecking..");
				sleep (5);
			} else {
				my @frd_dns_dn = grep (/^dn:\s+cn=DNS_$frd_hostname/i, @output);
				$frd_dns_server = $frd_dns_dn[0];
				$frd_dns_server =~ s/^dn:\s+//gi;
			}
		} while ($count < 20 && $ret != 0);

		if ($ret == 0 && $frd_dns_server ne "") {
			Log("Forest Root Domain DNS Server DN deduced as [$frd_dns_server]");
		} else {
			die("Failed to deduce the Forest Root Domain DNS Server DN");
		}

		#
		# Make sure the FRD DNS Server is not an authoritative for this zone
		# ALREADY. e.g., CDC or GDC is adding a new rev zone, where FRD DNS
		# is already owning this zone in TREE.
		#
		Log ("Executing .. /usr/bin/ldapsearch  -o ldif-wrap=no  -h $frd_provider".
			" -E $LDAP_SERVER_NOMAPPING_OID -D '$ldapadminname' -w * -s base".
			" -b '$frd_dns_server' dNIPZoneList -Z -LLL -x");
		my @output = `/usr/bin/ldapsearch  -o ldif-wrap=no  -h $frd_provider\\
			-E $LDAP_SERVER_NOMAPPING_OID -D '$ldapadminname' \\
			-w '$ENV{'ADM_PASSWD'}' -s base -b '$frd_dns_server'\\
			dNIPZoneList -Z -LLL -x 2>&1 `;
		$ret = $? >> 8;
		chomp(@output);
		if ($ret != 0) {
			die("Failed to check if Forest Root Domain DNS Server is".
				"already an authoritative DNS Server for zone [$zone_obj] ".
				"Error [@output: $ret]");
		} else {
			if (grep (/$zone_obj/i, @output) ) {
				Log("Forest Root Domain DNS Server is already a authoritative".
					" DNS Server for Zone [$zone_obj]");
				return;
			}
		}

		if (lc($frd_dns_server) ne lc($dns_server)) {
			Log("Adding Forest Root Domain DNS Server as authoritative for zone [$zone_obj]");
			util::file_append_str("\n", $ldif_file);
			util::file_append_str("dn: $zone_obj\n", $ldif_file);
			util::file_append_str("changetype: modify\n", $ldif_file);
			util::file_append_str("add: dNIPZoneServers\n", $ldif_file);
			util::file_append_str("dNIPZoneServers: $frd_dns_server\n", $ldif_file);

			util::file_append_str("\n", $ldif_file);
			util::file_append_str("dn: $frd_dns_server\n", $ldif_file);
			util::file_append_str("changetype: modify\n", $ldif_file);
			util::file_append_str("add: dNIPZoneList\n", $ldif_file);
			util::file_append_str("dNIPZoneList: $zone_obj\n", $ldif_file);

		}
	};

	if ($@) {
		Log(ERR, $@);
	}
}

#Create the forward and reverse zone
sub configure_zone_object
{

	my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $fwdzone_dnsmaster = registry::getReg("FWDZONE_DNSMASTER");
	my $revzone_dnsmaster = registry::getReg("REVZONE_DNSMASTER");
	my $forest_root = registry::getReg("FORESTROOT");
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $dns_server = registry::getReg("DNSSERVER");
	my $fwdzone_context = registry::getReg("FWDZONE_CONTEXT");
	my $revzone_context = registry::getReg("REVZONE_CONTEXT");
	my $is_replica = registry::getReg("IS_REPLICA");
	my $tempdir = registry::getReg("TEMPDIR");
	my $final_fwd_zone  = "$live_dsstatedir/dns/final-fwd-zone.ldif";
	my $zonefile = "$live_dsstatedir/dns/zone-object.ldif";

	my $final_rev_zone  = "$live_dsstatedir/dns/final-rev-zone.ldif";
	my $revzonefile = "$live_dsstatedir/dns/reverse-zone-object.ldif";
	my $update_zone_dn = "$live_dsstatedir/dns/update-zone-dn.ldif";
	my $zonemodfile = "$live_dsstatedir/dns/DNS-modify.ldif";
	my $revzonemodfile = "$live_dsstatedir/dns/DNS-modify-reverse-zones.ldif";

	my $fwd_zone = registry::getReg("DNSZONE");
	my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
	my $is_fwdzone_exist = registry::getReg("IS_FWDZONE_EXIST");
	my $is_revzone_exist = registry::getReg("IS_REVZONE_EXIST");
	my ($ret, $counter, $count);

	eval{
		my $frd_provider;
		if($install_type eq "frd" and $is_replica eq "FALSE") {
			$frd_provider = $dns_server_ip;
		}
		else {
			$frd_provider = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$forest_root  +short | cut -d " " -f4 | sed 's/\.\$//'`;
			if ($? || $frd_provider eq "") {
				my $error = $? >> 8;
				die ("Failed to get the domain provider for domain [$forest_root] [$error]");
			}
			chomp($frd_provider);
		}

		unlink($final_fwd_zone) if -e $final_fwd_zone;
		unlink($final_rev_zone) if -e $final_rev_zone;
		unlink($update_zone_dn) if -e $update_zone_dn;

		Log("   Creating zone ldif files in the directory");
		my ($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value)=(0, "", 0, "");
		# Preparing forward zone.
		if ($is_fwdzone_exist eq "FALSE") {
			Log("   Appending $zonefile to $final_fwd_zone");
			util::file_cat($zonefile, '>', $final_fwd_zone);
		}
		else {
			($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value) = util::is_policy_defined(zone => $fwd_zone, zone_context => $fwdzone_context, dns_server => $frd_provider, type_of_config => "INSTALL");

			Log("   Updating the existing forward zone $fwd_zone");
			util::file_append_str("dn: cn=$fwd_zone,$fwdzone_context\n", $final_fwd_zone);
			util::file_append_str("changetype: modify\n", $final_fwd_zone);

			# Whether dnipUpdatePolicy or dnipUpdateFilter ?
			# allow-update
			if($upd_filter_present) {
				util::file_append_str("replace: dNIPUpdateFilter\n", $final_fwd_zone);
				util::file_append_str("dNIPUpdateFilter: $upd_filter_value\n", $final_fwd_zone);
			} else {
				# update-policy
				if($upd_policy_present) {
					util::file_append_str("replace: dNIPUpdatePolicy\n", $final_fwd_zone);
				} else {
					util::file_append_str("add: dNIPUpdatePolicy\n", $final_fwd_zone);
				}
				util::file_append_str("dNIPUpdatePolicy: grant * sam * A NS SRV PTR AAAA KEY\n", $final_fwd_zone);
			}

			util::file_append_str("\n", $final_fwd_zone);
			util::file_append_str("dn: cn=$fwd_zone,$fwdzone_context\n", $final_fwd_zone);
			util::file_append_str("changetype: modify\n", $final_fwd_zone);
			util::file_append_str("add: dNIPZoneServers\n", $final_fwd_zone);
			util::file_append_str("dNIPZoneServers: $dns_server\n", $final_fwd_zone);
		}
		`sed -i "s/{DNSMaster}/$fwdzone_dnsmaster/g" $final_fwd_zone`;

		# Preparing reverse zone.
		if($is_revzone_exist eq "FALSE") {
			Log("   Appending $revzonefile to $final_rev_zone");
			util::file_cat($revzonefile, '>>', $final_rev_zone);
		} else {
			($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value) = util::is_policy_defined(zone => $rev_zone, zone_context => $revzone_context, dns_server => $frd_provider, type_of_config => "INSTALL");

			Log("   Updating the existing reverse zone $rev_zone");
			util::file_append_str("dn: cn=$rev_zone,$revzone_context\n", $final_rev_zone);
			util::file_append_str("changetype: modify\n", $final_rev_zone);
			# Whether dnipUpdatePolicy or dnipUpdateFilter ?
			# allow-update
			if($upd_filter_present) {
				util::file_append_str("replace: dNIPUpdateFilter\n", $final_rev_zone);
				util::file_append_str("dNIPUpdateFilter: $upd_filter_value\n", $final_rev_zone);
			} else {
				if($upd_policy_present) {
					util::file_append_str("replace: dNIPUpdatePolicy\n", $final_rev_zone);
				} else {
					util::file_append_str("add: dNIPUpdatePolicy\n", $final_rev_zone);
				}
				util::file_append_str("dNIPUpdatePolicy: grant * sam * NS PTR AAAA KEY\n", $final_rev_zone);
			}
			util::file_append_str("\n", $final_rev_zone);
			util::file_append_str("dn: cn=$rev_zone,$revzone_context\n", $final_rev_zone);
			util::file_append_str("changetype: modify\n", $final_rev_zone);
			util::file_append_str("add: dNIPZoneServers\n", $final_rev_zone);
			util::file_append_str("dNIPZoneServers: $dns_server\n", $final_rev_zone);
		}

		`sed -i "s/{DNSMaster}/$revzone_dnsmaster/g" $final_rev_zone`;

		# Non-FRD DC needs to be considered including FADC. where it may bring in
		# new reverse zone to Domain
		if($install_type ne "frd" or $is_replica eq "TRUE") {
			#
			# Make FRD the Zone server for any new reverse zone that a DC will exist in.
			# This enables FRD to resolve any reverse zone in the forest and
			# hence any WKS joining to the the domain will be able to update
			# its Reverse PTR record dynamically by forwarding
			# the dynamic update request to the reverse zone designated server.
			#
			dns_make_frd_zoneserver($frd_provider,
				"cn=$rev_zone,$revzone_context",
				$final_rev_zone);
		}

		if (! -e "$tempdir/add_dns_zone") {
			Log("Creating/Updating the forward zone object");
			# Here during dns record import we are going over GC port(#1636)
			# as these records might not be lying on DC with ip dns_server_ip.
			$ret = import::import_file(host=>$dns_server_ip, port=>1636, admin=>$ldapadminname, ssl=>"yes",
					pwd=>$ENV{'ADM_PASSWD'}, file=> $final_fwd_zone, cont_if_exist=>"true");
			if ($ret != 0) {
				Log("Retrying to locally import $final_fwd_zone");
				$ret = import::import_file(host=>"127.0.0.1", port=>636, admin=>$ldapadminname, ssl=>"yes",
				pwd=>$ENV{'ADM_PASSWD'}, file=> $final_fwd_zone, cont_if_exist=>"true");
			}
			if ($ret != 0) {
				die("Could not import $final_fwd_zone");
			}

			Log("Creating/Updating the reverse zone object");
			# Need to go over GC port(#1636) as we are directly contacting
			# frd_provider and new record may be with installing machine
			$ret = import::import_file(host=>$frd_provider, port=>1636, admin=>$ldapadminname, ssl=>"yes",
					pwd=>$ENV{'ADM_PASSWD'}, file=> $final_rev_zone, cont_if_exist=>"true");
			if ($ret != 0) {
			Log("Retrying caching/updating locally");
			$ret = import::import_file(host=>"127.0.0.1", port=>1636, admin=>$ldapadminname, ssl=>"yes",
					pwd=>$ENV{'ADM_PASSWD'}, file=> $final_rev_zone, cont_if_exist=>"true");
			}
			if ($ret != 0) {
				die("Could not import $final_rev_zone");
			}

			# Checking for forward zone creation
			Log ("Checking for the creation of forward zone [cn=$fwd_zone,$fwdzone_context]");
			Log ("Executing .. LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no -H ldaps://$frd_provider:1636".
				" -D '$ldapadminname' -w * -s base".
				" -b cn=$fwd_zone,$fwdzone_context dn -Z -LLL -x");
			$count = 0;
			do {

				if($count % 2 == 0) {
                                        Log(INFO, "Trying import on 1636");
                                        $port_dsfw = 1636;
                                }
                                else {
                                        Log(INFO, "Trying import on 636");
                                        $port_dsfw = 636;
                                }

				my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no -H ldaps://$frd_provider:$port_dsfw \\
					-D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' \\
					-s base -b 'cn=$fwd_zone,$fwdzone_context' \\
					dn -Z -LLL -x 2>&1 `;
				$ret = $? >> 8;
				if ($ret != 0) {
					Log ("Retrying check for forward zone locally");
					my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  \\
					-D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' \\
					-s base -b 'cn=$fwd_zone,$fwdzone_context' \\
					dn -Z -LLL -x 2>&1 `;
				}
				$count ++;
				chomp(@output);
				if ($ret != 0) {
					Log ("Check for forward zone record failed: [@output: $ret], rechecking");
					sleep (5);
				}

			} while ($count < 20 && $ret != 0);

			Log ("Forward zone [cn=$fwd_zone,$fwdzone_context] is created successfully \n") if ($ret == 0);

			# Checking for reverse zone creation
			$count = 0;
			Log ("Checking for the creation of reverse zone [cn=$rev_zone,$revzone_context] on [$dns_server_ip]");
			if(not ($install_type eq "frd" and $is_replica eq "FALSE")) {
								Log ("Executing .. /usr/bin/ldapsearch -o ldif-wrap=no -h $frd_provider -p $port_dsfw -D '$ldapadminname' -w * -s base -b cn=$rev_zone,$revzone_context dn -x -Z -LLL");
				do {
					if($count % 2 == 0) {
                                        Log(INFO, "Trying import on 1389");
                                        $port_dsfw = 1389;
                                }
                                else {
                                        Log(INFO, "Trying import on 389");
                                        $port_dsfw = 389;
                                }

					my @output = `/usr/bin/ldapsearch -o ldif-wrap=no -h $frd_provider -p $port_dsfw -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' -s base -b 'cn=$rev_zone,$revzone_context' dn -Z -x -LLL 2>&1 `;
					$ret = $? >> 8;
					chomp(@output);
					$count ++;
					if ($ret != 0) {
						Log ("Check for reverse zone record failed: [@output: $ret], rechecking");
						sleep (5);
					}
				} while ($count < 20 && $ret != 0);
			}

			Log ("Reverse zone [cn=$rev_zone,$revzone_context] is created successfully on  [$dns_server_ip]") if ($ret == 0);
			`touch $tempdir/add_dns_zone`;
		}
		else {
			Log("The zones were created already, skipping it.");
		}

		# Updating zones dn in DNS_DHCP group and DNS server.
		my $dnsmaster = registry::getReg("DNSMASTER");
		$counter = 0;

		if($is_replica eq "FALSE" or $dnsmaster eq "TRUE") {
			Log("   Appending $zonemodfile to $update_zone_dn");
			util::file_cat($zonemodfile, '>', $update_zone_dn);
		}
		if($is_revzone_exist eq "FALSE" or $dnsmaster eq "TRUE") {
			Log("   Appending $revzonemodfile to $update_zone_dn");
			util::file_cat($revzonemodfile, '>>', $update_zone_dn);
		}
		elsif($is_revzone_exist eq "TRUE"
			and $dnsmaster eq "FALSE"
			and is_reverse_zone_present($rev_zone, $revzone_context, $frd_provider) eq 0) {

			Log("   Appending $revzonemodfile to $update_zone_dn");
			util::file_cat($revzonemodfile, '>>', $update_zone_dn);
		}

		if(-e $update_zone_dn) {
			do {
				if ($counter > 0) {
					Log(INFO, "Retrying to import the file $update_zone_dn, retry count:$counter");
					sleep(5);
				}

				if($counter % 2 == 0) {
					Log(INFO, "Trying import on 1636");
					$port_dsfw = 1636;
				}
				else {
					Log(INFO, "Trying import on 636");
					$port_dsfw = 636;
				}
				# Here during dns record import we are going over GC port(#1636)
				# as these records might not be lying on DC with ip dns_server_ip.
				$ret = import::import_file(host=>$dns_server_ip, port=>$port_dsfw, admin=>$ldapadminname, ssl=>"yes",
						pwd=>$ENV{'ADM_PASSWD'}, file=> $update_zone_dn, cont_if_exist=>"true", edir_schematic_cntl=>"true");
				if($ret != 0) {
					Log(INFO, "Retrying to import locally");
					$ret = import::import_file(host=>"127.0.0.1", port=>$port_dsfw, admin=>$ldapadminname, ssl=>"yes",
					pwd=>$ENV{'ADM_PASSWD'}, file=> $update_zone_dn, cont_if_exist=>"true", edir_schematic_cntl=>"true");
				}

				$counter ++;

			} while ($ret !=0 and $counter < $retry_count);

			if ($ret != 0) {
				die("Could not import $update_zone_dn");
			}
		}

	};

	if ($@) {Log(ERR, $@)}
}

#Create the delegate zone
sub configure_delegate_zone_object
{

	my $my_node_name = registry::getReg("DNS Root");
	my $forest_root = registry::getReg("FORESTROOT");
	my $is_adc = registry::getReg("IS_REPLICA");
	my $my_domain_ip = registry::getReg("IPADDRESS");
	my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $dns_master = registry::getReg("DNS Master");
	my $install_type = registry::getReg("INSTALL_TYPE");
	# Flag to know if delegate zone is already present
	my $Zone_Exists = "FALSE";
	my @zone_ctx = ();
	my $forest_dns_ip;
	my $fwd_zone = registry::getReg("DNSZONE");
	my $delegate_Ldif_name;
	my $update_dnsserver = "/var/opt/novell/xad/ds/dns/RR/add_server_forwarder.ldif";
	my $counter = 0;
	my @output = ();
	my $ret = -1;
	my $count = 0;

	eval {

		if ($install_type  ne "frd" or $is_adc eq "TRUE") {

			# PDC need not be the root DNS provider. So perform dig and identify
			# the Forest root DOMAIN DSfW DNS Server IP.
			my @frd_ns = `dig NS $forest_root +short`;
			$ret = $?;
			if (($ret == 0) && (@frd_ns)) {
				chomp(@frd_ns);
				my @forest_root_NS = split(" ", "@frd_ns");
				my $nameserver_count = @forest_root_NS;

				if ($nameserver_count == 0) {
					# Probably there is a failure at previous dig on our FRD.
					die("Could not locate any DNS Server for Forest Root Domain $forest_root");
				}

				for(my $counter1 = 0; $counter1 < $nameserver_count; $counter1++) {
					$forest_dns_ip = `dig $forest_root_NS[$counter1] +short`;
					my $rc = $?;
					chomp($forest_dns_ip);
					if (($rc != 0) || ($forest_dns_ip eq "")) {
						Log(WARN, "Could not fetch IP for Forest Root Domain Controller DNS Server".
									" $forest_root_NS[$counter1]");
					} else {
						# Checking if the obtained DNS IP is that of FRD DSfW server.
						# Need not be a PDC though.

						if ($forest_dns_ip eq $my_domain_ip) {
							# we are not interested, FRD ADC IP case.
							next;
						}

						my $count = 5;
						do {
							@output = `/opt/novell/eDirectory/bin/ldapsearch\\
								-x -h $forest_dns_ip -s base -b ""`;
							$ret = $? >> 8;
							chomp(@output);
							if ($ret != 0){
								Log("RootDSE search on [ $forest_dns_ip ] failed,".
									"error code returned [ $ret ]\n Retrying validation...");
								$count--;
								sleep 3;
							}
						} while (($ret != 0) && ($count > 0));
						# Make sure the server is a DSfW server
						if (($ret == 0) && grep (/2.16.840.1.113719.1.513.7.1/, @output)) {
							Log("Identified the DNS Server for DSfW Forest Root".
								" Domain - $forest_dns_ip");
							last;

						} elsif ($ret == 0){
							Log("Identified that DNS Server [ $forest_dns_ip ] is a non-DSfW".
								" DNS Server. Skipping.");
						}
					}
				}
			} else {
				die("Could not locate any DNS Server for Forest Root Domain".
					" $forest_root [$ret]");
			}

			if ($forest_dns_ip eq "") {
				Log ("Failed to obtain Forest Root Domain Controller IP");
				goto fatal_error;
			}
		}

		#########################################################
		# Set the forwarder list on the DNS being configured    #
		#########################################################

		# We let in FRD ADC in this loop if it has its own DNS server
		# to add the FRD as forward list to itself.
		# Else it wont be able to resolve reverse zone that FRD is authoritative for.


		if (($install_type  ne "frd" or $is_adc eq "TRUE") and $dns_master eq "TRUE") {
			# Add zone forwardList on all new DNS server.
			# pack "ccC4", $ber_len, $FWD_value{List_Count}, $FWD_value{IP_Data_len}, split(/\./, "$IP_Value{Data}");
			my $finalbitstring = pack "ccC4", "1", "4", split(/\./, "$forest_dns_ip");
			my $ip_base64 = XAD::encodeBase64($finalbitstring);

			# Get this DNS server DN and set the forwarder on it. This applies to all Zones on this server.
			my $dns_server = registry::getReg("DNSSERVER");
			Log("   Appending Forest Root Domain nameserver forwarder details to $update_dnsserver");
			util::file_append_str("dn: $dns_server\n", $update_dnsserver);
			util::file_append_str("changetype: modify\n", $update_dnsserver);
			util::file_append_str("replace: dNIPFwdList\n", $update_dnsserver);
			# We forward all unknown domain queries to the Root DNS (FRD)
			# FRD will be one source of info all zones in Tree.
			util::file_append_str("dNIPFwdList:: $ip_base64\n", $update_dnsserver);

			# These records are getting updated while I tried setting forwarder using JC Console.
			# May be needed for internal house-keeping.. So blindly applying those.
			Log("   Appending  appropriate Server Options to $update_dnsserver");
			util::file_append_str("\n", $update_dnsserver);
			util::file_append_str("dn: $dns_server\n", $update_dnsserver);
			util::file_append_str("changetype: modify\n", $update_dnsserver);
			util::file_append_str("replace: dNIPSNMPTrapFlag\n", $update_dnsserver);
			util::file_append_str("dNIPSNMPTrapFlag: 1\n", $update_dnsserver);

			util::file_append_str("\n", $update_dnsserver);
			util::file_append_str("dn: $dns_server\n", $update_dnsserver);
			util::file_append_str("changetype: modify\n", $update_dnsserver);
			util::file_append_str("replace: dNIPDNSServerOptions\n", $update_dnsserver);
			util::file_append_str("dNIPDNSServerOptions: 1\n", $update_dnsserver);

			if(-e $update_dnsserver) {
				do {
					if ($counter > 0) {
						Log(INFO, "Retrying to import the file $update_dnsserver, retry count:$counter");
						sleep(5);
					}
					$ret = import::import_file(host=>$dns_server_ip, port=>1636, admin=>$ldapadminname, ssl=>"yes",
							pwd=>$ENV{'ADM_PASSWD'}, file=> $update_dnsserver, cont_if_exist=>"true", edir_schematic_cntl=>"true");
					if ($ret != 0) {
						Log(WARN,"Could not import $update_dnsserver");
					}
					$counter ++;

				} while ($ret !=0 and $counter < $retry_count);

				if ($ret != 0) {
					goto fatal_error;
				}
			} else {
				goto fatal_error;
			}

		}

		#############################################################################
		# Create a delegate zone for the Domain being configured in the parent zone #
		#############################################################################
		if ($install_type ne "frd") {
			my $ret = -1;
			my $count = 0;
			my $parent_domain = registry::getReg("Parent Domain");
			my $frd_zone_name = $forest_root;
			my @dns_locator;

			# This helps any domain in tree to resolve this new domain.
			# my_node_name will always be the domain FQDN stripping
			# the Parent domain e.g., for grand child domain it will be
			# gchild if fqdn is gchild.child.frd.com.

			# Obtain FRD forward zone object
			$frd_zone_name =~ s/\./_/g;

			if ($frd_zone_name eq $fwd_zone) {
				# Must be FRD/FRD-ADC -- We shouldn't have been here
				return;
			}

			$my_node_name =~ s/\.$parent_domain//gi;
			$delegate_Ldif_name = "/var/opt/novell/xad/ds/dns/RR/$my_node_name"."_Delegate.ldif";
			if (-e "$delegate_Ldif_name") {
				unlink("$delegate_Ldif_name");
			}

			# Obtain the frd zone context from locator object context
			my $dns_ncpserver = registry::getReg("DNS_NCPServer");
			Log ("Obtaining Forest Root Domain forward zone context for [cn=$frd_zone_name]");
			Log ("Obtained NCP Server Context [$dns_ncpserver]");

			# Obtain the locator object context
			Log ("Executing .. LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -H ldaps://$forest_dns_ip:1636".
									" -D '$ldapadminname' -w * -s base".
									" -b '$dns_ncpserver' dNIPLocatorPtr dn -Z -LLL -x");
			do {
				my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -x -Z -H ldaps://$forest_dns_ip:1636 \\
					-D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' \\
					-s base -b '$dns_ncpserver' dNIPLocatorPtr  -LLL`;
				$ret = $? >> 8;
				$count ++;
				chomp(@output);
				if ($ret != 0) {
					Log ("Failed to obtain the DNS locator object context: [@output: $ret], rechecking");
					sleep (5);
				} else {
					@dns_locator = grep(/dNIPLocatorPtr:\s+/i,@output);
					$dns_locator[0] =~ s/^dNIPLocatorPtr:\s+//gi;
					if ($dns_locator[0] eq "") {
						$ret = -1;
					}
				}
			} while ($count < 10 && $ret != 0);

			if ($ret != 0 or $dns_locator[0] eq "") {
				goto fatal_error;
			}

			$parent_domain =~ s/\./_/g;
			Log ("Obtaining zone context for [cn=$parent_domain]");
			Log ("Executing .. `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -H ldaps://$forest_dns_ip:1636".
					" -D '$ldapadminname' -w * -s base".
					" -b '$dns_locator[0]' dNIPDNSZones dn -Z -LLL -x");
			do {
				my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -x -Z -H ldaps://$forest_dns_ip:1636 \\
					-D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' \\
					-s base -b  '$dns_locator[0]' dNIPDNSZones -LLL`;
				$ret = $? >> 8;
				$count ++;
				chomp(@output);
				if ($ret != 0) {
					Log ("Failed to obtain Forest Root Domain DNS zone object context: [@output: $ret], rechecking");
					sleep (5);
				} else {
					@zone_ctx = grep(/dNIPDNSZones:\s+cn=$parent_domain/i, @output);
					$zone_ctx[0] =~ s/dNIPDNSZones:\s+//gi;
					if ($zone_ctx[0] eq "") {
						Log ("Failed to obtain the Parent Domain zone object context @zone_ctx, rechecking");
						$ret = -1;
					} else {
						# Make sure if we are ADC and there was no DNS configured for this domain yet
						# The delegate zone is created only on local DNS configuration and
						# for the case where ADC is the first DNS of the DOMAIN then a new
						# delegate zone has to be created.
						if ($is_adc eq "TRUE") {
							Log ("Executing .. LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -H ldaps://$forest_dns_ip:1636".
								" -D '$ldapadminname' -w * -s base".
								" -b 'cn=$my_node_name,$zone_ctx[0]' dn -Z -LLL -x");
							my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -x -Z -H ldaps://$forest_dns_ip:1636 \\
								-D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' -o ldif-wrap=no  \\
								-s base -b 'cn=$my_node_name,$zone_ctx[0]' dn -LLL`;
							$ret = $? >> 8;
							chomp(@output);
							if ($ret == $LDAP_NO_SUCH_OBJECT) {
								Log ("Additional DC is hosting first DNS for the domain and ".
									"requires a delegate zone to be created");
								$Zone_Exists = "FALSE";
								$ret = 0;
								goto PROCESS_DELEGATE_RR;
							} elsif (grep(/dn:\s+cn=$my_node_name/i, @output) && $ret == 0) {
								$Zone_Exists = "TRUE";
							}
						}
					}
				}

			} while ($count < 10 && $ret != 0);
PROCESS_DELEGATE_RR:
			if ($ret == 0 and $zone_ctx[0] ne "") {
				Log ("Parent Domain Forward zone context deduced as $zone_ctx[0]");
			} else {
				goto fatal_error;
			}

			dns::Process_delegate_RR($my_node_name, $my_domain_ip,
						 "$zone_ctx[0]", 0, $Zone_Exists, "add");
			Log ("Created delegate zone records $delegate_Ldif_name");


			# Need to update the SOA serial number to make sure zone db is upto date.
			my $soa_value_dns = `/opt/novell/xad/share/dcinit/provisionTools.sh soa-query -p $forest_dns_ip -a '$ldapadminname' -c '$zone_ctx[0]'`;
			chomp($soa_value_dns);
			if ($soa_value_dns gt 0) {
				util::file_append_str("\n", $delegate_Ldif_name);
				util::file_append_str("dn: $zone_ctx[0]\n", $delegate_Ldif_name);
				util::file_append_str("changetype: modify\n", $delegate_Ldif_name);
				util::file_append_str("replace: dNIPSOASerial\n", $delegate_Ldif_name);
				util::file_append_str("dNIPSOASerial: $soa_value_dns\n", $delegate_Ldif_name);
			} else {
				Log(WARN,"Failed to retreive dNIPSOASerial attribute. Forest Root Domain DNS cache needs to be manually refreshed");
			}

			if (! -e $delegate_Ldif_name) {
				goto fatal_error;
			}
			# Import the delegate zone records.
			$counter = 0;
			do {
				if ($counter > 0) {
					Log(INFO, "Retrying to import the file $delegate_Ldif_name, retry count:$counter");
					sleep(5);
				}
				$ret = import::import_file(host=>$forest_dns_ip, port=>1636, admin=>$ldapadminname, ssl=>"yes",
								  pwd=>$ENV{'ADM_PASSWD'}, file=> "$delegate_Ldif_name", cont_if_exist=>"true");
				if ($ret != 0) {
					Log(WARN,"Could not import $delegate_Ldif_name");
				}
				$counter ++;
			} while ($ret != 0 and $counter < $retry_count);

			if ($ret != 0) {
fatal_error:
				Log (WARN, "Make necessary setting manually for parent and child DNS servers to resolve each other.");
				die("Delegate zone creation at Forest Root Domain domain to this domain failed");
			}
		}

	};

	if ($@) {Log(ERR, $@)}
}



#Create the forward and reverse zone
sub configure_zone_rr_object
{

	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $dnsroot = registry::getReg("DNSROOT");
	my $ipaddress = registry::getReg("IPADDRESS");
	my $fwdzone_context = registry::getReg("FWDZONE_CONTEXT");
	my $revzone_context = registry::getReg("REVZONE_CONTEXT");
	my $reverse_dns_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
	my $is_replica = registry::getReg("IS_REPLICA");
	my $fwdzone_dnsmaster = registry::getReg("FWDZONE_DNSMASTER");
	my $revzone_dnsmaster = registry::getReg("REVZONE_DNSMASTER");
	my $soa_entry_file;

	my $final_fwdzone_records = "$live_dsstatedir/dns/final-fwdzone-records.ldif";
	my $final_revzone_records = "$live_dsstatedir/dns/final-revzone-records.ldif";
	my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
	my $ptr_rrset = registry::getReg("REVERS_PTR_IP");
	my $count = 0;

	eval{
		my $frd_provider;
		my $install_type = registry::getReg("INSTALL_TYPE");
		if($install_type eq "frd" and $is_replica eq "FALSE") {
			$frd_provider = $dns_server_ip;
		}
		else {
			my $forest_root = registry::getReg("FORESTROOT");
			$frd_provider = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$forest_root  +short | cut -d " " -f4 | sed 's/\.\$//'`;
			if ($? || $frd_provider eq "") {
				my $error = $? >> 8;
				die ("Failed to get the domain provider for domain [$forest_root] [$error]");
			}
			chomp($frd_provider);
		}

		if (-d "$live_dsstatedir/dns/RR") {
			unlink <$live_dsstatedir/dns/RR/*.ldif>;
		}
		else {
			mkpath("$live_dsstatedir/dns/RR");
		}

		if ($is_replica eq "FALSE") {

			if (-f "$live_dsstatedir/dns/RR/$dnsroot.zone") {
				Log("Removing $live_dsstatedir/dns/RR/$dnsroot.zone");
				unlink("$live_dsstatedir/dns/RR/$dnsroot.zone");
			}

			Log(MESG, "   Generating $live_dsstatedir/dns/RR/$dnsroot.zone");
			stage::reg_substitute("$templatedir/dns.zone", "$live_dsstatedir/dns/RR/$dnsroot.zone");
			`sed -i "s/{DNSMaster}/$fwdzone_dnsmaster/g"  $live_dsstatedir/dns/RR/$dnsroot.zone`;
			`sed -i "s/{DNSServerIP}/$dns_server_ip/g"  $live_dsstatedir/dns/RR/$dnsroot.zone`;


			if (-f "$live_dsstatedir/dns/RR/$ipaddress.zone") {
				Log("Removing $live_dsstatedir/dns/RR/$ipaddress.zone");
				unlink("$live_dsstatedir/dns/RR/$ipaddress.zone");
			}
			Log(MESG, "   Generating $live_dsstatedir/dns/RR/$ipaddress.zone");
			stage::reg_substitute("$templatedir/dc-reverse.zone", "$live_dsstatedir/dns/RR/$ipaddress.zone");
			`sed -i "s/{DNSMaster}/$revzone_dnsmaster/g"  $live_dsstatedir/dns/RR/$ipaddress.zone`;


			Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/$dnsroot.zone\n");
			dns::Process_DNS_Files("$live_dsstatedir/dns/RR/$dnsroot.zone", "$fwdzone_context", "$is_replica", "add");

			Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/$ipaddress.zone, $revzone_context, $is_replica, add\n");
			dns::Process_DNS_Files("$live_dsstatedir/dns/RR/$ipaddress.zone", "$revzone_context", "$is_replica", "add");
		}
		else {
			my $dns_zone = registry::getReg("DNSZONE");
			$soa_entry_file = registry::getReg("LIVE_DSSTATEDIR")."/dns/Modify-SOA-Entry.ldif";
			my $soa_value = registry::getReg("SOAVALUE");

			unlink($soa_entry_file) if (-e $soa_entry_file);
			util::file_append_str("dn: cn=$dns_zone,$fwdzone_context\n", $soa_entry_file);
			util::file_append_str("changetype: modify\n", $soa_entry_file);
			util::file_append_str("replace: dNIPSOASerial\n", $soa_entry_file);
			util::file_append_str("dNIPSOASerial: $soa_value\n", $soa_entry_file);

			stage::reg_substitute("$templatedir/additional-dc.zone", "$live_dsstatedir/dns/RR/ADC_$dnsroot.zone");

			stage::reg_substitute("$templatedir/dc-reverse.zone",  "$live_dsstatedir/dns/RR/$ipaddress.zone");
			`sed -i "s/{DNSMaster}/$revzone_dnsmaster/g"  $live_dsstatedir/dns/RR/$ipaddress.zone`;

			Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/ADC_$dnsroot.zone, $fwdzone_context, $is_replica, add\n");
			dns::Process_DNS_Files("$live_dsstatedir/dns/RR/ADC_$dnsroot.zone", $fwdzone_context, "TRUE", "add");

			Log(MESG, "   Generating ldifs from $live_dsstatedir/dns/RR/$ipaddress.zone, $revzone_context, $is_replica, add\n");
			dns::Process_DNS_Files("$live_dsstatedir/dns/RR/$ipaddress.zone", $revzone_context, "TRUE", "add");
		}

		if (! nds::check_certificate()) {
			nds::get_certificate();
		}

		unlink("$final_fwdzone_records") if -e "$final_fwdzone_records";
		unlink("$final_revzone_records") if -e "$final_revzone_records";

		if ($is_replica eq "TRUE") {
		    my $dnsmaster = registry::getReg("DNSMASTER");
		    my $zone_name = registry::getReg("DNS Host Name");
		    my $InstallMachineName = lc(registry::getReg ("InstallMachineName"));

		    $zone_name = substr($zone_name, (length($InstallMachineName) + 1), length($zone_name));

		    my $nsfile = "/var/opt/novell/xad/ds/dns/RR/\@_NS_"."$zone_name".".ldif";
		    my $nsupdatefile = "/var/opt/novell/xad/ds/dns/RR/\@_NS_update_"."$zone_name".".ldif";
		    unlink ("$nsfile") if ($dnsmaster eq "FALSE");
		    unlink ("$nsupdatefile") if ($dnsmaster eq "FALSE");
		}

		# Generating final forward/reverse zone ldif files.
		Log(MESG, "   Generating final forward/reverse zone ldif files [$final_fwdzone_records; $final_revzone_records]");
		my @flist = <$live_dsstatedir/dns/RR/*.ldif>;
		my @modifylist;
		my $revIP = $reverse_dns_zone;
		$revIP =~ s/_/\./g;

		foreach (@flist) {
			my $filename = $_;
			if($filename =~ /$revIP/) {
				util::file_cat($_, ">>", $final_revzone_records);
				next;
			}
			open(INFILE, $filename);
			my @content = <INFILE>;

			# Sorting based on changetype, changetype:add would always come first.
			if(grep(/changetype: modify/i, @content)) {
				push(@modifylist, $filename);
			}
			else {
				util::file_cat($_, ">>", $final_fwdzone_records);
			}
		}

		# All changetype: modify are appended at the tail.
		foreach(@modifylist)
		{
			util::file_cat($_, ">>", $final_fwdzone_records);
		}

		# Need to update the SOA serial number to make sure reverse zone db is upto date.
		my $soa_value_rev_dns = `/opt/novell/xad/share/dcinit/provisionTools.sh soa-query -p $dns_server_ip -a '$ldapadminname' -c 'cn=$reverse_dns_zone,$revzone_context'`;
		util::file_append_str("dn: cn=$reverse_dns_zone,$revzone_context\n", $final_revzone_records);
		util::file_append_str("changetype: modify\n", $final_revzone_records);
		util::file_append_str("replace: dNIPSOASerial\n", $final_revzone_records);
		util::file_append_str("dNIPSOASerial: $soa_value_rev_dns\n", $final_revzone_records);

		my ($ret, $counter);

		# Update forward zone records.
		$counter = 0;
		do {
			if ($counter > 0) {
				Log(INFO, "Retrying to import the file $final_fwdzone_records, retry count:$counter");
				sleep(5);
			}

			if($counter % 2 == 0) {
                                        Log(INFO, "Trying import on 1636");
                                        $port_dsfw = 1636;
                        }
                                else {
                                        Log(INFO, "Trying import on 636");
                                        $port_dsfw = 636;
                        }

			# Here during dns record import we are going over GC port(#1636)
			# as these records might not be lying on DC with ip dns_server_ip.
			$ret = import::import_file(host=>$dns_server_ip, port=>$port_dsfw, admin=>$ldapadminname, ssl=>"yes",
						      pwd=>$ENV{'ADM_PASSWD'}, file=> "$final_fwdzone_records", cont_if_exist=>"true");
			if ($ret != 0) {
				Log("Retrying locally");
				$ret = import::import_file(host=>"127.0.0.1", port=>$port_dsfw, admin=>$ldapadminname, ssl=>"yes",
						      pwd=>$ENV{'ADM_PASSWD'}, file=> "$final_fwdzone_records", cont_if_exist=>"true");
			}
			if ($ret != 0) {
				Log(WARN,"Could not import $final_fwdzone_records");
			}
			$counter ++;
		} while ($ret != 0 and $counter < $retry_count);
		if ($ret != 0) {
			die("Could not import $final_fwdzone_records");
		}

		# Update reverse zone records.
		$counter = 0;
		do {
			if ($counter > 0) {
				Log(INFO, "Retrying to import the file $final_revzone_records, retry count:$counter");
				sleep(5);
			}

			if($counter % 2 == 0) {
                                        Log(INFO, "Trying import on 1636");
                                        $port_dsfw = 1636;
                        }
                                else {
                                        Log(INFO, "Trying import on 636");
                                        $port_dsfw = 636;
                        }

			$ret = import::import_file(host=>$frd_provider, port=>$port_dsfw, admin=>$ldapadminname, ssl=>"yes",
						      pwd=>$ENV{'ADM_PASSWD'}, file=> "$final_revzone_records", cont_if_exist=>"true");
			if ($ret != 0) {
				Log(WARN,"Could not import $final_revzone_records");
			}
			$counter ++;
		} while ($ret != 0 and $counter < $retry_count);
		if ($ret != 0) {
			die("Could not import $final_revzone_records");
		}

		if ($is_replica eq "TRUE") {
			# SOA entry file will contain only one ldap entry. If fails task itself can be retried.
			# Most likely if the above ldap operation passes, this soa entry also would get added.
			# Going over GC port(#1636) as these records might not be lying on DC with ip dns_server_ip.
			my $rc = system("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$dns_server_ip:1636 -x -D ".
					"'$ldapadminname' ".
					"-w '$ENV{ADM_PASSWD}' -f $soa_entry_file");

			if ($rc != 0) {
				Log("Retrying locally");
				$rc = system("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -x -D ".
					"'$ldapadminname' ".
					"-w '$ENV{ADM_PASSWD}' -f $soa_entry_file");

			}
			if ($rc != 0) {
				die("Could not update the soa value");
			}
		}

		# Checking the reverse zone creation for all but FRD
		if(not ($install_type eq "frd" and $is_replica eq "FALSE")) {


			# Checking for reverse zone ptr record creation
			$ptr_rrset =~ s/\./_/g;
			Log ("Checking for the creation of reverse zone PTR record ".
				"[cn=$ptr_rrset,cn=$rev_zone,$revzone_context]");
			Log ("Executing .. LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  ".
				"-H ldaps://$frd_provider:$port_dsfw -D '$ldapadminname' -w * -s base".
				" -b 'cn=$ptr_rrset,cn=$rev_zone,$revzone_context' dn -Z -LLL -x");
			do {
				if($count % 2 == 0) {
						Log(INFO, "Trying import on 1636");
						$port_dsfw = 1636;
	                        }
                                else {
					Log(INFO, "Trying import on 636");
					$port_dsfw = 636;
				}

				my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -o ldif-wrap=no  -H ldaps://$frd_provider:$port_dsfw \\
					 -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' -s base \\
					 -b 'cn=$ptr_rrset,cn=$rev_zone,$revzone_context' dn -Z -LLL -x 2>&1`;
				chomp(@output);
				$ret = $? >> 8;
				$count ++;
				if ($ret != 0) {
					Log ("Check for reverse zone ptr record failed: [@output: $ret], rechecking");
					sleep (5);
				}

			} while ($count < 20 && $ret != 0);

			if ($ret == 0) {
				Log ("Reverse zone PTR record [cn=$ptr_rrset,cn=$rev_zone,$revzone_context] is successfully created on [$frd_provider]")
			}else {
				Log ("Reverse zone PTR record [cn=$ptr_rrset,cn=$rev_zone,$revzone_context] is not created on [$frd_provider]. \nThe DNS task might fail. Retry the DNS task in the DSfW provisioning wizard.")
			}
		}
	};

	if ($@) {Log(ERR, $@)}
}

1;
