#!/usr/bin/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2004
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         migrate.pl
#
#   FACILITY:
#
#         Domain Controller Migration Tool
#
#   ABSTRACT:
#
#         This module is responsible for the configuration defaults of a
#         migrated domain.
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package XAD;

use XAD::registry;
use XAD::replib;
use XAD::schema;

my $rcsId = '$Id$';

# Returns
#
#	-1	Attribute not in schema
#	0	Exclude attribute
#	1	Include attribute
#	2	Include attribute (binary)
#	3	Include attribute (linked)
#
sub actionForAttribute
{
	my ($schemaHash, $attribute) = @_;
	my $schemaElement;
	my $linkID;
	my $systemFlags;
#	my $syntax, $omSyntax, $omObjectClass;

	if (uc($attribute) eq "DN" ||
	    uc($attribute) eq "NAME" ||
	    uc($attribute) eq "WHENCREATED" ||
	    uc($attribute) eq "CHANGETYPE" ||
	    uc($attribute) eq "CONTROL" ||
	    uc($attribute) eq "MSDS-HASINSTANTIATEDNCS" ||
	    uc($attribute) eq "MSDS-HASMASTERNCS")
	{
		return 0;
	}

	$schemaElement = $schemaHash->{uc($attribute)};
	if (!defined($schemaElement)) {
		return -1;
	}

	$linkID = $schemaElement->{'LINKID'}->[0];
	if ($linkID != 0) {
		if ($linkID % 2) {
			return 0; # attribute is back-linked
		} else {
			return 3; # attribute is forward-linked
		}
	}

	$systemFlags = $schemaElement->{'SYSTEMFLAGS'}->[0];

	# XXX some hard-coded exceptions
	if (uc($attribute) =~ /^RID/) {
		$systemFlags &= ~(0x00000001);
	} elsif (uc($attribute) eq "USNCREATED" && $OFFLINE == 1) {
		$systemFlags &= ~(0x00000001);
	}

	if ($systemFlags & 0x00000004) {
		return 0; # attribute is constructed
	} elsif ($systemFlags & 0x00000001) {
		return 0; # attribute is non-replicated
	}

#	$syntax = $schemaElement->{'ATTRIBUTESYNTAX'}->[0];
#	$omSyntax = $schemaElement->{"OMSYNTAX"}->[0];
#	$omObjectClass = $schemaElement->{"OMOBJECTCLASS"}->[0];

	return 1; # attribute is normal
}

sub isPrintableString
{
	my ($value) = @_;

	return ($value =~ /[\x00-\x09\x0B-\x19\x7F-\xFF]/);
}

sub ldifAva
{
	my ($attr, $value) = @_;
	my $ret, $isPrintable;

	$isPrintable = isPrintableString($value);
	if ($isPrintable == 0) {
		$ret = $attr . ": " . $value;
	} else {
		$ret = $attr . ":: " . encodeBase64($value);
	}
	$ret .= "\n";

	return $ret;
}

sub processEntryForImport
{
	my ($schemaHash, $ldif) = @_;
	my $attribute;
	my $printedDn = 0;
	my $printedDnBackLink = 0;

	foreach $attribute (keys %{$ldif}) {
		my $value;
		my $action;
		my $canonicalAttribute;

		next if $attribute eq "";

		$action = actionForAttribute($schemaHash, $attribute);
		if ($action > 0) {
			$canonicalAttribute = $schemaHash->{$attribute}->{'LDAPDISPLAYNAME'}->[0];
		}

		if ($action == -1) {
			print STDERR "WARNING: could not find schema definition for attribute $attribute.\n";
		} elsif ($action == 0) {
			# nothing to do
		} elsif ($action == 1) {
			if ($printedDn == 0) {
				print OUTPUTFILE "dn: " . $ldif->{'DN'}->[0] . "\n";
				$printedDn = 1;
			}
			foreach $value (@{$ldif->{$attribute}}) {
				print OUTPUTFILE ldifAva($canonicalAttribute, $value);
			}
		} elsif ($action == 3) {
			if ($printedDnBackLink == 0) {
				print OUTPUTFILE_BL "dn: " . $ldif->{'DN'}->[0] . "\n";
				$printedDnBackLink = 1;
			}
			foreach $value (@{$ldif->{$attribute}}) {
				print OUTPUTFILE_BL ldifAva($canonicalAttribute, $value);
			}
		} else {
			die "Unknown schema action $action";
		}
	}

	if ($printedDn) {
		print OUTPUTFILE "\n";
	}
	if ($printedDnBackLink) {
		print OUTPUTFILE_BL "\n";
	}
}

# from http://www.openldap.org/lists/openldap-software/200003/msg00043.html
sub reverseDn
{
	my ($dn) = @_;
	my @rdns;
	my $i;

	if (!defined($dn)) {
		return "";
	}

	@rdns = split(/,/, $dn);
	for ($i = $[; $i < $#rdns; ) {
		if ($rdns[$i] =~ /(\"([^\"]|\\")*|\\)$/) {
			splice @rdns, $i-$[, 2, ($rdns[$i] . ',' . $rdns[$i+1]);
		} else {
			++$i;
		}
	}
	foreach my $rdn (@rdns) { # ignore whitespace preceding each RDN
		$rdn =~ s/^\s*//;
	}
	return join (',', (reverse @rdns));
}

sub byDnAndDependency
{
	my ($a_dn, $b_dn) = ($a->{'DN'}, $b->{'DN'});
	my ($a_oc, $b_oc) = ($a->{'OBJECTCLASS'}, $b->{'OBJECTCLASS'});
	my ($ret) = 0;

	# make sure groups are before users so primaryGroupId can be
	# validated
	my $a_dn_parent = $a_dn->[0]; $a_dn_parent =~ s/^([^,]+)//g;
	my $b_dn_parent = $b_dn->[0]; $b_dn_parent =~ s/^([^,]+)//g;

	if ($a_dn_parent eq $b_dn_parent) {
		my $a_is_group = grep(/group/i, @$a_oc);
		my $b_is_group = grep(/group/i, @$b_oc);
		$ret = $b_is_group - $a_is_group;
	}

	if ($ret == 0) {
		$ret = uc(reverseDn($a_dn->[0])) cmp uc(reverseDn($b_dn->[0]));
	}

	return $ret;
}

sub processLdifForImport
{
	my ($schemaFileName, $inputFileName, $outputFileName, $outputFileNameBackLink) = @_;
	my @schema;
	my @dit, @sortedDit;
	my $ldif;
	my %schemaHash;

	if (!(-f $schemaFileName)) {
		print STDERR "Could not open $schemaFileName.\n";
		return 1;
	}

	if (!(-f $inputFileName)) {
		print STDERR "Could not open $inputFileName.\n";
		return 1;
	}

	@schema = loadLdif($schemaFileName);
	%schemaHash = loadSchema(\@schema);
	@dit = loadLdif($inputFileName);
	@sortedDit = sort byDnAndDependency @dit;

	open(OUTPUTFILE, ">$outputFileName") || die "Could not open $outputFileName for writing";
	open(OUTPUTFILE_BL, ">$outputFileNameBackLink") || die "Could not open $outputFileName for writing";

	foreach $ldif (@sortedDit) {
		next unless defined($ldif->{'DN'});
		processEntryForImport(\%schemaHash, $ldif);
	}

	close(OUTPUTFILE_BL);
	close(OUTPUTFILE);

	return 0;
}

1;

