#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates
################################################################################
#
#   NAME:
#         nds.pm
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         eDirectory helper APIs
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package nds;

use XAD::dclib;
use Logger qw(:DEFAULT);
use XAD::registry;
use util;
my $LDAP_ERR_DUP = 20;

my $rcsId = '$Id$';

sub nds2LdapDN
{
	my ($nds) = @_;
	my $ldap;
	my @dn;

	@dn = split(/,/, $nds);
	$ldap = join(".", @dn);

	return $ldap;
}

sub ldap2NdsDN
{
	my ($ldap) = @_;
	my $nds;
	my @dn;

	@dn = split(/,/, $ldap);
	$nds = join(".", @dn);

	return $nds;
}

sub ldapDNParent
{
	my ($dn) = @_;
	my @dnComponents = split(/\,/, $dn);
	my $parent = join(",", @dnComponents[1 .. $#dnComponents]);

	die "DN $dn has no parent" if ($parent eq "" && $dn ne "");

	return $parent;
}

sub ndsDNParent
{
	my ($dn) = @_;
	my @dnComponents = split(/\./, $dn);
	my $parent = join(".", @dnComponents[1 .. $#dnComponents]);

	die "DN $dn has no parent" if ($parent eq "" && $dn ne "");

	return $parent;
}

sub rdn2TypeVal
{
	my ($dn) = @_;
	my @dnComponents = split(/,/, $dn);
	my @rdnComponents = split(/=/, $dnComponents[0]);

	return @rdnComponents;
}

sub check_certificate{

	if (-f "/etc/opt/novell/certs/SSCert.der") {
		return 1;
	}
	else {
		return 0; 
	}
}

sub get_certificate
{

	my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
	my $ndstreename = registry::getReg("NDSTREENAME");
	my $ndsadminname = registry::getReg("NDSADMINNAME");

	system("LD_LIBRARY_PATH=\"/opt/novell/eDirectory/lib:/opt/novell/eDirectory/lib/nds-modules:$ENV{'LD_LIBRARY_PATH'}\"".
		" /opt/novell/oes-install/util/getSSCert -a $dns_server_ip -t $ndstreename -u $ndsadminname -x '$ENV{ADM_PASSWD}'");

	if (! -f "/etc/opt/novell/certs/SSCert.der") {
		die("Could not get the certificates");
	}

}

#Loads the samspm using
sub load_samspm
{

	my $ndsstat = registry::getReg("NDSSTAT");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	Log(MESG, ">>> Loading samspm module");

	if (-e $tempdir."/load_samspm") {

		Log("The loading of samspm modules was already done.");
		return 0;
	}

	eval{
		my $rc = system("$ndsstat --config-file $live_etcdir/nds.conf -c \"load samspm\"");

		if ($rc != 0) {
			die("Could not load samspm, the command used is $ndsstat --config-file $live_etcdir/nds.conf -c \"load samspm\"");
		}

	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The samspm loaded successfully");

	`touch $tempdir/load_samspm`;
}



#Extends LUM schema using NAM.SCH. As ADPH adds NAM attribute (posixuseraccount,
#posixgroupaccout) to user and group object during samification.
#This must be done prior to extending AD specific schema contained in msds.sch.

sub extend_nam_schema{

	my $ndssch = registry::getReg("NDSSCH");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ndstreename = registry::getReg("NDSTREENAME");
	my $tempdir = registry::getReg("TEMPDIR");

	Log(MESG, ">>>Extending LUM chema");
	if (-e $tempdir."/extend_nam_schema") {
		Log("The extension of the nam schema already done.");
		return 0;
	}
eval{
	my $sch_server = "";
	if (defined $ENV{ROOT_SERVER}) {
		$sch_server = " -h $ENV{ROOT_SERVER}";
	}
	my $rc = system("$ndssch '$ndsadminname' -F /var/opt/novell/eDirectory/log/schema.log -t $ndstreename $sch_server /var/lib/novell-lum/NAM.SCH > /dev/null 2>&1");
	if ($rc != 0) {
		die("Could not extend lum schema, the command used is \
		\'$ndssch $ndsadminname -F /var/opt/novell/eDirectory/log/schema.log -t $ndstreename $sch_server /var/lib/novell-lum/NAM.SCH\'");
	}
};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Extended LUM schema successfully");

	`touch $tempdir/extend_nam_schema`;

}



#Extends the microsoft schema (msds.sch)
sub extend_schema {

	my $ndssch = registry::getReg("NDSSCH");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ndstreename = registry::getReg("NDSTREENAME");
	chomp($ndstreename);
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $tempdir = registry::getReg("TEMPDIR");

	Log(MESG, ">>>Extending eDirectory schema");

	if (-e $tempdir."/extend_schema") {
		Log("The nds schema extension was already done.");
		return 0;
	}

eval{

	my $sch_server = "";
	if (defined $ENV{ROOT_SERVER}) {
		$sch_server = " -h $ENV{ROOT_SERVER}";
	}
	Log("Executing ... $ndssch $ndsadminname -F /var/opt/novell/eDirectory/log/schema.log -t $ndstreename $sch_server $live_etcdir/msds.sch");
	
	my $rc = system("$ndssch '$ndsadminname' -F /var/opt/novell/eDirectory/log/schema.log -t $ndstreename $sch_server $live_etcdir/msds.sch > /dev/null 2>&1");
	if ($rc != 0) {
		die("Could not extend edir schema, the command used is \
		\'$ndssch $ndsadminname -F /var/opt/novell/eDirectory/log/schema.log -t $ndstreename $sch_server $live_etcdir/msds.sch\'");
	}
};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Extended eDirectory schema successfully with AD Schema level 2019");

	`touch $tempdir/extend_schema`;
}

sub configure_nmas_ipc_method{

	my $nmasinst = registry::getReg("NMASINST");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ndstreename = registry::getReg("NDSTREENAME");
	chomp($ndstreename);
	#my $ndstreename = shift;
	my $nmasmthdir = registry::getReg("NMASMTHDDIR");
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my $tempdir = registry::getReg("TEMPDIR");

	#Log(MESG, "ndstreename 2 = $ndstreename");

	if (-e $tempdir."/configure_nmas_ipc_method") {
		Log("The installation of IPCExternal NMAS methods was already done.");
		return 0;
	}

eval {
	Log(MESG, ">>>Installing IPCExternal NMAS method");
	`$nmasinst -addmethod '$ndsadminname' $ndstreename $nmasmthdir/IPCExternal/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}'`;
	my $rc = $? >> 8;

	if ($rc != 0) {
		die("Could not add nmas ipc method, the command used is \
		$nmasinst -addmethod $ndsadminname $ndstreename $nmasmthdir/IPCExternal/config.txt -h $installmachinename -w ****\'");
	}

};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "IPCExternal methods loaded successfully");

	`touch $tempdir/configure_nmas_ipc_method`;
}
sub configure_nmas_mskrb_method{

	my $nmasinst = registry::getReg("NMASINST");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ndstreename = registry::getReg("NDSTREENAME");
	chomp($ndstreename);
	#my $ndstreename = shift;
	#chomp($ndstreename);
	my $nmasmthdir = registry::getReg("NMASMTHDDIR");
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/configure_nmas_mskrb_method") {
		Log("The installation of kerberos NMAS methods already done.");
		return 0;
	}

eval {
	Log(MESG, ">>>Installing Kerberos NMAS method");
	`$nmasinst -addmethod '$ndsadminname' $ndstreename $nmasmthdir/Kerberos/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}'`;
	my $rc = $? >> 8;

	if ($rc != 0) {
		Log(MESG, "Could not add nmas mskrb method, the command used is
		$nmasinst -addmethod $ndsadminname $ndstreename $nmasmthdir/Kerberos/config.txt -h $installmachinename -w ****\'");
	}

};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The Kerberos NMAS method loaded successfully");

	`touch $tempdir/configure_nmas_mskrb_method`;

}
sub configure_nmas_spnego_method{

	my $nmasinst = registry::getReg("NMASINST");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ndstreename = registry::getReg("NDSTREENAME");
	chomp($ndstreename);
	#my $ndstreename = shift;
	#chomp($ndstreename);
	my $nmasmthdir = registry::getReg("NMASMTHDDIR");
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");
	my $tempdir = registry::getReg("TEMPDIR"); 

	#if($ndstreename eq "") {
	#	$ndstreename = `ndsstat | grep -i "Tree Name" | awk '{print \$3}'`;
	#	chomp($ndstreename);
	#	$ndstreename = shift;
	#}

	if (-e $tempdir."/configure_nmas_spnego_method") {
		Log("The installation of the negotiate NMAS methods already done.");
		return 0;
	}
eval{
	Log(MESG, ">>>Installing Negotiate NMAS method");
	`$nmasinst -addmethod '$ndsadminname' $ndstreename $nmasmthdir/Negotiate/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}'`;
	my $rc = $? >> 8;

	if ($rc != 0) {
		Log(MESG, "Could not add nmas spnego method, the command used is \
		$nmasinst -addmethod $ndsadminname $ndstreename $nmasmthdir/Negotiate/config.txt -h $installmachinename -w ****\'");
	}

};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Negotiate NMAS method loaded successfully");

	`touch $tempdir/configure_nmas_spnego_method`;
}


#Trigger the eDirectory backlinker using ndsstat command
sub trigger_backlinker{
	my $ndsstat = registry::getReg("NDSSTAT");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");

	Log(MESG, ">>>Triggering back linker");

eval{
	`$ndsstat --config-file $live_etcdir/nds.conf -c "load dstrace">/dev/null 2>&1`;
	if ($? != 0) {
		die("Could not trigger the backlinker, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"load dstrace\"");
	}

	`$ndsstat --config-file $live_etcdir/nds.conf -c "set ndstrace=*b"`;
	if ($? != 0) {
		die("Could not trigger the backlinker, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"set ndstrace=*b\"");
	}

	`$ndsstat --config-file $live_etcdir/nds.conf -c "unload dstrace">/dev/null 2>&1`;;
	if ($? != 0) {
		die("Could not trigger the backlinker, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"unload dstrace\"");
	}
	Log("   Waiting for the changes to take effect");
	sleep 10;
};
	if ($@) { Log(ERR, $@) }
}

sub trigger_limber {
	my $ndsstat = registry::getReg("NDSSTAT");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");

	Log(MESG, ">>>Triggering Limber");

eval{
	`$ndsstat --config-file $live_etcdir/nds.conf -c "load dstrace">/dev/null 2>&1`;
	if ($? != 0) {
		die("Could not trigger limber, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"load dstrace\"");
	}

	`$ndsstat --config-file $live_etcdir/nds.conf -c "set ndstrace=*L"`;
	if ($? != 0) {
		die("Could not trigger limber, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"set ndstrace=*L\"");
	}

	`$ndsstat --config-file $live_etcdir/nds.conf -c "unload dstrace">/dev/null 2>&1`;
	if ($? != 0) {
		die("Could not trigger limber, the command was, $ndsstat --config-file $live_etcdir/nds.conf -c \"unload dstrace\"");
	}
	Log(MESG, "   Waiting for the changes to take effect");
	sleep(30);
};
	if ($@) { Log(ERR, $@) }
}


sub configure_nmas_methods
{
	Log(MESG, ">>>Configuring NMAS methods");

	#my $ndstreename = registry::getReg("NDSTREENAME");
	#my $tempdir = registry::getReg("TEMPDIR");
	#`mkdir -p $tempdir`;

	#if($ndstreename eq "") {
        #       $ndstreename = `ndsstat | grep -i "Tree Name" | awk '{print \$3}'`;
        #}
        #chomp($ndstreename);
	#Log(MESG, "ndstreename = $ndstreename");
	
	configure_nmas_ipc_method();
	configure_nmas_mskrb_method();
	configure_nmas_spnego_method();

	#Need to trigger backlinker, since new nmas methods has been configured 
	#this syncs the security container to the local box
	trigger_backlinker();

	Log(MESG, "   NMAS methods are successfully configured");
}

sub set_nldap
{

	my $ldapconfig = registry::getReg("LDAPCONFIG");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $ndstreename = registry::getReg("NDSTREENAME");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $option = shift;

	Log("Executing $ldapconfig set \"$option\" --config-file $live_etcdir/nds.conf -t $ndstreename -a '$ndsadminname' -w ****");
	my $rc = system("$ldapconfig set \"$option\" --config-file $live_etcdir/nds.conf -t $ndstreename -a '$ndsadminname' -w '$ENV{ADM_PASSWD}' > /dev/null 2>&1");

	return $rc;
}

#Always takes comma separated interfaces as the input
sub prepare_interface_list($) {
	my $interfaces_to_add = shift or die("Invalid LDAP interface");
	my $live_etcdir = registry::getReg("LIVE_ETCDIR");
	my $ndstreename = registry::getReg("NDSTREENAME");
	my $ndsadminname = registry::getReg("NDSADMINNAME");
	my $ldapconfig = registry::getReg("LDAPCONFIG");
	my $rc = -1;
	my @ldapconfig_get_output = ();
	my $final_interface_string = "";
	my %final_hash = ();

	#Get the existing LDAP interfaces
	Log("Executing $ldapconfig get \"ldapInterfaces\"--config-file $live_etcdir/nds.conf".
		" -t $ndstreename -a '$ndsadminname' -w ****");
	@ldapconfig_get_output = `$ldapconfig get "ldapInterfaces" --config-file $live_etcdir/nds.conf \\
		-t $ndstreename -a '$ndsadminname' -w '$ENV{ADM_PASSWD}' 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		die("ldapconfig get command failed : [@ldapconfig_get_output]");
	}
	chomp(@ldapconfig_get_output);

	#Extract all the interfaces from the ldapconfig get output
	foreach my $entry (@ldapconfig_get_output) {
		if (!grep(/^ldapInterfaces:\s+/i, $entry)) {
			next;
		}
		#To have the back-ward compatibility forming the string
		#with comma separated interface values
                my $interface_value =  $entry;
                $interface_value =~ s/^ldapInterfaces:\s+//gi;
		$interface_value =~ s/\s+/,/gi;
		$interface_value =~ s/^,//gi;
		my @interface_value_array = ();

		@interface_value_array = split(/,/, $interface_value);

		foreach my $e (@interface_value_array) {
			if ($e eq "") {
				next;
			}
			$final_hash{$e}++;
		}
	}

	foreach my $e (split(/,/, $interfaces_to_add)) {
		$final_hash{$e}++;
	}

	foreach my $val (keys(%final_hash)) {
		$final_interface_string.=",".$val;
	}
	$final_interface_string =~ s/^,//g;
	Log("STRING VALUE : $final_interface_string");

	return $final_interface_string;
}

sub configure_nldap_settings
{

	my $tempdir = registry::getReg("TEMPDIR");

	if (-e $tempdir."/configure_nldap_settings") {
		Log("The configuration of the nldap setting was done earlier.");
		return 0;
	}

	Log(MESG, ">>>Configuring LDAP server");

	eval{
		my $rc = set_nldap("Require TLS for Simple Binds with Password=no");

		if ($rc != 0) {
			die("could not disable the option TLS for Simple Binds with Password=no");
		}

		my $ldap_url = registry::getReg("LDAPI_URL");

		my $interface_string = prepare_interface_list("ldap://:389,ldaps://:636,$ldap_url".
				",cldap://,ldap://:3268,ldaps://:3269");
		$rc = set_nldap("ldapInterfaces=$interface_string");

		if ($rc != 0) {
			die("could not set the ldap interface option");
		}

		refresh_nldap();
	};

	if ($@) { Log(ERR, $@) }

	sleep(30);

	Log(MESG, "   The NLDAP settings are done successfully");

	`touch $tempdir/configure_nldap_settings`;
}

sub nldap_enable_requiretls
{

	my $rc = set_nldap("Require TLS for Simple Binds with Password=yes");
	if ($rc != 0) {
		Log(WARN,"could not enable the option TLS for Simple Binds with Password=yes");
	}
}

sub uniqueDomainID_indexing() {

	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $index_ini = "$templatedir/nldap-UniqueDomainID-index.ini";
	my $replication_provider = registry::getReg("Replication Provider");
	
	eval{

		# Add value index for uniqueDomainID
		open FILE, ">$index_ini" or die ("failed to open the file $index_ini");
		print FILE "[NCPDN]\n";
		print FILE "changetype=modify\n";
		print FILE "add=indexDefinition\n";
		print FILE "indexDefinition=0\$uniqueDomainID\$2\$0\$0\$1\$uniqueDomainID\n";
		close(FILE);
		stage::gen_ldif($index_ini, '>', "$live_dsstatedir/domain/nldap-UniqueDomainID-index.ldif", "--offline", "--ncpdn");
		Log("Executing $ldapmodify -x -H ldaps://$replication_provider:1636 -D '$ldapadminname' -w ***** -f $live_dsstatedir/domain/nldap-UniqueDomainID-index.ldif");
		my @output = `$ldapmodify -x -H ldaps://$replication_provider:1636 -D '$ldapadminname' -w '$ENV{ADM_PASSWD}' -f $live_dsstatedir/domain/nldap-UniqueDomainID-index.ldif 2>&1`;
		my $error = $? >> 8;
		if ($error and $error != $LDAP_ERR_DUP) {
			chomp(@output);
			die("Index Definition addition for uniqueDomainID attribute on the NCP server object failed.\n [@output]");
		}

	};
	if ($@) { Log(ERR, $@); }
	Log("Index Definition addition for uniqueDomainID attribute on the NCP server object succeeded");
}

sub configure_nldap_objects
{
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldapmodify = "/opt/novell/eDirectory/bin/ldapmodify";
	my $ldapmodflags = "-x -e /etc/opt/novell/certs/SSCert.der -D '$ldapadminname'";
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	#Added for import_file function
	my $nldap_del_clslst_ldif="$live_dsstatedir/domain/nldap-delete-classlist.ldif";
	my $nldap_ldif="$live_dsstatedir/domain/nldap.ldif";


	Log(MESG, ">>>Configuring LDAP to eDirectory schema mappings and plugins");

	eval{

		Log(MESG, "Importing $nldap_del_clslst_ldif");
		my $local_server_ip = `/bin/hostname -i`;
		my $dns_server_ip = registry::getReg("DNS_SERVER_IP");
		chomp($local_server_ip);
		#Added to achieve retry functionality during configuration of SLAPI
		$rc = import::import_file(admin=>$ldapadminname,
			pwd=>$ENV{'ADM_PASSWD'},
			cert=>"/etc/opt/novell/certs/SSCert.pem",
			file=>$nldap_del_clslst_ldif,
			cont_if_exist=>"true",
			host=>$dns_server_ip);
		#If DNS is different from current Domain then import of $nldap_del_clslst_ldif file using local server ip instead of DNS server ip.
                if ($rc != 0) {
                        if ($rc == -1){
				Log(MESG, "Importing $nldap_del_clslst_ldif using local ip.");
				$rc = import::import_file(admin=>$ldapadminname,
				pwd=>$ENV{'ADM_PASSWD'},
				cert=>"/etc/opt/novell/certs/SSCert.pem",
				file=>$nldap_del_clslst_ldif,
				cont_if_exist=>"true",
				host=>$local_server_ip);
                        }
                        else {
				die("Could not import $nldap_del_clslst_ldif");
                        }
                }

		Log(MESG, "Importing $nldap_ldif\n");
		$rc = import::import_file(admin=>$ldapadminname,
			pwd=>$ENV{'ADM_PASSWD'},
			cert=>"/etc/opt/novell/certs/SSCert.pem",
			file=> $nldap_ldif,
			cont_if_exist=>"true",
			host=>$dns_server_ip);
		#If DNS is different from current Domain then import of $nldap_ldif file using local server ip instead of DNS server ip.
		if ($rc != 0) {
                        if ($rc == -1){
				Log(MESG, "Importing $nldap_ldif using local ip.");
				$rc = import::import_file(admin=>$ldapadminname,
				pwd=>$ENV{'ADM_PASSWD'},
				cert=>"/etc/opt/novell/certs/SSCert.pem",
				file=>$nldap_ldif,
				cont_if_exist=>"true",
				host=>$local_server_ip);
                        }
                        else {
				die("Could not import $nldap_ldif");
                        }
		}
		`/opt/novell/eDirectory/bin/ldapsearch -x -s base -b cn=schema attributetypes=uniquedomainid -LLL >&/dev/null`;
		$rc = $?;
		if ($rc == 0) {
			uniqueDomainID_indexing();
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   eDirectory schema mappings are successfully created");
}

sub refresh_nldap
{

	my $ldapconfig = registry::getReg("LDAPCONFIG");
	my $live_ndsconfdir = registry::getReg("LIVE_NDSCONFDIR");
	my $ndsadminname = registry::getReg("NDSADMINNAME");

	Log(MESG, ">>>Refreshing nldap server");

	eval{

		#print "$ldapconfig -R --config-file $live_ndsconfdir/nds.conf -a $ndsadminname -w $ENV{'ADM_PASSWD'}";
		my $rc = system("$ldapconfig -R --config-file $live_ndsconfdir/nds.conf -a '$ndsadminname' -w '$ENV{ADM_PASSWD}' > /dev/null 2>&1");

		if ($rc != 0) {
			my $error = $rc >> 8;
			die("Refreshing the LDAP server failed. [$error]");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   The NLDAP server successfully refreshed");
}

#Refresh the ldap server using ldapexop
sub refresh_nldap_exop
{
	my $ldapexop = registry::getReg("LDAPEXOP");
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $nldap_oid = registry::getReg("LDAP_NLDAP_EXOP_OID");

	Log(MESG, ">>>Refreshing nldap server");

	eval {

		Log("Executing ... $ldapexop -H $ldapi_url -Y EXTERNAL $nldap_oid");

		sleep(10);
		my $rc = system("$ldapexop -H $ldapi_url -Y EXTERNAL $nldap_oid");
		if ($rc != 0) {
			die("Could not refresh the ldap server");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   The NLDAP server successfully refreshed");
}

sub validate_loaded_plugin
{

	Log(MESG, ">>>Validating the loading of NAD plugin");
	my $nad_plugin_loaded=0;
	my $tries = 0;

	eval{
		while ( $nad_plugin_loaded != 1 && -e "/var/opt/novell/eDirectory/data/ndsd.pid" ) {
			sleep 1;
			`cat /proc/\`cat /var/opt/novell/eDirectory/data/ndsd.pid\`/maps | grep -i 'nad' >/dev/null 2>&1`;
			if ( $? == 0 ) {
				$nad_plugin_loaded=1;
			}
			$tries++;
			if( $tries % 100 == 0 ) {
				Log(MESG, "   Restarting ndsd");
				`systemctl stop ndsd`;
				`systemctl start ndsd`;
			}
			if ($tries == 301) {
				last;
			}
		}

		if ( $nad_plugin_loaded == 0 ){
			die("Plug-ins are not properly loaded, hence terminating the installation....");
		}
	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   NAD plugins are successfully loaded");
}

sub wait_for_samspm
{

	Log(MESG, ">>>Checking samspm status");

	do {
		die("Service ndsd is down") if (system("/usr/bin/systemctl status ndsd.service >/dev/null 2>&1"));
		Log("Waiting for samspm to be loaded .");
		`/opt/novell/eDirectory/bin/ndstrace -c modules | grep 'samspm.*Running' > /dev/null 2>&1`;

	} while ($? != 0 && sleep(1));

	Log(MESG, "   samspm status is loaded");

}

sub sync_gpo_nmas
{

	my $GPO2NMAS = registry::getReg("GPO2NMAS");
	my $ADMINISTRATOR_UID = registry::getReg("ADMINISTRATOR_UID");
	my $DOMAIN_ADMINS_GID = registry::getReg("DOMAIN_ADMINS_GID");
	my $LIVE_LOCALSTATEDIR = registry::getReg("LIVE_LOCALSTATEDIR");

	Log(MESG, ">>>Synchronizing Policies");
	eval{
		`$GPO2NMAS -g "{31B2F340-016D-11D2-945F-00C04FB984F9}" -f nmas`;
		if ($? != 0) {
			die("Could not sync group policies from nmas to SYSVOL");
		}

		`chown -R $ADMINISTRATOR_UID:$DOMAIN_ADMINS_GID $LIVE_LOCALSTATEDIR/sysvol`;

	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The GPO policies are in sync");
}

#The caller should not be ADC and that is not checked here
sub gen_uniqueID
{
	my $type = registry::getReg('INSTALL_TYPE');
	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $adm_principal = registry::getReg("ADM_PRINCIPAL");
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $lib	= registry::getReg("_LIB");
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldap_url = registry::getReg("LDAPI_URL");
	my $dm_uniquedomainid_ldif = "$live_dsstatedir/domain/domain-uniquedomainid.ldif";

	eval {

		Log(MESG, ">>>Setting UniqueDomainId for the domain root");

		if ($type eq "frd") {
			registry::setReg("UniqueDomainID", 1049076);
		}
		else {
			registry::setReg("UniqueDomainID", registry::getReg("TRUSTPOSIXOFFSET"));
		}
		stage::gen_ldif("$templatedir/domain-uniquedomainid.ini", '>', "$live_dsstatedir/domain/domain-uniquedomainid.ldif", "--defaultrootdomainobject");
		Log(MESG, "Importing $dm_uniquedomainid_ldif");
		my $rc = import::import_file(host=>$ldap_url,
			admin=>$ldapadminname,
			pwd=>$ENV{'ADM_PASSWD'},
			file=>$dm_uniquedomainid_ldif,
			cont_if_exist=>"true");
		if($rc != 0) {
			die("Could not import $dm_uniquedomainid_ldif");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Unique Domain ID successfully generated and set into domain root partition");
	
}

#Adds objects under configuration partition
sub create_site
{
	#A frd installation would pass nothing and child
	#installation would pass config-site.ldif....
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $config_site_ldif = "$live_dsstatedir/config/config-site.ldif";
        #Added for ldif modifications
        my $ldapadminname = registry::getReg("LDAPADMINNAME");

	if (! -e "$config_site_ldif") {
		Log(MESG, ">>>No additional site configuration needed");
		return 0;
	}

	Log(MESG, ">>>Importing Site Configuration");

	eval{
		Log(MESG, "Importing $config_site_ldif");
		#Added for ldif modification
		my $rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
				pwd=>$ENV{'ADM_PASSWD'},
				admin=>$ldapadminname,
				file=>$config_site_ldif,
				cont_if_exist=>"true");

		if ($rc != 0) {
			die("Could not import $config_site_ldif");
		}
	};

	if ($@) {
		Log(ERR, $@);
	}

	Log(MESG, "   Site configuration imported successfully");
}

1;

