#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2004
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         ntds.pl
#
#   FACILITY:
#
#         Domain Controller Initialization
#
#   ABSTRACT:
#
#         Parse the schema initialization file(s).
#
#   REVISION:
#
#         $Id$
#
#################################################################################

package XAD;

use XAD::registry;
use XAD::dclib;

my $rcsId = '$Id$';

$CATEGORY2RDN{"Container"}		= "CN";
$CATEGORY2RDN{"Organizational-Unit"} 	= "OU";
$CATEGORY2RDN{"Domain"}			= "DC";
$CATEGORY2RDN{"Organization"}		= "O";

my $CACHE;
my $OFFLINE = 0;
my $HIGHEST_USN = 1000;

# generate skeletal LDIF (no users)
sub readSections
{
	my %sections;
	my $cwd;

	open(SCHEMAINI, $_[0]) || die "Could not open $_[0]\n";
	while(<SCHEMAINI>) {
		next if (/^;/);
		next if (/^\s/);
		if (/^\[/) {
			chop;
			s/\[(.*)\](.*)/$1/;
			$cwd = $_;
#			print STDERR "Parsed section $cwd\n";
			$sections{$cwd} = "";
		} else {
			$sections{$cwd} .= $_;
		}
	}
	close(SCHEMAINI);
	return %sections;
}

sub ocToRdnAttr
{
	my $rdnAttr;
	my $oc = $_[0];

	$rdnAttr = $CATEGORY2RDN{$oc};
	if (!defined($rdnAttr)) {
		$rdnAttr = "CN";
	}
	return $rdnAttr;
}

sub makeDistinguishedNameAndRDN
{
	my ($key, $oc, $parentDn) = @_;
	my ($rdn, $dn);
	my $rdnAttr;

	$key = regSubstitute($key);

	if (defined($registry::DEFAULTMAP{$key})) {
		$dn = $registry::DEFAULTMAP{$key};
		($rdn) = split(/,/, $dn);
	} else {
		$rdnAttr = ocToRdnAttr($oc);
		$rdn = $rdnAttr . "=" . $key;
		$dn = $rdn;
		if ($parentDn ne "") {
			$dn .= "," . $parentDn;
		}
		$rdn = lc($rdnAttr) . "=" . $key;
	}
	return ($rdn, $dn);
}

sub makeDistinguishedName
{
	my ($rdn, $dn) = makeDistinguishedNameAndRDN(@_);
	return ($dn);
}

sub hex2Base64
{
	my $data = $_[0];

	return encodeBase64(pack("H*", $data));
}

sub base642Hex
{
	my $data = $_[0];

	return unpack("H*", decodeBase64($data));
}

sub maybePackAttributeValue
{
	my ($attr, $data) = @_;
	my $binary = 0;

	if (uc($attr) eq "OBJECTSID" || uc($attr) eq "SIDHISTORY" || uc($attr) eq "SECURITYIDENTIFIER") {
		$data = encodeBase64(packSid($data));
		$binary = 1;
	} elsif (($attr =~ /GUID$/i && uc($attr) ne "RIGHTSGUID") ||
		 uc($attr) eq "INVOCATIONID") {
		$data = encodeBase64(packGuid($data));
		$binary = 1;
	} elsif (uc($attr) eq "OBJECTCATEGORY") {
		$data = "CN=$data,$CACHE{'Schema DN Name'}";
	}

	return ($data, $binary);
}

sub ldifSubstitute
{
	my ($attr, $data) = @_;
	my $substData;
	my $binary = 0;
	my $sep;

	foreach (keys %registry::SYSDEFAULT) {
		$data =~ s/\$SYSDEFAULT=$_/$registry::SYSDEFAULT{$_}/g;
	}

	if ($data =~ /^0x(\w+)/) {
		# Hex data
		$substData = sprintf "%d", hex($data);
	} elsif ($data =~ /^\\x/) {
		# Binary data
		$substData = hex2Base64(substr($data, 2));
		$binary = 1;
	} elsif (uc($data) eq "TRUE" || uc($data) eq "FALSE") {
		# Normalize boolean data
		$substData = uc($data);
	} else {
		# String data
		$substData = XAD::regSubstitute($data);
		if ($substData =~ /(.*)<(.*)$/) {
			my $dn;
			$dn = $CACHE{$2};
			$substData = $1 . $dn;
		}
		($substData, $binary) = maybePackAttributeValue($attr, $substData);
	}
	if ($binary) {
		$sep = ":: ";
	} else {
		$sep = ": ";
	}

	return $attr . $sep . $substData;
}

sub sddlSubstitute
{
	# 603: RID of primary domain controller
	# 516: Domain Controllers RID
	return (sddl2hex($registry::REGISTRY{'Domain SID'}, 603, 516, $_[0]));
}

sub makeEmptyDaclSacl
{
	return (sddlSubstitute("DS"));
}

sub emitLdifForSection
{
	my ($key, $sections, $parentDn, $suppressBody) = @_;
	my @avas;
	my $line;
	my $dn;
	my @children;
	my $child;
	my $childrenOnly = 1;
	my $hasObjectGUID = 0;
	my $hasSecurityDescriptor = 0;
	my $printedSomething = 0;
	my $objectClass;
	my $rdn;
	my $rdnAttr, $rdnVal;
	my $changeType = 0;
	my $objectGUID;
	my @objectGUIDAliases;
	my $hasInstanceType = 0;
	my $hasUSNCreated = 0;
#	my $hasObjectCategory = 0;
	my $nativeSecurity = ($registry::REGISTRY{"Native Security"} eq "TRUE");
	my $nds = ($registry::REGISTRY{"NDS"} eq "TRUE");

	$key =~ s/(\s+)$//;

	if (!defined($sections->{$key})) {
		die "No section for $key, aborting\n";
	}

	$rdnVal = $key;

	@avas = split(/\n/, $sections->{$key});
	foreach $line (@avas) {
		my ($attr, $val) = split(/=/, $line, 2);
		# trailing sapace
		$attr =~ s/(\s+)$//;
		# leading space
		$val =~ s/^(\s+)//;
		# trailing comments
		$val =~ s/;(.*)$//;
		if (uc($attr) eq "RDN-OF-OBJECT") {
			$rdnVal = regSubstitute($val);
		} elsif (uc($attr) eq "OBJECTCATEGORY") {
			$objectClass = $val;
		} elsif (uc($attr) eq "CHANGETYPE") {
			$changeType = 1;
		} elsif (uc($attr) eq "CHILD") {
			push @children, $val;
		} else {
			$childrenOnly = 0;
		}
	}

	($rdn, $dn) = makeDistinguishedNameAndRDN($rdnVal, $objectClass, $parentDn);
	if (!defined($CACHE{$key})) {
		$CACHE{$key} = $dn;
	}
#	else {
#		print STDERR "WARNING: duplicate key $key\n";
#	}

	undef $rdnVal;

	if ($suppressBody == 0 && $childrenOnly == 0) {
		print "dn: $dn\n";
		# only print RDN for new entries
		if ($changeType == 0) {
			($rdnAttr, $rdnVal) = split(/=/, $rdn);
			print "$rdnAttr: $rdnVal\n";
		}
	}

	if ($suppressBody == 1) {
		@avas = ();
	}

	foreach $line (@avas) {
		# support RFC 2849 changelog entries; not sure if
		# slapadd will support these
		if ($line eq "-") {
			print $line . "\n";
			next;
		}
		my ($attr, $val) = split(/=/, $line, 2);
		$attr =~ s/(\s+)$//;
		$val =~ s/^(\s+)//;
		if ($attr eq "") {
			print STDERR "WARNING: malformed line \"$line\"\n";
			next;
		}
		if ($val eq "") {
			print STDERR "WARNING: malformed line \"$line\"\n";
			next;
		}

		if (uc($attr) eq "OBJECTGUID") {
			$objectGUID = $val;
			$hasObjectGUID = 1;
			next if $nds;
		} elsif (uc($attr) eq "INSTANCETYPE") {
			$hasInstanceType = 1;
			next if $nds;				# computed attrbiute in eDir
		} elsif (uc($attr) eq "OBJECTCATEGORY" ||	# computed attribute in eDir
			 uc($attr) eq "ISDELETED" ||		# computed attribute in eDir
		 	 uc($attr) eq "REVISION" ||		# eDir attribute with conflicting syntax
			 uc($attr) eq "OBJECTVERSION" || 	# eDir attribute with conflicting syntax
			 uc($attr) eq "MAXPWDAGE" || 		# computed from NMAS password policy, cannot be set
			 uc($attr) eq "MINPWDAGE" || 		# computed from NMAS password policy, cannot be set
			 uc($attr) eq "MINPWDLENGTH" || 	# computed from NMAS password policy, cannot be set
			 uc($attr) eq "PWDPROPERTIES" || 	# computed from NMAS password policy, cannot be set
			 uc($attr) eq "PWDHISTORYLENGTH") { 	# computed from NMAS password policy, cannot be set
			next if $nds;
		} elsif ($nds && uc($attr) eq "LOCKOUTDURATION") {
			$attr = "intruderLockoutResetInterval";
			$val /= -10000000; # convert from NDS to NDS time interval
		} elsif ($nds && uc($attr) eq "LOCKOUTOBSERVATIONWINDOW") {
			$attr = "intruderAttemptResetInterval";
			$val /= -10000000; # convert from NDS to NDS time interval
		} elsif ($nds && uc($attr) eq "LOCKOUTTHRESHOLD") {
			$attr = "loginIntruderLimit";
		} elsif (uc($attr) eq "USNCREATED") {
			$hasUSNCreated = 1;
		} elsif (uc($attr) eq "NTSECURITYDESCRIPTOR") {
			$val = sddlSubstitute($val);
			$hasSecurityDescriptor = 1;
		} elsif (uc($attr) eq "CHILD") {
			# children handled prior
			next;
		} elsif (uc($attr) eq "RDN-OF-OBJECT") {
			next;
		} elsif (uc($attr) eq uc($rdnAttr)) {
			# don't repeat RDN
			next;
		}
		if (uc($val) eq "<OBJECTGUID") {
			push(@objectGUIDAliases, $attr);
			next;
		}
		if (uc($val) eq "<RDN") {
			$val = $rdn;
		}
		print ldifSubstitute($attr, $val);
		print "\n";
		$printedSomething = 1;
	}

	# Because slapadd won't call SLAPI plugins, we need to generate
	# the objectGUID ourselves. An alternative would be to compute
	# it from the LCUP entryUUID.
	if ($changeType == 0 && !$childrenOnly) {
		if ($hasObjectGUID == 0 && ($OFFLINE || @objectGUIDAliases)) {
			$objectGUID = `uuidgen`;
			chop ($objectGUID);
			print ldifSubstitute("objectGUID", $objectGUID);
			print "\n";
			$printedSomething = 1;
		}
	
		foreach $line (@objectGUIDAliases) {
			print ldifSubstitute($line, $objectGUID);
			print "\n";
			$printedSomething = 1;
		}
	}

	if (!$childrenOnly && $OFFLINE) {
		if (!$hasInstanceType) {
			print "instanceType: 4\n";
			$printedSomething = 1;
		}

		if (!$hasSecurityDescriptor && $nativeSecurity) {
			print ldifSubstitute("nTSecurityDescriptor", makeEmptyDaclSacl()) . "\n";
			$printedSomething = 1;
		}

		if (!$hasUSNCreated) {
			print "uSNCreated: " . $HIGHEST_USN . "\n";
			print "uSNChanged: " . $HIGHEST_USN . "\n";
			$printedSomething = 1;
			$HIGHEST_USN++;
		}
	}

	if ($printedSomething) {
		print "\n";
	}

	# Recurse into children
	foreach $child (@children) {
		emitLdifForSection($child, $sections, $dn, 0);
	}
}

sub genDcSkelLdif
{
	my %sections = readSections($_[0]);
	my $treeRoot = $_[1];
	my $suppressBody = $_[2];

	%CACHE = %registry::REGISTRY;

	if ($registry::REGISTRY{'NDS'} eq "TRUE") {
		# offline is a NOOP for eDirectory (at least until we use ldif2dib)
	} else {
		$OFFLINE = $_[3];
	}

	emitLdifForSection($treeRoot, \%sections, NULL, $suppressBody);
}

1;
