#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2009
#	Novell, Inc.  All rights reserved.
################################################################################
#
#   NAME:
#         secure.
#
#   FACILITY:
#         All Kerberos and acl related modules are grouped here
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package secure;

use Logger qw(:DEFAULT);
use NDS::stage;
use util;
use File::Copy;
use File::Temp;
use NDS::import;
use Net::LDAP qw(LDAP_SUCCESS LDAP_TYPE_OR_VALUE_EXISTS);
use XAD::nds;
use Net::LDAP::Control;
use Net::LDAP::Util qw( ldap_error_text
			ldap_error_name
			ldap_error_desc
			);
use helpers::utility;

use constant RAND_STRING_LENGTH => 12;
use constant UPGRADE_ON => 1;
use constant UPGRADE_OFF => 0;
use constant LDAP_ERR_DUP => 20;

my $retry = 12;
my $myLocalhost = "127.0.0.1";

sub compute_effective_acls{

	my $templatedir = registry::getReg("TEMPLATEDIR");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");

	Log(MESG, ">>>Computing effective ACLs");

	my $ipaddress = registry::getReg("IPADDRESS");
	my $ndsexistingadminname = registry::getReg("NDSEXISTINGADMINNAME");
	my $mappeddomainnc = registry::getReg("MAPPEDDOMAINNC");

	my $adminname = registry::getReg("NDSADMINNAME");
	my $is_mapped = registry::getReg("MAPPED");
	my $install_type = registry::getReg("INSTALL_TYPE");

	if ($install_type eq "frd" && $is_mapped eq 0) {
		$ndsexistingadminname = $adminname;
		$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};
	}

	$out = `ComputeEffectiveAcls.pl -p $ipaddress -a "$ndsexistingadminname" -n "$mappeddomainnc" -w "Env:NDSEXISTINGADMINPASSWD"`;
	if ($? != 0) {
		Log($out);
		die("Could not computer the effective acl");
	}
	if (-s "/tmp/effective_acls"){
		stage::gen_ldif("/tmp/effective_acls", '>',  "$live_dsstatedir/domain/nds-domain-acls.ldif", "--defaultrootdomain");
	}
	elsif (-e "/tmp/effective_acls") {
		Log(MESG,  "   /tmp/effective_acls is empty");
		`touch $live_dsstatedir/domain/nds-domain-acls.ldif`;
	}
	unlink ("/tmp/effective_acls");

	Log(MESG,  "   Effective ACLs are computed");
}

sub bootstrap_krbtgt{

	my $xadroot = registry::getReg("XADROOT");
	my $setpwdflags = registry::getReg("SETPASSWORD_FLAGS");
	my $tempdir = registry::getReg("TEMPDIR");
	my $count = 0, $ret, $retry_count = 6;

	eval{
		if (! -e "$tempdir/bootstrap_krbtgt") {
			Log(MESG, ">>>Setting krbtgt password");

			do {
				$ret = system("$xadroot/sbin/setpassword $setpwdflags -r -u krbtgt");
				$count++;
				if ($ret != 0) {
					Log(INFO, "Could not set password for krbtgt, retrying $count/$retry_count times");
					sleep(10);
				}
			} while ($ret != 0 and $count < $retry_count);
			if ($ret != 0) {
				die("Could not set password for krbtgt, please retry after resolving the problem");
			}
			`touch $tempdir/bootstrap_krbtgt`;
		}
	};
	if ($@) { Log(ERR, $@) }
}

sub bootstrap_machine{

	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $XADROOT = registry::getReg("XADROOT");
	my $SETPASSWORD_FLAGS = registry::getReg("SETPASSWORD_FLAGS");
	my $INSTALLMACHINENAME = registry::getReg("Install_DCName");
	my $tempdir = registry::getReg("TEMPDIR");
	my $count = 0, $ret = 1, $retry_count = 6;

	eval{
		if (! -e "$tempdir/bootstrap_machine") {
			Log(MESG, ">>>Setting machine account password and configuring Kerberos keytab");

			NEXT: while($ret != 0 and $count <= $retry_count) {
				if ( -f "$live_dsstatedir/krb5kdc/krb5.keytab") {
					Log("Removing keytab file");
					unlink ("$live_dsstatedir/krb5kdc/krb5.keytab");
				}
				$ret = system("$XADROOT/sbin/setpassword $SETPASSWORD_FLAGS -r -k $live_dsstatedir/krb5kdc/krb5.keytab -u $INSTALLMACHINENAME\$");
				$count++;
				if ($ret != 0) {
					Log(INFO, "Could not set password for machine account, retrying $count/$retry_count times");
					sleep(10);
					next NEXT;
				}

				if (-e "$live_dsstatedir/krb5kdc/krb5.keytab") {
					chmod 0640, "$live_dsstatedir/krb5kdc/krb5.keytab";
					`chgrp named $live_dsstatedir/krb5kdc/krb5.keytab`;

				}
			}
			if ($ret != 0) {
				die("Could not set password for machine account, please retry after resolving the problem");
			}
			`touch $tempdir/bootstrap_machine`;
		}

	};
	if ($@) { Log(ERR, $@) }
}

sub check_schema_sync {
	my %args = @_;
	my $retry_count = $args{retry_count};
	my $counter = 0;
	my $rc = 0;

	do {
		if ($counter > 0) {
			Log(INFO, "Retrying as the schema was not synchronized, retry count:$counter");
			sleep(5);
		}

		$rc = system("/usr/bin/ldapsearch -b cn=schema -s base attributetypes=forcelogoff -x -LLL > /dev/null 2>&1");

		if ($rc != 0) {
			Log(WARN, "The schema is not extended yet.");
		}

		$counter ++;

	} while ($rc != 0 and $counter < $retry_count);

	return $rc;
}

sub create_administrator
{

	my $type = registry::getReg("INSTALL_TYPE");
	my $mapped = int(registry::getReg("MAPPED"));

	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $ldapadd_flags = registry::getReg("LDAPADD_FLAGS");
	my $ldapmodify_flags = registry::getReg("LDAPMODIFY_FLAGS");

	my $admin_acl_ldif = registry::getReg("LIVE_DSSTATEDIR")."/domain/administrator-acl-on-root.ldif";
	my $admin_ldif = registry::getReg("LIVE_DSSTATEDIR")."/domain/administrator.ldif";
	my $admin_dn = "cn=Administrator,cn=users,".registry::getReg("MAPPEDDOMAINNC");

	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my ($rc,$counter);
	my $retry_count = 10;
	my $password_set_failed = 0;

	eval{
		$rc = check_schema_sync(retry_count=>$retry);
		if ($rc != 0) {
			die("The schema is not synchronized yet, please synchronize it and try again.");
		}

		Log(MESG, ">>> Creating Administrator ");

		$counter = 0;
		do {
			if ($counter > 0) {
				Log(INFO, "Retrying to import the file $admin_ldif, retry count:$counter");
				sleep(5);
			}
			$rc = import::import_file(host=>"$myLocalhost", port=>636, admin=>$ldapadminname, ssl=>"yes",
						      pwd=>$ENV{'ADM_PASSWD'}, file=> $admin_ldif, cont_if_exist=>"true");

			if ($rc != 0) {
				Log(WARN, "Could not import $admin_ldif");
			}
			$counter++
		}while($rc != 0 and $counter < $retry_count);
		if ($rc != 0) {
			die("Could not import $admin_ldif");
		}
		sleep(20);

		my $password = $ENV{ADM_PASSWD_DOMAIN};

		$counter = 0;
		do {
			if ($counter > 0) {
				sleep(5);
			}
			#Added code for ldif modification
			my $tldif = `/bin/mktemp`;
			open TFILE, ">$tldif" or die("failed to open the file");
			print TFILE "dn: $admin_dn\n";
			print TFILE "changetype: modify\n";
			print TFILE "replace: userPassword\n";
			print TFILE "userPassword: $password\n";
			close(TFILE);
			Log("Importing $tldif");
			$rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
				pwd=>$ENV{'ADM_PASSWD'},
				admin=>registry::getReg("LDAPADMINNAME"),
				file=>$tldif,
				cont_if_exist=>"true");

			unlink($tldif);
			$counter++
		}while($rc != 0 and $counter < $retry_count);
		if ($rc != 0) {
			Log(WARN, "Assign credentials to the Administrator manually before provisioning");
			$password_set_failed = 1;
		}

		if ($type eq "frd" and $mapped == 0) {
			Log("Importing $admin_acl_ldif");
			$rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
				pwd=>$ENV{'ADM_PASSWD'},
				admin=>registry::getReg("LDAPADMINNAME"),
				file=>$admin_acl_ldif,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die("Could not assign permission for administrator.");
			}
		   }
	};

	if ($password_set_failed) {
		exit(25)
	}

	if ($@) { Log(ERR, $@); }
}


sub bootstrap_administrator{

	my $XADROOT = registry::getReg("XADROOT");
	my $SETPASSWORD_ADM_FLAGS = registry::getReg("SETPASSWORD_ADM_FLAGS");
	my $tempdir = registry::getReg("TEMPDIR");
	my $count = 0, $ret, $retry_count = 6;

	eval {
		if (! -e "$tempdir/bootstrap_administrator") {
			Log(MESG,">>>Setting Administrator password");

			do {
				$ret = system("$XADROOT/sbin/setpassword $SETPASSWORD_ADM_FLAGS -u Administrator");
				$count++;
				if ($ret != 0) {
					Log(WARN, "Could not set password for Administrator, retrying $count/$retry_count times");
					sleep(10);
				}
			}while ($ret != 0 and $count < $retry_count);
			if ($ret != 0) {
				die("Could not set password for Administrator, please retry after resolving the problem");
			}
			`touch $tempdir/bootstrap_administrator`;
		}
	};
	if ($@) { Log(ERR, $@) }
}

sub enable_krb5_local_lookup{

	my $krbfile = registry::getReg(LIVE_ETCDIR)."/krb5.conf";

	Log(MESG, ">>>Enabling krb5 local lookup");

	eval{
		if (-f $krbfile) {
			my $tempfile = "/tmp/krb5.conf.tmp";
			open (FILE, $krbfile) or die("Could not open the krb5.conf file");
			open (TEMPFILE, ">$tempfile") or die("Could not open the temporary file $tempfile");
			while (<FILE>) {
				my $data;
				if (/^#(\s+kdc =.*)/) {
					$data = $1."\n";
				}else {
					$data = $_;
				}
				print TEMPFILE  $data;
			}
			close(FILE);
			close(TEMPFILE);
			`mv $tempfile $krbfile`;
		}
		else {
			die("Could not find the $krbfile");
		}


	};

	if ($@) { Log(ERR, $@) }

	Log(MESG, "   Enabling krb5 for local lookup is successful");
}


sub acquire_domain_ticket{

	my $adm_principal = registry::getReg("ADM_PRINCIPAL");
	my $ip_address = registry::getReg("IPADDRESS");
	my $resolv_update = 0;
	my $rc = 0, $count = 0;

	#Running kinit to get the ticket.

	Log(MESG, ">>>Acquiring the domain ticket");
	eval{
		if (! -s $ENV{KRB5CCNAME}) {
			Log(MESG, "Getting the Domain ticket");
			do {
				if ($rc != 0) {
					Log(INFO, "Could not get the ticket, Retrying ...");
					$count ++;
					sleep(5);
				}
				$rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
			} while ($rc != 0 && $count < 10);	#Looping it 10 times to make sure KDC could come up.

			if ($rc != 0) {
				die("Could not get the ticket");
			}
 		}
	};
	if ($@) {
		#Kinit could not get the ticket.
		Log(ERR, $@);
	}

	Log(MESG, "   Acquired the ticket for $adm_principal");
}

sub samify_user_objects
{

	if (! registry::getReg("MAPPED")) {
		Log("Only Name Mapped scenario samification happens. Hence ignoring it.");
		return;
	}

	my $adm_principal = registry::getReg("ADM_PRINCIPAL");
	my $default_root_domain = registry::getReg("DEFAULTROOTDOMAIN");
	my $ldap_search = registry::getReg("LDAPSEARCH");
	my $ldap_search_flags = registry::getReg("LDAPSEARCH_FLAGS");
	my $ldap_modify = registry::getReg("LDAPMODIFY");
	my $ldap_modify_flags = registry::getReg("LDAPMODIFY_FLAGS");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $lib		= registry::getReg("_LIB");
	my $rc;

	eval{
		if (! (defined $ENV{ADM_PASSWD_DOMAIN} || defined $ENV{ADM_PASSWD})) {
			die("The credentials for doing the samification is not supplied.");
		}

		if (defined $ENV{ADM_PASSWD_DOMAIN}) {
			$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
		}

		Log(MESG, ">>>Adding SAM attributes to the existing user objects");

		$rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
		if ($rc != 0) {
			die ("Could not get the ticket for $adm_principal");
		}

		`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldap_search $ldap_search_flags -b "$default_root_domain" "(&(objectclass=user))" dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' > $live_dsstatedir/samify/intermediate_file1`;
		`grep 'dn: *' $live_dsstatedir/samify/intermediate_file1 | grep -v '^#.*' > $live_dsstatedir/samify/intermediate_file2`;

		`makeLdif.pl -t $template_dir/samify_user.ldif -a $live_dsstatedir/samify/intermediate_file2 -o $live_dsstatedir/samify/final_usr.ldif`;

		$rc = system("SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldap_modify $ldap_modify_flags -f $live_dsstatedir/samify/final_usr.ldif");

		if ($rc != 0) {
			die("Could not samify the user objects $ldap_modify $ldap_modify_flags");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The existing user object are now samified");
}

sub samify_group_objects
{

	if (! registry::getReg("MAPPED")) {
		Log("Only Name Mapped scenario samification happens. Hence ignoring it.");
		return;
	}

	my $default_root_domain = registry::getReg("DEFAULTROOTDOMAIN");
	my $ldap_search = registry::getReg("LDAPSEARCH");
	my $ldap_search_flags = registry::getReg("LDAPSEARCH_FLAGS");
	my $ldap_modify = registry::getReg("LDAPMODIFY");
	my $ldap_modify_flags = registry::getReg("LDAPMODIFY_FLAGS");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $template_dir = registry::getReg("TEMPLATEDIR");
	my $lib		= registry::getReg("_LIB");
	my $rc;

	eval{
		Log(MESG, ">>>Adding SAM attributes to the existing group objects");

		`SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldap_search $ldap_search_flags -b "$default_root_domain" "(&(objectclass=group)(!(sAMAccountName=*)))" dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' > $live_dsstatedir/samify/intermediate_file1`;
		`grep 'dn: *' $live_dsstatedir/samify/intermediate_file1 | grep -v '^#.*' > $live_dsstatedir/samify/intermediate_file2`;

		`makeLdif.pl -t $template_dir/samify_grp.ldif  -a $live_dsstatedir/samify/intermediate_file2 -o $live_dsstatedir/samify/final_grp.ldif`;

		$rc = system("SASL_PATH=/opt/novell/xad/$lib/sasl2 $ldap_modify $ldap_modify_flags -f $live_dsstatedir/samify/final_grp.ldif");

		if ($rc != 0) {
			die("Could not samify the existing group objects");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The existing groups are now samified");
}

sub can_samify
{

	my $adminname = registry::getReg("Administrator DN");
	my $counter = 0;
	my $retry_count = 5;
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $rc = 0;
	my $is_mapped = registry::getReg("MAPPED");
	my $install_type = registry::getReg("INSTALL_TYPE");
	if ($install_type eq "frd" && $is_mapped eq 0) {
		$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};
	}
	my $temp_file = mktemp("/tmp/tmp.XXXXXXXX");

	do {
		$rc = 0;
		if ($counter > 0) {
			Log (INFO, "Retrying to samify administrator object, retry count:$counter");
			sleep (10);
		}
		`echo -e "dn:$adminname\nchangetype:modify\nadd: Description\nDescription: xxxxxx\n" >$temp_file`;
		$rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
					admin=>$ldapadminname,
					pwd=>$ENV{'NDSEXISTINGADMINPASSWD'},
					file=>$temp_file,
					cont_if_exist=>"true");
		if ($rc != 0) {
			Log(WARN, "Could not import [$temp_file] for adding Description attribute");
		} else {
			unlink($temp_file);
		}

		`echo -e "dn:$adminname\nchangetype:modify\ndelete: Description\nDescription: xxxxxx\n" >$temp_file`;
		$rc = import::import_file(host=>registry::getReg("LDAPI_URL"),
					admin=>$ldapadminname,
					pwd=>$ENV{'NDSEXISTINGADMINPASSWD'},
					file=>$temp_file,
					cont_if_exist=>"true");
		if ($rc != 0) {
			Log(WARN, "Could not import [$temp_file] for deleting Description attribute");
		} else {
			unlink($temp_file);
		}

		sleep (5);

		`/usr/bin/ldapsearch -D '$ldapadminname' -x -w '$ENV{NDSEXISTINGADMINPASSWD}' -s base -LLL -b '$adminname' uniquedomainid | grep -i uniquedomainid`;
		if ($? != 0) {
			$rc = $?;
			#The samification did not happen, update the domain now
			util::send_domain_update_now_signal ();
		}
		$counter ++;
	}while ($rc != 0 && $counter < $retry_count);
	return $rc;
}

sub generate_random_string
{
	my $s_size = shift;
	my @alphanumeric = ('a'..'z', 'A'..'Z', 0..9);
	my $result = join '', map $alphanumeric[rand @alphanumeric], 0..($s_size-1);

	return $result;
}

#Added for upgrade scenario
sub upgrade_samification{
	my %args = @_;
	my $ld = ${$args{hndl}};
	my $dn = $args{dn};
	my $description = $args{desc};
	my $livedsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $lib = registry::getReg("_LIB");

	#Added code for ldif template
	my $ldif_file = "$livedsstatedir/samify/final_dn.ldif";
	open TFILE, ">$ldif_file" or die("failed to open the $ldif_file : $!");
	print TFILE "dn: $dn\n";
	print TFILE "changetype: modify\n";
	print TFILE "add: Description\n";
	print TFILE "Description: $description\n";
	print TFILE "-\n";
	print TFILE "changetype: modify\n";
	print TFILE "delete: Description\n";
	print TFILE "Description: $description\n";
	close(TFILE);

	#Description addition and deletion
	my @output = `SASL_PATH=/opt/novell/xad/$lib/sasl2 /usr/bin/ldapmodify -Q -Y GSSAPI -f $ldif_file 2>&1`;
	my $error = $? >> 8;
	if ($error and $error != LDAP_ERR_DUP) {
		helpers::logWrite(1, "samifying ". $dn);
		helpers::logWrite(1, "Samifying dn failed: ".$error." : [".@output."]");
	} else {
		Log ("samifying ". $dn . "\n");
	}

	unlink($ldif_file);
}

sub samify_all_domain_objects
{

	my $admin_name = registry::getReg("LDAPADMINNAME");
	my $domain_nc = registry::getReg("Domain NC");
	my $is_mapped = registry::getReg("MAPPED");
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $cntl1 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.6",
				   value	=> "Violate domain boundary",
				   critical => 0
				 );

	my $cntl2 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.8",
				   value	=> "Search Partition Boundary",
				   critical => 0
				 );

	my $crossref_dn = registry::getReg("CrossRef_DN");
	my $description = generate_random_string(RAND_STRING_LENGTH);
	my $upgrade_on = shift || UPGRADE_OFF;
	my $ld;

	eval{
		Log(MESG, ">>>Adding UniqueDomainID to the existing objects");

		if ($upgrade_on eq UPGRADE_ON) {
			$ld = Net::LDAP->new("$myLocalhost", scheme=>"ldap", port=>389, timeout=>120) or die("Could not create connection");
			my $msg = $ld->bind;
			$msg->code && die("Could not bind to LDAP with error code: [ ". $msg->code .
			" ] and error message: [ ". $msg->error. " ]");
		} else {
			if (! (defined $ENV{ADM_PASSWD_DOMAIN} || defined $ENV{ADM_PASSWD})) {
				die("The credentials for doing the samification is not supplied.");
			}

			if (defined $ENV{ADM_PASSWD_DOMAIN}) {
				$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
			}

			if ($install_type eq "frd" && $is_mapped eq 0) {
				$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};
			}

			if (!(defined $ENV{NDSEXISTINGADMINPASSWD}) && $is_mapped eq 1) {
				die("The tree administrator credentials is not supplied during samification");
			}

			$ld = import::connectandbind (admin=>$admin_name, pwd=>$ENV{NDSEXISTINGADMINPASSWD});
		}
		helpers::logAttrsSet("samification.log", 2);

		my $mesg = $ld->search (base=>$crossref_dn,
				filter=> "(objectclass=*)",
				scope=>'base',
				timelimit=>30,
				attrs=>['domainpartitionlist'],
				deref=>'never');
		$mesg->code && die("LDAP search failed on [$crossref_dn] with error code:[ ".
			$mesg->code." ]and error message:[ ".$mesg->error." ]");

		my @partitions = $mesg->entry->get_value('domainpartitionlist');
		if (scalar @partitions == 0) {
			Log ("The domainpartitionlist is empty, putting only local domain nc [$domain_nc]");
			push (@partitions, $domain_nc) ;

		}

		foreach my $samifying_partition (@partitions) {
			Log("Generating sAMAccountName attribute for objects under $samifying_partition");
			my $mesg = $ld->search (base=>$samifying_partition,
					filter=>"(objectclass=*)",
					scope=>'sub',
	#				timelimit=>30,
					attrs=>['dn'],
					deref=>'never',
					control=>[$cntl1,$cntl2]);
			$mesg->code && die("LDAP search failed on [$samifying_partition] with error code: [ ".
				$mesg->code ." ]and error message: [ ". $mesg->error." ]");

			#In reference to the Bug 681363, the samify task
			#logs all the samification dn etries and status
			#into samify.log currently getting created under
			#location /var/opt/novell/xad/log.
			#
			helpers::logWrite(1, "Performing samification on $samifying_partition\n");
			if ($upgrade_on eq UPGRADE_ON) {
				foreach my $entry ($mesg->entries) {
					upgrade_samification(hndl=>\$ld, dn=>$entry->dn(), desc=>$description);
				}
			} else {
				foreach my $entry ($mesg->entries) {
					#Add description
					$mesg = $ld->modify ($entry->dn(), add => { description=>$description});
					if (ldap_error_name($mesg) ne "LDAP_SUCCESS") {
						helpers::logWrite(1, "samifying ". $entry->dn());
						helpers::logWrite(1, "Samification> ".ldap_error_name($mesg)." : ".ldap_error_text($mesg));
					} else {
						helpers::logWrite(3, "samifying ". $entry->dn() . "\n");
					}

					#Delete description
					$mesg = $ld->modify ($entry->dn(), delete => { description=>$description});
				}
			}
			helpers::logWrite(1, "Samification on partition [$samifying_partition] completed\n");

		}
		$ld->unbind;

	};

	if ($@) {
		Log(WARN, $@);
		Log(MESG, "   Some of the objects are not samified for more details".
				" Please check the samification.log");
	} else {
		helpers::logWrite(1, "All the objects under this domain are now samified\n");
	}
}

sub remove_invalid_uniquedomainid
{

	my $admin_name = registry::getReg("LDAPADMINNAME");
	my $domain_nc = registry::getReg("Domain NC");
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $ldapi_url = registry::getReg("LDAPI_URL");
	my $uniquedomainid = 0;

	#This module is called only for child domain installation
	if ($install_type ne "child") {
		return;
	}

	eval{
		if (! defined $ENV{NDSEXISTINGADMINPASSWD}) {
			die("The credentials are not supplied.");
		}

		Log(MESG, ">>>Removing invalid UniqueDomainID value from the existing objects");

		$uniquedomainid = registry::getReg("TRUSTPOSIXOFFSET");

		my $ld = import::connectandbind (host=>$ldapi_url, admin=>$admin_name, pwd=>$ENV{NDSEXISTINGADMINPASSWD});
		my $mesg = $ld->search (base=>$domain_nc, filter=>"(&(objectclass=*)(!(uniquedomainid=$uniquedomainid)))",
					scope=>'sub', timelimit=>30, attrs=>['dn'], deref=>'never');
		$mesg->code && die $mesg->error;

		foreach my $entry ($mesg->entries) {
			Log ("Removing invalid uniqueDomainID for ". $entry->dn() . "\n");

			#Delete uniqueDomainID
			$mesg = $ld->modify ($entry->dn(), delete => ['uniqueDomainId']);
		}

		$ld->unbind;

	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   All the invalid uniqueDomainIds are removed from this domain");
}

#Remove the parent to child trust
sub remove_parent_child_trust
{

	my $parent_name = registry::getReg("Parent Domain");
	my $parent_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_name`;
	if ($?) {
		my $error = $? >> 8;
		Log(ERR, "Failed to get the domain provider for [$parent_name] [$error]");
	}
	chomp($parent_provider);
	my $domain_name = registry::getReg("Root Domain");
	my $xtrust = registry::getReg("XTRUST");
	my $retry = 0;


	Log(MESG, ">>>Removing parent child trust");
RETRY:
	$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_PARENT");
	Log("Executing $xtrust -h $parent_provider --delete-trust $domain_name");

	my $rc = system("$xtrust -h $parent_provider --delete-trust $domain_name");
	if ($rc != 0) {
		$retry += 1;
		handle_trust_failure() if ($retry == 1);
		goto RETRY if ($retry == 1);
		Log (WARN, "Removing parent child trust failed.");
	}
}

#Remove the child to parent trust
sub remove_child_parent_trust
{

	my $parent_name = registry::getReg("Parent Domain");
	my $xtrust = registry::getReg("XTRUST");
	my $retry = 0;

	Log(MESG, ">>>Removing child parent trust");

RETRY:
	$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
	Log("Executing $xtrust -h $myLocalhost --delete-trust $parent_name");

	my $rc = system("$xtrust -h $myLocalhost --delete-trust $parent_name");
	if ($rc != 0) {
		$retry += 1;
		handle_trust_failure() if ($retry == 1);
		goto RETRY if ($retry == 1);
		Log (WARN, "Removing child parent trust failed.");
	}
}

sub remove_trusts
{
	Log(MESG, ">>>Removing the existing trust");

	remove_parent_child_trust();
	remove_child_parent_trust();

	Log(MESG, "   Removed the trust between parent and child");
}

sub generate_shared_secret_export
{

	#We generate a random string and export this with the variable SHAREDSECRET
	#which will be used during the TDO creation.
	my $shared_secret =`/opt/novell/xad/sbin/xtrust -q --generate-random-string`;
	chomp($shared_secret);

	$ENV{SHAREDSECRET} = $shared_secret;

}

#Makes a parent to child trust
sub add_parent_child_trust
{

	my $parent_name = registry::getReg("Parent Domain");
	my $parent_provider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_name`;
	if ($?) {
		my $error = $? >> 8;
		Log(ERR, "Failed to get the domain provider for domain [$parent_name] [$error]");
	}
	chomp($parent_provider);
	my $domain_name = registry::getReg("Root Domain");
	my $xtrust = registry::getReg("XTRUST");
	my $trust_attr = "-a WITHIN_FOREST"; #Use TRUST_ATTRIBUTE_WITHIN_FOREST
	my $netbios_name = registry::getReg("NETBIOSNAME");
	my $domain_sid = registry::getReg("DOMAINSID");
	my $retry = 0;

	eval{

		Log(MESG, ">>>Adding parent to child trust");
RETRY:
		$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_PARENT");
		Log("Executing $xtrust -q $trust_attr -f $netbios_name -s $domain_sid -e SHAREDSECRET -h $parent_provider -i -o -t uplevel --add-trust $domain_name");

		my $rc = system("$xtrust -q $trust_attr -f $netbios_name -s $domain_sid -e SHAREDSECRET -h $parent_provider -i -o -t uplevel --add-trust $domain_name");

		if ($rc != 0) {
			$retry += 1;
			handle_trust_failure() if ($retry == 1);
			goto RETRY if ($retry == 1);
			die("Could not create the parent to child trust");
		}
	};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Parent to child trust created successfully");

}

#Makes a child to parent trust
sub add_child_parent_trust
{
	my $parent_domain = registry::getReg("PARENTDOMAIN");
	my $dns_root = registry::getReg("DNSROOT");
	my $trust_attr_parent = "";
	my $trust_attr_root   = "";
	my $domain_context_root = grep(/$parent_domain/,$dns_root);
	my $forest_nc = registry::getReg("Forest NC");
	my $parent_nc = registry::getReg("Parent NC");

	if ($domain_context_root ne "") {
		$trust_attr_parent = "-a TREE_PARENT";
	}

	$parent_nc = lc($parent_nc);
	$forest_nc = lc($forest_nc);

	if ($forest_nc eq $parent_nc) {
		$trust_attr_root = "-a TREE_ROOT";
	}

	my $xtrust = registry::getReg("XTRUST");
	my $trust_attr = "-a WITHIN_FOREST"; #Use TRUST_ATTRIBUTE_WITHIN_FOREST
	my $domain_sid = registry::getReg("DOMAINSID");

	$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_PARENT");
	my $netbios_parent = `/opt/novell/xad/share/dcinit/provisionQuery.sh $parent_domain "NetBIOS Name"`;
	my $domainsid_parent  = `/opt/novell/xad/share/dcinit/provisionQuery.sh $parent_domain "Dom Sid"`;
	my $retry = 0;
	chomp($netbios_parent, $domainsid_parent);

	$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
	eval{

		Log(MESG, ">>>Adding child to parent trust");
RETRY:
		$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
		Log("Executing $xtrust -q $trust_attr $trust_attr_parent $trust_attr_root -f $netbios_parent -s $domainsid_parent -e SHAREDSECRET -h $myLocalhost -i -o -t uplevel --add-trust $parent_domain");

		my $rc = system("$xtrust -q $trust_attr $trust_attr_parent $trust_attr_root -f $netbios_parent -s $domainsid_parent -e SHAREDSECRET -h $myLocalhost -i -o -t uplevel --add-trust $parent_domain");

		if ($rc != 0) {
			$retry += 1;
			handle_trust_failure() if ($retry == 1);
			goto RETRY if ($retry == 1);
			die("Could not create child to parent trust");
		}

	};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The child to parent trust is successfull");

}

sub handle_trust_failure {

	my $parent_domain = registry::getReg("Parent Domain");
	my $adm_principal = "";
	my $rc = 0;

	eval {
		$ENV {KRB5CCNAME} = registry::getReg("KRB5CCNAME_PARENT");
		if (not defined $ENV{ADM_PASSWD_PARENT} ||
			$ENV{ADM_PASSWD_PARENT} eq "") {
			die("The credentials of parent domain not available");
		}

		$ENV {ADM_PASSWD} = $ENV{ADM_PASSWD_PARENT};
		my $parent_admin_samname = registry::getReg("SAMACCOUNTNAME_PARENT");
		$adm_principal = "'$parent_admin_samname'\@$parent_domain";
		$rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
		if ($rc != 0) {
			die ("Could not get the ticket for parent $adm_principal");
		}

		if ( (not defined $ENV{ADM_PASSWD_DOMAIN}) ||
			($ENV{ADM_PASSWD_DOMAIN} eq "")){
			die("The credentials for getting the ticket not supplied.");
		}

		$ENV {KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
		$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
		$adm_principal = registry::getReg("ADM_PRINCIPAL");

		$rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
		if ($rc != 0) {
			die ("Could not get the ticket for $adm_principal");
		}
	};

	if ($@) { Log (ERR, $@) }
}

sub add_trusts
{
	registry::setReg("TRUST_ATTRIBUTE_WITHIN_FOREST", "-a WITHIN_FOREST");

	Log(MESG, ">>>Adding the trust between parent and child");

	add_parent_child_trust();
	add_child_parent_trust();

	Log(MESG, "   The trust between parent and child is created successful");
}

sub bind_cross_ref
{

	my $adm_principal = registry::getReg("ADM_PRINCIPAL");
	my $xtrust = registry::getReg("XTRUST");
	my $parent_name = registry::getReg("Parent Domain");
	my $domain_name = registry::getReg("Root Domain");
	my $forest_name = registry::getReg("Forest Root");
	my $rc;

	eval{
		#Kinit is already run
		#$rc = system("kinit -E $adm_principal");
		#die("Could not run kinit") if ($rc != 0);

		Log(MESG, ">>>Binding cross reference");
		$rc = system("$xtrust -T $parent_name --bind-cross-ref $domain_name");
		die("Could not bind cros ref") if ($rc != 0);

		$rc = system("$xtrust -T $forest_name -r --bind-cross-ref $domain_name");
		die("Could not bind cross ref") if ($rc != 0);
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The cross refrence binding between parent and child are done");
}

sub migrate_nkdc_realm {

	my $ldapadmin = registry::getReg("LDAPADMINNAME");
	my $migrate_nkdc = registry::getReg("MIGRATE_NKDC");
	my $nkdc_realm = registry::getReg("NKDC Realm");
	my $rc = 0;

	chomp($nkdc_realm);

	Log(MESG, ">>>Migrating the NKDC realm users");

	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};

	eval {
		if ($nkdc_realm eq '""') {
			Log("The NKDC realm to be migrated is not defined in xad configuration file.");
			return 0;
		}

		if (not defined $ENV {ADM_PASSWD} || $ENV{ADM_PASSWD} eq "") {
			die("The credentials for migrating the nkdc users not given.");
		}
		if (registry::getReg("MAPPED")) {

			Log(MESG, "Migrating nkdc realm users from $nkdc_realm");
			$rc = system("$migrate_nkdc -r $nkdc_realm -D $ldapadmin -p 389");
			die ("could not migrate novell kdc users") if ($rc != 0);
		} else {
			die("The NKDC migration is done only for name mapped setups.");
		}
	};

	if ($@) {Log(ERR, $@); }

	Log(MESG, "   NKDC realm users from $nkdc_realm are successfully migrated");
}

#
# Join the domain - this will generate the keytab for this machine
#
# For eDirectory, use the -S option to bind the NCP server object
# to the newly created AD server object. (We assume that the NCP
# server object for a newly created server is located in
# DEFAULTPARENTNC.)
#
# Actually, the function name should be domain_add as the
# provision --join operation is changed to provision --add operation
# This is done because provision --join operation that also sets
# password (using kpasswdd server) failed in a different subnet scenario.
# However, retaining the same function name

sub domain_join
{

	my $xadroot = registry::getReg("XADROOT");
	my $install_machinename = registry::getReg("INSTALLMACHINENAME");
	my $nds_temp_container = registry::getReg("NDSTEMPCONTAINER");
	my $provider = registry::getReg("PROVIDER");
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $machine_guid = registry::getReg("MACHINEGUID");
	my $ipaddress = registry::getReg("IPADDRESS");
	my $netbios_name = registry::getReg("Install_DCName");
	my $domain_name = registry::getReg("Root Domain");
	my $domain_nc = registry::getReg ("Mapped Domain NC");

	my $sitename = util::find_sitename($domain_name);

	#Make sure the Install Machine NetBIOS name is unique.
	`/opt/novell/xad/share/dcinit/provisionTools.sh netbios-query -p $ipaddress -n $netbios_name`;
	if ($?) {
		Log(WARN, "Install Machine NetBIOS name already exists. The domain join operation may fail.");
	}

	Log(MESG, ">>>Adding the domain");

	Log("Removing old machine account (if necessary)");

	my @sitelist = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -s sub -b cn=sites,cn=configuration,$domain_nc "(objectclass=site)" dn -LLL | grep "^dn:"`;
	chomp(@sitelist);
	foreach my $site (@sitelist) {
		$site =~ s/^dn:\s*//;
		$site =~ s/,cn=Sites,.*//i;
		$site =~ s/cn=//i;

		Log("Executing command $xadroot/sbin/provision -q -m $netbios_name -s $site -S CN=$install_machinename,$nds_temp_container --remove $provider");

		system("$xadroot/sbin/provision -q -m $netbios_name -s $site -S CN=$install_machinename,$nds_temp_container --remove $provider");
	}

	if (-e "$live_dsstatedir/krb5kdc/krb5.keytab") {
		Log("Removing old keytab file");
		unlink("$live_dsstatedir/krb5kdc/krb5.keytab");
	}


	Log("Adding this machine to the domain");

	eval {
		#Acquire the TGT before joining to the domain.
		#TODO Need to set KRB5CCNAME_DOMAIN during reg set.
		#Right Now its at default location of /tmp/krb5cc_0
		#$ENV {KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
		my $adm_principal = registry::getReg("Domain Administrator Principal");

		$rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
		if ($rc != 0) {
			die ("Could not get the ticket for $adm_principal");
		}

		#Joining to the domain.
		Log("Executing ... $xadroot/sbin/provision -q -g \"$machine_guid\" -i $ipaddress ".
				"-k $live_dsstatedir/krb5kdc/krb5.keytab -m $netbios_name ".
				"-S 'CN=$install_machinename,$nds_temp_container' ".
				"-s $sitename --add $provider");

		my $rc = system("$xadroot/sbin/provision -q -g \"$machine_guid\" -i $ipaddress ".
				"-k $live_dsstatedir/krb5kdc/krb5.keytab -m $netbios_name ".
				"-S 'CN=$install_machinename,$nds_temp_container' ".
				"-s $sitename --add $provider");
		if ($rc !=0) {
			die("Could not add this machine to the domain");
		}

	};

	if ($@) {Log(ERR, $@); }

	Log(MESG, ">>>Successfully added the domain controller");
}

sub remove_ticket_cache {

	Log("   Clearing the credential cache");
	`/opt/novell/xad/bin/kdestroy`;

	if (registry::getReg("KRB5CCNAME_PARENT")) {
		Log("   Clearing the parent domain credential cache");
		$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_PARENT");
		`/opt/novell/xad/bin/kdestroy`;
	}

	if (registry::getReg("KRB5CCNAME_DOMAIN")) {
		Log("   Clearing the local credential cache");
		$ENV{KRB5CCNAME} = registry::getReg("KRB5CCNAME_DOMAIN");
		`/opt/novell/xad/bin/kdestroy`;
	}
}

sub associate_passwd_policy ($){

	my $retain_passwd_policy = shift;
	my ($container, $passwd_policy_dn) = "";
	my $domain_nc = registry::getReg("Mapped Domain NC");
	my $temp_ldif = `/bin/mktemp`;
	my $ldapmodify = registry::getReg("LDAPMODIFY");
	my $ldapi_url = registry::getReg("LDAPI_URL");

	if ($retain_passwd_policy eq "yes") {
		Log("   Password policies should be retained, so doing nothing");
		return ;
	}

	if ((registry::getReg("INSTALL_TYPE") ne "frd") ||
		(registry::getReg("MAPPED") == 1)) {
		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	}

	my $ldapadminname = registry::getReg("NDSADMINNAME");
	$ldapadminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
	my $ldapmodify_flags = " -H $ldapi_url -x -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}'";

	if ($retain_passwd_policy eq "" || not defined ($retain_passwd_policy)) {
		$retain_passwd_policy = "no";
	}
	eval {
		Log("Executing ... $ldapmodify $ldapmodify_flags -f $temp_ldif");


		if ($retain_passwd_policy eq "no") {
			$container = registry::getReg("Mapped Domain NC");
			$passwd_policy_dn = "nspmPasswordPolicyDN: CN=Domain Password Policy,CN=Password Policies,CN=System,$domain_nc";
			Log(MESG, "Associating [$container] with password policy [$passwd_policy_dn]");

			open (TEMP, ">$temp_ldif") or die ("Failed to open the temporary file");
			print TEMP "\ndn: $container\n";
			print TEMP "changeType: modify\n";
			print TEMP "replace: nspmPasswordPolicyDN\n";
			print TEMP $passwd_policy_dn."\n";
			close(TEMP);

			my $rc = import::import_file(host=>$ldapi_url,
				admin=>$ldapadminname,
				file=>$temp_ldif,
				cont_if_exist=>"true");

			if ($rc != 0) {
				die ("Failed to update the container [$container]");
			}

			unlink($temp_ldif);
		}
	};

	if ($@) {
		Log(ERR, $@);
	}
}

#
# Here we get the list of existing eDirectory users and attempt to
# get the unicodepwd and supplementalcredentials attribute populated
#
sub check_and_generate_AD_keys(){
	my $live_dsstatedir = registry::getReg("LIVE_DSSTATEDIR");
	my $mapped = registry::getReg("MAPPED");
	my $domain_nc = registry::getReg("Mapped Domain NC");

	my @non_samified_users;
	my @non_unique_users;
	my $non_samified_users_count = 0;
	my $non_unique_users_count = 0;
	my $admin_name = registry::getReg("LDAPADMINNAME");
	my $machine_name = registry::getReg("INSTALLMACHINENAME");
	my $is_mapped = registry::getReg("MAPPED");
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $cntl1 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.6",
				   value	=> "Violate domain boundary",
				   critical => 0
				 );
	my $cntl2 = Net::LDAP::Control->new(
				   type	 => "2.16.840.1.113719.1.513.4.8",
				   value	=> "Search Partition Boundary",
				   critical => 0
				 );
	my $crossref_dn = registry::getReg("CrossRef_DN");
	my $ld = "";

	eval{
		Log(MESG, ">>> Generating AD SAM keys for pre-existing partitions");

		if (! (defined $ENV{ADM_PASSWD_DOMAIN} || defined $ENV{ADM_PASSWD})) {
			die("The credentials needed for samification is not supplied.");
		}

		if (defined $ENV{ADM_PASSWD_DOMAIN}) {
			$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_DOMAIN};
		}

		if ($install_type eq "frd" && $is_mapped eq 0) {
			$ENV{NDSEXISTINGADMINPASSWD} = $ENV{ADM_PASSWD};
		}

		if (!(defined $ENV{NDSEXISTINGADMINPASSWD}) && $is_mapped eq 1) {
			die("The tree administrator credentials is not supplied");
		}

		$ld = import::connectandbind (admin=>$admin_name, pwd=>$ENV{NDSEXISTINGADMINPASSWD});

		my $mesg = $ld->search (base=>$crossref_dn,
				filter=> "(objectclass=*)",
				scope=>'base',
				timelimit=>30,
				attrs=>['domainpartitionlist'],
				deref=>'never');
		$mesg->code && die("LDAP search failed on [$crossref_dn]".
			" with error code: [ ".$mesg->code." ] and error message: [ ".
			$mesg->error. " ]");

		my @partitions = $mesg->entry->get_value('domainpartitionlist');
		if (scalar @partitions == 0) {
			Log("Found no extra partitions in domain for AD SAM Key generation");
			push (@partitions, $domain_nc);
		}

		foreach my $samifying_partition (@partitions) {
			helpers::logWrite(1, "\nPerforming AD SAM Key generation on users under [$samifying_partition]");
			my $filter = "(objectclass=user)";
			Log(">>> Examining partition [ $samifying_partition ] for pre-existing users..");
			if ( lc($samifying_partition) eq lc($domain_nc)) {
				# Skip user objects that are created by us in default domain NC.
				$filter = "(&(objectclass=user)(!(|(name=guest)(name=Administrator)".
					"(name=OESCommonProxy_*)(name=krbtgt)(name=$machine_name)".
					"(name=xad_provisioning_user))))";
			}
			my $mesg = $ld->search (base=>$samifying_partition,
					filter=>$filter,
					scope=>'sub',
					timelimit=>30,
					attrs=>['dn'],
					deref=>'never',
					control=>[$cntl1,$cntl2]);
			$mesg->code && die("LDAP search failed on [$samifying_partition] ".
				"with error code: [". ldap_error_name($mesg). "] error message: [".
				ldap_error_text ($mesg)."] [". $mesg->dn."]");
			my @entries = $mesg->entries;
			foreach $entry (@entries) {
				my $dn = $entry->dn();
				helpers::logWrite(3, "Processing user $dn\n");
				# Do not look for accounts without objectSid, not needed
				$mesg = $ld->modify($dn, delete => ['objectSid'] );

			}
		}
		helpers::logWrite(1, "AD SAM Key generation complete..");
		$ld->unbind;
	};

	if ($@) {
		Log(WARN, $@);
		Log(ERR, "   Unable to add AD SAM Keys on pre-existing user objects in domain");
	}

}

;
