#!/usr/bin/perl -I/opt/novell/xad/lib/perl -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib64/perl/XAD
################################################################################
# Domain Services for Windows - Commonly used subroutines for WINS
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2013
#       Novell, Inc.  All rights reserved.
################################################################################

package checkdupws;
use strict;
use warnings;
use Term::ReadKey;
use Logger qw(:DEFAULT);
use IO::File;
use File::Temp;
use Install::check;
use File::Basename;
use XAD::registry;


############################################################################
# output of ./checkdupws.pl:
#
# console output:
#
# In case of duplicate workstation:
# frd:/opt/novell/xad/share/dcinit # ./checkdupws.pl --history
# 192.168.56.31	win1
# 192.168.56.30	win1
#
# In case of no duplicate workstation:
# frd:/opt/novell/xad/share/dcinit # ./checkdupws.pl --current
# No duplicate workstations detected
#
# File checkdupws.log content:
#
# In case of duplicate workstation:
# frd:/opt/novell/xad/share/dcinit # ./checkdupws.pl --history
# --------------------------------------------
#
# 2013-09-25 12:04:07
# DUPLICATE WORKSTATIONs - Since the beginning
#
# 192.168.56.31 win1
# 192.168.56.30 win1
#
# ----------------------------------------------
#
# In case of no duplicate workstation:
# frd:/opt/novell/xad/share/dcinit # ./checkdupws.pl --current
# ---------------------------------------------
#
# 2013-09-25 12:04:07
# DUPLICATE WORKSTATIONs - Since last '-clear-cache'
#
# No duplicate workstations detected
#
# -----------------------------------------------
#
# Logging information related to checkdupws
#
# 1. Duplicate workstation logs
#
# These should be logged using TEXT function.
#
# It is logged in console as well as file
#
# 2. Title
#
# Describes the kind of text being printed.(duplicate workstations or scheduling) along with the timestamp.
#
# Its logged in file only.
#
# 3. Log_Debug
#
# These should be used to print log in console only when --debug option is passed
#
# In log file it always prints the logs irrespective of --debug option.
#
# These are just informational logs used to understand the flow of execution.
#
# 4. Log_Warning
#
# These should be used to print Warning in console as well as file without --debug option
#
# These messages are for scenario where partial "duplicate workstation" information is returned.
#
# For e.g when adc is down.
#
# 5. Decorator
#
# Its to provide text boundary. "--------------" in this case.
#
# 6. Log(ERR,...)
#
# For fatal information like directory structure not getting created.
#
# In general no timestamp should come on console.
#
############################################################################

# Global variable's for WINS

my $wins_dir = "/var/opt/novell/xad/wins/";

#TIME
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;


our ($help,$current,$history,$clear_cache,$schedule,$unschedule,$initialize,$is_debug) = (0, 0, 0, 0, 0, 0, 0, 0);
my $title_suffix = "";
my $var_xad_dir = registry::getReg("LIVE_LOCALSTATEDIR");
my $etc_xad_dir = registry::getReg("LIVE_ETCDIR");
my $duplicate_workstation_logdir = "$var_xad_dir/log";
my $duplicate_workstation_logfile = "checkdupws.log";
my $wins_kdc_log_file = "checkdupws_kdc.log";
my $duplicate_workstation_log = $duplicate_workstation_logdir."/".$duplicate_workstation_logfile;
my @duplicate_workstations = ();
my %dupws_hash = ();
my @global_dc_array = ();
my $wins_kdc_log_collection = "$wins_dir/dcs/*/$wins_kdc_log_file";
my $local_wins_kdc_log = "$wins_dir/$wins_kdc_log_file";
my $wins_kdc_file_string = "\tkdc = FILE:$local_wins_kdc_log\n";
my $krb5_config_filename = "$etc_xad_dir/krb5.conf";
my $kdc_logbase = "$wins_dir/dcs/";
my $kdc_logs = "$kdc_logbase/*/kdc.log*";
my $xad_log = "$var_xad_dir/log";
my $collect_kdclogs_log = "$xad_log/collectkdc.log";
my $collectkdclogbin = "/opt/novell/xad/sbin/collectkdclogs";

#******************************************************************************
# Constant's for WINS
#******************************************************************************


#Script return values
use constant {
	XAD_UNKNOWN_ERROR => -1,
	XAD_KRB5_NOT_RUNNING => 3,
	XAD_FAILED_TO_GET_DC => 9,
};

#Constants script returns
use constant {
	XAD_CHECK_SUCCESS => 0,
	XAD_ALREADY_RUNNING => 1,
	#For operations that need to run only once
	XAD_ALREADY_DONE => 2,
};

#Constants for determining calling environment
use constant {
	#Parameter called if execution by terminal
	XAD_CONSOLE => 1,
	#Parameter called if execution by provisioning
	#Its the default behavior
	XAD_PROVISIONING => 2,
};

#Constants for determining whether krb5.conf is updated
use constant {
	XAD_KRB5_UPDATED => 1,
	XAD_KRB5_NOT_UPDATED => 0,
};


#Environment in which the module is called
my $environment = XAD_PROVISIONING;
my $krb5_conf_update_flag = XAD_KRB5_NOT_UPDATED;


sub Log_Debug {
	my $string = shift;

	if ($environment == XAD_PROVISIONING) {
		Log($string);
		return;
	}

	Logger::unset_mask(LOG_CONSOLE);
	Log($string);
	Logger::set_mask(LOG_CONSOLE);

	if ($is_debug == 1) {
		print STDERR "$string\n";
	}
}

sub Log_Warning {
	my $string = shift;

	if ($environment == XAD_PROVISIONING) {
		Log($string);
		return;
	}

	Logger::unset_mask(LOG_CONSOLE);
	Log($string);
	Logger::set_mask(LOG_CONSOLE);

	print STDERR "$string\n";
}



sub get_domain() {
	my $domain_name = `/bin/hostname -d`;
	chomp($domain_name);

	if ($domain_name eq "") {
		Log_Warning("Failed to get domain name");
		Log(ERR, "Command /bin/hostname -d failed");
	}

	return $domain_name;
}

sub get_host() {
	my $domain_host = `/bin/hostname`;
	chomp($domain_host);

	if ($domain_host eq "") {
		Log_Warning("Failed to get hostname");
		Log(ERR, "Command /bin/hostname failed");
	}

	return $domain_host;
}



#This should be called each time, before calling the
#subroutines checkDuplicateWorkStation.
#This sets the parameters as per the requirement
# of the mentioned subroutines.
sub clear_and_set(@) {
	my %args = @_;

	$checkdupws::initialize = $args{init} || 0;
	$checkdupws::current = $args{cur} || 0;
	$checkdupws::history = $args{his} || 0;
	$checkdupws::schedule = $args{sch} || 0;
	$checkdupws::unschedule = $args{unsch} || 0;
	$checkdupws::clear_cache = $args{clr_ch} || 0;
	$checkdupws::is_debug = $args{debug} || 0;
}

#******************************************************************************
# Subroutine's to find duplicate workstation using WINS on a DSfW server
#******************************************************************************

# Wrapper function for executing commands.
# Filters out unnecessary error messages
sub exec_cmd($) {
	my $cmd = shift;

	my $output = `$cmd 2>/dev/null`;
	return $output;
}


sub add_string_in_krb5() {
	my $krb5_output_filename = mktemp("/tmp/krb5out.XXXXXXXX");
	my $ret = -1;

	open(KRB5_CONFIG_FILE, "<$krb5_config_filename");
	open(TEMPFILE,">$krb5_output_filename");
	my $is_logging = 0;

	while (<KRB5_CONFIG_FILE>) {
		if ($is_logging == 2) {
			print TEMPFILE $_;
			next;
		}
		if ($_ =~ m/\[logging\]/) {
			$is_logging = 1;
			print TEMPFILE $_;
			next;
		}
		if ($is_logging == 1) {
				#my $string = $_;
				$is_logging = 2;
				print TEMPFILE $_;
				print TEMPFILE $wins_kdc_file_string;
				next;
		}
		print TEMPFILE $_;
	}

	# close the file handles
	close(KRB5_CONFIG_FILE);
	close(TEMPFILE);

	chmod(0644, $krb5_output_filename) or
		die("Cannot change permissions of file $krb5_output_filename:$!\n");

	exec_cmd("mv $krb5_output_filename $krb5_config_filename");
	$ret = $?;
	if($ret != 0) {
		print STDERR "Cannot move $krb5_output_filename to $krb5_config_filename:$!\n";
		return ($ret, "Cannot move $krb5_output_filename to $krb5_config_filename:$!");
	}

	return $ret;
}

sub restart_krb5 {
	exec_cmd("rcxad-krb5kdc status");

	my $status = $? >> 8;

	if ($status == XAD_KRB5_NOT_RUNNING) {
		Log_Debug("Returning from restart_krb5 without restarting");
		return;
	}
	Log_Debug("Stopping krb5 service");
	exec_cmd("rcxad-krb5kdc stop");

	# Reset the LD_LIBRARY_PATH env variable so that following services pick up the
	# system provided kerebos and gssapi libraries.

	my $LD_LIBRARY_PATH = $ENV{'LD_LIBRARY_PATH'};
	$ENV{'LD_LIBRARY_PATH'} = "";
	Log_Debug("Starting krb5 service");
	exec_cmd("rcxad-krb5kdc start");
	my $service_xadkrb5kdc_status = $?;
	Log_Debug("krb5 service started with code $service_xadkrb5kdc_status");
	$ENV{'LD_LIBRARY_PATH'} = $LD_LIBRARY_PATH;
}


sub log_duplicatews_hash() {
	DECORATOR("-----------------------------------");
	TITLE("DUPLICATE WORKSTATIONS $title_suffix");

	if (scalar(%dupws_hash) eq 0 ) {
		TEXT("No duplicate workstations detected");
	}

	while (my ($k,$v) = each %dupws_hash ) {
		TEXT("$k\t$v");
	}
	DECORATOR("\n-----------------------------------");
}

sub addto_duplicatews_hash($$) {
	my $ipaddress = shift;
	my $computer_name = shift;

	$dupws_hash{"$ipaddress"} = $computer_name;
}

#Return duplicate ip address of the computer name
#Return -1 if it is not a duplicate
sub get_duplicate($$) {
	my $ipaddress = shift;
	my $computer_name = shift;
	my $domain_name = `/bin/hostname -d`;
	chomp($domain_name);

	my $calculated_ip_address = exec_cmd("/usr/bin/dig $computer_name.$domain_name +short");
	chomp($calculated_ip_address);

	if ("$calculated_ip_address" eq "$ipaddress" || "$calculated_ip_address" eq "") {
		return XAD_UNKNOWN_ERROR;
	}

	return $calculated_ip_address;
}

sub create_dupws_hash() {
    my ($pair_string);

    foreach $pair_string (@duplicate_workstations) {
	my @pair_array  = split(" ", $pair_string);

	my $ipaddress = $pair_array[0];
	my $computername = $pair_array[1];
	my $calculated_ip_address = get_duplicate($ipaddress,$computername);

	if ($calculated_ip_address ne -1) {
		addto_duplicatews_hash("$ipaddress","$computername");
		addto_duplicatews_hash("$calculated_ip_address","$computername");
	} elsif ($history == 1) {
		#With --history option, even corrected workstations should be displayed
		addto_duplicatews_hash("$ipaddress","$computername");
	}
    }
}

#Check from file
sub check_kdc_log($) {
	my $kdc_logs_pattern = shift;

	#In the kdc.log search for Decrypt integrity check
	#The next line should should have an AS_REQ.  In the line
	#look for the ip address and workstation name.
	#A workstation can be identified by a $ after the name and
	#before the @domain name.
	#Sample kdc log can be something like
	#Nov 19 11:12:29 frd krb5kdc[23215](info): preauth (timestamp) verify failure: Decrypt integrity check failed
	#Nov 19 11:12:32 frd krb5kdc[23215](info): AS_REQ (6 etypes {18 17 23 24 -135 3}) 172.16.43.138: PREAUTH_FAILED: verylate$@winsut.com for krbtgt/winsut.com@winsut.com, Preauthentication failed
	#Nov 19 11:12:29 frd krb5kdc[23215](info): preauth (timestamp) verify failure: Decrypt integrity check failed
	#Nov 19 11:12:32 frd krb5kdc[23215](info): AS_REQ (6 etypes {18 17 23 24 -135 3}) 172.16.43.139: PREAUTH_FAILED: verylate2$@winsut.com for krbtgt/winsut.com@winsut.com, Preauthentication failed
	#Output of the command can beas follows(Multiple entries having same name are filtered out)
	#172.16.43.138       verylate
	#172.16.43.139       verylate2

	my $duplicate_domain_ws = exec_cmd(
		"zgrep -A1 -i 'Decrypt integrity check failed' $kdc_logs_pattern".
		"| grep -v 'Decrypt integrity check failed' |awk -F ')' '{print \$3}'".
		"| grep -v '^\$' |awk -F 'for' '{print \$1}' |sort -n | uniq -i".
		"| sort -n | tr ':' ' '  | grep '\$\@'  |  awk '{print \$1,\$3}'".
		"| sed 's/\$\@/ /' | awk '{printf \"%-20s %-20s \\n\",\$1,\$2}'");
	if ($? != 0) {
		Log(ERR, "While checking $kdc_logs_pattern");
		return XAD_UNKNOWN_ERROR;
	}

	@duplicate_workstations = split("\n", $duplicate_domain_ws);

	create_dupws_hash();

	log_duplicatews_hash();

}

sub check_all_duplicates() {
	collect_KDC_logs("--pullkdc");
	check_kdc_log("$kdc_logs");
	return XAD_CHECK_SUCCESS;
}


sub current_run() {
	collect_KDC_logs("--pullkdc-custom");
	$title_suffix = "- Since last '-clear-cache' run";
	check_kdc_log("$wins_kdc_log_collection");
	return XAD_CHECK_SUCCESS;
}

sub DECORATOR($) {
	my $decorator = shift;
	Logger::unset_mask(LOG_TIMESTAMP | LOG_CONSOLE);
	$decorator .= "\n";
	Logger::Log($decorator);
	Logger::set_mask(LOG_CONSOLE | LOG_TIMESTAMP  | LOG_TIMESTAMP);
}

sub TITLE($) {
	my $title = shift;
	Logger::unset_mask(LOG_CONSOLE);
	$title = "\n$title\n";
	Logger::Log($title);
	Logger::set_mask(LOG_CONSOLE | LOG_TIMESTAMP  | LOG_TIMESTAMP);
}

sub TEXT($) {
	my $text = shift;
	Logger::unset_mask(LOG_TIMESTAMP);
	Logger::Log($text);
	Logger::set_mask(LOG_CONSOLE | LOG_TIMESTAMP  | LOG_TIMESTAMP);
}

sub clearcache_run() {
	DECORATOR("-----------------------------------");
	TITLE("DUPLICATE WORKSTATIONS - clear cache ");

	#Make it empty
	exec_cmd("cat >$local_wins_kdc_log </dev/null");
	if ($? != 0) {
	       Log(ERR, "While truncating $local_wins_kdc_log");
	}

	my $ret = collect_KDC_logs("--pushkdc-custom");

	if ($ret == XAD_CHECK_SUCCESS) {
	       TEXT("Cache of duplicate workstations cleared.");
	       DECORATOR("-----------------------------------");
	       return XAD_CHECK_SUCCESS;
	} else {
	       TEXT("Cache of duplicate workstations partially cleared.");
	       DECORATOR("-----------------------------------");
	       return XAD_UNKNOWN_ERROR;
	}
}

sub unschedule_run() {
	my $title_suffix = "- Unschedule run";
	my $cron_command = "/usr/bin/crontab";
	my $cron_listcommand = "/usr/bin/crontab -l";
	my $cron_tempfile = mktemp("winsXXXXXX");
	my $cron_reschedule_tempfile = mktemp("winsXXXXXX");

	DECORATOR("-----------------------------------");
	TITLE("DUPLICATE WORKSTATIONS $title_suffix");

	exec_cmd("$cron_listcommand >$cron_tempfile");
	exec_cmd("cat $cron_tempfile | grep checkdupws.pl | grep xad");
	if ($? == XAD_CHECK_SUCCESS) {
		#Delete existing schedule for checkdupws.pl
		exec_cmd("sed -e '/checkdupws.pl/d' $cron_tempfile >$cron_reschedule_tempfile");
		exec_cmd("$cron_command $cron_reschedule_tempfile");
		TEXT("checkdupws.pl sucessfully unscheduled");
	} else {
		TEXT("checkdupws.pl is not scheduled");
	}

	unlink("$cron_tempfile");
	unlink("$cron_reschedule_tempfile");

	return XAD_CHECK_SUCCESS;
}

sub schedule_run() {
	my $title_suffix = "- Schedule Run in $schedule hours";
	my $cron_command = "/usr/bin/crontab";
	my $cron_listcommand = "/usr/bin/crontab -l";
	my $cron_tempfile = mktemp("winsXXXXXX");
	my $cron_reschedule_tempfile = mktemp("winsXXXXXX");
	my $reschedule = 0;

	my $schedule_minutes = $schedule * 60;

	DECORATOR("-----------------------------------");
	TITLE("DUPLICATE WORKSTATIONS $title_suffix");

	exec_cmd("$cron_listcommand >$cron_tempfile");
	exec_cmd("cat $cron_tempfile | grep checkdupws.pl | grep xad");
	if ($? == XAD_CHECK_SUCCESS) {
		#Delete existing schedule for checkdupws.pl
		exec_cmd("sed -e '/checkdupws.pl/d' $cron_tempfile >$cron_reschedule_tempfile");
		$reschedule = 1;
	}

	if ($reschedule == XAD_CHECK_SUCCESS) {
		#An entry needs to be appended to the existing cron entries
		exec_cmd( "echo '*/$schedule_minutes * * * * ".
			"/opt/novell/xad/share/dcinit/checkdupws.pl --current'>>$cron_tempfile");
		exec_cmd("$cron_command $cron_tempfile");
		TEXT("checkdupws.pl sucessfully scheduled for $schedule hours");
	} else {
		exec_cmd("echo '*/$schedule_minutes * * * * ".
			"/opt/novell/xad/share/dcinit/checkdupws.pl --current'>>$cron_reschedule_tempfile");
		exec_cmd("$cron_command $cron_reschedule_tempfile");
		TEXT("checkdupws.pl sucessfully rescheduled for $schedule hours");
	}
	unlink("$cron_reschedule_tempfile");
	unlink("$cron_tempfile");

	return XAD_CHECK_SUCCESS;
}

sub execute_options() {
	if($help == 1) {
		Usage();
		return 0;
	} elsif($clear_cache == 1) {
		return clearcache_run();
	} elsif($current == 1) {
		return current_run();
	} elsif($history == 1) {
		$title_suffix = "- Since the beginning";
		return check_all_duplicates();
	} elsif(!$schedule  eq "") {
		return schedule_run();
	} elsif($unschedule == 1) {
		return unschedule_run();
	} else {
		Usage();
	}

	return XAD_CHECK_SUCCESS;
}

sub Usage() {
	print <<USAGE;

This script identifies workstations with a duplicate name in a domain.
You can also schedule the script to be run after a specified interval.

This script writes the log messages to/var/opt/novell/xad/log/wins_duplicate_check.log

	--clear-cache		Clears the duplicate workstation cache. You must run this
				option after correcting all the duplicate workstation names.

	--current		Displays information about workstations with duplicate names
				that has joined the domain since the script was last run
				with the --clear-cache option.

	--history		Displays information about workstations with duplicate names
				that have joined the domain.

	--debug			Displays debug log information on console.

	--remove-schedule	Cancels the scheduled run of the script that was set using
				the --schedule=time option.

	--schedule=time		Schedules the script to be run periodically in the specified
				interval. This option runs the script with --current option.
				Specify the time interval in 'hours'.

USAGE

	exit 0;
}

sub check_and_init_WINS_kdc_log {
	`grep $local_wins_kdc_log $krb5_config_filename`;

	if ($? != XAD_CHECK_SUCCESS) {
		Log_Debug("Adding $duplicate_workstation_log to $krb5_config_filename");
		return add_string_in_krb5();
	} else {
		#krb5 conf already updated
		#with WINS specific logs
		$krb5_conf_update_flag = XAD_KRB5_UPDATED;
		return XAD_ALREADY_DONE;
	}

}


#Returns array of domain controller attached to the domain.
#Format of the returned dcs is <dc server name>.<domain name>
sub get_dcs() {
	Log_Debug("Creating other domain controller directories");
	my $domain_name = get_domain();
	my $dc_string = exec_cmd(
		"dig -t SRV _ldap._tcp.dc._msdcs.$domain_name +short".
		" | awk '{print \$4}' | sed 's/.\$//g'");
	my $ret = $?;
	chomp($dc_string);
	if ($ret != 0 || $dc_string eq "") {

		if ($environment == XAD_PROVISIONING) {
			return XAD_CHECK_SUCCESS;
		}

		Log_Debug("Problem while getting domain controllers");
		return XAD_FAILED_TO_GET_DC;
	}

	@global_dc_array = split("\n", $dc_string);

	return XAD_CHECK_SUCCESS;
}

#Make the domain controller specific directories in the path
#specified in variable $kdc_logbase
#Each directory name is of the format <hostname>.<domain name>
sub make_dc_specific_dir() {
	my $dir = "";

	my $ret = get_dcs();
	if ($ret != XAD_CHECK_SUCCESS) {
		#Its logged as warning, since even without directory creation,
		#checkdupws.pl will give most of duplicate workstations
		return ($ret, "Problem while creating directories for domain controllers");
	}

	my $hostname = get_host();
	my $domain_name = get_domain();

	if (!-d $kdc_logbase) {
		exec_cmd("mkdir -p  $kdc_logbase");
	}

	foreach $dir (@global_dc_array) {
		chomp($dir);

		#Checking if the dir is referring to local host
		if ("$hostname.$domain_name" eq $dir) {
			#Creating soft links if the dir is referring to local host
			if (-l "$kdc_logbase/$dir") {
				#Link already exists
				Log_Debug("Symbolic link $kdc_logbase/$dir already exists");
				next;
			}

			#Link created to xad log in /var path
			exec_cmd("ln -s  $xad_log $kdc_logbase/$dir");
			if ($? != 0) {
				Log_Debug("Problem while creating link to $xad_log");
			}
			#Link created to xad/wins/<wins kdc log> in var path
			exec_cmd("ln -s  $local_wins_kdc_log $kdc_logbase/$hostname.$domain_name/$wins_kdc_log_file");
			if ($? != 0) {
				Log_Warning("Warning : Problem while creating link to $local_wins_kdc_log");
			}
			next;
		}

		#Creating new directory if dir is referring to the remote host
		if (-d "$kdc_logbase/$dir") {
			#Directory already exists
			Log_Debug("Directory $kdc_logbase/$dir already exists");
			next;
		}
		exec_cmd("mkdir $kdc_logbase/$dir");
		if ($? != 0) {
			Log_Warning("Warning : Problem while creating directory $kdc_logbase/$dir");
		}
	}

	return XAD_CHECK_SUCCESS;
}

#Pulling kdc logs from other domain controllers
sub collect_KDC_logs($) {
	my $ret = 0;
	my $option = shift;

	#Environment variable LD_LIBRARY_PATH needs to
	#changed for collectkdclogs to execute properly
	my $lpath = $ENV{LD_LIBRARY_PATH};
        $ENV{LD_LIBRARY_PATH} = "";

	exec_cmd("$collectkdclogbin $option");
	$ret = $?;

	if ($ret != 0) {
		Log_Warning("Warning : Problem while collecting kdc logs");
		#Complete the checkdupws.pl run even if some
		#domain controllers are not reachable
		#However, update the administrator about the
		#partial nature of run
		Log_Warning("Warning : Continuing to execute with partial information");
		Log_Warning("Check $collect_kdclogs_log logs for details");
	}

	$ENV{LD_LIBRARY_PATH} = $lpath;

	return $ret;
}

sub create_WINS_dir() {
	if (-d $wins_dir) {
		return 0;
	}

	exec_cmd("mkdir -p $wins_dir");
	if ($? != 0) {
		return (XAD_UNKNOWN_ERROR, "Failed to create directory $wins_dir");
	}

	if (!(-f "/opt/novell/xad/sbin/collectkdclogs")) {
		exec_cmd("ln -f /opt/novell/xad/sbin/sysvolsync $collectkdclogbin");
		if ($? != 0) {
			return (XAD_UNKNOWN_ERROR, "Failed to create link for /opt/novell/xad/sbin/sysvolsync");
		}
	}

	return XAD_CHECK_SUCCESS;
}

sub duplicate_ws_init {
	my $file = "$duplicate_workstation_logdir/$duplicate_workstation_logfile";
	my $ret = -1;
	my $mesg = "";

	if (! -d "$duplicate_workstation_logdir") {
		`mkdir -p $duplicate_workstation_logdir`;
		if ($? != 0) {
			# print STDERR "Error while creating log directory $pathname\n";
			return(XAD_UNKNOWN_ERROR, "Error while creating log directory $duplicate_workstation_logdir");
		}
	}

	set_filename("$file");

	($ret, $mesg) = check_and_init_WINS_kdc_log();
	if ($ret != XAD_ALREADY_DONE and $ret != 0) {
		return($ret, $mesg);
	} else {
		Log_Debug("Updated $krb5_config_filename file with WINS configuration");
	}

	($ret, $mesg) = create_WINS_dir();
	if ($ret != XAD_CHECK_SUCCESS) {
		return($ret, $mesg);
	}
	if ($krb5_conf_update_flag != XAD_KRB5_UPDATED) {
		restart_krb5();
	}

	#Appropriate warning is given to the administrator inside these functions
	($ret, $mesg) = make_dc_specific_dir();
	if ($ret != XAD_CHECK_SUCCESS) {
		return($ret, $mesg);
	}

	return $ret;
}

#Exits with XAD_ALREADY_RUNNING code if instance of the script is already running
sub is_running() {
	my $script_path = $0;
	my $script_name = basename( $script_path );
	my $script_pid = $$;

	my $ret_string = `ps -eaf | grep $script_name | grep -v grep | grep -v $script_pid`;
	if ($ret_string eq "") {
		return XAD_CHECK_SUCCESS;
	} else {
		print STDERR "Script already running\n";
		return XAD_ALREADY_RUNNING;
	}
}

sub checkDuplicateWorkStation($) {
	$environment = shift;
	my $retrunValue = -1;
	my $mesg = "";

	#Exit with non-0 code if instance of the script is already running
	my $returnValue = is_running();

	($returnValue, $mesg) = duplicate_ws_init();
	if ($returnValue !=0 ) {
		if ($returnValue != XAD_FAILED_TO_GET_DC) {
			Log(ERR, "$mesg");
		}
	}

	#No other operation should be done if --init option is passed
	if($initialize == 1) {
		print "Initialization done\n";
		return 0;
	}

	$returnValue = execute_options();

	return $returnValue;
}
1;
