#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Health Check Utility
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2008-2009
#   Novell, Inc.  All rights reserved.
################################################################################

package helpers;

use warnings;
use strict ;
use File::Temp;

#log level and file descriptor to use, these are global scope variables
my($configuredLogLevel, $outputFD, $logFile); 

$configuredLogLevel = 0;
$logFile = "healthCheck.log"; # full path

sub timeStamp {
    my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
    my @weekDays   = qw(Sun Mon Tue Wed Thu Fri Sat);
    my ($year, $theTime);

    my ($second, $minute, $hour, $dayOfMonth, $month, $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime(time);
    $year = 1900 + $yearOffset;
    $theTime = "$hour:$minute:$second, $weekDays[$dayOfWeek] $months[$month] $dayOfMonth, $year";
    return ($theTime);
} # isn't this already present under another package... can't we use that func..


sub logWrite {
    my($level, $message) = @_;
    my $time ;

    chomp($message);

    ($time) = timeStamp();
    
    if ($level <= $configuredLogLevel){
        print STDOUT "[$time, $level]: $message \n";
    } 

    # Concatanation is done as user can also supply the file name. By default it is "healtCheck.log".
    my $file = "/var/opt/novell/xad/log/".$logFile; 
    if ( -e $file) {
        print "[$time, $level]: $message \n";
    } 
    else {
	logAttrsSet("healthCheck.log", 5);
        print "[$time, $level]: $message \n";
    }
}

sub logAttrsSet {
    my ($logLevel, $logFile);
    my $maxLevel = 5;

    $logFile = "";
    $logLevel = 10;

    ($logFile, $logLevel) = @_;

    if (!$logFile) {
       $logFile = $ENV{'HEALTH_CHECK_LOG_FILE'};
       if (!$logFile) {
           $logFile = "healthCheck.log"; # full path needed..
       }
    }

    if ($logFile) {
        open($outputFD, "+>>", "/var/opt/novell/xad/log/$logFile");
        select ($outputFD);
    }
    else {
        die "The log file name is not present";
    }

    if ($ENV{'HEALTH_CHECK_LOG_LEVEL'}) {
#print STDOUT "[".timeStamp(). "] Looking for log level in the environment variable";
        $logLevel = $ENV{'HEALTH_CHECK_LOG_LEVEL'};
            
    }

    if (int($logLevel) <= int($maxLevel)) {
        $configuredLogLevel = int ($logLevel);
#printf "[".timeStamp()."] The log level is now $configuredLogLevel \n";
    }
    else {
        $configuredLogLevel = 1;
#printf "[".timeStamp()."] The default log level is $configuredLogLevel \n";
    }

    logWrite(4, "Selected the log level - [$configuredLogLevel] for log file [] \n");
}



# Subroutine to get the certificate
# This is used to do the ldap related commands.
sub getCert {
    my ($adminDN, $admPasswd) = @_;
    my ($ndsTreeName, $ipAddress, $error);

    if (-e "/etc/opt/novell/certs/SSCert.der") {
	logWrite(5, "The certificate file already exists");
	return 0;
    }
    
    if (!$adminDN) {
        $adminDN = $ENV{'NDSEXISTINGADMINNAME'};
        if (!$adminDN) {
            logWrite(1, "Failed to get the certificate as NDS admin dn not supplied");
            return (1);
        }
    
        $admPasswd = $ENV{'NDSEXISTINGADMINPASSWD'};
        if (!$admPasswd) {
            logWrite(1, "Failed to get the certificate as NDS admin password is not supplied ");
            return (2);
        }
    }

	# converting the ldap format to dns format (cn=admin,o=novell --> cn=admin.o=novell)
    $adminDN =~ s/,/\./g;

    logWrite(5, "Get Certificates: The admin DN [$adminDN]");

    $ndsTreeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
    chomp($ndsTreeName);
    if (!$ndsTreeName) {
        logWrite(1, "Failed to get the NDS tree name");
        return (3);
    }
        
    $ipAddress = `/opt/novell/xad/share/dcinit/printConfigKey.pl "IP Address"`; 
    if (!$ipAddress) {
        logWrite(1, "Failed to get the DNS server IP Address.");
        return (4);
    }
    chomp($ipAddress); 

    $error = validateIP($ipAddress);
    if ($error != 0) {
        logWrite(1, "The IP Address [$ipAddress] is not properly formatted.");
        return (5);
    }
        
    $error = checkCredentials($ipAddress, $adminDN, $admPasswd);
    if ($error != 0) {
        logWrite(1, "The credential check failed while validating the credentials. <$error>");
        return (6);
    }

    logWrite(5, "Getting the certificate: [$ipAddress], [$ndsTreeName], [$adminDN]");

    system("LD_LIBRARY_PATH=/opt/novell/eDirectory/lib:/opt/novell/eDirectory/lib/nds-modules:\$LD_LIBRARY_PATH /opt/novell/oes-install/util/getSSCert -a $ipAddress -t $ndsTreeName -u $adminDN -x $admPasswd 2>&1");

    $error = $? >> 8;
    if ($error) {
	helpers::logWrite(1, "Failed to get the SSL Certificate. <$error>");
	return (7);
    }

    logWrite(5, "The certificate generation is successfull");


    return (0);
}


# Subrotine gives the final installation type given
# 1. Type of installation (NM or NNM)
# 2. Server type (frd, child, adc)
sub getInstallationType {
    my ($installationType, $domainType);
    my ($mappedDomNC, $forestNC, $domainNC, $isReplica);

    $mappedDomNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
    $forestNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC"`;
    $domainNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Domain NC"`;
    $isReplica = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Is Replica"`;

    chomp($mappedDomNC, $forestNC, $domainNC, $isReplica);

    if (lc($domainNC) eq lc($mappedDomNC)) {
        $installationType = "nnm";
    }
    else {
        $installationType = "nm";
    }

    if ($isReplica eq "TRUE") {
        $domainType = "adc";
    }
    else {
        if (lc($forestNC) eq lc($domainNC)) {
            $domainType = "frd";
        }
        else {
            $domainType = "child";
        }
    }

    if ($domainType eq "frd") {
        return ($installationType, $domainType, ($installationType eq "nm") ? "nmfrd": "nnmfrd");
    }
    
    if ($domainType eq "child") {
        return ($installationType, $domainType, ($installationType eq "nm") ? "nmchild": "nnmchild");
    }
    
    if ($domainType eq "adc") {
        return ($installationType, $domainType, ($installationType eq "nm") ? "nmadc": "nnmadc");
    }

    return 0;
}


# Subroutine to get the NM parent domain
sub getNMForestName {
    my ($parentIP, $sslPort) = @_;
    my ($adminName, $forestRoot, $ipAddress, $password, $error, $output);

    $adminName = $ENV{'NDSEXISTINGADMINNAME'};
    if (!$adminName) {
        logWrite(1, "The forest admin name is not supplied");
        return 0;
    }

    $password = $ENV{'NDSEXISTINGADMINPASSWD'};
    if (!$password) {
        logWrite(1, "The forest admin password is not supplied");
        return 0;
    }

    $forestRoot = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest Root"`;
    if ($forestRoot !~ /\./) { # this contains a dot, but error doesn't have any dot
        logWrite(1, "Failed to get the forest NC $forestRoot");
        return 0;
    }

    chomp($adminName, $password, $forestRoot);
 
    $error = getCert($adminName, $password);
    if ($error) {
        logWrite(1,"Failed to get the certificate to get the forest name");
        return 0;
    }

    $ipAddress = `/opt/novell/xad/share/dcinit/printConfigKey.pl "IP Address"`;
    if (!$ipAddress) {
        logWrite(1, "Failed to get the IP Address of the local server.");
        return 0;
    }
    chomp($ipAddress); # validate IP address for correct format...

    validateIP($ipAddress);	

    logWrite(5, "The ldap command for getting the forest name: [$ipAddress], [$adminName], [$forestRoot]");

    $adminName =~ s/\./,/g;

    #TODO: check if this works on the base other then tree root/rootdse. It can be a very costly operation for large trees and hang for several hours
    $output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -E "2.16.840.1.113719.1.513.4.5" -H ldaps://$ipAddress:$sslPort -D '$adminName' -w '$password' -b "" '(&(objectclass=crossRef)(!(cn=Enterprise*))(dnsRoot=$forestRoot))' ncName -x -LLL | grep 'ncName: '`; 

    chomp($output);
    if ($output =~ /ncName:(\s*)(.*)/) {
        logWrite(3, "The forest root name is [$2]"); 
	my $forestName = $2;
	$forestName =~ s/st=/s=/gi;	
	logWrite(1, "The forest name is $forestName");
        return $forestName;
    }
    else {
        logWrite(1, "Failed to get the forest name from the ldap command");
        return 0;
    }
}

# Functions to validate the IP address.
sub validateIP {
    my $IP = shift;
    return -1 unless defined($IP);
    if ($IP =~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/)
    {
        # IP address is in valid format
        if($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255)
        {
			if ($1 == 0 && $2 == 0 && $3 == 0 && $4 == 0) {
				logWrite(1, "The validate for the IP Address failed");
				return -4;
			}
            # Range check passed 
            logWrite(5, "The validation of the IP passes.");
            return 0;
        }
        else
        {
            # Range check failed
            logWrite(1, "The range check validation failed.");
            return -2;
        }
    }
    else
    {
        #IP address is not in valid format
        logWrite(1, "The IP address is not in valid format [$IP]");
        return -3;
    }

    return 0;
}

# Function to check the credentials of the parent server. 
sub checkCredentials {
    my ($ipAddress, $adminDN, $adminPasswd) = @_;
    my ($ndsLoginRet, $status, $errorCode) = "";

    $adminDN =~ s/,/./g;
    $adminDN =~ s/st=/s=/gi;
    logWrite(5, "The ndslogin parameters: [$adminDN], [$ipAddress]");

    # Check for occurance of ndslogin utility
    if(!(-e "/opt/novell/eDirectory/bin/ndslogin")) {
        logWrite(1, " Missing file '/opt/novell/eDirectory/bin/ndslogin'. This is required for validating IP address");
        return 1;
    }

    $ndsLoginRet = `/opt/novell/eDirectory/bin/ndslogin -h $ipAddress '$adminDN' -p '$adminPasswd' 2>&1`;
    chomp $ndsLoginRet;
	$errorCode = $?;

    $status = `echo '$ndsLoginRet' | awk -F ":" '{print \$2}'`;
    chomp $status;

    logWrite(4, "The output obtained after ndslogin [$status], [$ndsLoginRet], $?");
	if ($errorCode) {
		if($status =~ m/failed/) {
			if ($status =~ /\((\W)(\d*)\)/) {
				logWrite(5, "The error code for the failure of the ndslogin [$1$2]");
				$errorCode = $1.$2;
			}
			chomp $errorCode;

			logWrite(5," A failure was found, check for the error code [$errorCode]");

			# Invalid Password or User Name
			if ($errorCode eq "-669" || $errorCode eq "-601") {
				logWrite(2, " Invalid credentials. Exiting. <$errorCode>");
			}
			else {
				logWrite(2, "Failed while doing nds login. <$errorCode>");
			}    
			return $errorCode;
		}
		# eDirectory is down
		elsif($ndsLoginRet =~ m/login cannot be done/) {
			logWrite (1, "eDirectory login cannot be done. Please check whether eDirectory server is running on [$ipAddress]");
			return 4;
		}
		logWrite(1, "NDS login failed with error [$errorCode]");
		return 5;
	}
    else {
        logWrite(2, "The validation of the admin DN [$adminDN] using [ndslogin] is success");
    }

    return 0;
}


# Function to validate the paramaters.
sub validateParams {
    my ($adminDN, $ipAddress, $instType, $domType) = @_;
    my ($adminPasswd, $error);

    if ($instType eq "nnm" && $domType eq "frd") {
        if (!$ipAddress) {
            $ipAddress = `/opt/novell/xad/share/dcinit/printConfigKey.pl "IP Address"`;
            chomp($ipAddress);
        }
    }
	
    logWrite(5, "The inputs are [$adminDN], [$ipAddress, [$instType], [$domType]");

    $error = validateIP($ipAddress);
    if ($error) {
        logWrite(1, "The IP Address supplied is not proper. [$ipAddress]. <$error>");
        return 90;
    } 

	if (!$adminDN) {
		logWrite(1, "The Admin DN is not supplied");
		return 91;
	}

	if ($domType eq "adc") {
		$adminPasswd = $ENV {'ADM_PASSWD'};
		if (!$adminPasswd) {
			logWrite(1, "The password for the admin dn [$adminDN] is not supplied");
			return 92;
		}
	}
	elsif ($domType eq "frd") {
		if ($instType eq "nm") {
			$adminPasswd = $ENV {'NDSEXISTINGADMINPASSWD'};
			if (!$adminPasswd) {
				logWrite(1, "The password for the admin dn [$adminDN] is not supplied");
				return 92;
			}
		}
		elsif ($instType eq "nnm") {
			$adminPasswd = $ENV{'ADM_PASSWD'};
			if (!$adminPasswd) {
				logWrite(1, "The password for the admin dn for the dn [$adminDN] is not supplied");
				return 93;
			}

		}
		else {
			logWrite(1, "Invalid installation type specified. <$instType>");
			return 94;
		}
	}
	elsif ($domType eq "child") {
		$adminPasswd = $ENV{'ADM_PASSWD_PARENT'};
		if (!$adminPasswd) {
			logWrite(1, "The password is not supplied for the parent for the $instType installation");
			return 95;
		}
		if ($instType eq "nm") {
			if (not defined $ENV{'NDSEXISTINGADMINPASSWD'} ) {
				logWrite(1, "The tree admin name or the password is not supplied");
				return 96;
			}

			if (not defined $ENV{'NDSEXISTINGADMINNAME'}) {
				my $ndsadminname = `/opt/novell/xad/share/dcinit/printConfigKey.pl "NDSEXISTINGADMINNAME"`;
				chomp($ndsadminname);
				$ENV{'NDSEXISTINGADMINNAME'} = $ndsadminname;
			}
		}
	}
    else {
        logWrite(1, "Invalid domain type specified.<$domType> ");
        return 97;
    }

    logWrite(5, "The credential check: admin DN [$adminDN], ip [$ipAddress]");
    $error = checkCredentials($ipAddress, $adminDN, $adminPasswd);
    if ($error) {
        logWrite(1, "Failed to validate the credentials supplied. <$error>");
        return 98;
    }

    logWrite(1, "Successfully validated the input [$adminDN], [$ipAddress]");
    return 0;
}


# The function to check the hierarchy validation where the all the containers in the hierarchy are
# all partitioned.

sub hierarchyCheck {
    my ($instType) = @_;
    my ($parentNC, $mappedDomNC, $parentCont, $mappedParent) ;

    if (!($instType eq "nmchild")) {
        logWrite(1, "The hierarchy validation is applicable only for name mapped child domains.");
        return 0;
    }

    $parentNC = lc(`/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent NC"`);
    $mappedDomNC = lc(`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`);

    chomp($parentNC, $mappedDomNC);
    logWrite(5, "The parent domain NC [$parentNC], mapped Domain NC [$mappedDomNC]");
    $parentCont = "";
    $mappedParent = "";

    if ($mappedDomNC =~ /^\.*?(\S*?),\.*?(\S*?),.*/) {
        logWrite(5, "The child domain [$1], parent Domain [$2]");
        $parentCont = $2;
    }

    if ($parentNC =~ /^dc=(\S*?),dc.*/) {
        $mappedParent = $1;
        logWrite(5, "The mapped container [$1]");
    }

    if (!$parentCont || !$mappedParent) {
        logWrite(1, "Failed to obtain the parentCont [$parentCont] or mappedParent [$mappedParent]");
        return 1;
    }
    
    if (!(lc($parentCont) eq lc($mappedParent))) {
        logWrite(1, "The parent container [$parentCont] is not matching mapped parent [$mappedParent]");
        return 2;
    }
    
    logWrite(3, "The mapped parent[$mappedParent] and parent container [$parentCont] are same.");
    return 0;
}

sub isAclSupported {
	my $partition = $_[0];
	my $xaddir = registry::getReg("LocalStateDir");
	my $testFile = `mktemp "$xaddir/test.XXXXXXXX"`;
	chomp($testFile);
	my $rc = 0;
	if (!(-e $testFile)) {
		logWrite(1, "Failed to create a temporary file in $xaddir");
		goto out;
	}

	my @retval = `/usr/bin/setfacl -m g:users:rwx $testFile 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to set ACL entry on [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	}
	@retval = `/usr/bin/getfacl $testFile | /usr/bin/grep -q "group:users:rwx" >/dev/null 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to set of ACL entry on [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	} else {
		unlink($testFile);
		return 0;
	}

out:
	#remove file
	unlink($testFile);
	return 2;
}

sub isXattrSupported {
	my $partition = $_[0];
	my $xaddir = registry::getReg("LocalStateDir");
	my $testFile = `mktemp "$xaddir/test.XXXXXXXX"`;
	chomp($testFile);

	if (!(-e $testFile)) {
		logWrite(1, "Failed to create a temporary file in $xaddir");
		return 2;
	}
	my @retval = `/usr/bin/setfattr -n user.test -v test $testFile >/dev/null 2>&1`;
	my $rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to set extended attributes on [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	}

	@retval = `/usr/bin/setfattr -n security.test -v test2 $testFile >/dev/null 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to set extended attributes on [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	}

	@retval = `/usr/bin/getfattr -d $testFile | /usr/bin/grep 'user.test="test"'  >/dev/null 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to read extended attributes from [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	}

	@retval = `/usr/bin/getfattr -n security.test -d $testFile | \\
			  /usr/bin/grep 'security.test="test2"' >/dev/null 2>&1`;
	$rc = $?;
	if ($rc != 0) {
		chomp(@retval);
		logWrite(1, "Failed to read extended attributes from [$testFile]. Error:-@retval");
		unlink($testFile);
		return 1;
	}
	#Remove testfiles created
	unlink($testFile);
	return 0;
}
1;
