#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates
################################################################################
#
#   NAME:
#	  util.pm
#
#   FACILITY:
#	  Utilities to validate, update the configurations.
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

package util;

use POSIX;     # for setlocale()
use Locale::gettext;
use strict;
use warnings;
use File::Copy;
use Logger qw(:DEFAULT);
use XAD::registry;
use File::Temp;
use crosspartitionntp;
use Config::IniFiles;
use NDS::stage;
use XAD::nds;

#Error returns added for cross partition NTP configuration
my $ERR_NO_CROSS_PARTITION_NTP_CONFIG_NEEDED = -101;
my $ERR_NTP_LOCATION_SHOULD_MOVED_OR_DELETED = -102;
my $DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";

#TIME
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;

#Return value if cldap query times out
use constant {
	XAD_CLDAP_TIMEOUT => 124,
};

#Options for cache string
use constant {
              EXISTS   => 1,
              NOT_EXISTS  => 2,
};

#return 0 if its found
sub chkfor_reg {
	my $file = shift;
	`grep -q '\$REGISTRY' $file`;
	return $?;
}

sub grepfor {
	my $str = shift;
	my $file = shift;
	`grep -q $str $file`;
	return $?;
}

#return in the form of owner:group
sub get_owner {
	my $file = shift;

	my $owner = `ls -lnd $file | awk '{print \$3}'`;

	return $owner;
}

sub get_grp {
	my $file = shift;

	my $grp = `ls -lnd $file | awk '{print \$4}'`;

	return $grp;
}

#Takes argument this way Sourcefile, operator, [>|>>], destinationfile
#eg: file_cat($temp.ini, '>', $temp.ldif)
sub file_cat {
	my $source = shift;
	my $op = shift;
	my $dest = shift;

	open(SOURCE, '<', $source) or die("Coult not open the source file $source");
	open(DEST, $op, $dest) or die("Could not open the destination file");
	while (my $line = <SOURCE>) {
		print DEST "$line";
	}
	print DEST "\n";

	close(SOURCE);
	close(DEST);
}

#file_append_str("TEXT TO BE APPENDED", $file);
sub file_append_str {
	my $str = shift;
	my $file = shift;

	open(DEST, '>>', $file);
	print DEST $str;
	close DEST;
}

sub update_nsswitch
{
	Log(MESG, ">>>Updating nsswitch.conf");

	my $file = "/etc/nsswitch.conf";
	my $filetmp = "/etc/nsswitch.conf.tmp";

eval{
	open(FILE, $file) or die("Could not open the file $file, $!");

	my @content = <FILE>;

	if (! grep(/^passwd: files winbind/, @content) || ! grep(/^group: files winbind/, @content)) {

		open(FILETEMP, ">$filetmp") or die("Could not open the temporary file $file.tmp");

		foreach my $line (@content) {

			#files winbind has to appear in the list first
			if ($line =~ /^passwd:/ || $line =~ /^group:/) {

				#Removing files winbind if they are occuring already in tail like "passwd: compat nam files winbind"
				#would become passwd: compat nam"
				$line =~ s/files//;
				$line =~ s/winbind//;

				#Only insers files winbind in the front, "passwd: compat nam", would become passwd: files winbind compat nam"
				#It has to be done this way. Because nam already updates this file before provisioning. So keep whatever was entered earlier
				#And just only insert "files winbind" in the front.
				$line =~ /(passwd|group):\s+(.*)/;
				$line = "$1: files winbind $2\n";
			}
			#$_ =~ s/^passwd:.*$/passwd: files winbind/;
			#$_ =~ s/^group:.*$/group: files winbind/;
			print FILETEMP $line;
		}

		close(FILETEMP);
		close(FILE);

		unlink($file) or die("Could not update the file $file $!");
		move($filetmp, $file) or die("Could not update the file $file $!");
	}
};
	if ($@) {

		if (-e $filetmp) {
			unlink($filetmp);
		}
		Log(ERR, $@);
	}

	Log(MESG, "   nsswitch.conf is updated successfully");
}

sub stop_ndsd{

	Log(MESG,">>>Stopping eDirectory");
	#if ( -x "/etc/init.d/ndsd" ){ `/etc/init.d/ndsd stop`; }
	if ( -x "/usr/lib/systemd/system/ndsd.service" ) { `/usr/bin/systemctl stop ndsd.service`; }

}

sub check_and_start_ndsd{
	my $ldap_search = registry::getReg("LDAPSEARCH");
	my $eDirStatus = "DOWN";

	Log(MESGNONEWLN,">>>Waiting for eDirectory service to come up ");
	do {
		`$ldap_search -x -b "" -s base -l 60 > /dev/null 2>&1`;
		if ($? == 0) {
			$eDirStatus = "UP";
		} else {
			Log(MESGNONEWLN, ".");
	 		`systemctl start ndsd`;
			sleep (2);
		}
	} while ($eDirStatus ne "UP");
	Log(MESG, "");
	
	return $eDirStatus;
}

sub start_ndsd{
	Log(MESG,">>>Starting eDirectory");

	if ( -x "/usr/lib/systemd/system/ndsd.service" ) {
	#	`/usr/bin/systemctl start ndsd.service`;
	#	if ( $? != 0 ){
	#		die(">>> Cannot start ndsd. Aborting the installation...");
	#	}
	#}
	#if ( -x "/etc/init.d/ndsd" ){
	 	`systemctl start ndsd`;
		if ( $? != 0 ){
			die(">>> Cannot start ndsd. Aborting the installation...");
		}
	}
		

	check_and_start_ndsd();
	sleep (10);

}

sub disable_fips()
{
    my $ret = 0;
    `sed 's/n4u.server.fips_tls=1/n4u.server.fips_tls=0/g' </etc/opt/novell/eDirectory/conf/nds.conf >/tmp/nds.conf.nofips`;
    $ret = $?;
    `cp /tmp/nds.conf.nofips /etc/opt/novell/eDirectory/conf/nds.conf`;
    $ret = $?;
    if ($ret != 0) {
                        Log("Error while disabling FIPS");
    }
}
sub enable_fips()
{
    my $ret = 0;
    `sed 's/n4u.server.fips_tls=0/n4u.server.fips_tls=1/g' </etc/opt/novell/eDirectory/conf/nds.conf >/tmp/nds.conf.with-fips`;
    $ret = $?;
    `cp /tmp/nds.conf.with-fips /etc/opt/novell/eDirectory/conf/nds.conf`;
    $ret = $?;
    if ($ret != 0) {
                        Log("Error while enabling FIPS");
    }
}

sub retrieve_xad_sysconfig {
	Log("Retrieving /etc/sysconfig/novell/xad from /etc/sysconfig/novell/xad_oes2018_sp3 for upgrade");
	copy("/etc/sysconfig/novell/xad_oes2018_sp3","/etc/sysconfig/novell/xad");
}
sub retrieve_smb_conf() {

        my $upgrade_backup_file = "/etc/samba/smb.conf.oes2018-upgrade-save";
        my $smb_conf_file = "/etc/samba/smb.conf";

        Log("Retrieving $smb_conf_file from $upgrade_backup_file for upgrade");
        copy($upgrade_backup_file,$smb_conf_file);

}

# To update the entry from ntpd to chronyd in w32tm_client.service file.
sub update_w32tm_client_service(){
        Log(MESG,">>>Updating /usr/lib/systemd/system/w32tm_client.service file for chronyd entry.");
        my $w32tm_client_service_tmp_file = mktemp("/tmp/w32tm-tmp.XXXXXXXX");
        open(FFR, "</usr/lib/systemd/system/w32tm_client.service") or Log(WARN,"cannot open /usr/lib/systemd/system/w32tm_client.service file");
        open(FFW, ">$w32tm_client_service_tmp_file") or Log(WARN,"Cannot open temp file $w32tm_client_service_tmp_file");
        while(<FFR>){
                if($_ =~ m/ntpd\.service/) {
                        $_ =~ s/ntp/chrony/gi;
                }
                print FFW "$_";
        }
        close(FFR);
        close(FFW);
        copy("/usr/lib/systemd/system/w32tm_client.service","/usr/lib/systemd/system/w32tm_client.service.$stamp") or Log(WARN, "w32tm_client.service backup has failed");
        copy("$w32tm_client_service_tmp_file","/usr/lib/systemd/system/w32tm_client.service") || Log(WARN,"Error in copying w32tm_client.service file");
        unlink("$w32tm_client_service_tmp_file");
        Log(MESG,">>>Successfuly update w32tm_client.service for chronyd entry.");
        `systemctl daemon-reload`;
        `systemctl restart w32tm_client.service`;
}

# Subroutine for updating "Site" variable with current site in /etc/opt/novell/xad/xad.ini file.
sub update_site_name_in_xad_ini {
	Log(MESG,">>>Starting to update site name in xad.ini file");
        my $file_change = "";
        my $server_name = `hostname`;
        chomp($server_name);
	Log(INFO,"Restarting ndsd for update site in xad.ini");
        `systemctl stop ndsd`;
        `systemctl start ndsd`;
        my $current_site_entry = `ldapsearch -x -b "" -s base namingcontexts_edir -o ldif-wrap=no | grep -ie cn=$server_name, | grep -ie cn=Sites,`;
        my $current_site_name = (split('=', (split(',', (split(' ', $current_site_entry))[1]))[2]))[1];
	Log(MESG,"current site name : $current_site_name");
        my $xad_ini_file = "/etc/opt/novell/xad/xad.ini";
        my $xad_ini_tmp_file = mktemp("/tmp/xad-ini-tmp.XXXXXXXX");
        open(FFR, "</$xad_ini_file") or Log(WARN,"cannot open $xad_ini_file file");
        open(FFW, ">$xad_ini_tmp_file") or Log(WARN,"Cannot open $xad_ini_tmp_file file");
        while(<FFR>){
                if ($_ =~ /^\s*Site\s*=\s*/i) {
                        chomp($_);
			$_ =~ s/\s+//g;
                        if( $_ ne "Site=$current_site_name" ){
                                print FFW "Site = $current_site_name\n";
                                $file_change = "YES";
                                next;
                        }
                }
                print FFW "$_";
        }
        close(FFR);
        close(FFW);
        if ($file_change eq "YES") {
                copy("$xad_ini_file","$xad_ini_file.$stamp") || Log(WARN,"Error in copying $xad_ini_file file for update_site_name_in_xad_ini() in util.pm");
                copy("$xad_ini_tmp_file","$xad_ini_file") ||  Log(WARN,"Error in copying $xad_ini_tmp_file file for update_site_name_in_xad_ini() in util.pm");
        }
        unlink("$xad_ini_tmp_file");
}

#Subroutine for adding "default_ccache_name = FILE:/tmp/krb5cc_0" string in /etc/krb5.conf file under libdefaults section.
#The purpose of adding this string is to enable running of rpcclient command in the server, without export.
sub add_cache_string_in_krb5_conf_file(){
        my $krb5_tmp_filename = mktemp("/tmp/krb5-tmp.XXXXXXXX");
        open(FFR, "</etc/krb5.conf") or Log(WARN,"cannot open krb5.conf file");
        open(FFW, ">$krb5_tmp_filename") or Log(WARN,"Cannot open temp file");
        while(<FFR>){
                if($_ =~ m/\[libdefaults\]/) {
                        print FFW $_;
                        print FFW "\tdefault_ccache_name = FILE:/tmp/krb5cc_0\n";
                        next;
                }
                print FFW "$_";
        }
        close(FFR);
        close(FFW);
        copy("$krb5_tmp_filename","/etc/krb5.conf") || Log(WARN,"Error in copying krb5.conf file");
        unlink("$krb5_tmp_filename");
}

# Subroutine for checking that cache string "default_ccache_name = FILE:/tmp/krb5cc_0" is already exist or not.
# Based on condition it will return either EXISTS or NOT_EXISTS.
sub check_cache_string_in_krb5_conf_file(){
        open(FFR, "</etc/krb5.conf") or Log(WARN,"cannot open krb5.conf file");
        my $cache_string = NOT_EXISTS;
        while(<FFR>){
                if($_ =~ m/default_ccache_name = FILE:\/tmp\/krb5cc_0/) {
                        $cache_string = EXISTS;
                }
        }
        close(FFR);
        return $cache_string;
}

# Subroutine for creating backup file of krb5.conf file with timestamp.
sub backup_krb5_conf_file(){
        copy("/etc/krb5.conf","/etc/krb5.conf.$stamp") or Log(WARN,"krb5.conf file save failed");
}

# Subroutine for calling at rpm update and will check cache string exist or not, if not then will create backup file and will add cache string.
sub add_cache_string_krb5{
        my $ans = check_cache_string_in_krb5_conf_file();
        if($ans eq NOT_EXISTS){
                backup_krb5_conf_file();
                add_cache_string_in_krb5_conf_file();
        }
}

#Subroutine for creating backup file of etc/opt/xad/krb5.conf file with timestamp
sub backup_xad_krb5_conf_file(){
        copy("/etc/opt/novell/xad/krb5.conf","/etc/opt/novell/xad/krb5.conf.$stamp") or Log(WARN, "xad/krb5.conf file save failed");
}

#subroutine for creating symlink(soft link) of "/etc/opt/novell/xad/krb5.conf" with /etc/krb5.conf file if it is not.
sub symlink_krb5_conf_file{
        if (!-l "/etc/krb5.conf") {
                backup_krb5_conf_file();
                backup_xad_krb5_conf_file();
                copy("/etc/krb5.conf","/etc/opt/novell/xad/krb5.conf") or Log(WARN,"krb5.conf file save failed");
                unlink("/etc/krb5.conf");
                symlink("/etc/opt/novell/xad/krb5.conf", "/etc/krb5.conf");
        }
}

# Subroutine for checking that NTPD_RUN_CHROOTED is set or not in the /etc/sysconfig/chronyd file, if it is set then this subroutime will make NTPD_RUN_CHROOTED as unset .
sub ntpd_run_chrooted_var_change{
        open(FFR, "</etc/sysconfig/chronyd") or Log(WARN,"cannot open ntp file in ntpd_run_chrooted_var_change function()");
        my $chroot_var = NOT_EXISTS;
        while(<FFR>){
                if($_ =~ /NTPD_RUN_CHROOTED=\"yes\"/i) {
                        $chroot_var = EXISTS;
                }
        }
        close(FFR);
        if($chroot_var eq EXISTS){
                my $ntp_tmp_filename = mktemp("/tmp/ntp-tmp.XXXXXXXX");
                open(FFR, "</etc/sysconfig/chronyd");
                open(FFW, ">$ntp_tmp_filename");
                while(<FFR>){
                        if($_ =~ /NTPD_RUN_CHROOTED=\"yes\"/i) {
                                $_ =~ s/yes/no/gi;
                                print FFW "$_";
                                next;
                        }
                        print FFW "$_";
                }
                close(FFW);
                close(FFR);
                copy("/etc/sysconfig/chronyd","/etc/sysconfig/chronyd.$stamp") or Log(WARN, "ntp backup has failed");
                copy("$ntp_tmp_filename","/etc/sysconfig/chronyd") or Log(WARN,"ntp file has not copied");
                unlink("$ntp_tmp_filename");
        }
}

# Subroutine for removing the "authprov /opt/novell/xad/lib64/libw32time.so 131072:4294967295 global" variable from /etc/chrony.conf file.
# The variable no longer needed for ntp service.
# This fix is the part of D-6370.
sub remove_authprov_in_ntp_conf_file(){
	Log(MESG,">>>Removing authprov from /etc/chrony.conf file.");
	my $ntp_tmp_filename = mktemp("/tmp/ntp-tmp.XXXXXXXX");
	open(FFR, "</etc/chrony.conf") or Log(WARN,"cannot open chrony.conf file");
	open(FFW, ">$ntp_tmp_filename") or Log(WARN,"Cannot open temp file");
	while(<FFR>){
		if($_ =~ m/authprov/) {
			next;
		}
		print FFW "$_";
	}
	close(FFR);
	close(FFW);
	copy("/etc/chrony.conf","/etc/chrony.conf.$stamp") or Log(WARN, "chrony.conf backup has failed");
	copy("$ntp_tmp_filename","/etc/chrony.conf") || Log(WARN,"Error in copying chrony.conf file");
	unlink("$ntp_tmp_filename");
	Log(MESG,">>>Successfuly removed authprov from /etc/chrony.conf file.");
	`/usr/bin/systemctl restart ntpd.service`;
}
#Subroutine for crating rpc path for chronyd service.
sub rpc_path_chrony{
	`mkdir -p /var/lib/chrony/var/opt/novell/xad/rpc`;
}

# Subroutine for updating chrony.conf file with require parameter.
sub ntp_to_chrony_conf{

        my $DEFAULT_NTP_CONF="/etc/chrony.conf";
        my $TEMPLATEDIR = registry::getReg("TEMPLATEDIR");

	Log(MESG,">>>Updating parameters in chrony.conf file from ntp.conf file.");

	copy("$DEFAULT_NTP_CONF","$DEFAULT_NTP_CONF.$stamp") or Log(WARN, "$DEFAULT_NTP_CONF backup has failed");

        my $pool_server_ip = `grep -E "^[[:space:]]*server " /etc/ntp.conf | grep iburst | awk '{print \$2}'`;
        chomp($pool_server_ip);

        `echo "\npool $pool_server_ip iburst" >> $DEFAULT_NTP_CONF`;

        my $err= system("/opt/novell/xad/share/dcinit/provisionTools.sh ntpconfig -t $TEMPLATEDIR/ntp-master.conf");

        if ($err != 0) { die("ntp_to_chrony_conf ntpconfig failed with err code $err"); }

	Log(MESG,">>>Updating parameters in chrony.conf file from ntp.conf file are done.");

	`systemctl restart chronyd.service`;
}

# Subroutine for checking that smb.conf file is proper or not.
# Based on condition it will return either EXISTS or NOT_EXISTS.
sub check_smb_conf_file(){
	open(FFR, "</etc/samba/smb.conf.oes2018-upgrade-save") or Log(WARN,"cannot open smb.conf.oes2018-upgrade-save file");
	my $dsfw_smb_conf_file = EXISTS;
	while(<FFR>){
		if($_ =~ m/workgroup = WORKGROUP/) {
			$dsfw_smb_conf_file = NOT_EXISTS;
			last;
		}
	}
	close(FFR);
	return $dsfw_smb_conf_file;
}

# Subroutine for creating the new smb.conf file if backup file is not proper.
sub retrieve_smb_conf_backup{
	Log(MESG, ">>>Retrieving smb.conf backup file");
	my $templatedir = registry::getReg("TEMPLATEDIR");
        my $ans = check_smb_conf_file();
        if($ans eq NOT_EXISTS){
		Log(MESG, "Generating /etc/samba/smb.conf.oes2018-upgrade-save-backup file");
                stage::reg_substitute("$templatedir/smb.conf", "/etc/samba/smb.conf.oes2018-upgrade-save-backup");
                make_netlogon_share_writable("/etc/samba/smb.conf.oes2018-upgrade-save-backup");
		Log(MESG, "Copying /etc/samba/smb.conf.oes2018-upgrade-save-backup to /etc/samba/smb.conf.oes2018-upgrade-save");
		copy("/etc/samba/smb.conf.oes2018-upgrade-save-backup","/etc/samba/smb.conf.oes2018-upgrade-save") || Log(WARN,"Error in copying /etc/samba/smb.conf.oes2018-upgrade-save-backup file");
		unlink("/etc/samba/smb.conf.oes2018-upgrade-save-backup");
		Log(MESG,"Successfully removed /etc/samba/smb.conf.oes2018-upgrade-save-backup");
		
        }
	# Retrieve smb.conf file
	retrieve_smb_conf();
}

#Parameters update in smb.conf file for samba upgrade.
sub update_parameters_in_smb_conf() {
	#Updating the smb.conf

	Log (">>>Starting to update parameters in smb.conf file.");

	my $smb_file = "/etc/samba/smb.conf";

	copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");

	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) {
		Log ("The open of $smb_file failed : @Config::IniFiles::errors");
		return -1;
	}

	Log ("Updating $smb_file file...");
	Log ("Updating section - [global]...");

	#Remove 'auth methods = guest winbind'
	my $var_value = "auth methods";
	my $val = $cfg->val('global', $var_value);
	if (defined $val) {
		Log ("Removing $var_value.....");
		$cfg->delval ('global', $var_value);
	}

	#Remove 'idmap uid = 1-4294967295'
	$var_value = "idmap uid";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		Log ("Removing $var_value.....");
		$cfg->delval ('global', $var_value);
	}

	#Remove 'idmap gid = 1-4294967295'
	$var_value = "idmap gid";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		Log ("Removing $var_value.....");
		$cfg->delval ('global', $var_value);
	}

	#Remove 'client use spnego principal = Yes'
	$var_value = "client use spnego principal";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		Log ("Removing $var_value.....");
		$cfg->delval ('global', $var_value);
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updated $smb_file file.");
	return 0;
}

sub restart_services{
	Log(MESG, ">>>Restarting services");

	#my @stop_services = qw(smb winbind nmb xadsd xad-kpasswdd xad-krb5kdc rpcd nscd novell-named);
	#TBD Logic for ndsd check as per systemd status
	my @stop_services = qw(xadsd xad-kpasswdd xad-krb5kdc rpcd nscd novell-named smb nmb winbind);
	#my @start_services = qw(novell-named nscd rpcd xad-krb5kdc xad-kpasswdd xadsd nmb winbind smb);
	my @start_services = qw(novell-named nscd rpcd xad-krb5kdc xad-kpasswdd xadsd smb nmb winbind);
	my $service_start_status = 0;
	my $LD_LIBRARY_PATH = "";
	my ($counter, $rc) = (0,0);
	#Wait for 60s before doing doing a timeout for the cldap query
	my $ldapsearch_timeout = 60;
	#cldap timeout flag
	#'0' means no timeout happened
	my $cldap_timeout_flag = 0;
	my $temp_file = mktemp("/tmp/cldap.XXXXXXXX");

	eval {

		foreach (@stop_services) {

			Log("Stopping the service $_");

			if($_ eq "smb" or $_ eq "nmb" or $_ eq "winbind" or $_ eq "nscd") {
				`systemctl stop $_`;
			}
			else {
				`/usr/bin/systemctl stop $_.service`;
			}
		}

		
		#XXX Not so neat way to do. The xadsd should have went down gracefully.
		`pgrep xadsd`;
		if ($? == 0) {
			`pkill -9 xadsd`;
		}

		# Stop w32tm_client if exists
		`pgrep w32tm_client`;
		if ($? == 0){
			`killall -9 w32tm_client >> /var/opt/novell/xad/log/w32tm_client.log 2>&1 &`;
		}

	#	disable_fips();
		stop_ndsd();

		#XXX Not so neat way to do. The ndsd should have went down gracefully.
		`pgrep ndsd`;
		if ($? == 0) {
			`pkill -9 ndsd`;
		}

		start_ndsd();

		while ($counter < 5) {
			my $dns_host_name = registry::getReg("DNS Host Name");
			my $dns_domain = registry::getReg("DNS Root");
			my $ip_address_for_netlogon = registry::getReg("IP Address");

			#The flag should be '1' if cldap query consistently timesout
			$cldap_timeout_flag = 0;
			Log ("Executing .. timeout $ldapsearch_timeout /usr/bin/ldapsearch ".
			"-H cldap://$ip_address_for_netlogon:389".
			" \"(&(DnsDomain=$dns_domain)(Host=$dns_host_name)(NtVer=\\006))\" -b \"\"  -s base".
			"  netlogon 2>&1 | grep -i netlogon::");

			my @output = `timeout $ldapsearch_timeout /usr/bin/ldapsearch -H \\
			cldap://$ip_address_for_netlogon:389 "(&(DnsDomain=$dns_domain)\\
			(Host=$dns_host_name)(NtVer=\006))" -b ""  -s base  netlogon 2>&1 >$temp_file`;
			chomp(@output);
			$rc = $? >> 8;
			if ($rc == XAD_CLDAP_TIMEOUT) {
				Log ("The CLDAP query timed out");
				$cldap_timeout_flag = 1;
				$counter = $counter + 1;
				#Reloading nldap server to make cldap query work
				&refresh_nldap_server();
				next;
			}
			elsif ($rc != 0) {
				Log ("The CLDAP query didn't work : [ $rc, @output], retrying");
				$counter = $counter + 1;
				#Reloading nldap server to make cldap query work
				&refresh_nldap_server();
				next;
			}

			@output = `cat $temp_file | grep -i netlogon::`;
			$rc = $? >> 8;
			chomp(@output);

			if ($rc == 0) {
				Log ("The CLDAP query worked : [@output], continuing");
				last;
			} else {
				Log("netlogon attribute not found. retrying");
				$counter = $counter + 1;
				#Reloading nldap server and rechecking netlogon attribute
				&refresh_nldap_server();
				next;
			}
		}
		unlink($temp_file);

		if ($counter == 5) {
			if($cldap_timeout_flag == 1) {
				Log ("The CLDAP query is timing out");
				Log (ERR,"Please add both the configuration and schema\n".
				" replicas locally on this server.");
			}
			else {
				Log (ERR, "Could not get the proper response for the CLDAP query, see ndsd.log for more information");
			}
		}

		Log (WARN, "Enabling the systemd service for w32tm_client");
		`/usr/bin/systemctl enable w32tm_client.service >/dev/null 2>&1`;
		if ($?) {
			my $error = $? >> 8;
			Log (ERR, "Failed to enable the service w32tm_client: $error");
		}

		`systemctl daemon-reload`;
		Log("Completed systemctl daemon-reload with return value $?");

		foreach (@start_services) {

			if ($_ eq "novell-named") {
				if (registry::getReg("DNS Master") eq "FALSE") {
					Log(MESG, "The server is not DNS master, no need to start it.");
					next;
				}

#				`/etc/init.d/micasad status`;
#				if ($?) {
#					Log(WARN, "The micasad daemon was not running so starting it.");
#					`/etc/init.d/micasad start`;
#				}
				if (-e "/etc/nam.conf") {
					`/usr/bin/systemctl status namcd.service`;
					if ($?) {
						Log(WARN, "The namcd daemon was not running so starting it.");
						`/usr/bin/systemctl start namcd.service`;
					}
				}
				`/usr/bin/systemctl status novell-xregd.service`;
				if ($?) {
					Log(WARN, "The novell-xregd was not running so starting it.");
					`/usr/bin/systemctl start novell-xregd.service`;
				}
			}

			Log("Starting the service $_");

			if($_ eq "smb" or $_ eq "nmb" or $_ eq "winbind" or $_ eq "nscd") {
				`/usr/bin/systemctl start $_`;
			}

                        if ($_ eq "smb" || $_ eq "nmb" || $_ eq "winbind")
                        {
                                Log (WARN, "Enabling the systemd service for $_");
                                `/usr/bin/systemctl enable $_.service >/dev/null 2>&1`;
                                if ($?) {
                                        my $error = $? >> 8;
                                        Log (ERR, "Failed to enable the service $_: $error");
                                }
                        }
			
			# Reset the LD_LIBRARY_PATH env variable so that following services pick up the
			# system provided kerebos and gssapi libraries. 
			if(($_ eq "sshd") or ($_ eq "rsyncd")) {
				$LD_LIBRARY_PATH = $ENV{'LD_LIBRARY_PATH'};
				$ENV{'LD_LIBRARY_PATH'} = "";
				`/usr/bin/systemctl start $_.service`;
				$service_start_status = $?;
				Log(WARN, "Service $_ started with code $service_start_status");

				# Set the LD_LIBRARY_PATH env variable as the service has been started
				$ENV{'LD_LIBRARY_PATH'} = $LD_LIBRARY_PATH;
			} else {
				# Start the service as normal.
				`/usr/bin/systemctl start $_.service`;
				$service_start_status = $?;
				Log(WARN, "Service $_ started with code $service_start_status");
			}

			if ($_ eq "rpcd" || $_ eq "xad-krb5kdc" || $_ eq "xad-kpasswdd" || $_ eq "xadsd" )
			{
				Log (WARN, "Enabling the systemd service for $_");
				`/usr/bin/systemctl enable $_.service >/dev/null 2>&1`;
				if ($?) {
					my $error = $? >> 8;
					Log (ERR, "Failed to enable the service $_: $error");
				}
			}
				
			if ($service_start_status != 0) {
				my $DEFAULTROOTDOMAIN = registry::getReg("Domain NC");
				my $LDAPFORESTNC = registry::getReg("LDAPFORESTNC");

				if ($DEFAULTROOTDOMAIN ne $LDAPFORESTNC ){

					if ( $_ eq "rpcd" || $_ eq "xadsd" ){
						die("Failed to start $_. Aborting the installation...");
					}

				}


				if ( $_ eq "xad-krb5kdc" ){

					my $rc = 0;
			                my $count = 3;

			                do {
						Log(WARN,"Failed to start $_. Retrying...");
		                                `/usr/bin/systemctl restart xad-krb5kdc.service`;
						$rc = $?;
						sleep 2;

						$count--;


					}while (($count >= 0) && ($rc != 0)) ;

					if ($count == 0) {
						Log(INFO,"*"x80);
						Log(WARN,"$_ did not start. Please start it manually after upgrade");
						Log(INFO,"*"x80);
					}
				}


				Log(WARN, "   Failed in starting $_. Proceeding with other services..");

			}

		}

                #Starting w32tm_client
		Log (INFO, "Starting w32tm_client");
		`/usr/bin/systemctl start w32tm_client.service`;
		$service_start_status = $?;
		Log(WARN, "Service w32tm_client started with code $service_start_status");

		if ($service_start_status) {
			my $error = $service_start_status >> 8;
			Log (WARN, "Service w32tm_client failed with code : $error");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The DSfW services successfully restarted");

}

# Subroutine for retrieving samba's deamons latest apparmor profile files.
sub retrieve_latest_samba_apparmor_profile()
{
        Log(MESG, ">>>Retrieving samba's latest apparmor profile files.");

	if (-e "/etc/apparmor.d/usr.sbin.nmbd.rpmnew") {
		Log(MESG, ">>>Retrieving /etc/apparmor.d/usr.sbin.nmbd from /etc/apparmor.d/usr.sbin.nmbd.rpmnew file.");
		copy("/etc/apparmor.d/usr.sbin.nmbd","/etc/apparmor.d/usr.sbin.nmbd.$stamp") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.nmbd file.");
		copy("/etc/apparmor.d/usr.sbin.nmbd.rpmnew","/etc/apparmor.d/usr.sbin.nmbd") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.nmbd.rpmnew file.");
	}

	if (-e "/etc/apparmor.d/usr.sbin.smbd.rpmnew") {
		Log(MESG, ">>>Retrieving /etc/apparmor.d/usr.sbin.smbd from /etc/apparmor.d/usr.sbin.smbd.rpmnew file.");
		copy("/etc/apparmor.d/usr.sbin.smbd","/etc/apparmor.d/usr.sbin.smbd.$stamp") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.smbd file.");
		copy("/etc/apparmor.d/usr.sbin.smbd.rpmnew","/etc/apparmor.d/usr.sbin.smbd") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.smbd.rpmnew file.");
	}

	if (-e "/etc/apparmor.d/usr.sbin.winbindd.rpmnew") {
		Log(MESG, ">>>Retrieving /etc/apparmor.d/usr.sbin.winbindd from /etc/apparmor.d/usr.sbin.winbindd.rpmnew file.");
		copy("/etc/apparmor.d/usr.sbin.winbindd","/etc/apparmor.d/usr.sbin.winbindd.$stamp") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.winbindd file.");
		copy("/etc/apparmor.d/usr.sbin.winbindd.rpmnew","/etc/apparmor.d/usr.sbin.winbindd") || Log(WARN,"Error in copying /etc/apparmor.d/usr.sbin.winbindd.rpmnew file.");
	}
	
	Log(MESG,"Successfully retrieved samba's latest apparmor profile files.");
}


sub update_apparmor_nmbd()
{
	Log(MESG, ">>>Updating the usr.sbin.nmbd apparmor profile file");

	my $ret = 0;
	my $already_existing = 0;
	my $linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.nmbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	my $path="/var/log/samba/** r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.nmbd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.nmbd`;
		`sed -i -e "s/\\/var\\/log\\/samba\\/\\*\\* r,/\\ \\ \\/var\\/log\\/samba\\/\\*\\* r,/g" /etc/apparmor.d/usr.sbin.nmbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.nmbd for nmbd log file.");
		}
		else {
			Log(MESG, "The usr.sbin.nmbd apparmor entry update successful");
		}
	}
}

sub update_apparmor_smbd()
{
	Log(MESG, ">>>Updating the usr.sbin.smbd apparmor profile file");

	my $ret = 0;
	my $already_existing = 0;
	my $linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	my $path="/var/opt/novell/xad/sysvol/** rw,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/var\\/opt\\/novell\\/xad\\/sysvol\\/\\*\\* rw,/\\ \\ \\/var\\/opt\\/novell\\/xad\\/sysvol\\/\\*\\* rw,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd. This update is essential for proper functioning of sysvol sync");
		}
	}

	$ret = 0;
	$already_existing = 0;
        $linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
        chomp($linenumber);
        $path="/var/opt/novell/xad/sysvol/domain/Policies/** k,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/var\\/opt\\/novell\\/xad\\/sysvol\\/domain\\/Policies\\/\\*\\* k,/\\ \\ \\/var\\/opt\\/novell\\/xad\\/sysvol\\/domain\\/Policies\\/\\*\\* k,/g" /etc/apparmor.d/usr.sbin.smbd`;
		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for smbd group policies.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/var/log/samba/** r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/var\\/log\\/samba\\/\\*\\* r,/\\ \\ \\/var\\/log\\/samba\\/\\*\\* r,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for smbd log file.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/var/opt/novell/xad/ds/krb5kdc/krb5.keytab rk,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/var\\/opt\\/novell\\/xad\\/ds\\/krb5kdc\\/krb5.keytab rk,/\\ \\ \\/var\\/opt\\/novell\\/xad\\/ds\\/krb5kdc\\/krb5.keytab rk,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for krb5.keytab.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/opt/novell/lib64/liboescredstore.so* rm,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/opt\\/novell\\/lib64\\/liboescredstore.so\\* rm,/\\ \\ \\/opt\\/novell\\/lib64\\/liboescredstore.so\\* rm,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for liboescredstore.so.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/opt/novell/lib64/libccs2.so* rm,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/opt\\/novell\\/lib64\\/libccs2.so\\* rm,/\\ \\ \\/opt\\/novell\\/lib64\\/libccs2.so\\* rm,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for libccs2.so.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/media/nss/** rwkl,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/media\\/nss\\/\\*\\* rwkl,/\\ \\ \\/media\\/nss\\/\\*\\* rwkl,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for /media/nss.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/etc/opt/novell/xad/krb5.conf r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/etc\\/opt\\/novell\\/xad\\/krb5.conf r,/\\ \\ \\/etc\\/opt\\/novell\\/xad\\/krb5.conf r,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for krb5.conf.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/etc/nam.conf rk,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.smbd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/etc\\/nam.conf rk,/\\ \\ \\/etc\\/nam.conf rk,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd for nam.conf.");
		}
		else {
			Log(MESG, "The usr.sbin.smbd apparmor entry update successful");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /etc/ /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$linenumber=$linenumber-1;
	$path="signal send set=term peer=samba-bgqd,";
	`grep -F "$path"\ \ /etc/apparmor.d/usr.sbin.smbd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a \n" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/signal send set=term peer=samba-bgqd,/\\ \\ signal send set=term peer=samba-bgqd,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd. This update is essential for proper functioning of samba-bgqd peer");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /samba/pdb /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/usr/lib*/samba/samba-dcerpcd Px -> samba-dcerpcd,";
	`grep -F "$path"\ \ /etc/apparmor.d/usr.sbin.smbd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/usr\\/lib\\*\\/samba\\/samba-dcerpcd Px -> samba-dcerpcd,/\\ \\ \\/usr\\/lib\\*\\/samba\\/samba-dcerpcd Px -> samba-dcerpcd,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd. This update is essential for proper functioning of samba-dcerpcd");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /samba/pdb /etc/apparmor.d/usr.sbin.smbd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/usr/lib*/samba/samba-bgqd Px -> samba-bgqd,";
	`grep -F "$path"\ \ /etc/apparmor.d/usr.sbin.smbd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.smbd`;
		`sed -i -e "s/\\/usr\\/lib\\*\\/samba\\/samba-bgqd Px -> samba-bgqd,/\\ \\ \\/usr\\/lib\\*\\/samba\\/samba-bgqd Px -> samba-bgqd,/g" /etc/apparmor.d/usr.sbin.smbd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.smbd. This update is essential for proper functioning of samba-bgqd");
		}
	}
}

sub update_apparmor_samba_bgqd()
{
	Log(MESG, ">>>Updating the /etc/apparmor.d/samba-bgqd apparmor profile file");

	my $ret = 0;
	my $already_existing = 0;
	my $linenumber=`grep -n "  /usr/" /etc/apparmor.d/samba-bgqd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	my $path="/opt/novell/lib64/liboescredstore.so* rm,";
	`grep -F "$path" /etc/apparmor.d/samba-bgqd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/samba-bgqd`;
		`sed -i -e "s/\\/opt\\/novell\\/lib64\\/liboescredstore.so\\* rm,/\\ \\ \\/opt\\/novell\\/lib64\\/liboescredstore.so\\* rm,/g" /etc/apparmor.d/samba-bgqd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for /etc/apparmor.d/samba-bgqd for liboescredstore.so.");
		}
		else {
			Log(MESG, "The /etc/apparmor.d/samba-bgqd apparmor entry update successful for liboescredstore.so.");
		}
	}

	$ret = 0;
        $already_existing = 0;
        $linenumber=`grep -n "  /usr/" /etc/apparmor.d/samba-bgqd | head -1 | awk -F: '{print \$1}'`;
        chomp($linenumber);
        $path="/opt/novell/lib64/libccs2.so* rm,";
        `grep -F "$path" /etc/apparmor.d/samba-bgqd`;
        $already_existing = $?;
        if ($already_existing != 0){
                `sed -i "$linenumber a $path" /etc/apparmor.d/samba-bgqd`;
                `sed -i -e "s/\\/opt\\/novell\\/lib64\\/libccs2.so\\* rm,/\\ \\ \\/opt\\/novell\\/lib64\\/libccs2.so\\* rm,/g" /etc/apparmor.d/samba-bgqd`;

                $ret = $?;
                if ($ret != 0) {
                        Log("Could not update the apparmor profile for /etc/apparmor.d/samba-bgqd for libccs2.so.");
                }
                else {
                        Log(MESG, "The /etc/apparmor.d/samba-bgqd apparmor entry update successful for libccs2.so.");
                }
        }

        $ret = 0;
        $already_existing = 0;
        $linenumber=`grep -n "  /usr/" /etc/apparmor.d/samba-bgqd | head -1 | awk -F: '{print \$1}'`;
        chomp($linenumber);
        $path="/etc/nam.conf rk,";
        `grep -F "$path" /etc/apparmor.d/samba-bgqd`;
        $already_existing = $?;
        if ($already_existing != 0){
                `sed -i "$linenumber a $path" /etc/apparmor.d/samba-bgqd`;
                `sed -i -e "s/\\/etc\\/nam.conf rk,/\\ \\ \\/etc\\/nam.conf rk,/g" /etc/apparmor.d/samba-bgqd`;

                $ret = $?;
                if ($ret != 0) {
                        Log("Could not update the apparmor profile for /etc/apparmor.d/samba-bgqd for nam.conf.");
                }
                else {
                        Log(MESG, "The /etc/apparmor.d/samba-bgqd apparmor entry update successful for nam.conf.");
                }
        }


}

sub update_apparmor_winbindd()
{
	Log(MESG, ">>>Updating the usr.sbin.winbindd apparmor profile file");

	my $ret = 0;
	my $already_existing = 0;
	my $linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	my $path="/var/opt/novell/xad/ds/krb5kdc/krb5.keytab rk,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.winbindd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/var\\/opt\\/novell\\/xad\\/ds\\/krb5kdc\\/krb5.keytab rk,/\\ \\ \\/var\\/opt\\/novell\\/xad\\/ds\\/krb5kdc\\/krb5.keytab rk,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd for krb5.keytab.");
		}
	}
	
	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/var/log/samba/** r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.winbindd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/var\\/log\\/samba\\/\\*\\* r,/\\ \\ \\/var\\/log\\/samba\\/\\*\\* r,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd for winbindd log file.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/dev/tty* rw,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.winbindd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/dev\\/tty\\* rw,/\\ \\ \\/dev\\/tty\\* rw,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd for tty.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/etc/opt/novell/xad/openldap/ldap.conf r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.winbindd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/etc\\/opt\\/novell\\/xad\\/openldap\\/ldap.conf r,/\\ \\ \\/etc\\/opt\\/novell\\/xad\\/openldap\\/ldap.conf r,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd for ldap.conf.");
		}
	}

	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /var/ /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/etc/opt/novell/xad/krb5.conf r,";
	`grep -F "$path" /etc/apparmor.d/usr.sbin.winbindd`;
        $already_existing = $?;
        if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/etc\\/opt\\/novell\\/xad\\/krb5.conf r,/\\ \\ \\/etc\\/opt\\/novell\\/xad\\/krb5.conf r,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd for krb5.conf.");
		}
		else {
			Log(MESG, "The usr.sbin.winbindd apparmor entry update successful");
		}
	}
	$ret = 0;
	$already_existing = 0;
	$linenumber=`grep -n /samba/pdb /etc/apparmor.d/usr.sbin.winbindd | head -1 | awk -F: '{print \$1}'`;
	chomp($linenumber);
	$path="/usr/lib*/samba/samba-dcerpcd Px -> samba-dcerpcd,";
	`grep -F "$path"\ \ /etc/apparmor.d/usr.sbin.winbindd`;
	$already_existing = $?;
	if ($already_existing != 0){
		`sed -i "$linenumber a $path" /etc/apparmor.d/usr.sbin.winbindd`;
		`sed -i -e "s/\\/usr\\/lib\\*\\/samba\\/samba-dcerpcd Px -> samba-dcerpcd,/\\ \\ \\/usr\\/lib\\*\\/samba\\/samba-dcerpcd Px -> samba-dcerpcd,/g" /etc/apparmor.d/usr.sbin.winbindd`;

		$ret = $?;
		if ($ret != 0) {
			Log("Could not update the apparmor profile for usr.sbin.winbindd. This update is essential for proper functioning of samba-dcerpcd");
		}
	}

}

sub update_cron_tab{

	Log(MESG, ">>>Updating the crontab file");

	eval{
		my $rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-GPO2NMAS");
		if ($rc != 0) {
			die("Could not update the crontab for GPO2NMAS");
		}

		$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-FGP2NMAS");
		if ($rc != 0) {
			die("Could not update the crontab for FGP2NMAS");
		}

		$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh update-crontab-KDC");
		if ($rc != 0) {
			die("Could not update the cron entry for kdc");
		}

		$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh update-crontab-SYSVOLSYNC");
		if ($rc != 0) {
			die("Could not update the cron entry for sysvol sync");
		}

		$rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-SITESANDSUBNETS");
		if ($rc != 0) {
			die("Could not update the crontab for SITESANDSUBNETS");
		}
	};
	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The crontab entry update successful");
}

sub update_crontab_sitesandsubnets{

	Log(MESG, ">>>Updating the sites and subnets crontab file");

	eval{
		my $rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-SITESANDSUBNETS");
		if ($rc != 0) {
			die("Could not update the crontab for SITESANDSUBNETS");
		}
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   The sites and subnets crontab entry update successful");
}

sub update_crontab_sitesandsubnets_string{
	Log(MESG, ">>>Updating the sites and subnets command in crontab file");

	my $CRONCOMMAND = "/usr/bin/crontab";
	my $CRONLISTCOMMAND = "/usr/bin/crontab -l";
	my $CRONTEMPFILE = mktemp("/tmp/crontempfile.XXXXXXXX");

	`$CRONLISTCOMMAND > $CRONTEMPFILE`;
	`cat $CRONTEMPFILE | grep -i "/usr/bin/python3 /opt/novell/xad/sbin/sites_dnsupdate.py" | grep -iv "^#" >/dev/null 2>&1`;
	if($?){
		Log(MESG, "Replacing the command..");
		`sed -i "s/\\\/usr\\\/bin\\\/python\\\ /\\\/usr\\\/bin\\\/python3\\\ /g" $CRONTEMPFILE`;
		`$CRONCOMMAND $CRONTEMPFILE`;
	}
	`rm $CRONTEMPFILE`;
	Log(MESG, "   The sites and subnets command in crontab file update successful");
}

sub update_resolv_conf {
	my $out = "";
	my $ip_address = registry::getReg("IP Address");
	chomp($ip_address);

	my $dns_master = registry::getReg("DNS Master");
	chomp($dns_master);

	Log(MESG, ">>>Updating the resolv.conf");

	if ($dns_master eq "FALSE") {
		Log("The $ip_address is not the master DNS server.");
		return 0;
	}

	#Read the local ip_address is the first entry in resolv.conf
	eval {
		my $nameserver_entry_present = `sed -n '/^nameserver /p' /etc/resolv.conf | sed 1q`;

		if (length($nameserver_entry_present) > 0) {
			$out = `awk '/^nameserver/ {if(\$2 ~ /$ip_address/ && entry==0) {found=1} entry++} END{printf found}' /etc/resolv.conf`;
			if ($out ne "1") {
				# save the original resolv.conf file
				copy("/etc/resolv.conf", "/etc/resolv.conf.save");
				
				# Remove the local DNS server entry that are not first in the order, if any
				`sed -i -e "/^nameserver\\s*$ip_address/d" /etc/resolv.conf`;	

				#Updating the local ip on top of any nameserver entry
				#`sed -i '0, /^nameserver/ inameserver $ip_address' /etc/resolv.conf`;
				`sed -i -e '0,/^nameserver/ { /^nameserver/i nameserver $ip_address\
				}' /etc/resolv.conf`;
				if($? != 0) {
					die("Could not update the resolv.conf file");
				} else {
					Log(MESG, "The resolv.conf is successfully updated");
				}
			} else {
				Log(MESG, "The resolv.conf file is up-to-date");
			}
		} else {
					
			copy("/etc/resolv.conf", "/etc/resolv.conf.save");
					
			# nameserver entry missing in /etc/resolv.conf, so append this only entry
			`echo "nameserver $ip_address" >> /etc/resolv.conf`;
			Log(MESG, "The resolv.conf is successfully updated");
		}
	};

	if ($@) { Log (ERR, $@); }	
}


# Subroutine to update the /etc/pam.d/sshd file
# Fix in install to complete the fix for bug 431826
sub update_pamd_sshd {
	my $temp_file = `/bin/mktemp`;
	chomp($temp_file);

	eval {
		open(TEMP, ">>", $temp_file) or die("Failed to open the temporary file");

		print TEMP "auth sufficient pam_nam.so\n";
		print TEMP "account sufficient pam_nam.so\n";
		print TEMP "password sufficient pam_nam.so\n";
		print TEMP "session optional pam_nam.so\n";

		open(SSHD, "/etc/pam.d/sshd") or die ("Failed to open the file sshd");
		my @old_contents = <SSHD>;

		print TEMP @old_contents;
		close(SSHD);
		close(TEMP);
		move ($temp_file, "/etc/pam.d/sshd");
	};
	if ($@) { Log (ERR, $@); }	
	Log(MESG, "   The /etc/pam.d/sshd successfully updated");
}

## files ####
#############
my $ssh_config_file = "/etc/ssh/ssh_config";
my $sshd_config_file = "/etc/ssh/sshd_config";
my $rsyncd_config_file = "/etc/rsyncd.conf";
my $krb5_config_file = "/etc/opt/novell/xad/krb5.conf";

## error codes ##
#################
my $SSH_CONFIG_FILE_NOT_FOUND = -2;
my $SSHD_CONFIG_FILE_NOT_FOUND = -3;
my $RSYNCD_CONFIG_FILE_NOT_FOUND = -4;
my $KRB5_CONFIG_FILE_NOT_FOUND = -5;

## other vars ##
################
my $debug = 1; # Set to non zero to get some debug information.

## Setting the Locale ##
########################
setlocale(LC_MESSAGES, "");
bindtextdomain("xad", "/opt/novell/xad/share/locale");
textdomain("xad");


# file processing functions #
#############################

sub process_ssh_config()
{

	eval {

		Log(MESG, ">>>Updating ssh configuration");

		# check if file exists for not
		if(!(-e $ssh_config_file))
		{
			# ssh_config_file does not exists error out..
			die(" $ssh_config_file not found. This file is essential for proper functioning of Domain Services for Windows, exiting...");
			#exit($SSH_CONFIG_FILE_NOT_FOUND);
		}

		# open file and look for GSSAPI options i.e. 'GSSAPIAuthentication'
		# and 'GSSAPIDelegateCredentials'

		my $tempfile = `/bin/mktemp`;
		chomp $tempfile;

		open(SSH_CONFIG_FILE, $ssh_config_file) or die "Cannot open $ssh_config_file file:$!\n";
		Log("processing $ssh_config_file file") if($debug);

		open(TEMPFILE, "+>$tempfile") or die "Cannot open $tempfile temporary file:$!\n";
		Log("temporary file:$tempfile") if($debug);

		my $readline = "";

		while($readline = <SSH_CONFIG_FILE>)
		{
			chomp $readline;

			# look for the pattern 'GSSAPIAuthentication' and 'GSSAPIDelegateCredentials'
			if($readline =~ /^\#*\s*GSSAPIAuthentication/)
			{
				# found GSSAPIAuthentication
				$readline =~ s/\#*\s*GSSAPIAuthentication\s*(.*)/GSSAPIAuthentication yes/;
			}
			if($readline =~ /^\#*\s*GSSAPIDelegateCredentials/)
			{
				# found GSSAPIDelegateCredentials
				$readline =~ s/\#*\s*GSSAPIDelegateCredentials\s*(.*)/GSSAPIDelegateCredentials no/;
			}

			# print it into	 $tempfile
			print TEMPFILE $readline;
			print TEMPFILE "\n";
		}
		# close the files
		close(SSH_CONFIG_FILE);
		close(TEMPFILE);

		chmod(0644, $tempfile) or die "Cannot change permissions of file $tempfile:$!\n";

		# move the old $ssh_config_file to $ssh_config_file_org
		#move($ssh_config_file, "$ssh_config_file"."_org") or die "cannot move $ssh_config_file:$!\n";
		move($tempfile, $ssh_config_file) or die "Cannot move $tempfile to $ssh_config_file::$!\n";
	};

	if ($@) { Log (ERR, $@); }
	Log(MESG, "ssh configuration is updated");

}


sub process_sshd_config()
{

	eval {

		Log(MESG, ">>>Updating sshd configuration");
		# check if file exists for not
		if(!(-e $sshd_config_file))
		{
			# ssh_config_file does not exists error out..
			die(sprintf (gettext("%s not found. This file is essential for proper functioning of Domain Services for Windows, exiting..."), $sshd_config_file));
			#exit($SSHD_CONFIG_FILE_NOT_FOUND);
		}

		# open file and look for GSSAPI options i.e. 'GSSAPIAuthentication'
		# and 'GSSAPICleanupCredentials'

		my $tempfile = `/bin/mktemp`;
		chomp $tempfile;

		open(SSHD_CONFIG_FILE, $sshd_config_file) or die "Cannot open $sshd_config_file file:$!\n";
		Log("processing $sshd_config_file file") if($debug);

		open(TEMPFILE, "+>$tempfile") or die "Cannot open $tempfile temporary file:$!\n";
		print STDERR "temporary file:$tempfile\n" if($debug);

		my $readline = "";

		while($readline = <SSHD_CONFIG_FILE>)
		{
			chomp $readline;

			# look for the pattern 'GSSAPIAuthentication' and 'GSSAPICleanupCredentials'
			if($readline =~ /^\#*\s*GSSAPIAuthentication/)
			{
				# found GSSAPIAuthentication
				$readline =~ s/\#*\s*GSSAPIAuthentication\s*(.*)/GSSAPIAuthentication yes/;
			}
			if($readline =~ /^\#*\s*GSSAPICleanupCredentials/)
			{
				# found GSSAPICleanupCredentials
				$readline =~ s/\#*\s*GSSAPICleanupCredentials\s*(.*)/GSSAPICleanupCredentials yes/;
			}

			# print it into	 $tempfile
			print TEMPFILE $readline;
			print TEMPFILE "\n";
		}
		# close the files
		close(SSHD_CONFIG_FILE);
		close(TEMPFILE);

		chmod(0640, $tempfile) or die "Cannot change permissions of file $tempfile:$!\n";

		# move the old $sshd_config_file to $sshd_config_file_org
		#move($sshd_config_file, "$sshd_config_file"."_org") or die "cannot move $sshd_config_file:$!\n";
		move($tempfile, $sshd_config_file) or die "Cannot move $tempfile to $sshd_config_file:$!\n";
	};

	if ($@)  { Log(ERR, $@); }
	Log(MESG, "sshd configuration updated successfully");
}


sub process_krb5_config()
{

	eval {

		Log(MESG, "Updating kr5b config");

		# check if file exists for not
		if(!(-e $krb5_config_file))
		{
			# krb5_config_file does not exists error out..
			Log(ERR, sprintf (gettext("%s not found. This file is essential for proper functioning of Domain Services for Windows, exiting..."), $krb5_config_file));
			#exit($KRB5_CONFIG_FILE_NOT_FOUND);
		}

		# open file and look for rdns option
		# create a temporary file
		my $tempfile = `/bin/mktemp`;
		chomp $tempfile;

		open(KRB5_CONFIG_FILE, $krb5_config_file) or die "Cannot open $krb5_config_file:$!\n";
		Log("processing $krb5_config_file file") if($debug);

		open(TEMPFILE, "+>$tempfile") or die "Cannot open $tempfile temporary file:$!\n";
		Log("temporary file:$tempfile") if($debug);

		my $readline = "";

		while($readline = <KRB5_CONFIG_FILE>)
		{
			chomp $readline;

			# look for the pattern rdns
			if($readline =~ /^\#*\s*rdns/)
			{
				# rdns entry found
				$readline =~ s/\#*\s*rdns\s*=\s*(.*)/\trdns = true/;
			}

			# print readline in $tempfile
			print TEMPFILE $readline;
			print TEMPFILE "\n";
		}

		# close the file handles
		close(KRB5_CONFIG_FILE);
		close(TEMPFILE);

		chmod(0644, $tempfile) or die "Cannot change permissions of file $tempfile:$!\n";

		# move the old $krb5_config_file to $krb5_config_file_org
		#move($krb5_config_file, "$krb5_config_file"."_org") or die "cannot move $krb5_config_file:$!\n";
		move($tempfile, $krb5_config_file) or die "Cannot move $tempfile to $krb5_config_file:$!\n";
	};

	if ($@)  { Log(ERR, $@); }
	Log(MESG, "krb5 configuration updated successfully");

}


sub process_rsyncd_config()
{
	# check if file exists for not
	if(!(-e $rsyncd_config_file))
	{
		# rsyncd_config_file not found exit
		Log(ERR, sprintf (gettext("%s not found. This file is essential for proper functioning of Domain Services for Windows, exiting..."), $rsyncd_config_file));
		#exit($RSYNCD_CONFIG_FILE_NOT_FOUND);
	}

	# create the data part to be added to rsyncd config file
	my $rsyncd_section1 = <<"SEC1";

[sysvol-sync]
uid = root
gid = root
path = /var/opt/novell/xad/
munge symlinks = no
comment = SYSVOL synchronization rsyncd module
read only = no
hosts allow = 127.0.0.1, localhost
max connections = 1
ignore nonreadable = yes
transfer logging = yes
use chroot = false

SEC1

	my $rsyncd_section2 = <<"SEC2";

[msdfs]
uid = root
gid = root
path = /var/opt/novell/xad/msdfs/
comment =  Transfer the msdfs links during PDC role change
hosts allow = 127.0.0.1, localhost
read only = no
max connections = 1
transfer logging = yes
ignore nonreadable = yes

SEC2

	my $rsyncd_section3 = <<"SEC3";

[pdc-role-change]
uid = root
gid = root
path = /etc/samba/
comment = PDC Master role change processing
hosts allow = 127.0.0.1, localhost
read only = no
max connections = 1
transfer logging = yes
ignore nonreadable = yes

SEC3

	my $rsyncd_section4 = <<"SEC4";

[xad-ini-change]
path = /etc/opt/novell/xad/
uid = root
gid = root
comment =  Transfer the xad.ini file to new PDC
hosts allow = 127.0.0.1, localhost
read only = yes
max connections = 1
transfer logging = yes
ignore nonreadable = yes

SEC4

	eval {

		# check if the file already contains these sections 
		open(RSYNCD_CONFIG_FILE, "+<$rsyncd_config_file") or die "Cannot open $rsyncd_config_file file:$!\n";

		my $readline = "";
		my ($Is_section1_present, $Is_section2_present, $Is_section3_present, $Is_section4_present) = 0;

		 Log("processing $rsyncd_config_file.") if($debug);

		while($readline = <RSYNCD_CONFIG_FILE>)
		{
			chomp $readline;
			# print STDERR "readline : $readline\n" if($debug);

			if($readline =~ /^\[sysvol-sync\]/)
			{
				# section1 is present, set the flag to true
				$Is_section1_present = 1;
			}
			if($readline =~ /^\[msdfs\]/)
			{
				# section2 is present, set the flag to true
				$Is_section2_present = 1;
			}
			if($readline =~ /^\[pdc-role-change\]/)
			{
				# section 3 is present, set the flag to true
				$Is_section3_present = 1;
			}
			if($readline =~ /^\[xad-ini-change\]/)
			{
				# section 4 is present, set the flag to true
				$Is_section4_present = 1;
			}
		}

		if(!($Is_section1_present))
		{
			# section1 was not present in $rsyncd_config_file file, add it
			print RSYNCD_CONFIG_FILE $rsyncd_section1;
		}
		if(!($Is_section2_present))
		{
			# section2 was not present in $rsyncd_config_file file, add it
			print RSYNCD_CONFIG_FILE $rsyncd_section2;
		}
		if(!($Is_section3_present))
		{
			# section3 was not present in $rsyncd_config_file file, add it
			print RSYNCD_CONFIG_FILE $rsyncd_section3;
		}
		if(!($Is_section4_present))
		{
			# section4 was not present in $rsyncd_config_file file, add it
			print RSYNCD_CONFIG_FILE $rsyncd_section4;
		}
		close(RSYNCD_CONFIG_FILE);
	};

	if ($@)  { Log(ERR, $@); }
	Log(MESG, "sshd configuration updated successfully");

}

#Restart novell-named after updating the new DNS Server's association with zones.
sub novell_named_restart
{
        my $service_start_status;

	# Checking required start and Restarting novell-named.
	if (registry::getReg("DNS Master") eq "FALSE") {
		Log(MESG, "The server is not DNS master, no need to start it.");
		return ;
	}

	if (-e "/etc/nam.conf") {
		`/usr/bin/systemctl status namcd.service`;
		if ($?) {
			Log(WARN, "The namcd daemon was not running so starting it.");
			`/usr/bin/systemctl start namcd.service`;
		}
	}

	`/usr/bin/systemctl status novell-xregd.service`;
	if ($?) {
		Log(WARN, "The novell-xregd was not running so starting it.");
		`/usr/bin/systemctl start novell-xregd.service`;
	}

	`/usr/bin/systemctl start novell-named.service`;
	$service_start_status = $?;
	Log(WARN, "Service novell-named restarted after updating the new DNS Server's association with zones with code $service_start_status");
}

#restart ssh and rsync after updating the respective configuration files
sub ssh_rsync_restart()
{
        my $LD_LIBRARY_PATH = "";
        my $service_start_status;

        $LD_LIBRARY_PATH = $ENV{'LD_LIBRARY_PATH'};
        $ENV{'LD_LIBRARY_PATH'} = "";
        `/usr/bin/systemctl restart sshd.service`;
        $service_start_status = $?;
        Log(WARN, "Service sshd restarted after configuration changes with code $service_start_status");
        `/usr/bin/systemctl restart rsyncd.service`;
        $service_start_status = $?;
        Log(WARN, "Service rsyncd restarted after configuration changes with code $service_start_status");
        $ENV{'LD_LIBRARY_PATH'} = $LD_LIBRARY_PATH;
}

#Arg1: the ncp dn to extract the ip address
#Arg2: the ip address of the nds server where the ldapsearch has to happen
sub getIpFromNcp
{
	my $ncp_dn = shift;
	my $server = shift;
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldapsearch = "/opt/novell/eDirectory/bin/ldapsearch";
	my $ldap_search_flags = "";
	my $launch_config_wizard = registry::getReg("LAUNCH_CONFIG_WIZARD");

	if (uc($launch_config_wizard) eq "YES" and defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "")  {
		$ldapadminname = $ENV{LDAPEXISTINGADMINNAME};
	}

	if (defined $ENV{ADM_PASSWD} and $ldapadminname ne "") {
		$ldap_search_flags = "-E 2.16.840.1.113719.1.513.4.5 -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' ";
	}
	my ($str, $out);

	eval {
		#Log("Executing : LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch -H ldaps://$server:636 $ldap_search_flags -b $ncp_dn");
		#my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch -H ldaps://$server:636 $ldap_search_flags -b "$ncp_dn"`;

		Log("Executing : LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch -H ldaps://$server:636 $ldap_search_flags -b $ncp_dn -x -LLL\n");
		my @output = `LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch -H ldaps://${server}:636 $ldap_search_flags -b "$ncp_dn" -x -LLL`;
		if ($? != 0) {
			die("Could not get the ncp server $ncp_dn details");
		}

		foreach (@output) {
			#The format looks like "networkAddress:: OCMCDKRjZIM=";
			if ($_ =~ /^networkAddress/ ) {
				$_ =~ /(.*)::(.*)/;
				 $str = $2;
				 $str =~ s/^\s+//;
				 $str =~ s/\s+$//;

				 if (length($str) < 13)  {
					#There are tcp, udp, http, https, ldap, ldap networkAddress available. We will catch up only tcp or udp
					last;
				 }

			}
		}

		my $binary = XAD::decodeBase64($str);
		my @lst  = split('', $binary);
		$out = unpack("C", $lst[4])."."
			.unpack("C", $lst[5])."."
			.unpack("C", $lst[6])."."
			.unpack("C", $lst[7]);

		Log("Extracted the ip addres of $ncp_dn, ip = $out");
	};

	if ($@) { Log(ERR, $@); }
	return $out;
}

sub replace_line_in_file {
	my $replaced_line_pattern = shift;
	my $replacing_line = shift;
	my $file_name = shift;
	
	Log("Replacing line pattern $replaced_line_pattern by $replacing_line in file $file_name");

	eval{
		if (-f $file_name) {
			my $tempfile = "/tmp/tempfile.tmp";
			open (FILE, $file_name) or die("Could not open the file $file_name");
			open (TEMPFILE, ">$tempfile") or die("Could not open the temporary file $tempfile");
			while (<FILE>) {
				my $line;
				if(grep(/$replaced_line_pattern/, $_)) {
					$line = $replacing_line."\n";
				}else {
					$line = $_;
				}
				print TEMPFILE  $line;
			}
			close(FILE);
			close(TEMPFILE);
			move($tempfile, $file_name);
		}
		else {
			die("Could not find the $file_name");
		}
	};

	if ($@) { Log(ERR, $@) }
}

sub send_domain_update_now_signal {
	my $domain = shift;

	if (! defined ($domain) || $domain eq "") {
		Log ("Sending domainUpdateNow signal to localhost");
		`echo -e "dn: \nreplace: domainUpdateNow\ndomainUpdateNow: 1\n" | /usr/bin/ldapmodify -x`;
	} 
	else {
		my @out = `dig $domain +short`;
		foreach my $dc (@out) {
			chomp ($dc);
			Log ("Sending domainUpdateNow signal to $dc");
			`echo -e "dn: \nreplace: domainUpdateNow\ndomainUpdateNow: 1\n" | /usr/bin/ldapmodify -x -h $dc`; 
		}
	}
}
sub get_upgrade_to_string {
        my $file = shift;
        my $ret = "";
        my $find="^[^=]*=";
        $ret = `cat $file | grep -i "VERSION"`;
        $ret=~s/$find//;
        chomp($ret);
        $ret=~ tr/"//d;
	$ret=~ s/^\s+//;
        return lc($ret);
}

sub get_upgrade_from_string {
	my $file = shift;
	my $ret = "";
	my $find="^[^=]*=";
	$ret = `cat $file | grep -i "UPGRADE_FROM="`;
	$ret=~s/$find//;
	chomp($ret);
	$ret=~ tr/"//d;
	return lc($ret);
}

sub get_service_configured {
	my $file = shift;
	my $ret = "";

	open FILE, '<', $file;
	while (<FILE>) {
	    if ($_ =~ /^\s*SERVICE_CONFIGURED\s*=\s*"?\s*(yes|no)?\s*"?/) {
		$ret = $1;
		last;
	    }
	}
	close FILE;
	return lc($ret);
}

sub get_upgrade_from_to {
	my ($upgrade_from, $upgrade_to) = "";
	if (-f "/etc/sysconfig/novell/oes-ldap" and
		get_upgrade_from_string ("/etc/sysconfig/novell/oes-ldap") eq "oes2023") {

		$upgrade_from = "oes2023";
		goto upgrade_goto;

	} elsif (-f "/etc/sysconfig/novell/edir_oes2018_sp3" and
            get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp3") eq "yes") {

            $upgrade_from = "oes2018sp3";
            goto upgrade_goto;
	} elsif (-f "/etc/sysconfig/novell/edir_oes2018_sp2" and
            get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp2") eq "yes") {

            $upgrade_from = "oes2018sp2";
            goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir_oes2018_sp1" and
            get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp1") eq "yes") {

            $upgrade_from = "oes2018sp1";
            goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir_oes2018" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes2018") eq "yes") {

	    $upgrade_from = "oes2018";
	    goto upgrade_goto;
	} elsif (-f "/etc/sysconfig/novell/edir_oes2015_sp1" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes2015_sp1") eq "yes") {

	    $upgrade_from = "oes2015sp1";
	    goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir_oes11_sp3" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes11_sp3") eq "yes") {

	    $upgrade_from = "oes11sp3";
	    goto upgrade_goto;
        }  elsif (-f "/etc/sysconfig/novell/edir_oes2015" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes2015") eq "yes") {

	    $upgrade_from = "oes2015";
	    goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir_oes11_sp2" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes11_sp2") eq "yes") {

	    $upgrade_from = "oes11sp2";
	    goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir_oes11_sp1" and
	    get_service_configured ("/etc/sysconfig/novell/edir_oes11_sp1") eq "yes") {

	    $upgrade_from = "oes11sp1";
	    goto upgrade_goto;
        } elsif (-f "/etc/sysconfig/novell/edir2_oes11" and
	    get_service_configured ("/etc/sysconfig/novell/edir2_oes11") eq "yes") {

	    $upgrade_from = "oes11";
	    goto upgrade_goto;
        } elsif ( -f "/etc/sysconfig/novell/edir2_sp3" and
	    get_service_configured ("/etc/sysconfig/novell/edir2_sp3") eq "yes") {

	    $upgrade_from = "sp3";
	    goto upgrade_goto;
	} elsif ( -f "/etc/sysconfig/novell/edir2_sp2" and
	    get_service_configured ("/etc/sysconfig/novell/edir2_sp2") eq "yes") {

	    $upgrade_from = "sp2"; # sp2
	    goto upgrade_goto;
	} elsif (-f "/etc/sysconfig/novell/edir" and
	    get_service_configured ("/etc/sysconfig/novell/edir") eq "yes") {
	    $upgrade_from = "sp1";
	    goto upgrade_goto;
	}


upgrade_goto:
	if (-f "/etc/sysconfig/novell/edir" and
                get_service_configured ("/etc/sysconfig/novell/edir") eq "no") {
		if ( -f "/etc/novell-release" and get_upgrade_to_string ("/etc/novell-release") eq "2023") {	
			$upgrade_to = "oes2023";
		}

		if (  -f "/etc/novell-release" and get_upgrade_to_string ("/etc/novell-release") eq "2023.1") {
			$upgrade_to = "oes23.4";
		}
 
	        if ($upgrade_from eq "sp1")  {
                        Log (ERR, "The direct upgrade from SP1 to OES2023 is not supported.")
                }
	}
	if (-f "/etc/sysconfig/novell/edir_oes2018_sp3" and
                get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp3") eq "no") {

                $upgrade_to = "oes2018sp3";
                if ($upgrade_from eq "sp1")  {
                        Log (ERR, "The direct upgrade from SP1 to OES2018SP3 is not supported.")
                }

        }
	if (-f "/etc/sysconfig/novell/edir_oes2018_sp2" and
                get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp2") eq "no") {

                $upgrade_to = "oes2018sp2";
                if ($upgrade_from eq "sp1")  {
                        Log (ERR, "The direct upgrade from SP1 to OES2018SP2 is not supported.")
                }

        }
	if (-f "/etc/sysconfig/novell/edir_oes2018_sp1" and
	        get_service_configured ("/etc/sysconfig/novell/edir_oes2018_sp1") eq "no") {

	        $upgrade_to = "oes2018sp1";
	        if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES2018SP1 is not supported.")
		}

	}
	if (-f "/etc/sysconfig/novell/edir_oes2018" and
		get_service_configured ("/etc/sysconfig/novell/edir_oes2018") eq "no") {

		$upgrade_to = "oes2018";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES2018 is not supported.")
		}

	}
	if (-f "/etc/sysconfig/novell/edir_oes2015_sp1" and
		get_service_configured ("/etc/sysconfig/novell/edir_oes2015_sp1") eq "no") {

		$upgrade_to = "oes2015sp1";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES2015SP1 is not supported.")
		}

	}
	if (-f "/etc/sysconfig/novell/edir_oes2015" and
		get_service_configured ("/etc/sysconfig/novell/edir_oes2015") eq "no") {

		$upgrade_to = "oes2015";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES2015 is not supported.")
		}

	}
	if (-f "/etc/sysconfig/novell/edir_oes11_sp2" and
		get_service_configured ("/etc/sysconfig/novell/edir_oes11_sp2") eq "no") {

		$upgrade_to = "oes11sp2";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES11SP2 is not supported.")
		}

	} elsif (-f "/etc/sysconfig/novell/edir_oes11_sp1" and
		get_service_configured ("/etc/sysconfig/novell/edir_oes11_sp1") eq "no") {

		$upgrade_to = "oes11sp1";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES11SP1 is not supported.")
		}

	} elsif(-f "/etc/sysconfig/novell/edir2_oes11" and
	    get_service_configured ("/etc/sysconfig/novell/edir2_oes11") eq "no") {

		$upgrade_to = "oes11";
		if ($upgrade_from eq "sp1")  {
			Log (ERR, "The direct upgrade from SP1 to OES11 is not supported.")
		}

	} elsif ( -f "/etc/sysconfig/novell/edir2_sp3" and
	    get_service_configured ("/etc/sysconfig/novell/edir2_sp3") eq "no") {
		$upgrade_to = "sp3";
	}
	#From sp1 to sp2 need not to be taken care here. The upgrade_sysvol will be called straight away.
	return ($upgrade_from, $upgrade_to);
}

sub generate_xad_domain_flag {

	my $flag = 0;

	if (not (-e "/etc/opt/novell/xad/xad.ini")) {
		return $flag;
	}

	open FILE, '<', "/etc/opt/novell/xad/xad.ini";
	while (<FILE>) {
	    if ($_ =~ /^\s*Mapped Domain NC/i) {
		$flag = $flag | 1; #0000 0001 is used if its a mapped domain
	    }
	    elsif ($_ =~ /^\s*XADRETAINPOLICIES\s*=\s*yes/i) {
		$flag = $flag | 2; #0000 0010 is used if retain policy is yes
	    }
	}
	close (FILE);

	return $flag;
}

# Subroutine for adding "Zone Container" variable in /etc/opt/novell/xad/xad.ini file.
# The purpose of adding this variable, if DNS zone is outside the OU=OESSystemObjects container then, we can initialize the variable with right context.
sub add_zone_container_in_xad_ini {
        my $zone_container = "";
        my $dnsserver_context = "";
        my $xad_ini_file = "/etc/opt/novell/xad/xad.ini";
        my $xad_ini_tmp_file = mktemp("/tmp/xad-ini-tmp.XXXXXXXX");
        open(FFR, "</$xad_ini_file") or Log(WARN,"cannot open $xad_ini_file file");
        open(FFW, ">$xad_ini_tmp_file") or Log(WARN,"Cannot open $xad_ini_tmp_file file");
	# The code copies xad.ini, and initializes new variable with name "Zone Container".
        while(<FFR>){
                if ($_ =~ /^\s*DNSSERVER_CONTEXT\s*=/i) {
                        $dnsserver_context = $_;
                }
                elsif ($_ =~ /^\s*Zone Container\s*=/i) {
                        $zone_container = $_;
                }
                print FFW "$_";
        }
        close(FFR);
        if ($zone_container eq "") {
                copy("$xad_ini_file","$xad_ini_file.$stamp") || Log(WARN,"Error in copying $xad_ini_file file");
                $dnsserver_context =~ s/DNSSERVER_CONTEXT/Zone Container/g;
                print FFW $dnsserver_context;
                close(FFW);
                copy("$xad_ini_tmp_file","$xad_ini_file") ||  Log(WARN,"Error in copying $xad_ini_file file");
                unlink("$xad_ini_tmp_file");
                return;
        }
        close(FFW);
        unlink("$xad_ini_tmp_file");
}

# Update ServicePrincipalName - sharepoint issue
sub update_spn(){
	my $ERR_DUP = 20;
	my $ret = -1;
	my $defaultnamingcontext = registry::getReg('Domain NC');
	my $netbiosname = registry::getReg('Netbios Name');
	my $hname = registry::getReg("DNS Unqualified Host Name");
	my $dname = registry::getReg('DNS Root');

	my $spn="ldap/$hname/$netbiosname";

	my $file="/var/opt/novell/xad/ds/domain/domain-controller-update.ldif";
	open Handle,">$file" or Log(ERR, "Failed to open $file");
	print Handle "dn: cn=$hname,ou=domain controllers,$defaultnamingcontext\n";
	print Handle "changetype: modify\n";
	print Handle "add: servicePrincipalName\n";
	print Handle "servicePrincipalName: $spn\n";
	close Handle;
	my @output=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -f $file 2>&1`;
	chomp(@output);
	my $err = $?>>8 ;
	if ($err == $ERR_DUP) {
		Log("ServicePrincipalName attribute on the DC object already has \"$spn\" set.");
		$ret = 0;
		goto out;
	} elsif ($err != 0) {
		Log("Updating ServicePrincipalName attribute on DC object failed : @output");
		goto out;
	}
	$ret = 0;
	Log("Updating ServicePrincipalName attribute on DC object succeeded");
out:
	return $ret;
}

#Subroutine to restore the firewall during OES2023 migration
sub restore_firewall() {

        my $zone = `firewall-cmd --get-default-zone`;
        chomp($zone);
        `firewall-cmd --state  > /dev/null 2>&1`;
        if ($? != 0){
                Log("Firewall is off. So Restore the firewall using firewall offline cmd");
                `firewall-offline-cmd --add-rich-rule='rule family="ipv4" protocol value=icmp accept'`;
                if ($? != 0) {
                        Log("Adding firewall rich rule failed");
                }
                my @xad_tcp_ports = (445, 464, 389, 636, 88, 53, 135, 3268, 3269, 137, 139, 8025, 1389, 1636);
                my @xad_udp_ports = (445, 464, 389, 88, 53, 135, 123, 137, 138, 8025);

                foreach my $tcp_port (@xad_tcp_ports){
                        `firewall-offline-cmd  --add-port=$tcp_port/tcp`;
                }
                foreach my $udp_port (@xad_udp_ports){
                        `firewall-offline-cmd --add-port=$udp_port/udp`;
                }

        } else {
                Log("Restore the firewall");
                `firewall-cmd --zone=$zone --permanent --add-rich-rule='rule family="ipv4" protocol value=icmp accept'`;
                if ($? != 0) {
                        Log("Adding firewall rich rule failed");
                }
                `firewall-cmd --reload`;
                if ($? != 0){
                        Log("Reloading firewall is failed");
                }
                my @xad_tcp_ports = (445, 464, 389, 636, 88, 53, 135, 3268, 3269, 137, 139, 8025, 1389, 1636);
                my @xad_udp_ports = (445, 464, 389, 88, 53, 135, 123, 137, 138, 8025);

                foreach my $tcp_port (@xad_tcp_ports){
                        `firewall-cmd --zone=$zone --permanent --add-port=$tcp_port/tcp`;
                        `firewall-cmd --reload`;
                }
                foreach my $udp_port (@xad_udp_ports){
                        `firewall-cmd --zone=$zone --permanent --add-port=$udp_port/udp`;
                        `firewall-cmd --reload`;
                }
        }
}


# Subroutine to update new nmas-methods.
sub update_nmas_methods() {
	Log("Updating nmas-methods");
	my $password;
	# Name-mmaped tree admin
	my $admin=`/opt/novell/xad/share/dcinit/printConfigKey.pl "NDSEXISTINGADMINNAME" 2>/dev/NULL`;
	if (($? != 0)) {
		#non-name mapped tree administrator
		$admin=`/opt/novell/xad/share/dcinit/printConfigKey.pl "NDSDOMAINADMIN"`;
		$password = $ENV{'ADM_PASSWD'};
	} else {
		$password = $ENV{NDSEXISTINGADMINPASSWD};
	}
	chomp($admin);
	my $tree_name=`/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	chomp($tree_name);
	my $nmasmthd_path="/opt/novell/xad/share/nmasmthd";
	my $host=`hostname`;
	chomp($host);
	my $installmachinename = registry::getReg("INSTALLMACHINENAME");

	Log("Configuring nmas-methods");

	#Configure EXTERNAL NMAS method
	Log("Executing... /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/IPCExternal/config.txt -h $host -w *****");
	`/opt/novell/eDirectory/bin/nmasinst -addmethod "$admin" "$tree_name" "$nmasmthd_path/IPCExternal/config.txt" -h "$host" -w "$password"`;
	my $rc = $? >> 8;

        if ($rc != 0) {
                die("Could not add nmas ipc method, the command used is /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/IPCExternal/config.txt -h $installmachinename -w ****\'");
        }

	#Configure NEGOTIATE NMAS method
	Log("Executing... /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/Negotiate/config.txt -h $host -w *****");
	`/opt/novell/eDirectory/bin/nmasinst -addmethod "$admin" "$tree_name" "$nmasmthd_path/Negotiate/config.txt" -h "$host" -w "$password"`;
	$rc = $? >> 8;

        if ($rc != 0) {
                die("Could not add nmas spnego method, the command used is /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/Negotiate/config.txt -h $installmachinename -w ****\'");
        }

	#Configure KERBEROS NMAS method
	Log("Executing... /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/Kerberos/config.txt -h $host -w *****");
	`/opt/novell/eDirectory/bin/nmasinst -addmethod "$admin" "$tree_name" "$nmasmthd_path/Kerberos/config.txt" -h "$host" -w "$password"`;
	$rc = $? >> 8;

        if ($rc != 0) {
                die("Could not add nmas mskrb method, the command used is /opt/novell/eDirectory/bin/nmasinst -addmethod $admin $tree_name $nmasmthd_path/Kerberos/config.txt -h $installmachinename -w ****\'");
        }

	nds::trigger_backlinker();

	stop_ndsd();
	sleep 3;
	start_ndsd();
	`/usr/bin/systemctl restart xad-krb5kdc.service`;
	sleep 3;
	`/usr/bin/systemctl restart xad-kpasswdd.service`;
	sleep 3;
	Log("nmas-methods updated.");	
}

sub retry_trigger_backlinker() {
	`pgrep krb5kdc`;
	if ($?) {
		Log ("krb5kdc service not running, retrying trigger_backlinker() for nmas-methods.");
		my $retry_cnt = 0;
		while ($retry_cnt < 3) {
			Log ("retry..$retry_cnt");
			nds::trigger_backlinker();
			stop_ndsd();
			sleep 3;
			start_ndsd();
			`/usr/bin/systemctl restart xad-krb5kdc.service`;
			sleep 3;
			`/usr/bin/systemctl restart xad-kpasswdd.service`;
			sleep 3;
			`pgrep krb5kdc`;
			if ($?) {
				$retry_cnt = $retry_cnt + 1;
			} else {
				last;
			}
		}
	}
}

# Subroutine to add dsearch plugin to extensionInfo of
# LDAP server object and wait until it is loaded.
sub load_dsearch_plugin() {
	my $temp_ldif = `/bin/mktemp`;
	my $ldapmodify = registry::getReg("LDAPMODIFY");

	# Updating LDAP Server object with extensionInfo for dsearchplugin
	my $default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
	if ($? == 0) {
		chomp($default_container);
		registry::setReg("DEFAULT_CONTAINER", $default_container);
	}
	else {
		registry::setReg("DEFAULT_CONTAINER", "ou=OESSystemObjects");
	}

	update_nmas_methods();
	retry_trigger_backlinker();

	my $dn = "cn=LDAP Server - ".registry::getReg("InstallMachineName").
	",$default_container,".registry::getReg("Domain NC");

	open FILE, ">$temp_ldif" or die("failed to open the file");
	print FILE "dn: $dn\n";
	print FILE "add: extensionInfo\n";
	print FILE "extensionInfo: 8#object#dsearch_object_init#dsearch-plugin\n";	

	Log("Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Y EXTERNAL -f $temp_ldif ");
	my @output = system("LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -Q -Y EXTERNAL -f $temp_ldif");
	my $error = $? >> 8;
	my $LDAP_ERR_DUP = 20;
	my $count = 0;
	my $retry_count = 1;
	if ($error and $error != $LDAP_ERR_DUP) {
		chomp(@output);
		unlink($temp_ldif);
		Log(ERR, "Failed to update $dn with extensionInfo:$error [@output]");
	}

	if ($error == $LDAP_ERR_DUP) {
		return 0;
	}

	Log ("Updated LDAP Server with extensionInfo");
	unlink($temp_ldif);

	stop_ndsd();
	start_ndsd();

	sleep (25);

	while (1) {
		`pgrep ndsd`;
		if ($?) {
			if ($count == 5) {
				Log ("After updating dsearch plugin in extensioninfo eDirectory is not starting, aborting");
				last;
			}

			Log ("ndsd not running, starting it");
			start_ndsd();
			$count = $count + 1;
			sleep 5;
			next;
		}
		
		Log("cat /proc/`pidof ndsd`/maps | grep dsearch");
		my @output = `cat /proc/\`pidof ndsd\`/maps | grep dsearch`;
		chomp(@output);
		Log ("The output of the command execution is : [@output]");
		if ($? == 0) {
			Log("The dsearch plugin is part of extensionInfo of [$dn]");
			last;
		}
		$retry_count = $retry_count + 1;
		if ($retry_count >= 20) {
			Log("The dsearch plugin is not loading, aborting load.");
			last;
		}
		sleep(5);
	}
}

#Subroutine to create a file called "profile" in /etc/SuSEConfig
#and add relevant attributes to it as needed.
sub process_suseconfig_profile(){
	my $suseconfig_file = "/usr/sbin/config.postfix";
	system("grep", "/opt/novell/xad/bin",$suseconfig_file);
	if( $? !=0 ){
		eval{
        	    #opening the file in append mode,so that if it already exits we will not overwrite the file
		    open FILE_HANDLE, ">>$suseconfig_file"  or die("failed to open the file $suseconfig_file");
        	    print FILE_HANDLE "\nexport PATH=\$PATH:/opt/novell/xad/bin\n";
 		    close(FILE_HANDLE);
        	    chmod(0644, $suseconfig_file) or die "Cannot change permissions of file $suseconfig_file\n";
		};
		if($@) { Log(ERR, $@); }
	}
}

# Return the sitename for a domain provided
sub find_sitename ($) {
	my $domainname = shift;

	#Returns the site name in which DC is configured or needs to be configured
	my $configured_site_name = registry::getReg("Site");
	if (defined $configured_site_name) {
		return $configured_site_name;
	}

	my $output = `net lookup dsgetdcname $domainname  | grep dc_site_name | grep -v "*"`;
	chomp($output);
	if ($?) {
		die ("Couldn't locate the sitename for domain [$domainname]");
	}
	$output =~ s/\s*dc_site_name\s*:\s+'(.*)'//i;
	return $1;
}
#
## derive_newxadsd_location
##
my $ntp_partition="";
my $default_xadsd_location="/var/opt/novell/xad/rpc";

sub derive_newxadsd_location {
	my $xadsd_location=$default_xadsd_location;
	my $newpartition="";

	#Get Where NTP Partition Mounted On
	$ntp_partition=`df -P /var/lib/chrony | tail -1 | awk '{print \$6}'`;
	chomp($ntp_partition);

	# case to handle the condition where the /var/opt/novell/xad/rpc is not yet created
	if (! -d "$xadsd_location") {
		my $dir_location = `dirname $xadsd_location`;
		chomp($dir_location);
		$newpartition=`df -P $dir_location | tail -1 | awk '{print \$6}'`;
		chomp($newpartition);
		if ("$newpartition" eq "$ntp_partition") {
			return $xadsd_location;
		} else {
			$xadsd_location = $dir_location;
		}
	}

	while ("$xadsd_location" ne "/var") {
		$newpartition=`df -P $xadsd_location | tail -1 | awk '{print \$6}'`;
		chomp($newpartition);
		if ("$newpartition" eq "$ntp_partition") {
			return $xadsd_location;
		}
		$xadsd_location=`dirname $xadsd_location`;
		chomp($xadsd_location);
	}

	my $ntp_location=$xadsd_location;
	foreach my $comp ("", "/lib", "/ntp") {
		$ntp_location=$ntp_location.$comp;
		$newpartition=`df -P $ntp_location | tail -1 | awk '{print \$6}'`;
		chomp($newpartition);
		if ("$newpartition" eq "$ntp_partition") {
			return $ntp_location;
		}
	}
}

#
# get the new XADSD pipe directory location
#
sub populate_newxadsd_location {
	my $l_newxadsd_location="";
	my $IS_NTP_CHROOT="";
	$IS_NTP_CHROOT=`sed -n 's/NTPD_RUN_CHROOTED="\\(.*\\)"/\\1/p' /etc/sysconfig/chronyd`;
	chomp($IS_NTP_CHROOT);

	$l_newxadsd_location=derive_newxadsd_location();

	#Check for the default NTP configuration
	if ("$l_newxadsd_location" eq $default_xadsd_location) {
		Log("This is a default NTP configuration, no cross partition NTP configuration is needed.");
		registry::setReg("NcalRpcDir", "$default_xadsd_location");
		return $ERR_NO_CROSS_PARTITION_NTP_CONFIG_NEEDED;
	}

	if ( -d "$l_newxadsd_location/newxad") {
		my $partition=`df -P "$l_newxadsd_location/newxad" | tail -1 | awk '{print \$6}'`;
		chomp($partition);
		if ("$partition" eq "$ntp_partition") {
			if (-d "$l_newxadsd_location/newxad/rpc") {
				$partition=`df -P "$l_newxadsd_location/newxad/rpc" | tail -1 | awk '{print \$6}'`;
				chomp($partition);
				if ("$partition" eq "$ntp_partition") {
					if ("$IS_NTP_CHROOT" eq "no") {
						$l_newxadsd_location = "/var/opt/novell/xad/rpc";
						registry::setReg("NcalRpcDir", "$l_newxadsd_location");
					} else {
						registry::setReg("NcalRpcDir", "$l_newxadsd_location/newxad/rpc");
					}
				} else {
					Log("The directory structure /var/lib/chrony and $l_newxadsd_location/newxad/rpc are in different partitions.");
					Log("$l_newxadsd_location/newxad/rpc should either be moved or deleted");
					return $ERR_NTP_LOCATION_SHOULD_MOVED_OR_DELETED;
				}
			} else {
				`mkdir -p "$l_newxadsd_location/newxad/rpc"`;
				if ("$IS_NTP_CHROOT" eq "no") {
					$l_newxadsd_location = "/var/opt/novell/xad/rpc";
					registry::setReg("NcalRpcDir", "$l_newxadsd_location");
				} else {
					registry::setReg("NcalRpcDir", "$l_newxadsd_location/newxad/rpc");
				}
			}
		} else {
			Log("The directory structure /var/lib/chrony and $l_newxadsd_location/newxad are in different partitions.");
			Log("$l_newxadsd_location/newxad should either be moved or deleted");
			return $ERR_NTP_LOCATION_SHOULD_MOVED_OR_DELETED;
		}
	} else {
		`mkdir -p "$l_newxadsd_location/newxad/rpc"`;
		if ("$IS_NTP_CHROOT" eq "no") {
			$l_newxadsd_location = "/var/opt/novell/xad/rpc";
			registry::setReg("NcalRpcDir", "$l_newxadsd_location");
		} else {
			registry::setReg("NcalRpcDir", "$l_newxadsd_location/newxad/rpc");
		}
	}

	return 0;
} # end of populate_newxadsd_location

sub update_ntp_values($$$$$){
	my $new_ncalrpcdir = shift;
	my $updating_file = shift;
	my $updating_path = shift;
	my $non_deduced = shift;
	my $search_string = shift;
	my $localstate_dir = registry::getReg("LIVE_LOCALSTATEDIR");

	#Decide to use ${LOCALSTATEDIR} or /var/opt/novell/xad
	if ($non_deduced) {
		$localstate_dir = '${LOCALSTATEDIR}';
	}

	my $append_command='#NTP_BEGIN
NCALRPCDIR='.$localstate_dir.'/rpc
IS_NTP_CHROOT=$(sed -n \'s/NTPD_RUN_CHROOTED="\(.*\)"/\1/p\' /etc/sysconfig/chronyd) 
if [ "X${IS_NTP_CHROOT}" == "Xyes" ]
then
	NCALRPCDIR='.$new_ncalrpcdir.'
fi
export RPC_NCALRPC_DIR=${NCALRPCDIR}
#NTP_END';

	#Unlink the existing file from tmp
	if (-e "/tmp/$updating_file") {
		unlink("/tmp/$updating_file");
	}

	#Backup the file
	copy("$updating_path/$updating_file", "$updating_path/$updating_file.bkp") or die("Failed to backup $updating_path/$updating_file : $!");

	open(IFH, "<$updating_path/$updating_file") or die("Could not open file $updating_path/$updating_file");
	open(OFH, ">/tmp/$updating_file") or die("Could not open file /tmp/$updating_file");

	while (my $line=<IFH>) {
		#Logic for deleting the NTP block
		#If it is present
		if (defined($line) and $line =~ /#NTP_BEGIN/) {
			while(1) {
				$line=<IFH>;
				if ($line =~ /#NTP_END/) {
					$line=<IFH>;
					last;
				}
			}
		}

		#Logic for deleting lines 
		#NCALRPCDIR=$REGISTRY=NcalRpcDir
		#export RPC_NCALRPC_DIR=$NCALRPCDIR
		if (defined($line) and ($line =~ /^NCALRPCDIR=.*/ or $line =~ /export.*RPC_NCALRPC_DIR=.*/)) {
			#Skip the line
			next;
		}

		#Append NTP block to line 'export KRB5CCNAME'
		if (defined($line) and $line =~ /^export.*$search_string.*$/) {
			$line=~s/(export.*$search_string.*)/$1\n\n$append_command/;
		}

		if (defined($line)) {
			print OFH $line;
		}
	}

	close(IFH);
	close(OFH);

	chmod(0755, "/tmp/$updating_file") or die ("Cannot change permissions of file /tmp/$updating_file:$!\n");
	move("/tmp/$updating_file", "$updating_path/$updating_file") or die("Could not update the file $updating_path/$updating_file $!");
}

sub partition_server_object{
	my $ipaddress = registry::getReg("IPADDRESS");
	my $localmachinename = registry::getReg("InstallMachineName");
	my $mappedNDStempcontainer = registry::getReg("Mapped NDS Temp Container");
	my $ldapadminname = registry::getReg("LDAPADMINNAME"); 
	my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";
	my $certFile_der = "/etc/opt/novell/certs/SSCert.der";
	my $NCPserverFDN = "NULL";
	my $forestNC = registry::getReg("ACTUALFORESTNC");
	my $servernc = "";
	my $mapped_domain_nc = registry::getReg("MAPPEDDOMAINNC");
	my $ldap_out_file = mktemp("/tmp/ldap_out.XXXXXXXX");

	if (uc(registry::getReg("LAUNCH_CONFIG_WIZARD")) eq "YES") {
		if ($ENV{'LDAPEXISTINGADMINNAME'} ne "") {
			$ldapadminname=$ENV{'LDAPEXISTINGADMINNAME'};
		}
	}

	Log(INFO, "Executing... LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname'".
			" -w '$ENV{'ADM_PASSWD'}' -s base -b '$mapped_domain_nc'".
			" -E 2.16.840.1.113719.1.513.4.5 -LLL defaultNamingContext");
	my @error_output = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' \\
				-w '$ENV{'ADM_PASSWD'}' -s base -b '$mapped_domain_nc' \\
				-E "2.16.840.1.113719.1.513.4.5" -LLL defaultNamingContext > $ldap_out_file`;
	if ($? != 0) {
		chomp(@error_output);
		unlink($ldap_out_file);
		Log(ERR, "LDAP Search failed with error [@error_output]");
	}
	my $default_naming_context = `cat $ldap_out_file \\
					| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '| grep -i "^dn:"`;
	chomp($default_naming_context);
	$default_naming_context =~ s/^dn:\s+//g;
	Log(INFO, "Default naming context : $default_naming_context");

	my $default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
        if ($? == 0) {
                chomp($default_container);
        }
        else {
               $default_container="ou=OESSystemObjects";
        }
	$servernc = "cn=$localmachinename,$default_container,$default_naming_context";
	Log(INFO, "Server NC : $servernc");

	Log(INFO, "Executing... LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname'".
			" -w '$ENV{'ADM_PASSWD'}' -s sub -b '$servernc'".
			" serverReference -E 2.16.840.1.113719.1.513.4.5 -LLL");
	@error_output = `LDAPTLS_CACERT=$certFile_pem /opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' \\
				-w '$ENV{'ADM_PASSWD'}' -s sub -b '$servernc' \\
				serverReference -E "2.16.840.1.113719.1.513.4.5" -LLL > $ldap_out_file`;
	if ($? != 0) {
		chomp(@error_output);
		unlink($ldap_out_file);
		Log(ERR, "LDAP Search failed with error [@error_output]");
	}
	my $serverPartitionName = `cat $ldap_out_file \\
					| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; ' \\
					| grep -i serverreference`;
	$serverPartitionName =~ s/^serverreference:\s+//gi;
	chomp($serverPartitionName);
	Log(INFO, "Server partition name : $serverPartitionName");

	if (-f $ldap_out_file) {
		unlink($ldap_out_file);
	}

	Log(MESG, ">>>Performing Partition");

	eval{
		replops::split_partition($ipaddress, $serverPartitionName, 
						$ldapadminname, "env:ADM_PASSWD",
						636, $NCPserverFDN, $certFile_der);
	};

	if ($@) { Log(ERR, $@) }
	Log(MESG, "   Server partitioned successfully");
}

#This part of the code is taken from earlier
#revision 5554 to reload the NLDAP server
#This is done to handle the issue mentioned
#in Bug - 823734

sub refresh_nldap_server() {
	Log (INFO, "Refreshing the nldap server");
	`/opt/novell/eDirectory/sbin/nldap -u >/dev/null 2>&1`;
	sleep(3);
	`/opt/novell/eDirectory/sbin/nldap -l >/dev/null 2>&1`;
	sleep(10);
	`/usr/bin/ldapsearch -s base -b "" -x >/dev/null 2>&1`;
}

sub write_newxadsd_location{
	my $rc = 0;
	my $old_ncalrpcdir=registry::getReg("NcalRpcDir");
	my $xad_profile_file="/etc/profile.d/novell-xad.sh";

	# Populate the new xadsd location
	eval{
		$rc = &populate_newxadsd_location();

		if($rc == -102){
			return $rc;
		}

		#Compare the old value to new value
		#populate the right value into /etc/opt/novell/xad/xad.ini
		my $new_ncalrpcdir=registry::getReg("NcalRpcDir");

		if ($old_ncalrpcdir ne $new_ncalrpcdir) {

			if ($old_ncalrpcdir eq "")
			{
				open(CONFIG,">>$DCINIT_CONFIG") or
				croak("could not open $DCINIT_CONFIG\n");
				print CONFIG "NcalRpcDir = $new_ncalrpcdir\n";
				close(CONFIG);
			}
			else
			{
				&replace_line_in_file('^NcalRpcDir.*',
				"NcalRpcDir = $new_ncalrpcdir",
				"$DCINIT_CONFIG");
			}

			#Change for /etc/profile.d/novell-xad.sh
			#Adding the NCALRPCDIR=new NTP partition
			&update_ntp_values($new_ncalrpcdir, 'novell-xad.sh',
			'/etc/profile.d', 0, 'MANPATH=');
		}
		crosspartitionntp::update_smb_conf();
	};

	if($@) {
		Log(ERR,"While populating new xadsd variables");
		Log("$@");
	}
}

# Checks on the zones whether the dNIPUpdateFilter or dNIPUpdatePolicy attribute is
# present or not
#
# input argumnets:
# zone - name of the zone
# context - the context where the 'zone' is created
# dns_server - Ip address of the DNS server
# type_of_config - types can be INSTALL, UPGRADE
#
# returns values in an array with
# subscript [0] dNIPUpdatePolicy present or not (0/1)
# subscript [1] dNIPUpdatePolicy value
# subscript [2] dNIPUpdateFilter present or not (0/1)
# subscript [3] dNIPUpdateFilter value
sub is_policy_defined{
	my %args = @_;
	my $find_on_zone = $args{zone};
	my $zones_context = $args{zone_context};
	my $dns_server_ip = $args{dns_server} || "localhost";
	my $type_of_conf = $args{type_of_config};
	my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my @policies_present = (0, "", 0, "");
	my @updateFilter = ();
	my @updatePolicy = ();
	my $policycheck_command = "";
	my $policycheck_command_external = "";

	# ensure that the Zone is provided in the input argument
	if(!defined($find_on_zone)){
		if(uc($type_of_conf) eq "INSTALL"){
			Log(ERR, "invalid zone provided");
		} else {
			Log(WARN, "invalid zone provided");
			goto RETURN;
		}
	}

	# ensure that the Zone Context is in the input argument list
	# else chances that the Zone argument is with full context
	# separate the zone and the context.
	if((!defined($zones_context) or $zones_context eq "")) {
		if(grep(/,/, $find_on_zone)){
			$zones_context = (split(',', $find_on_zone, 2))[1];
			$find_on_zone = (split(',', $find_on_zone, 2))[0];
			$find_on_zone =~ s/cn=//g;
		} else {
			if(uc($type_of_conf) eq "INSTALL"){
				Log(ERR, "invalid zone context provided");
			} else {
				Log(WARN, "invalid zone context provided");
				goto RETURN;
			}
		}
	}

	# Check and process the correct ldapsearch request for
	# INSTALL, UPGRADE and other type of configurations
	if(uc($type_of_conf) eq "UPGRADE"){
		$policycheck_command = "SASL_PATH=/opt/novell/xad/lib64/sasl2 /usr/bin/ldapsearch".
			" -Q -Y GSSAPI -s base -b \"cn=$find_on_zone,$zones_context\"".
			" dnipUpdateFilter dnipUpdatePolicy -LLL -o ldif-wrap=no";
	} elsif(uc($type_of_conf) eq "INSTALL"){
		$policycheck_command = "LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H ldaps://$dns_server_ip:1636 -D \"$ldapadminname\"".
			" -w \"$ENV{'ADM_PASSWD'}\" -s base -b \"cn=$find_on_zone,$zones_context\" -LLL -x -Z".
			" dnipUpdateFilter dnipUpdatePolicy -o ldif-wrap=no";
	} else{
		$policycheck_command = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
			" /usr/bin/ldapsearch -Q -Y EXTERNAL -s base -b \"cn=$find_on_zone,$zones_context\"".
			" dnipUpdateFilter dnipUpdatePolicy -LLL -o ldif-wrap=no";
	}

	$policycheck_command_external = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
			" /usr/bin/ldapsearch -Q -Y EXTERNAL -s base -b \"cn=$find_on_zone,$zones_context\"".
			" dnipUpdateFilter dnipUpdatePolicy -LLL -o ldif-wrap=no";

	Log(INFO, "Executing ... $policycheck_command");
	my @policycheck_result = `$policycheck_command 2>&1`;
	my $return_status = $? >> 8;
	if ($return_status != 0) {
		Log(INFO, "Executing ... $policycheck_command_external");
		@policycheck_result = `$policycheck_command_external 2>&1`;
		$return_status = $? >> 8;
	}
	if ($return_status != 0) {
		if(uc($type_of_conf) eq "INSTALL"){
			Log(ERR, "ldapsearch for DNS update policy failed with status [$return_status] [@policycheck_result]");
		} else {
			Log(WARN, "ldapsearch for DNS update policy failed with status [$return_status] [@policycheck_result]");
			goto RETURN;
		}
	}

	if(@updatePolicy = grep(/^dnipUpdatePolicy:\s+/i, @policycheck_result)) {
		$policies_present[0]=1;
		chomp(@updateFilter);
		$policies_present[1] = (split(/:\s+/, $updateFilter[0]))[1];
	}

	if(@updateFilter = grep(/^dnipUpdateFilter:\s+/i, @policycheck_result)) {
		$policies_present[2]=1;
		chomp(@updateFilter);
		$policies_present[3] = (split(/:\s+/, $updateFilter[0]))[1];
	}

RETURN:
	return @policies_present;
}

sub make_netlogon_share_writable($) {

	my $samba_conf_file = shift;
	my $val = "";

	Log("   Updating netlogon configuration in $samba_conf_file file");
	Log("   Making Writable = Yes for netlogon share");

	my $cfg = Config::IniFiles->new (-file=> $samba_conf_file, -nocase=>1);

	if (!defined $cfg) {
		Log(WARN, "Could not open the file $samba_conf_file");
	}

        #Writable = yes for [netlogon]
	$val = $cfg->val('netlogon', 'writable');
	if (!defined $val) {
                $cfg->newval ('netlogon', 'writable', 'yes');
        }else {
                $cfg->setval ('netlogon', 'writable', 'yes');
        }

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g'     $samba_conf_file`;
	`sed -i 's/=/ = /g'     $samba_conf_file`;
}

# Replace the /etc/openldap/ldap.conf with /etc/opt/novell/xad/openldap/ldap.conf
sub replace_ldapconf() {
	# Replacing the ldap.conf
	my $localtime=localtime;
	my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
	my $stamp=$year.$month.$date."-".$time;
	my $old_ldap_file = "/etc/openldap/ldap.conf";
	my $new_ldap_file = "/etc/opt/novell/xad/openldap/ldap.conf";

	copy($old_ldap_file, $old_ldap_file.$stamp) or Log (ERR, "Failed to backup the ldap conf file: $!");
	file_cat($new_ldap_file, '>', $old_ldap_file) or die "$old_ldap_file replace failed: $!";
	Log ("Replacing an existing file $old_ldap_file with $new_ldap_file successfully");
}

#AES entries to be updated in /etc/krb5.conf file
my %replacement_maps_aes =(
    'permitted_enctypes[\h]*=[\h]*arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'permitted_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc[\h]+aes128-cts-hmac-sha1-96' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'default_tgs_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tgs_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5'
);

#Update the krb5.conf with the aes256 parameters
sub replace_in_file_aes {
        my $file = shift;
        open(IN,"<$file") or die $!;
        my @input_lines = <IN>;
        close(IN);

        my @output_lines;
        foreach(@input_lines)
        {
                foreach my $aes256_entry (keys %replacement_maps_aes){
                        $_=~s/$aes256_entry/$replacement_maps_aes{$aes256_entry}/g;
                }
                push(@output_lines,$_);
        }

        open(OUT,">$file") or die $!;
        print OUT @output_lines;
        close(OUT);
}

#Current update supported only from non-AES level
sub update_krb5Conf_aes {
	my $localtime=localtime;
	my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
	my $stamp=$year.$month.$date."-".$time;
	my $krb5_conf_file = "/etc/krb5.conf";

        copy($krb5_conf_file,"/tmp/krb5.conf-aes") or die "$krb5_conf_file copy failed: $!";
        copy($krb5_conf_file,"/etc/krb5.conf.$stamp") or die "$krb5_conf_file save failed: $!";
        replace_in_file_aes("/tmp/krb5.conf-aes");
        copy("/tmp/krb5.conf-aes",$krb5_conf_file) or die "$krb5_conf_file replace failed: $!";
        Log("Domain Controller config file updated to support AES encryption");
}

sub update_domain_to_aes {
	update_krb5Conf_aes;

	#bootstrap_krbtgt;
	my $command = "/opt/novell/xad/sbin/setpassword -NDOS -r -u krbtgt";

	Log(INFO, "Executing : $command");
	my @command_output = `$command 2>&1`;
	if (($?>>8) != 0) {
		Log("Could not set password for krbtgt");
		Log("error : [@command_output]");
		Log("Please try rerunning");
	}

	# setpassword command for server object
	my $server_name = `/bin/hostname`;
	chomp($server_name);
	$command = "/opt/novell/xad/sbin/setpassword -NDOS -r -u $server_name\$ -k /var/opt/novell/xad/ds/krb5kdc/krb5.keytab";
	Log(INFO, "Executing : $command");
	@command_output = `$command 2>&1`;
	if (($?>>8) != 0) {
		Log("Could not set password for server object");
		Log("error : [@command_output]");
		Log("Please try rerunning");
	}
	else {
		Log("AES credentials added to domain objects successfully.");
	}
}

#Fine Grained Password Policy Subroutines
sub isTreeAtLevel2012(){
	my $treeat2012l=0;
        my $at2012level = 1;
        my $rootDomain = registry::getReg("Root Domain");;
	#my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	my $totalServersFound = 0;

	chomp($ldapadminname);
        chomp($rootDomain);
        $rootDomain =~ s/\./,dc=/g;
        $rootDomain = "dc=".$rootDomain;

        eval{
                #Find all server objects in the domain
                my @server_result =`/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "ou=Domain Controllers,$rootDomain" -s sub "(objectClass=computer)" -Z -o ldif-wrap=no -LLL serverReferenceBL 2>&1 | grep -i "^serverReferenceBL"`;

                my $error = $? >> 8;
                if ($error) {
                        Log(WARN,"Could not get the domain controllers: $error: [@server_result]");
                        return;
                }

                $totalServersFound = $#server_result + 1;
                foreach my $serv (@server_result){
                        my $serverDn = (split(/^serverReferenceBL: /, $serv))[1];
                        chomp($serverDn);
                        my $name = (split(/=/, ((split(/,/, $serv))[0])))[1];

                        #Search for msDS-Behavior-Version on each of the server object
                        my $output = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        if(($?>>8) == 0){
                                my $version = (split(/:/, $output))[1];
                                if($version <5){
                                        Log(WARN, "Server $name is not at 2012 level.");
                                        $at2012level = 0;
                                }else{
                                        Log(INFO, "Server $name is at 2012 level.");
                                }
                        }else{
                                Log(INFO, "Server $name is not a valid server.");
                        }
                }

                if(!$at2012level){
                        Log(WARN, "Not all servers are at level 2012");
                        $treeat2012l=0;
                }else{
                        $treeat2012l=1;
                }


                #if it is single server and is the PDC
                #we are going to anyway allow the server to update msDS-Behavior-Version
                #hence checking the server 
                my $type = registry::getReg("INSTALL_TYPE");
                my $is_replica = registry::getReg("IS_REPLICA");

                if($type eq "frd" and $is_replica eq "FALSE" and $totalServersFound==1){
                        $treeat2012l = 1;
                }
        };

        if ($@) { Log(WARN,$@); }

	return $treeat2012l;
}

sub post_schema_extension(){
        my $mapped_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
        my $mapped_server_dn_name =`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Server DN Name"`;
        my $post_extn_ldif = mktemp("/tmp/post_behav_ldif.XXXXXXXXXX");
        my $is_replica = registry::getReg("IS_REPLICA");
	#my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	my $tree_at_level = isTreeAtLevel2012();

        Log(INFO,"Preparing domain for extended schema at 2012 level(post)");
	chomp($ldapadminname);
        chomp($mapped_nc);
        chomp($mapped_server_dn_name);

        open FILE, ">$post_extn_ldif" or die ("failed to open the file $post_extn_ldif");
        if($is_replica eq "FALSE"){
                if($tree_at_level){
                        print FILE "dn:$mapped_nc\n";
                        print FILE "changetype: modify\n";
                        print FILE "replace: msDS-Behavior-Version\n";
                        print FILE "msDS-Behavior-Version: 5\n";
                        print FILE "-\n";
                        print FILE "replace: nTMixedDomain\n";
                        print FILE "nTMixedDomain: 0\n";
                        print FILE "\n";
                }else{
                        Log(WARN, "Domain functional level it not updated to 2012, as few of the servers are not at level 2012 or greater.");
                        Log(WARN, "Please update the associated server's to 2012 level and rerun the script on PDC.");
                }
        }

        #dn: cn=NTDS Settings,cn=TEST1,cn=Servers,cn=Default-First-Site-Name,cn=Sites,cn=Configuration,dc=nonschem,dc=edu
        print FILE "dn: cn=NTDS Settings,$mapped_server_dn_name\n";
        print FILE "changetype: modify\n";
        print FILE "replace: msDS-Behavior-Version\n";
        print FILE "msDS-Behavior-Version: 5\n";
        print FILE "\n";

        close FILE;

        eval{
                Log(INFO, "Executing...(post) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $post_extn_ldif");
                my @output = `/usr/bin/ldapmodify -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -Z -f $post_extn_ldif`;
                my $error = $? >> 8;
                if ($error) {
                        die("Could not modify the objects: $error: [@output]");
                }
        };

        if ($@) { Log(WARN,$@) }

        unlink($post_extn_ldif);
}

sub post_forest_functional_level(){
        my $post_forest_func_ldif = mktemp("/tmp/post_forest_func_ldif.XXXXXXXXXX");
        my $netbios_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Netbios Name"`;
        my $mapped_server_dn_name =`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Server DN Name"`;
        my $forest_nc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
        my $is_replica = registry::getReg("IS_REPLICA");
	#my $ldapadminname = registry::getReg("LDAPADMINNAME");
	my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	my $tree_at_level = isTreeAtLevel2012();

        #if an ADC do not apply the modifications
        if($is_replica eq "TRUE"){
                return;
        }

        Log(INFO,"Preparing forest for extended schema at 2012 level(post)");
	chomp($ldapadminname);
        chomp($netbios_name);
        chomp($mapped_server_dn_name);
        chomp($forest_nc);

        if($tree_at_level){
                open FILE, ">$post_forest_func_ldif" or die ("failed to open the file $post_forest_func_ldif");
                #dn: cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
                print FILE "dn: cn=Partitions,cn=Configuration,$forest_nc\n";
                print FILE "changetype: modify\n";
                print FILE "replace: msDS-Behavior-Version\n";
                print FILE "msDS-Behavior-Version: 5\n";
                print FILE "\n";
                #dn: cn=NONSCHEM,cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
                print FILE "dn: cn=$netbios_name,cn=Partitions,cn=Configuration,$forest_nc\n";
                print FILE "changetype: modify\n";
                print FILE "replace: msDS-Behavior-Version\n";
                print FILE "msDS-Behavior-Version: 5\n";
                print FILE "-\n";
                print FILE "replace: ntMixeddomain\n";
                print FILE "ntMixeddomain: 0\n";
                close FILE;

                eval{
                        Log(INFO, "Executing...(post forest) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $post_forest_func_ldif");
                        my @output = `/usr/bin/ldapmodify -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -Z -f $post_forest_func_ldif`;
                        my $error = $? >> 8;
                        if ($error) {
                                die("Could not modify the objects: $error: [@output]");
                        }
                };

                if ($@) { Log(WARN,$@) }
        }else{
                Log(WARN, "Forest functional level it not updated to 2012, as few of the servers are not at level 2012 or greater.");
                Log(WARN, "Please update associated server's to 2012 schema level and then rerun the script on PDC.");
        }

        unlink($post_forest_func_ldif);
}

1;
