#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# DSfW server update script for updating changes needed to be done for moving to
# the newer OES Version
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#   Micro Focus or one of its affiliates
################################################################################
package util_update;

use warnings;

use Logger qw(:DEFAULT);
use File::Copy;
use File::Temp;
use File::Basename;
use XAD::registry;
use NDS::import;
use util;
use Config::IniFiles;
use NDS::stage;

my $lib = registry::getReg("_LIB");

# string constants for AD_schema_levels and corresponding ms-ds-behavior-versions
use constant {
	MS_DS_BEHAVIOR_VERSION_2012  => 5,
	MS_DS_BEHAVIOR_VERSION_2016  => 7,

	AD_SCHEMA_LEVEL_2012  => 2012,
	AD_SCHEMA_LEVEL_2016  => 2016,
	AD_SCHEMA_LEVEL_2019  => 2019,
};

#Obtain the time stamp
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
$time =~ s/://g;
my $stamp=$year.$month.$date."-".$time;

my $LDAP_ERR_DUP = 20;
my $LDAP_ERR_ALREADY_EXISTS = 68;
my $LDAP_ERR_NO_SUCH_ATTR = 16;
my $type_of_install="INSTALL";

sub update_ndsd_log_location() {
	my $nds_conf_file = registry::getReg("NdsConfDir")."/nds.conf";
	my $nds_log_dir = registry::getReg("NdsLogDir");
	my $xad_log_file = registry::getReg("LocalStateDir")."/log/ndsd.log";
	my $nds_log_file = "$nds_log_dir/ndsd.log";
	my $backup_file = "$xad_log_file.$stamp.bz2";

	Log("Preparing nds.conf for OES11SP2");
	copy($nds_conf_file, "$nds_conf_file.bak");
	util::replace_line_in_file("n4u.server.log-file=*",
			"n4u.server.log-file=$nds_log_file",
			$nds_conf_file);
	`/usr/bin/killall -HUP ndsd 2>/dev/null`;
	if (-e $xad_log_file) {
		`tar -czf $backup_file $xad_log_file >/dev/null 2>&1`;
		unlink $xad_log_file or warn ("Failed to unlink file: $xad_log_file");
		move($backup_file, $nds_log_dir) or
			warn("Could not move the file $backup_file");
	}
	Log("Log file for ndsd has been set to : $nds_log_file.");
}

#Update the dNIPUpdatePolicy to allow dynamic update of NS records
sub update_dnipupdate_policy {
	#This is a fix for BUG: 830547

	if (registry::getReg("DNS Master") eq "FALSE") {
		Log ("This Server is not a DNS master hence skipping update of dnIPUpdatePolicy");
		return;
	}

	my $fwd_zone = registry::getReg("DNSZONE");
	my $rev_zone = registry::getReg("IP_ADDRESS_UNDERSCORE_FORM");
	my $dns_server = registry::getReg("DNSSERVER");
	my $tmp_file = mktemp("/tmp/moddnupdate.XXXXXXXX");
	my $rc = 0;
	my $upgrade = registry::getReg("UPGRADE");
	my $dnipzones_command = "";
	my $policycheck_command = "";
	my $updateFilter = 0;
	my $updateFilterValue = "";
	my $ldap_out_file = mktemp("/tmp/ldapoutput.XXXXXXXX");

	if ($upgrade eq "YES") {
		Log ("The server is being Upgraded");
		$dnipzones_command = "SASL_PATH=/opt/novell/xad/lib64/sasl2 /usr/bin/ldapsearch".
			" -Q -Y GSSAPI -s sub -b \"$dns_server\" dNIPZoneList -LLL -o ldif-wrap=no";
	} else {
		$dnipzones_command = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf".
			" /usr/bin/ldapsearch -Q -Y EXTERNAL -b \"$dns_server\" dNIPZoneList -LLL".
			" -o ldif-wrap=no";
	}

	Log ("Executing :- $dnipzones_command > $ldap_out_file");
	my @error_output = `$dnipzones_command > $ldap_out_file`;
	my $err = $? >> 8;
	if ($err != 0) {
		chomp(@error_output);
		unlink($ldap_out_file);
		Log (ERR, "LDAP search failed with error [@error_output] [$err]\n");
	}

	my @zone_list = `cat $ldap_out_file \\
			| grep -i "dNIPZoneList" | awk -F\" \" '{print \$2}'`;
	$err = $? >> 8;
	chomp(@zone_list);
	if ($err != 0) {
		Log (ERR, "Error encountered while parsing zone list [@zone_list] [$err]");
	} else {
		unlink($ldap_out_file);
		Log ("Successfully searched for the dns zones.\n@zone_list");
	}

	my $install_type = registry::getReg("INSTALL_TYPE");

	my ($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value) = (0, "", 0, "");
	foreach my $zone (@zone_list) {
		if (uc($upgrade) eq "YES") {
			($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value) = util::is_policy_defined(zone => "$zone", type_of_config => "UPGRADE");
		} else {
			($upd_policy_present, $upd_policy_value, $upd_filter_present, $upd_filter_value) = util::is_policy_defined(zone => "$zone");
		}

		if($upd_filter_present) {
			my $update_command = "dn: $zone\nchangetype: modify\n".
				"replace: dNIPUpdateFilter\ndNIPUpdateFilter: $upd_filter_value\n";
			$rc = `echo -e "$update_command" >>$tmp_file`;
		} else {
			my $update_command = "dn: $zone\nchangetype: modify\nreplace: dNIPUpdatePolicy".
				"\ndNIPUpdatePolicy: grant * sam *";
			my $grant = "";

			if ($zone =~ /\Q$rev_zone\E/i) {
				$grant = "NS PTR AAAA KEY\n";
			} elsif ($zone =~ /\Q$fwd_zone\E/i) {
				$grant = "A NS SRV PTR AAAA KEY\n";
			} elsif (lc($install_type) eq "frd") {
				if (index($zone, "in-addr.arpa") != -1) {
					$grant = "NS PTR AAAA KEY\n";
				} else {
					$grant = "A NS SRV PTR AAAA KEY\n";
				}
			}

			if ($grant ne "") {
				$rc = `echo -e "$update_command $grant" >>$tmp_file`;
			}
		}
	}

	# Modify the attribute on DNS zone
	my $ldapmodify = "";
	if (-e $tmp_file) {
		if ($upgrade eq "YES") {
			$ldapmodify = registry::getReg("LDAPMODIFY");
			$ldapmodify = "SASL_PATH=/opt/novell/xad/lib64/sasl2 $ldapmodify -c -Q -Y GSSAPI";
		} else {
			$ldapmodify = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -c -Q";
		}
		Log ("Modifying the dNIPUpdatePolicy attribute of the DNS zones.");
		Log ("Executing:- $ldapmodify -f $tmp_file 2>&1");
		my @output = `$ldapmodify -f $tmp_file 2>&1`;
		$err = $? >> 8;
		if (($err != 0) and ($err != $LDAP_ERR_DUP)) {
			chomp (@output);
			Log (WARN, "Could not modify the dNIPUpdatePolicy  attribute".
				" of DNS zones. Error:- [@output] [$err]");
		} else {
			Log ("Successfully modify the dNIPUpdatePolicy attribute of DNS zones.");
		}
	}
}

# Update the Bugs Fixed in Install of OES11SP2
sub update_gplink_attribute {
	# This is the fix for BUG: 788300.
	# There is gplink attribute on Domain Controllers container which points
	# to non-existing gpo policy. Hence removing it.
	my $mapped_domain_nc = registry::getReg("Mapped Domain NC");
	my $tmp_file = mktemp("/tmp/delgplink.XXXXXXXX");
	my $baseDN = "ou=Domain Controllers,$mapped_domain_nc";
	my $ldapmodify = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -c -Q";
	my $rc = `echo -e "dn: $baseDN\nchangetype: modify\ndelete: gplink" > $tmp_file`;
	Log ("Executing $ldapmodify -f $tmp_file 2>&1") ;
	my @output = `$ldapmodify -f $tmp_file 2>&1`;
	my $err = $? >> 8;
	if (($err != 0) and ($err != $LDAP_ERR_NO_SUCH_ATTR)) {
		chomp (@output);
		Log (WARN, "Could not remove the gplink attribute on the domain controller container: [@output] : [$err]");
	} else {
		Log ("Successfully deleted the gplink attribute on the domain controller container");
	}
}

# Update the WINBIND script
sub update_winbind_script {
	Log(INFO, "Modifying winbind init script");

	eval {
		if (-f "/etc/init.d/winbind") {
			copy("/etc/init.d/winbind", "/etc/init.d/winbind.$stamp")
				or Log(WARN, "Failed to take the backup of winbind rc script: $!");

			util::replace_line_in_file (
					'^# Required-Stop:  \$network \$remote_fs \$syslog',
					"# Required-Stop:  \$network \$remote_fs \$syslog xadsd",
					"/etc/init.d/winbind");

			my @winbind_start = `sed -n -e '/^\\t\\+start)/,/startproc /p' /etc/init.d/winbind 2>&1`;
			if ($? == 0 and !grep(/^\s+ulimit\s+-n\s+\d+$/, @winbind_start)) {
				util::replace_line_in_file ('^\tstart\)',
					"\tstart)\n\t\t#Limit on maximum open files\n\t\tulimit -n 8192",
					"/etc/init.d/winbind");
			} else {
				Log(WARN, "Failed to apply ulimit setting in".
					" /etc/init.d/winbind startup script".
					" [$!] [@winbind_start] ..Continuing...");
			}

			chmod 0755, "/etc/init.d/winbind";

		} else {
			die("winbind init script not found");
		}
	};

	if ($@) {
		Log(ERR, $@);
	}
}


#Update the winbind script with performance tuning parameters
sub update_smbconf() {
	#Updating the smb.conf
	my $smb_file = "/etc/samba/smb.conf";

	copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");

	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) {
		Log ("The open of $smb_file failed : @Config::IniFiles::errors");
		return -1;
	}

	Log ("Updating section - [global]");
	# winbind max clients = 2000
	my $var_value = "winbind max clients";
	my $val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '2000');
	}

	# winbind cache time = 3600
	$var_value = "winbind cache time";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '3600');
	}

	# name cache timeout = 3600
	$var_value = "name cache timeout";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '3600');
	}

	# winbind max domain connections = 5
	$var_value = "winbind max domain connections";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '5');
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updating $smb_file file with tuning parameters");
	return 0;
}

#Update samba smb.conf with latest changes wrt to 4.6
sub upgrade_samba_smb_conf() {
	#Updating the smb.conf
	my $smb_file = "/etc/samba/smb.conf";

	copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");

	my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
	if (!defined $cfg) {
		Log ("The open of $smb_file failed : @Config::IniFiles::errors");
		return -1;
	}

	Log ("Updating $smb_file file with latest samba 4.6 entries");
	Log ("Updating section - [global]");
	# security = user
	my $var_value = "security";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, 'user');
	}
	else {
		$cfg->setval ('global', $var_value, 'user');
	}

	#Remove 'idmap config  : backend = ad'
	$var_value = "idmap config  : backend";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		$cfg->delval ('global', $var_value);
	}

	#  idmap config $REGISTRY=Netbios Name : backend = ad
	$var_value = "idmap config ".uc(registry::getReg("Netbios Name"))." : backend";
	my $val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, 'ad');
	}
	else {
		$cfg->setval ('global', $var_value, 'ad');
	}

	#Remove 'idmap config  : range = 1 - 4294967295'
	$var_value = "idmap config  : range";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		$cfg->delval ('global', $var_value);
	}

	# idmap config $REGISTRY=Netbios Name : range = 1 - 4294967295
	$var_value = "idmap config ".uc(registry::getReg("Netbios Name"))." : range";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, '1 - 4294967295');
	}
	else {
		$cfg->setval ('global', $var_value, '1 - 4294967295');
	}

	#Remove 'send spnego principal = Yes'
	$var_value = "send spnego principal";
	$val = $cfg->val('global', $var_value);
	if (defined $val) {
		$cfg->delval ('global', $var_value);
	}

	# require strong key = no
	$var_value = "require strong key";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		$cfg->newval ('global', $var_value, 'no');
	}
	else {
		$cfg->setval ('global', $var_value, 'no');
	}

	# template homedir = /home/%U
        $var_value = "template homedir";
        $val = $cfg->val('global', $var_value);
        if (!defined $val) {
                $cfg->newval ('global', $var_value, '/home/%U');
        }
        else {
                $cfg->setval ('global', $var_value, '/home/%U');
        }

	Log ("Updating section - [netlogon]");
	#Remove 'share modes = No'
	$var_value = "share modes";
	$val = $cfg->val('netlogon', $var_value);
	if (defined $val) {
		$cfg->delval ('netlogon', $var_value);
	}

	Log ("Updating section - [sysvol]");
	#Remove 'share modes = No'
	$var_value = "share modes";
	$val = $cfg->val('sysvol', $var_value);
	if (defined $val) {
		$cfg->delval ('sysvol', $var_value);
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
	`sed -i 's/=/ = /g' $smb_file`;

	Log ("Updated $smb_file file with latest samba 4.6 entries");
	return 0;
}

#Badlock Fix
sub update_winbind_sealed_pipes_param()
{
	my $samba_conf_file = "/etc/samba/smb.conf";

	copy($samba_conf_file, $samba_conf_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");

	my $cfg = Config::IniFiles->new (-file=> $samba_conf_file, -nocase=>1);
	my $var_value = "winbind sealed pipes";

	if (!defined $val) {
		$cfg->newval ('global', $var_value, 'false');
	}
	else {
		$cfg->setval ('global', $var_value, 'false');
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g'     $samba_conf_file`;
	`sed -i 's/=/ = /g'     $samba_conf_file`;

	Log ("Updated $samba_conf_file file with badlock fix parameters");
	return 0;
}

#Update samba smb.conf with latest wins configuration of samba-4.10
sub update_wins_parameter_smb_conf() {
        #Updating the smb.conf

	Log (">>>Starting to update wins parameter in smb.conf file with value: name resolve order = wins lmhosts host bcast");

        my $smb_file = "/etc/samba/smb.conf";

        copy($smb_file, $smb_file.$stamp) or Log (ERR, "Failed to backup the samba conf file: $!");

        my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
        if (!defined $cfg) {
                Log ("The open of $smb_file failed : @Config::IniFiles::errors");
                return -1;
        }

        Log ("Updating $smb_file file with wins parameter");
        Log ("Updating section - [global]");
        
	# name resolve order = wins lmhosts host bcast
        $var_value = "name resolve order";
        $val = $cfg->val('global', $var_value);
        if (defined $val) {
                $cfg->newval ('global', $var_value, 'wins lmhosts host bcast');
        }
        
        $cfg->RewriteConfig();
        `sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
        `sed -i 's/=/ = /g' $smb_file`;

        Log ("Updated $smb_file file with latest wins parameter");
        return 0;
}

sub changeACLonsysvoldir() {
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $gp_creator_owner_gid = registry::getReg("GP_CREATOR_OWNER_GID");
	my ($rc, $error) =(0, 0);

	eval {
		Log(INFO, "Executing ... /usr/bin/setfacl -m g:$gp_creator_owner_gid".
				":rwx $live_local_statedir/sysvol");
		`/usr/bin/setfacl -m g:$gp_creator_owner_gid:rwx $live_local_statedir/sysvol`;
		$rc = $?;
		if ($rc != 0) {
			$error = $rc>>8;
			Log(INFO, "Failed to change ACL on $live_local_statedir/sysvol [$error]");
		}
	};

	if ($@) {
		Log(ERR, $@);
	}
}

#BUG 830989 User who is member of Group Policy Creater Owners
#group only, can't create new GPO
#Accepts 'upgrade or UPGRADE' and 'migration or MIGRATION' as input
sub changeACLofUserinGPCO($){
	my $operation_type = shift;
	my $ldapmodify = "";
	my $auth_mechanism = "EXTERNAL";
	my %mechanism_path = (
				"EXTERNAL" => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf",
				"GSSAPI" => "SASL_PATH=/opt/novell/xad/$lib/sasl2"
			);
	my $acl_change_ini = mktemp("/tmp/temp_gpco_ini.XXXXXXXX");
	my $acl_change_ldif = mktemp("/tmp/temp_gpco_ldif.XXXXXXXX");

	if (lc($operation_type) eq "upgrade") {
		$auth_mechanism = "GSSAPI";
	}
	$ldapmodify = "$mechanism_path{$auth_mechanism} /usr/bin/ldapmodify -Y $auth_mechanism -c -Q";

	open(FH, ">$acl_change_ini") or die("Could not open file $acl_change_ini [$!]");
	print FH "[DEFAULTROOTDOMAIN]\n";
	print FH "CHILD=System\n";
	print "\n";
	print FH "[System]\n";
	print FH "CHILD=Policies\n";
	print "\n";
	print FH "[Policies]\n";
	print FH "changetype=modify\n";
	print FH "replace=ACL\n";
	print FH ";\n";
	print FH "; DS_ATTR_COMPARE | DS_ATTR_READ | DS_ATTR_SELF\n";
	print FH ";\n";
	print FH "ACL=11#subtree#CN=Group Policy Creator Owners,\$REGISTRY=USERSCONTAINER#[All Attributes Rights]\n";
	print FH ";\n";
	print FH "; DS_ENTRY_BROWSE | DS_ENTRY_ADD | DS_ENTRY_RENAME\n";
	print FH ";\n";
	print FH "ACL=11#subtree#CN=Group Policy Creator Owners,\$REGISTRY=USERSCONTAINER#[Entry Rights]";
	close(FH);

	Log("Generating $acl_change_ldif");
	`/opt/novell/xad/share/dcinit/genLdif.pl $acl_change_ini --defaultrootdomain > $acl_change_ldif`;

	Log(INFO, "Executing ... $ldapmodify -f $acl_change_ldif");
	my @output = `$ldapmodify -f $acl_change_ldif 2>&1`;
	my $status = $?;
	my $error = $status>>8;
	if ($error and ($error != $LDAP_ERR_DUP and !grep(/duplicate acl/i, @output))) {
		chomp(@output);
		Log(ERR, "Failed to update ACLs on the Policies Container $error [@output]");
	}
	unlink($acl_change_ini);
	unlink($acl_change_ldif);
}

sub update_crontabentries{
	my $croncommand="/usr/bin/crontab";
	my $cronlistcommand="/usr/bin/crontab -l";
	my $crontempfile=mktemp("/tmp/cronentry.XXXXXXXX");
	my @replace_entries = ("/opt/novell/xad/sbin/gposync.sh",
			"/opt/novell/xad/sbin/sysvolsync",
			"/opt/novell/xad/sbin/fgpsync.sh");
	my $log_dir = "/var/opt/novell/xad/log";

	# Collect the current entries from crontab file
	`$cronlistcommand > $crontempfile`;

	# Read the data into an array
	open(FH, "$crontempfile") or die("Could not open $crontempfile");
	my @cronfiledata = <FH>;
	close(FH);

	chomp(@cronfiledata);

	# Loop through the entries to check whether it contains
	# /dev/null or not
	foreach my $repl_entry (@replace_entries) {
		# Get the entry in the current cronatb file
		my @entry = grep(m#$repl_entry#, @cronfiledata);
		if(($#entry+1) and defined($entry[0])
				and !grep(m#1>/dev/null#, $entry[0])) {

			Log(INFO, "Entry [$entry[0]] stdout redirected to /dev/null".
				" in file [$crontempfile]");
			my @filename = fileparse($repl_entry, qr/\.[^.]*/);
			my $redirections = "1>/dev/null 2>>${log_dir}/$filename[0].log";
			`sed -i 's#$repl_entry#$repl_entry $redirections#g' $crontempfile`;
		}
	}

	`$croncommand $crontempfile`;

	#Remove the temporary file
	`rm -f "$crontempfile"`;
}

# Updating the following ACL's
# ACL: 6#entry#$computer_cn#operatingSystem
# ACL: 6#entry#$computer_cn#operatingSystemVersion
# ACL: 6#entry#$computer_cn#operatingSystemServicePack
sub update_acls_on_computer_objects($) {
	my $operation_type = shift;
	my $auth_mechanism = "EXTERNAL";
	my %mechanism_path = (
			"EXTERNAL" => "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf",
			"GSSAPI" => "SASL_PATH=/opt/novell/xad/$lib/sasl2"
			);

	my $domain_nc = registry::getReg('Domain NC');
	chomp($domain_nc);
	my $live_local_statedir = registry::getReg("LIVE_LOCALSTATEDIR");
	my $ldif_file = "${live_local_statedir}/ds/domain/update_computer_acls.ldif";

	if (lc($operation_type) eq "upgrade") {
		$auth_mechanism = "GSSAPI";
	}

	my $ldapsearch = "$mechanism_path{$auth_mechanism} /usr/bin/ldapsearch -Y $auth_mechanism -Q";
	my $ldapmodify = "$mechanism_path{$auth_mechanism} /usr/bin/ldapmodify -Y $auth_mechanism -c -Q";

	# Search all the computer objects in the domain
	my @ldap_output=`$ldapsearch -s sub -b $domain_nc 'objectclass=msds:computer' dn -LLL -o ldif-wrap=no`;

	if ($? != 0) {
		my $err = $? >> 8;
		chomp(@ldap_output);
		Log(INFO, "Failed to search the computer objects : $err : [@ldap_output]");
		return;
	}
	chomp(@ldap_output);

	# Collect all the computer objects except the object's
	# found in ou=Domain Controllers container
	my @all_computers = grep (!/ou=domain controllers,/ig, @ldap_output);
	@all_computers = grep (/^dn:/ig, @all_computers);

	unless(open LDIFFILE,">$ldif_file") {
		Log(INFO, "Could not open the ldif file $ldif_file : $!");
		return;
	}

	# Prepare the LDIF file to update the ACL's on computer objects
	foreach my $computer (@all_computers) {
		my $computer_cn = $computer;
		$computer_cn =~ s/^dn: //i;

		Log(INFO, "Updating ACL's on computer [$computer_cn]");

		print LDIFFILE "$computer\n";
		print LDIFFILE "changeType: modify\n";
		print LDIFFILE "add: ACL\n";
		print LDIFFILE "ACL: 6#entry#$computer_cn#operatingSystem\n";
		print LDIFFILE "ACL: 6#entry#$computer_cn#operatingSystemVersion\n";
		print LDIFFILE "ACL: 6#entry#$computer_cn#operatingSystemServicePack\n\n";
	}
	close LDIFFILE;

	Log(INFO, "Updating the ACLs on the computer objects");

	my @output = `$ldapmodify -f $ldif_file 2>&1`;
	my $err = $? >> 8;
	if ($err and ($err != $LDAP_ERR_DUP and
		!grep(/duplicate acl/i, @output))) {

		chomp (@output);
		Log(INFO, "Failed to update ACL's on computer object : $err: [@output]");
	} else {
		Log(INFO, "Successfully updated the ACL's on the computer objects ");
	}
}

# Creates ini, the ldif file, and does ldapmodify for the following ACL
# Following are the ACLs added
# 1. For mac10.10 integration
# ACL=3#subtree#[Root]#pwdChangedTime
# 2. For Remote Desktop license server integration
# ACL=7#subtree#[Root]#terminalServer
sub update_oes2015_acls () {

        my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        my $upgrade_acl_ini = "/opt/novell/xad/share/dcinit/templates/nds-domain-upgrade-acls.ini";
        my $upgrade_acl_ldif = "$LIVE_DSSTATEDIR/domain/nds-domain-oes2015-acls.ldif";
        my $ldapmodify = registry::getReg("LDAPMODIFY");
        my $mapped_domain_nc = registry::getReg("Mapped Domain NC");

        Log("Upgrading ACLs for attributes as a part of oes2015 upgrade");
        Log("Attributes are: \"pwdChangedTime and \" terminalServer");
        open FILE, ">$upgrade_acl_ini" or die ("failed to open the file $upgrade_acl_ini");
        print FILE "[DEFAULTROOTDOMAIN]\n";
        print FILE "CHILD=Domain NC ACLs and Policy\n";
        print FILE "[Domain NC ACLs and Policy]\n";
        print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
        print FILE "changetype=modify\n";
	print FILE "add=ACL\n";
	# The ACL enables pwdChangedTime(mapped from pwdLastSet) attribiutes to be read by
	# "identity" corresponding to "Workstation account"
	print FILE "ACL=3#subtree#[Root]#pwdChangedTime\n";
	# The ACL enables terminalServer attribiute to be read and modified by
	# the remote desktop license server
	print FILE "ACL=7#subtree#[Root]#terminalServer";
	close(FILE);

       `/opt/novell/xad/share/dcinit/ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
       `genLdif.pl /tmp/effective_acls --defaultrootdomain >$upgrade_acl_ldif`;
        unlink "/tmp/effective_acls";
        Log("Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $upgrade_acl_ldif");
        my @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $upgrade_acl_ldif 2>&1`;
        my $error = $? >> 8;
        if ($error and $error != $LDAP_ERR_DUP) {
                chomp(@output);
                Log("Failed to update ACLs on the domain object [@output]");
        }
        Log("Updated the ACLs for attributes as the part of oes2015 upgrade");
}

#Change smb.conf and also permissions of scripts folder in sysvol
#No changes in smb.conf if its adc, the change is only for pdc
sub netlogon_share_changes() {
	my $samba_conf_file = "/etc/samba/smb.conf";
	my $xad_scripts_path = "/var/opt/novell/xad/sysvol/domain/scripts";

	Log("Doing changes for netlogon share access");

	my $type = registry::getReg('INSTALL_TYPE');
	my $is_replica = registry::getReg("IS_REPLICA");

	Log("Executing ... chmod 0771, \"$xad_scripts_path\"");
	chmod 0771, "$xad_scripts_path";

	if ($is_replica eq "TRUE") {
		#adc install
		Log("No [netlogon] specific changes needed for smb.conf in adc");
        }
        else {
		#frd or cdc install
		Log("Changes in smb.conf being done for pdc");
		util::make_netlogon_share_writable("$samba_conf_file");
        }
}

# Changes will be reflected in /etc/chrony.conf
# with the recent changes for libw32time.so
# additional configurable parameters are added.
sub update_ntp_with_w32time($){
        my $file_name = shift;

        Log("Updating the file $file_name with w32time changes.");

        eval{
                if (-f $file_name) {
                        my $ntpconf_backup = mktemp("$file_name.XXXXXXXXXX");

                        Log("Backup of file chrony.conf taken into $ntpconf_backup");
                        copy("$file_name", $ntpconf_backup) or
                                Log("Coult not copy the /etc/chrony.conf to $ntpconf_backup");

                        # With the recent changes in the libw32time.so file
                        # following are the two lines of configuration needed in /etc/chrony.conf file.
                        # ntpsigndsocket /var/lib/chrony/var/opt/novell/xad/rpc/
                        # restrict -4 default notrap nomodify nopeer noquery mssntp
                        # The old authprov modification is kept as it is, if required further it will be deleted.

                        # collect file data, and use it for duplicate entry search
                        open(FHN, "<$ntpconf_backup") or Log("Failed to open $ntpconf_backup");
                        my @file_data = <FHN>;
                        close(FHN);

                        my $ntpsigndsocket_append = "ntpsigndsocket \/var\/lib\/ntp\/var\/opt\/novell\/xad\/rpc\/";
                        my $restrict_append = "restrict -4 default notrap nomodify nopeer noquery mssntp";

                        # If not present append "ntpsigndsocket /var/lib/chrony/var/opt/novell/xad/rpc/"
                        if(grep(/^ntpsigndsocket.*\/var\/lib\/ntp\/var\/opt\/novell\/xad\/rpc\/$/, @file_data)==0){
                                Log("appending ntpsigndsocket /var/lib/chrony/var/opt/novell/xad/rpc/");
                                `echo '$ntpsigndsocket_append' >>$file_name`;
                        }

                        # If not present append "restrict -4 default notrap nomodify nopeer noquery mssntp"
                        if(grep(/^$restrict_append$/, @file_data)==0){
                                Log("appending restrict -4 default notrap nomodify nopeer noquery mssntp");
                                `echo '$restrict_append' >>$file_name`;
                        }

                        `sed -i -e '/^authprov/{/\\s*global\\s*/! s/.*/& global/}' $file_name`;
                }
                else {
                        die("Could not find the $file_name");
                }
        };

        if ($@) { Log(ERR, $@) }
}

#AES entries to be updated in /etc/krb5.conf file
my %replacement_maps_aes =(
    'permitted_enctypes[\h]*=[\h]*arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'permitted_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5[\h]+des-cbc-md5[\h]+des-cbc-crc[\h]+aes128-cts-hmac-sha1-96' => 'permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5 des-cbc-md5 des-cbc-crc',
    'default_tgs_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tgs_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tgs_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5',
    'default_tkt_enctypes[\h]*=[\h]*aes128-cts[\h]+arcfour-hmac-md5' => 'default_tkt_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 arcfour-hmac-md5'
);

#Update the krb5.conf with the aes256 parameters
sub replace_in_file_aes {
        my $file = shift;
        open(IN,"<$file") or die $!;
        my @input_lines = <IN>;
        close(IN);

        my @output_lines;
        foreach(@input_lines)
        {
                foreach my $aes256_entry (keys %replacement_maps_aes){
                        $_=~s/$aes256_entry/$replacement_maps_aes{$aes256_entry}/g;
                }
                push(@output_lines,$_);
        }

        open(OUT,">$file") or die $!;
        print OUT @output_lines;
        close(OUT);
}

#Current update supported only from non-AES level
sub update_krb5conf_aes {
	my $localtime=localtime;
	my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
	my $stamp=$year.$month.$date."-".$time;
	my $krb5_conf_file = "/etc/krb5.conf";

        copy($krb5_conf_file,"/tmp/krb5.conf-aes") or die "$krb5_conf_file copy failed: $!";
        copy($krb5_conf_file,"/etc/krb5.conf.$stamp") or die "$krb5_conf_file save failed: $!";
        replace_in_file_aes("/tmp/krb5.conf-aes");
        copy("/tmp/krb5.conf-aes",$krb5_conf_file) or die "$krb5_conf_file replace failed: $!";
        Log("Domain Controller config file updated to support AES encryption");
}

sub update_domain_to_aes {
        update_krb5conf_aes;

	my $type = registry::getReg("INSTALL_TYPE");

	if($type eq "frd") {

		my $installmachinename = registry::getReg("INSTALLMACHINENAME");

		my $treeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
		if(not defined($treeName) or $treeName eq ""){
			Log(WARN, "failed to fetch the tree name.");
			return 1;
		}
		chomp($treeName);

		my $treeAdminName="";

		if (defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "") {
			$treeAdminName = $ENV{LDAPEXISTINGADMINNAME};
		}

		#non-name mapped tree administrator
		$treeAdminName = registry::getReg("LDAPDOMAINADMINNAME");
		my $is_replica = registry::getReg("IS_REPLICA");
		if ($type eq "frd" and $is_replica eq "FALSE") {
			if (not registry::getReg("MAPPED")) {
				#name mapped tree administrator
				$treeAdminName = registry::getReg("LDAPDOMAINADMIN");
			}
		}

		chomp($treeAdminName);
		$treeAdminName=~s/,/\./g;

		Log(INFO, "Executing : /opt/novell/eDirectory/bin/nmasinst -addmethod $treeAdminName $treeName /opt/novell/xad/share/nmasmthd/Kerberos/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}' ");
		my @command_output = `/opt/novell/eDirectory/bin/nmasinst -addmethod $treeAdminName $treeName /opt/novell/xad/share/nmasmthd/Kerberos/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}' 2>&1`;
		if (($?>>8) != 0) {
			Log(WARN, "nmasinst - Kerberos failed with error [@command_output]");
		} else {
			Log(MESG, "update_domain_to_aes performing nds::trigger_backlinker.");
			nds::trigger_backlinker();
		}

		Log(INFO, "Executing : /opt/novell/eDirectory/bin/nmasinst -addmethod $treeAdminName $treeName /opt/novell/xad/share/nmasmthd/Negotiate/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}' ");
		@command_output = `/opt/novell/eDirectory/bin/nmasinst -addmethod $treeAdminName $treeName /opt/novell/xad/share/nmasmthd/Negotiate/config.txt -h $installmachinename -w '$ENV{'ADM_PASSWD'}' 2>&1`;
		if (($?>>8) != 0) {
			Log(WARN, "nmasinst - Negotiate failed with error [@command_output] ");
		}
	}

	#bootstrap_krbtgt;
	my $command = "/opt/novell/xad/sbin/setpassword -NDOS -r -u krbtgt";

	Log(INFO, "Executing : $command");
	my @command_output = `$command 2>&1`;
	if (($?>>8) != 0) {
		Log("Could not set password for krbtgt");
		Log("error : [@command_output]");
		Log("Please try rerunning");
	}

        # setpassword command for server object
	execute_setpassword_command();
}

sub is_domain_at_desired_level($$){
	my $ms_ds_behavior_version = shift;
	my $ad_schema_level = shift;

	my $domain_at_desired_level = 1;
	my $rootDomain = registry::getReg("Root Domain");
        my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;

        chomp($ldapadminname);
        chomp($rootDomain);
        $rootDomain =~ s/\./,dc=/g;
        $rootDomain = "dc=".$rootDomain;

        eval{
                #Find all server objects in the domain
                my @server_result=();
                if(uc($type_of_install) eq "INSTALL"){
                        @server_result =`/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "ou=Domain Controllers,$rootDomain" -s sub "(objectClass=computer)" -Z -o ldif-wrap=no -LLL serverReferenceBL 2>&1 | grep -i "^serverReferenceBL"`;
                }else{
                        @server_result =`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "ou=Domain Controllers,$rootDomain" -s sub "(objectClass=computer)" -Z -o ldif-wrap=no -LLL serverReferenceBL 2>&1 | grep -i "^serverReferenceBL"`;

                }

                my $error = $? >> 8;
                if ($error) {
                        Log(WARN,"Could not get the domain controllers: $error: [@server_result]");
                        return;
                }

                my $totalServersFound = $#server_result + 1;
                foreach my $serv (@server_result){
                        my $serverDn = (split(/^serverReferenceBL: /, $serv))[1];
                        chomp($serverDn);
                        my $name = (split(/=/, ((split(/,/, $serv))[0])))[1];

                        #Search for msDS-Behavior-Version on each of the server object
                        my $output = "";
                        if(uc($type_of_install) eq "INSTALL"){
                                $output = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }else{
                                $output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }
                        if($? == 0){
                                my $version = (split(/:/, $output))[1];
                                if($version != $ms_ds_behavior_version){
                                        Log(WARN, "Server $name is not at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                        $domain_at_desired_level = 0;
                                }else{
                                        Log(INFO, "Server $name is at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                }
                        }else{
                                Log(INFO, "Server $name is not a valid server.");
                        }
                }
		if(!$domain_at_desired_level){
                        Log(WARN, "Domain functional level it not updated to DS_BEHAVIOR_VERSION $ms_ds_behavior_version, as not all domain servers are at DS_BEHAVIOR_VERSION $ms_ds_behavior_version or greater.");
                }

                #if it is single server and is the PDC
                #we are going to anyway allow the server to update msDS-Behavior-Version
                #hence checking the server
                my $type = registry::getReg("INSTALL_TYPE");
                my $is_replica = registry::getReg("IS_REPLICA");

                if( (lc($type) eq "frd" or lc($type) eq "child") and $is_replica eq "FALSE" and $totalServersFound==1){
                        $domain_at_desired_level = 1;
                }
        };

        if ($@) { Log(WARN,$@); }

        return $domain_at_desired_level;


}
sub is_domain_at_level2012(){
	my $domainAt2012l = is_domain_at_desired_level(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
	return $domainAt2012l;
}

sub is_domain_at_level2016(){
        my $domainAt2016l = is_domain_at_desired_level(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2016);
        return $domainAt2016l;
}

#For the forest level validation
# First all of the servers present in the tree needs
# to be at the correct schema level
# Second all of the domains should be at the same
# updated schema level

sub is_forest_at_desired_level($$){
	my $ms_ds_behavior_version = shift;
	my $ad_schema_level = shift;

        my $forest_at_desired_level = 1;
        my $totalServers = 0;
        my $forestNC = registry::getReg("Forest NC");
        my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
        chomp($ldapadminname);

        #Server object validation
        eval{
                #Find all server objects in the domain
                my @server_result = ();
                if(uc($type_of_install) eq "INSTALL"){
                        @server_result =`/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "cn=Configuration,$forestNC" -s sub "(objectClass=server)" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i "^dn:"`;
                }else{
                        @server_result =`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "cn=Configuration,$forestNC" -s sub "(objectClass=server)" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i "^dn:"`;
                }

                my $error = $? >> 8;
                if ($error) {
                        Log(WARN,"Could not get the domain controllers: $error: [@server_result]");
                        return;
                }

                my $totalServers = $#server_result + 1;
                foreach my $serv (@server_result){
                        my $serverDn = (split(/^dn: /, $serv))[1];
                        chomp($serverDn);
                        my $name = (split(/=/, ((split(/,/, $serv))[0])))[1];

                        #Search for msDS-Behavior-Version on each of the server object
                        my $output = "";
                        if(uc($type_of_install) eq "INSTALL"){
                                $output = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }else{
                                $output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "cn=NTDS Settings,$serverDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }
                        if($? == 0){
                                my $version = (split(/:/, $output))[1];
                                if($version != $ms_ds_behavior_version){
                                        Log(WARN, "Server $name is not at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                        $forest_at_desired_level = 0;
                                }else{
                                        Log(INFO, "Server $name is at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                }
                        }else{
                                Log(INFO, "Server $name is not a valid server.");
                        }
                }

                if(!$forest_at_desired_level){
                        Log(WARN, "Forest functional level is not updated to DS_BEHAVIOR_VERSION $ms_ds_behavior_version, as not all forest servers are at DS_BEHAVIOR_VERSION $ms_ds_behavior_version or greater.");
                }
        };

        if ($@) { Log(WARN,$@); }

        # Domain validation
        eval{
                #Find all domain contexts in the forest
                my @domain_contexts = ();
                if(uc($type_of_install) eq "INSTALL"){
                        @domain_contexts = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "$forestNC" -s sub "(objectClass=Partition)" xad-Domain-Flag -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -B 1 xad-Domain-Flag | grep -i "^dn:"`;
                }else{
                        @domain_contexts = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "$forestNC" -s sub "(objectClass=Partition)" xad-Domain-Flag -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -B 1 xad-Domain-Flag | grep -i "^dn:"`;
                }

                my $error2 = $? >>8;
                if ($error2){
                        Log(WARN, "Could not get the Domain Contexts in the forest: $error2: [@domain_contexts]");
                        return;
                }

                my $total_contexts = $#domain_contexts +1;
                foreach my $context (@domain_contexts){
                        my $contextDn = (split(/^dn: /, $context))[1];
                        chomp($contextDn);

                        #Search for msDS-Behavior-Version on each of the domain context
                        my $output = "";
                        if(uc($type_of_install) eq "INSTALL"){
                                $output = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -b "$contextDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }else{
                                $output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -b "$contextDn" -s base -Z -o ldif-wrap=no -LLL msDS-Behavior-Version 2>&1 | grep -i "msDS-Behavior-Version"`;
                        }
                        if($? == 0){
                                my $version = (split(/:/, $output))[1];
                                if($version != $ms_ds_behavior_version){
                                        Log(WARN, "Domain $contextDn is not at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                        $forest_et_desired_level = 0;
                                }else{
                                        Log(INFO, "Domain $contextDn is at DS_BEHAVIOR_VERSION $ms_ds_behavior_version level.");
                                }
                        }else{
                                Log(INFO, "Domain $contextDn is not a valid domain.");
                        }
                }

                if(!$forest_at_desired_level){
                        Log(WARN, "Forest functional level is not updated to DS_BEHAVIOR_VERSION $ms_ds_behavior_version, as not all domains in the Forest are at DS_BEHAVIOR_VERSION $ms_ds_behavior_version.");
                }
        };

        if ($@) { Log(WARN,$@); }

        return $forest_at_desired_level;
}


sub is_forest_at_level2012(){
        my $forestAt2012l = is_forest_at_desired_level(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
        return $forestAt2012l;
}

sub is_forest_at_level2016(){
        my $forestAt2016l = is_forest_at_desired_level(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2016);
        return $forestAt2016l;
}

sub post_schema_extension(){
	post_schema_extension_params(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
}


sub post_schema_extension_params($$){
	my $ms_ds_behavior_version = shift;
	my $ad_schema_level = shift;

        my $mapped_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
        my $post_extn_ldif = mktemp("/tmp/post_behav_ldif.XXXXXXXXXX");
        my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;

        Log(INFO,"Preparing domain for DS_BEHAVIOR_VERSION $ms_ds_behavior_version level(post)");
        chomp($ldapadminname);
        chomp($mapped_nc);

        open FILE, ">$post_extn_ldif" or die ("failed to open the file $post_extn_ldif");

        print FILE "dn:$mapped_nc\n";
        print FILE "changetype: modify\n";
        print FILE "replace: msDS-Behavior-Version\n";
        print FILE "msDS-Behavior-Version: $ms_ds_behavior_version\n";
        print FILE "-\n";
        print FILE "replace: nTMixedDomain\n";
        print FILE "nTMixedDomain: 0\n";
        print FILE "\n";

        close FILE;

        eval{
                my @output = ();
                if(uc($type_of_install) eq "INSTALL"){
                        Log(INFO, "Executing...(post) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $post_extn_ldif");
                        @output = `/usr/bin/ldapmodify -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -Z -f $post_extn_ldif`;
                }else{
                        Log(INFO, "Executing...(post) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $post_extn_ldif");
                        @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $post_extn_ldif`;
                }
                my $error = $? >> 8;
                if ($error) {
                        die("Could not modify the objects: $error: [@output]");
                }
        };

        if ($@) { Log(WARN,$@) }

        unlink($post_extn_ldif);
}

sub post_forest_functional_level(){
	post_forest_functional_level_params(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
}

sub post_forest_functional_level_params($$){
	my $ms_ds_behavior_version = shift;
        my $ad_schema_level = shift;

	my $post_forest_func_ldif = mktemp("/tmp/post_forest_func_ldif.XXXXXXXXXX");
        my $netbios_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Netbios Name"`;
        my $forest_nc = `/opt/novell/xad/share/dcinit/provisionTools.sh actual-forest-nc`;
        my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;

        Log(INFO,"Preparing forest for extended functional level DS_BEHAVIOR_VERSION $ms_ds_behavior_version (post)");
        chomp($ldapadminname);
        chomp($netbios_name);
        chomp($forest_nc);

        open FILE, ">$post_forest_func_ldif" or die ("failed to open the file $post_forest_func_ldif");
        #dn: cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
        print FILE "dn: cn=Partitions,cn=Configuration,$forest_nc\n";
        print FILE "changetype: modify\n";
        print FILE "replace: msDS-Behavior-Version\n";
        print FILE "msDS-Behavior-Version: $ms_ds_behavior_version\n";
        print FILE "\n";
        #dn: cn=NONSCHEM,cn=Partitions,cn=Configuration,dc=nonschem,dc=edu
        print FILE "dn: cn=$netbios_name,cn=Partitions,cn=Configuration,$forest_nc\n";
        print FILE "changetype: modify\n";
        print FILE "replace: msDS-Behavior-Version\n";
        print FILE "msDS-Behavior-Version: $ms_ds_behavior_version\n";
        print FILE "-\n";
        print FILE "replace: ntMixeddomain\n";
        print FILE "ntMixeddomain: 0\n";
        close FILE;

        eval{
                my @output = ();
                if(uc($type_of_install) eq "INSTALL"){
                        Log(INFO, "Executing...(post forest) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $post_forest_func_ldif");
                        @output = `/usr/bin/ldapmodify -D $ldapadminname -w  '$ENV{'ADM_PASSWD'}' -Z -f $post_forest_func_ldif`;
                }else{
                        Log(INFO, "Executing...(post forest) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $post_forest_func_ldif");
                        @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $post_forest_func_ldif`;
                }

                my $error = $? >> 8;
                if ($error) {
                        die("Could not modify the objects: $error: [@output]");
                }
        };
        if ($@) { Log(WARN,$@) }

        unlink($post_forest_func_ldif);
}

sub dc_schema_version_update(){
        dc_schema_version_update_2012();
}

sub dc_schema_version_update_2012(){
	dc_schema_version_update_params(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
}

sub dc_schema_version_update_2016(){
        dc_schema_version_update_params(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2016);
}

sub dc_schema_version_update_2019(){
        dc_schema_version_update_params(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2019);
}

sub dc_schema_version_update_params($$){
        my $ms_ds_behavior_version = shift;
        my $ad_schema_level = shift;

        my $mapped_server_dn_name =`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Server DN Name"`;
        my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
        chomp($ldapadminname);
        chomp($mapped_server_dn_name);
        my $dcupdate_ldif = mktemp("/tmp/dcupdate_ldif.XXXXXXXXXX");

        my $dcver = "";
        if(uc($type_of_install) eq "INSTALL"){
                $dcver = `/usr/bin/ldapsearch -D $ldapadminname -w '$ENV{'ADM_PASSWD'}' -s base -b "cn=NTDS Settings,$mapped_server_dn_name" msDS-Behavior-Version -o ldif-wrap=no -Z -LLL | grep msDS-Behavior-Version | awk -F": " '{print \$2}'`;
        }else{
                $dcver = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -Y EXTERNAL -Z -s base -b "cn=NTDS Settings,$mapped_server_dn_name" msDS-Behavior-Version -o ldif-wrap=no -Z -LLL | grep msDS-Behavior-Version | awk -F": " '{print \$2}'`;
        }

        if($dcver < $ms_ds_behavior_version){
                open FILE, ">$dcupdate_ldif" or die ("failed to open the file $dcupdate_ldif");

                #dn: cn=NTDS Settings,cn=TEST1,cn=Servers,cn=Default-First-Site-Name,cn=Sites,cn=Configuration,dc=nonschem,dc=edu
                print FILE "dn: cn=NTDS Settings,$mapped_server_dn_name\n";
                print FILE "changetype: modify\n";
                print FILE "replace: msDS-Behavior-Version\n";
                print FILE "msDS-Behavior-Version: $ms_ds_behavior_version\n";
                print FILE "\n";
                close FILE;

                Log(INFO,"Updating the DC to $ad_schema_level schema level and DS_BEHAVIOR_VERSION $ms_ds_behavior_version");
                eval{
                        my @output = ();


                        if(uc($type_of_install) eq "INSTALL"){
                                Log(INFO, "Executing...(post) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $dcupdate_ldif");
				#To avoid space in admin context, $ldapadminname puted inside quotation.
                                @output = `/usr/bin/ldapmodify -D '$ldapadminname' -w '$ENV{'ADM_PASSWD'}' -Z -f $dcupdate_ldif`;
                        }else{
                                my $type = registry::getReg("INSTALL_TYPE");
                                if($type eq "frd") {
                                        Log(INFO, "Executing...(post) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $dcupdate_ldif");
                                        @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $dcupdate_ldif`;
                                }
				if($type eq "child") {
					#Running kinit to get the ticket.
					Log(MESG, ">>>Acquiring the domain ticket");

					my $domain_name = `/bin/dnsdomainname` || croak ("/bin/dnsdomainname failed\n");
					chomp($domain_name);
					my $admin_principal = "Administrator\@".uc($domain_name);

					my $rc = system("/opt/novell/xad/bin/kinit -E $admin_principal");

					if ($rc != 0) {
						Log ("Could not get the ticket");
					}
					Log(MESG, "   Acquired the ticket for $admin_principal");

					Log(INFO, "Executing...(post) SASL_PATH=/opt/novell/xad/lib64/sasl2 /usr/bin/ldapmodify -c -Q -Y GSSAPI -Z -f $dcupdate_ldif");
					@output = `SASL_PATH=/opt/novell/xad/lib64/sasl2 /usr/bin/ldapmodify -c -Q -Y GSSAPI -Z -f $dcupdate_ldif`;
				}

                        }
                        my $error = $? >> 8;
                        if ($error) {
                                die("Could not modify the objects: $error: [@output]");
                        }
                };

                if ($@) { Log(ERR,$@) }

                unlink($dcupdate_ldif);
        }
}

sub check_services(){
	my @services=qw(ndsd);
	my $m_status = 0;

	foreach my $service (@services){
		`rc$service status >/dev/null 2>&1`;
		if($?>0){
			Log(WARN, "$service is not up and running, bringing up the service.");
			`rc$service start >/dev/null 2>&1`;
			if($? > 0){
				Log(WARN, "$service is not coming up, brining it manually.");
				$m_status = 1;
			}else{
				Log(INFO, "$service is up and running.");
			}
		}
	}
	return $m_status;
}

sub create_msdsprincipalname_and_acl(){
	my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	chomp($ldapadminname);
	my $vardir = "/var/opt/novell/xad/ds";
	my $templatedir = "/opt/novell/xad/share/dcinit/templates";
	my $principal_domain_acls_ini = "$templatedir/principal-domain-acls.ini";
	my $principal_domain_acls_ldif = "$vardir/domain/principal-domain-acls.ldif";

	open FILE, ">$principal_domain_acls_ini" or die ("failed to open the file $principal_domain_acls_ini");
	print FILE "[DEFAULTROOTDOMAIN]\n";
	print FILE "CHILD=Ms DS Principal Name\n";
	print FILE "\n";
	print FILE "[Ms DS Principal Name]\n";
	print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
	print FILE "changetype=modify\n";
	print FILE "add=msDS-PrincipalName\n";
	print FILE ";\n";
	print FILE "; Adding the msDS-PrincipalName with value netbios name\n";
	print FILE "; This is specifically used by the NSS AD\n";
	print FILE ";\n";
	print FILE "msDS-PrincipalName=\$REGISTRY=Netbios Name\\\n";
	print FILE "-\n";
	print FILE "add=ACL\n";
	print FILE ";\n";
	print FILE "; Allow public to have compare access to cn attribute\n";
	print FILE ";\n";
	print FILE "; DS_ATTR_COMPARE\n";
	print FILE ";\n";
	print FILE ";ACL=1#subtree#[Public]#cn\n";
	print FILE ";\n";
	print FILE "; Allow authenticated users compare/read access\n";
	print FILE ";\n";
	print FILE "; DS_ATTR_COMPARE | DS_ATTR_READ\n";
	print FILE ";\n";
	print FILE "ACL=3#subtree#[Root]#msDS-PrincipalName\n";
	close FILE;
	Log(INFO,"Generating principal-domain-acls.ldif");

	if(uc($type_of_install) eq "MIGRATE"){
		`/opt/novell/xad/share/dcinit/genLdif.pl $principal_domain_acls_ini --offline --defaultrootdomain > $principal_domain_acls_ldif`;
	}else{
		stage::gen_ldif("$principal_domain_acls_ini", '>', "$principal_domain_acls_ldif", "--offline", "--defaultrootdomain");
	}

	eval{
		my @output =();
		my $counter = 0;
		while($counter<5){
			if ($counter > 0) {
				Log(INFO, "Retrying to add msDS-PrincipalName attribute on domain, retry count:$counter");
				sleep (3);
			}

			if(uc($type_of_install) eq "INSTALL"){
				Log(INFO, "Executing...(principal) /usr/bin/ldapmodify -D $ldapadminname -w  ***** -Z -f $principal_domain_acls_ldif");
				@output = `/usr/bin/ldapmodify -D $ldapadminname -w  '${'ADM_PASSWD'}' -Z -f $principal_domain_acls_ldif`;
			}else{
				Log(INFO, "Executing...(principal) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $principal_domain_acls_ldif");
				@output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $principal_domain_acls_ldif`;
			}

			my $error = $? >> 8;
			if ($error) {
				Log(WARN, "Failed to add msDS-PrincipalName to the objects: $error: [@output]");
			}else{
				Log(INFO, "Successfully add msDS-PrincipalName to the objects.");
				last;
			}
			$counter++;
		}

		if($counter == 5){
			Log(INFO, "*"x80);
			Log(INFO, "Failed to add msDS-PrincipalName attribute to domain");
			Log(INFO, "Execute the command...LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $principal_domain_acls_ldif after the upgrade/migration.");
			Log(INFO, "*"x80);
		}
	};

	if ($@) { Log(WARN,$@) }
}

sub update_functional_levels_params($$){
	my $ms_ds_behavior_version = shift;
	my $ad_schema_level = shift;

        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        #Update domain functional level
        #Are all the DC's at same functional level?
        #check on domain dsfw.com
        my $domainAt = is_domain_at_desired_level($ms_ds_behavior_version, $ad_schema_level);

        if($is_replica eq "FALSE"){
                if ($domainAt){
                        #Domain updated with the latest schema version
                        #if and only all dc's are the the same schema level
                        post_schema_extension_params($ms_ds_behavior_version, $ad_schema_level);
                }

        }else{
                Log(WARN, "This server is not domain's PDC. So not updating the Domain Functional Level.");
                return;
        }

        #Update forest functional level
        #check all the DC's for same functional level across the forest
        #check for all domains at same functional level
        my $forestAt = is_forest_at_desired_level($ms_ds_behavior_version, $ad_schema_level);

        if($type eq "frd" && $is_replica eq "FALSE"){
                if ($forestAt){
                        #Update the forest to the latest schema level
                        post_forest_functional_level_params($ms_ds_behavior_version, $ad_schema_level);
                }
        }else{
                Log(WARN, "This server is not PDC. So not updating the Forest Functional Level.");
        }
}

sub update_functional_levels(){
        update_functional_levels_2012();
}

sub update_functional_levels_2012(){
        update_functional_levels_params(MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
}

sub update_functional_levels_2016(){
        update_functional_levels_params(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2016);
}

sub update_functional_levels_2019(){
        update_functional_levels_params(MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2019);
}

#Subroutine will hep you to create the Password Settings container in a DSfW
#environment.
sub create_password_settings_container(){
        my $mapped_nc = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC"`;
        my $psc_creation_ldif = mktemp("/tmp/passwd_cont_ldif.XXXXXXXXXX");
	my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
	chomp($ldapadminname);

        Log(INFO,"Creating the Password Settings Container");
        chomp($mapped_nc);

        #check if the container is already created.
	my $command = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch".
		" -Q -Y EXTERNAL -s base -b \"cn=System,$mapped_nc\" -s sub \"(cn=Password Settings Container*)\" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i \"Password Settings Container\"";

#        my $output = `/usr/bin/ldapsearch -D $ldapadminname -w ${'ADM_PASSWD'} -b "cn=System,$mapped_nc" -s sub "(cn=Password Settings Container*)" -Z -o ldif-wrap=no -LLL dn 2>&1 | grep -i "Password Settings Container"`;
	my $output = `$command`;
        my $e = $? >> 8;
        if($e==0){
                Log(INFO, "Executing...(Password Settings Container) container already exists.");
                return 0;
        }

        open FILE, ">$psc_creation_ldif" or die ("failed to open the file $psc_creation_ldif");
        print FILE "dn: CN=Password Settings Container,CN=System,$mapped_nc\n";
        print FILE "changetype: add\n";
        print FILE "cn: Password Settings Container\n";
        print FILE "objectClass: top\n";
        print FILE "objectClass: msDS-PasswordSettingsContainer\n";
        print FILE "instanceType: 4\n";
        print FILE "showInAdvancedViewOnly: TRUE\n";
        print FILE "name: Password Settings Container\n";
        print FILE "systemFlags: -1946157056\n";
        print FILE "objectCategory: CN=ms-DS-Password-Settings-Container,CN=Schema,CN=Configuration,$mapped_nc\n";
        print FILE "\n";
        close FILE;

        eval{
		my $counter = 0;
		while($counter<5){
			if($counter>0){
				Log(INFO, "Retrying to create the Password Settings Container. retry count:$counter");
				sleep (3);
			}

			Log(INFO, "Executing...(Password Settings Container) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $psc_creation_ldif");
			my @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $psc_creation_ldif 2>&1`;
			my $error = $? >> 8;
			if ($error) {
				Log(WARN, "Failed to create Password Settings Container: $error: [@output]");
			}else{
				Log(INFO, "Successfully created the Password Settings Container.");
				last;
			}
			$counter++;
		}

		if($counter == 5){
			Log(INFO, "*"x80);
			Log(INFO, "Failed to create Password Settings Container.");
			Log(INFO, "Execute the command...(Password Settings Container) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $psc_creation_ldif after the upgrade/migration.");
			Log(INFO, "*"x80);
		}else{
			unlink($psc_creation_ldif);
		}
        };

        if ($@) { Log(WARN,$@); }

}

sub extend_schema_params($){
	my $ad_schema_level = shift;
        my $tAdmin="";
	my $vardir = "/var/opt/novell/xad/ds";
	my $schemaFile="/etc/opt/novell/xad/msds.sch";
	my $templatedir = "/opt/novell/xad/share/dcinit/templates";
	my $schemaTool="/opt/novell/eDirectory/bin/ndssch";
	my $schemaLogFile="/var/opt/novell/eDirectory/log/schema.log";

        Log(INFO,"Extending schema to $ad_schema_level level");
        $treeName = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
        if(not defined($treeName) or $treeName eq ""){
                Log(WARN, "failed to fetch the tree name.");
                return 1;
        }
        chomp($treeName);

        if (defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "") {
                $tAdmin = $ENV{LDAPEXISTINGADMINNAME};
        }

        #non-name mapped tree administrator
        $tAdmin = registry::getReg("LDAPDOMAINADMINNAME");
        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        if ($type eq "frd" and $is_replica eq "FALSE") {
                if (not registry::getReg("MAPPED")) {
                        #name mapped tree administrator
                        $tAdmin = registry::getReg("LDAPDOMAINADMIN");
                }
        }
        chomp($tAdmin);
        $tAdmin=~s/,/\./g;

	#Generate the schema.ldif and schema-bl.ldif
	Log(INFO,"Generating schema.ldif");
	`/opt/novell/xad/share/dcinit/genLdif.pl $templatedir/schema.ini --offline --schema > $vardir/schema/schema.ldif`;

	if (util::chkfor_reg("$vardir/schema/schema.ldif") == 0) {
		die("Not all the REGISTRY values are replaced in $vardir/schema/schema.ldif $?")
	};

        Log(INFO,"Generating schema-bl.ldif");
	`/opt/novell/xad/share/dcinit/genLdif.pl $templatedir/schema-bl.ini --schema > $vardir/schema/schema-bl.ldif`;

        if (util::chkfor_reg("$vardir/schema/schema-bl.ldif") == 0) {
                die("Not all the REGISTRY values are replaced in $vardir/schema/schema-bl.ldif")
        };

	#Code to generate the msds.sch file
	`/opt/novell/xad/share/dcinit/aggregateSchema.pl $vardir/schema/schema.ldif --ndsschema > $schemaFile`;

	if (util::chkfor_reg("$schemaFile") == 0) { die("Not all the REGISTRY values are replaced in $schemaFile")};

        my @status = ();
        if(not defined($ENV{ADM_PASSWD}) or $ENV{ADM_PASSWD} eq ""){
		Log(INFO, "*********************************************************************");
		Log(INFO, "Please enter the password for $tAdmin");
		Log(INFO, "*********************************************************************");
                @status = `$schemaTool $tAdmin -F $schemaLogFile -t $treeName $schemaFile 2>&1`;
        } else {
                @status = `$schemaTool $tAdmin -F $schemaLogFile -t $treeName $schemaFile -p $ENV{ADM_PASSWD} 2>&1`;
        }
        my $error = $? >> 8;
        if ($error){
                chomp(@status);
                Log(ERR,"Failed to extend schema to $ad_schema_level level : [@status]");
                return 1;
        }

        Log(INFO,"Schema extension to $ad_schema_level level completed");
}

sub update_server_to_desired_level($$$){
        $type_of_install = shift;
	$ms_ds_behavior_version = shift;
	$ad_schema_level = shift;
        #Check for ndsd and novell-named service
        my $stat  = check_services();

        if($stat>0){
                Log(INFO, "Either ndsd or novell-named service is not up and running.\n
                        please bring up these services manually and then re-run the script.");
                return 1;
        }
	# Call to extend schema seems redundant as import of ldifs in later code handles it.
	# Can we remove this call ?? 
        if(uc($type_of_install) eq "MIGRATE"){
                extend_schema_params($ad_schema_level);
        }

        Log(INFO, "Please wait, server is getting updated with $ad_schema_level schema and $ms_ds_behavior_version functional level.");
        my $type = registry::getReg("INSTALL_TYPE");
        my $is_replica = registry::getReg("IS_REPLICA");

        Log(INFO,"Restarting ndsd for upgrade");

        `systemctl stop ndsd`;
        `systemctl start ndsd`;

        # Only FRD
        if($type eq "frd" and $is_replica eq "FALSE"){
                #my $ldapadminname = registry::getReg("LDAPADMINNAME"); #"cn=Administrator,cn=Users,dc=dsfwx,dc=edu";
                my $ldapadminname = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
                my $NATIVE_REPL = registry::getReg("NATIVE_REPL");
                my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
                my $schema_ldif="$LIVE_DSSTATEDIR/schema/schema.ldif";
                my $schema_bl_ldif="$LIVE_DSSTATEDIR/schema/schema-bl.ldif";
                chomp($ldapadminname);
                my @output=();
                my $error=0;

                Log(INFO, "Executing...(Importing schema.ldif) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -a -c -Z -f $schema_ldif");
                @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -a -c -Z -f $schema_ldif >/dev/null 2>&1`;
                $error = $? >> 8;
                if ($error != $LDAP_ERR_ALREADY_EXISTS and $error != $LDAP_ERR_DUP and $error != 0) {
                        die("Failed to import schema.ldif : $error: [@output]");
                }

                Log(INFO, "Executing...(Importing schema-bl.ldif) LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -a -c -Z -f $schema_bl_ldif");
                @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -a -c -Z -f $schema_bl_ldif >/dev/null 2>&1`;
                $error = $? >> 8;
                if ($error != $LDAP_ERR_ALREADY_EXISTS and $error != $LDAP_ERR_DUP and $error != 0) {
                        die("Failed to import schema-bl.ldif : $error: [@output]");
                }
        }

        #sleep for 10 seconds
        sleep(10);

        #Update the schema version to the level on DC.
        dc_schema_version_update_params($ms_ds_behavior_version, $ad_schema_level);

        #update domain and forest functional level
        update_functional_levels_params($ms_ds_behavior_version, $ad_schema_level);

        Log(INFO, "System is updated to $ad_schema_level schema level, please check the log file for more details.");
}

sub update_server_to2012($){
        $type_of_install = shift;

        update_server_to_desired_level($type_of_install, MS_DS_BEHAVIOR_VERSION_2012, AD_SCHEMA_LEVEL_2012);
        #add msDS-PrincipalName on the root domain controller
        my $domainAt = is_domain_at_level2012();
        if($domainAt){
                create_msdsprincipalname_and_acl();
        }else{
                Log(WARN, "ms-DS-PrincipalName is not set as not all domain servers are at 2012 level.");
        }

        #update AES capabilities on the server
        update_domain_to_aes();

        #update the crontab file
        Log(INFO, "Adding the entry for fine grained password policy to crontab.");
        my $rc = system("/opt/novell/xad/share/dcinit/provisionTools.sh  update-crontab-FGP2NMAS");
        if ($rc != 0) {
                Log(WARN, "Could not update the crontab for Fine grained password policy.");
        }

        #Creating PasswordSettings Container for Fine Grained Policies
        create_password_settings_container();

        Log(INFO, "systemd changes updated successfully...");
}

sub update_server_to2016($){
        $type_of_install = shift;
        update_server_to_desired_level($type_of_install, MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2016);
}

sub update_server_to2019($){
        $type_of_install = shift;
        update_server_to_desired_level($type_of_install, MS_DS_BEHAVIOR_VERSION_2016, AD_SCHEMA_LEVEL_2019);
}

#For setpassword
use constant {
	SETP_MIXED => 0,
	SETP_NOT_MIXED => 1,
};

sub is_fqdn_mixed($) 
{
	my $mystring = shift;
	my $uc_mystring = uc $mystring;
	my $lc_mystring = lc $mystring;

	if($mystring eq $lc_mystring or $mystring eq $uc_mystring)
	{
		return SETP_NOT_MIXED;
	}

	return SETP_MIXED;
}

sub get_user_name()
{
	my $server = `/bin/hostname`;
	chomp($server);
	my $user_detail = "$server\$";

	return $user_detail;
}

sub get_principal_names()
{
	my $domain = `/bin/hostname -d`;
	chomp($domain);
	my $fqdn = `/bin/hostname --fqdn`;
	chomp($fqdn);

	my $uc_domain = uc $domain;
	my $uc_fqdn = uc $fqdn;

	my $lc_domain = lc $domain;
	my $lc_fqdn = lc $fqdn;

	my $principal_names_pre = "$lc_fqdn/$lc_domain,$uc_fqdn/$lc_domain,$uc_fqdn/$uc_domain,$lc_fqdn/$uc_domain";

	my $principal_names = "";

	if(is_fqdn_mixed($fqdn) == SETP_MIXED) 
	{
		$principal_names = "$principal_names_pre,$fqdn/$domain";
	}
	else 
	{
		$principal_names = $principal_names_pre;

	}

	return $principal_names;
}

#Use the setpassword command to configure additional
#principal names for the server
sub execute_setpassword_command()
{
	my $principal_names = get_principal_names();
	my $user_detail = get_user_name();

	my $command = "/opt/novell/xad/sbin/setpassword -NDOSf -r -u $user_detail -E $principal_names -k /var/opt/novell/xad/ds/krb5kdc/krb5.keytab";

	Log(INFO, "Executing : $command");
	my @command_output = `$command 2>&1`;
	if(($?>>8) != 0){
		Log("Could not set password for $user_detail");
		Log("error : [@command_output]");
		Log("Please try rerunning");
	}
}

#specific modifications for Windows 10 machines
sub update_for_win10()
{
	execute_setpassword_command();
	my $domain = lc `/bin/hostname -d`;
	chomp($domain);

	#Execute only for pdc
	my $is_replica = registry::getReg("IS_REPLICA");

	if ($is_replica eq "FALSE") {
		Log(INFO, "Executing : /usr/bin/ln -s $domain/Policies /var/opt/novell/xad/sysvol/sysvol/Policies");
		my @command_output = `/usr/bin/ln -s $domain/Policies /var/opt/novell/xad/sysvol/sysvol/Policies`;
		if(($?>>8) != 0){
			Log(WARN, "/usr/bin/ln -s $domain/Policies /var/opt/novell/xad/sysvol/sysvol/Policies failed with error [@command_output]");
		}
	}
}

#Workaround for smbLoris vulnerability
sub update_max_smbd_param {

        my $smb_file = "/etc/samba/smb.conf";
        my $section = "";
        my $val = "";

        my $cfg = Config::IniFiles->new (-file=> $smb_file, -nocase=>1);
        if (!defined $cfg) {
                Log("Could not open the file $smb_file");
                return -1;
        }

        Log ("Updating in $smb_file for max smbd processes");

        #idmap config addition
        my $var_value = "max smbd processes";
        $val = $cfg->val('global', $var_value);
        if (!defined $val) {
                $cfg->newval ('global', $var_value, '1000');
        }else {
                $cfg->setval ('global', $var_value, '1000');
        }

        $cfg->RewriteConfig();
        `sed -i 's/^[a-zA-Z]/\t&/g' $smb_file`;
        `sed -i 's/=/ = /g' $smb_file`;

}

#Upgrade ACL's for Software Installation through GPO
sub software_installation_acl(){
	my $acl_ldif_file=mktemp("/tmp/acl_for_root.XXXXXXXXXX");
	my $root_domain=registry::getReg("Root Domain");
	$root_domain =~ s/\./,dc=/g;
	$root_domain = "dc=$root_domain";

	eval{
		#do a search for 'appSchemaVersion attribute on the root object
		Log(INFO, "Executing : LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Y EXTERNAL -b \"$root_domain\" -s base -Z -LLL -o ldif-wrap=no | grep -i \"ACL.*appSchemaVersion\"");
		my @output=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Y EXTERNAL -b "$root_domain" -s base -Z -LLL -o ldif-wrap=no | grep -i "ACL.*appSchemaVersion"`;

		#if 'appSchemaVersion' attribute is not present on the root object
		# then we need to add all additional ACL's
		if(not grep(/ACL.*appSchemaVersion/, @output)){
			#collect the ACL's
			my @all_acls = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Y EXTERNAL -b \"$root_domain\" -s base -Z -LLL -o ldif-wrap=no ACL`;
			my $first_line_written = 0;
			open(LDIF_FD, ">$acl_ldif_file") or die("could not open file $acl_ldif_file");

			#write all acl's into a LDIF file
			foreach my $acl (@all_acls){
				if(not $acl =~ /^$/){
					print LDIF_FD "$acl";
				}

				if($first_line_written == 0){
					print LDIF_FD "changetype: modify\n";
					print LDIF_FD "replace: ACL\n";
					$first_line_written = 1;
				}
			}

			#set of additional ACL's needs to be added
			print LDIF_FD "#additional ACL's to enable Software Installation through GPO\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#appSchemaVersion\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#msiScriptName\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#canUpgradeScript\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#packageFlags\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#packageType\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#msiScriptPath\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#lastUpdateSequence\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#localeID\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#machineArchitecture\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#packageName\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#versionNumberHi\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#versionNumberLo\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#installUiLevel\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#vendor\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#url\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#revision\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#productCode\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#categories\n";
			print LDIF_FD "ACL: 3#subtree#[Root]#msiFileList\n";

			close(LDIF_FD);

			Log(INFO, "Executing : LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $acl_ldif_file");
			my @result = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Q -Y EXTERNAL -Z -f $acl_ldif_file`;
			if(($?>>8) > 0){
				Log(WARN, "ldapmodify command failed with error [@result]");
			}else{
				Log(INFO, "successfully added the additional ACL's on root object");
			}

			unlink($acl_ldif_file);
		}
	};

        if ($@) {
                Log(WARN, $@);
        }
}

sub apply_2018sp2_acls (){

        my $upgrade_acl_ini = "/tmp/nds-domain-upgrade-acls.ini";
        my $LIVE_DSSTATEDIR = registry::getReg("LIVE_DSSTATEDIR");
        my $ldapmodify = registry::getReg("LDAPMODIFY");
        my $mapped_domain_nc = registry::getReg("Mapped Domain NC");

        if (-e $upgrade_acl_ini) {
                Log("Skipping the updation of 2018sp2 acls as nds-domain-upgrade-acls.ini is already present");
                return;
        }
        # Logic : Check for pre-existing values and if present OR the existing values with the new ones
        Log("About to upgrade ACLs as a part of oes2018sp2 upgrade");
        open FILE, ">$upgrade_acl_ini" or die ("failed to open the file $upgrade_acl_ini");
        print FILE "[DEFAULTROOTDOMAIN]\n";
        print FILE "CHILD=Domain NC ACLs and Policy\n";
        print FILE "[Domain NC ACLs and Policy]\n";
        print FILE "RDN-Of-Object=DEFAULTROOTDOMAIN\n";
        print FILE "changetype=modify\n";
        print FILE "add=ACL\n";

	print FILE "ACL=3#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#groupMember\n";
	print FILE "ACL=3#subtree#[Public]#msDS-PhoneticDisplayName\n";
	print FILE "ACL=7#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#msTSManagingLS\n";
	print FILE "ACL=7#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#msTSLicenseVersion\n";
	print FILE "ACL=7#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#msTSLicenseVersion2\n";
	print FILE "ACL=7#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#msTSLicenseVersion3\n";
	print FILE "ACL=7#subtree#CN=Domain Computers,CN=users,$mapped_domain_nc#msTSExpireDate\n";

        close(FILE);

       `/opt/novell/xad/share/dcinit/ComputeEffectiveAcls.pl -n $mapped_domain_nc`;
       `genLdif.pl $upgrade_acl_ini --defaultrootdomain > $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif`;

        Log("Executing ... LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif");
        my @output = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf $ldapmodify -c -Q -Y EXTERNAL -f $LIVE_DSSTATEDIR/domain/nds-domain-upgrade-acls.ldif 2>&1`;
        my $error = $? >> 8;
        if ($error and $error != $LDAP_ERR_DUP) {
                chomp(@output);
		`rm -f $upgrade_acl_ini`;
                Log("Failed to update ACLs on the domain object [@output]");
        }
        Log("Updated the  ACLs on the domain object ");
}



1;
