#!/usr/bin/perl -I/opt/novell/xad/lib/perl -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib64/perl/XAD
################################################################################
# Domain Services for Windows - Commonly used subroutines for WINS
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2013
#       Novell, Inc.  All rights reserved.
################################################################################

package wins_common;
use strict;
use warnings;
use Term::ReadKey;
use Logger qw(:DEFAULT);
use IO::File;
use File::Temp;
use Install::check;

#Merged from configure_dsfw_wins.pl
use Config::IniFiles;
use File::Copy;
use XAD::dns;

#Merged from checkdupws.pl
use File::Basename;
use XAD::registry;

#******************************************************************************
# Global variable's for WINS
#******************************************************************************

my $wins_srv = "_netbios._tcp.wins._msdcs";
my $wins_srv_result = "$wins_srv IN SRV  0 100 389";
my $wins_edir_record = '\\#netbios_#tcp_wins_#msdcs';
my $wins_configure_log = "wins_configure.log";
my $wins_configure_dir = "/var/opt/novell/xad/log";
my $samba_conf_file = "/etc/samba/smb.conf";
my $wins_dat_file = "/var/lib/samba/wins.dat";
my $wins_tdb_file = "/var/lib/samba/wins.tdb";
my $wins_dir = "/var/opt/novell/xad/wins/";
my $global_zone_context = " ";
my $global_frdDNS_provider = " ";
my $global_cmd = "";
my @global_output = ();
my $operation = "";

#Merged fron configure_dsfw_wins.pl
my $fh;
my $masks;
my $global_password = "";
my $global_tree_administrator = "";
my $global_srv_verify_retries = 0;

#TIME
my $localtime=localtime;
my ($day,$month,$date,$time,$year)=split(/ /,$localtime);
my $stamp=$year.$month.$date."-".$time;

#Configuration status determining variables
our ($is_configured, $is_configured_upgrade, $only_configure, $only_postcheck, $is_debug) =(0, 0, 0, 0, 0);



#******************************************************************************
# Constant's for WINS
#******************************************************************************

#Operational options available
use constant {
	      CONFIGURE   => 1,
	      UNCONFIGURE => 2,
	      ADD => 3,
	      DEL => 4,
};

#Script return values
use constant{
	XAD_WINS_SUCCESS => 0,
	XAD_WINS_PRESENT_OUTSIDE =>1,
	XAD_WINS_PRESENT  => 2,
	XAD_HOSTNAME_FAILURE => 6,
	XAD_UNKNOWN_ERROR => -1,
	XAD_PDC_REJECT => 5,
	LDAP_INSUFFICIENT_ACCESS => 12800,
	XAD_WINS_NOT_CONFIGURED => 7,
	XAD_DNS_UNREACHABLE => 8,
	XAD_FAILED_TO_GET_DC => 9,
	XAD_WINS_FILES_NOT_CREATED => 10,
	XAD_WINS_SRV_RECORD_NOT_FOUND => 11,
};

#Constants script returns
use constant {
	XAD_CHECK_SUCCESS => 0,
	XAD_ALREADY_RUNNING => 1,
	#For operations that need to run only once
	XAD_ALREADY_DONE => 2,
	XAD_SCHEDULE_FAIL => 6,
};

#Constants for determining calling environment
use constant {
	#Parameter called if execution by terminal
	XAD_CONSOLE => 1,
	#Parameter called if execution by provisioning
	#Its the default behavior
	XAD_PROVISIONING => 2,
};

#Environment in which the module is called
our $environment = XAD_PROVISIONING;

#******************************************************************************
# Base and util Subroutine's for WINS
#******************************************************************************

# Wrapper function for executing commands.
# Stores command name($global_cmd),command output or error messages(@global_output) in global variables()
# Returns the return value of the first command in the piped sequence
# Desgined for ldap commands that may or may not be followed by filters.
# However complex commands that have '||' operator will fail
sub exec_ldap_cmd($) {
	$global_cmd = shift;
	my $filter_cmd = "";
	my $main_cmd = "";
	my $temp_output = "";

	my $maincmd_tempfile = mktemp("/tmp/command.XXXXXXXX");

	my $first_pipe_index = index($global_cmd, "|");

	#ldap command is not piped
	if($first_pipe_index == -1) {
		$main_cmd = $global_cmd;
	}
	#ldap command is piped
	else {
		my $main_cmd_end = $first_pipe_index - 1;
		my $filter_cmd_beg = $first_pipe_index + 1;

		#Command before first '|', ldap command that is
		$main_cmd = substr($global_cmd, 0, $main_cmd_end);
		#Command after first '|', filter command that is
		$filter_cmd = substr($global_cmd, $filter_cmd_beg);
	}

	`$main_cmd >$maincmd_tempfile 2>&1`;
	my $ret = $?;

	$temp_output = `cat $maincmd_tempfile`;
	if ($ret == 0) {
		if ($main_cmd ne $global_cmd) {
			#piped command
			$temp_output = `cat $maincmd_tempfile | $filter_cmd`;
		}
	}

	@global_output = split "\n", $temp_output;

	unlink($maincmd_tempfile);

	return $ret;
}

sub Log_Console {
	my $string = shift;

	if ($is_debug == 0 and $environment == XAD_CONSOLE) {
		Logger::set_mask(LOG_CONSOLE);
		Log($string);
		Logger::unset_mask(LOG_CONSOLE);
		return;
	}

	#Case where environment is provisioning
	#Or where debug is enabled for Console execution
	Log($string);
}



sub get_domain() {
	my $domain_name = `/bin/hostname -d`;
	chomp($domain_name);

	if ($domain_name eq "") {
		Log_Console("Failed to get domain name");
		Log(ERR, "Command /bin/hostname -d failed");
	}

	return $domain_name;
}

sub get_host() {
	my $domain_host = `/bin/hostname`;
	chomp($domain_host);

	if ($domain_host eq "") {
		Log_Console("Failed to get hostname");
		Log(ERR, "Command /bin/hostname failed");
	}

	return $domain_host;
}

sub getTreeAdministrator() {
	my $tree_administrator = "";

	if (defined $ENV{LDAPEXISTINGADMINNAME} and $ENV{LDAPEXISTINGADMINNAME} ne "") {
		#Get from environment variable
		$tree_administrator = $ENV{LDAPEXISTINGADMINNAME};
		return $tree_administrator;
	}

	#non-name mapped tree administrator
	$tree_administrator = registry::getReg("LDAPDOMAINADMINNAME");
	my $type = registry::getReg("INSTALL_TYPE");
	my $is_replica = registry::getReg("IS_REPLICA");

	if ($type eq "frd" and $is_replica eq "FALSE") {
		if (not registry::getReg("MAPPED")) {
			#name mapped tree administrator
			$tree_administrator = registry::getReg("LDAPDOMAINADMIN");
		}
	}

	Log(INFO, "From registry - Returning Tree Administrator $tree_administrator");
	return $tree_administrator;
}

sub getPassword() {
	Log(INFO, "Retrieving password for Administrator");
	my $password;
	my $export_variable;
	my $tree_administrator;

	my $type = registry::getReg("INSTALL_TYPE");
	my $is_replica = registry::getReg("IS_REPLICA");

	$password = $ENV{NDSEXISTINGADMINPASSWD};
	$export_variable = "NDSEXISTINGADMINPASSWD";

	if ($type eq "frd" and $is_replica eq "FALSE") {
		if (not registry::getReg("MAPPED")) {
			$password = $ENV{ADM_PASSWD};
			$export_variable = "ADM_PASSWD";
		}
	}

	if (defined $password and $password ne "") {
		#Password is exported as an environment variable
		Log(INFO, "Returning password from the environment");
		return $password;
	}

	if ($global_password ne "") {
		#It is already taken as input from user
		Log(INFO, "Returning previously taken password");
		return $global_password;
	}

	Log(INFO, "Taking password input from user");

	$tree_administrator = getTreeAdministrator();
	print STDERR "Enter password for $tree_administrator :";
	ReadMode('noecho');
	$global_password = ReadLine(0);
	chomp($global_password);
	print STDERR "\n";
	ReadMode('normal');

	return $global_password;
}

sub Log_Array($) {
	my @array = @_;
	my $line  = "";

	foreach $line (@array) {
		chomp($line);
		Log(INFO, $line);
	}
}

#Getting the IP address of the server where different zones' objects are located
#Currently this is the pdc of the "root domain" of DSfW forest
sub get_frdDNS_provider() {
	my $frdDNS_provider;
	my $dig_output;
	my $install_type = registry::getReg("INSTALL_TYPE");
	my $dns_server_ip =  registry::getReg("DNS_SERVER_IP");
	my $is_replica = registry::getReg("IS_REPLICA");
	my $ret = 0;

	if ($global_frdDNS_provider ne " ") {
		Log(INFO, "Returning Cached frd provider - $global_frdDNS_provider");
		return $global_frdDNS_provider;
	}

	if($install_type eq "frd" and $is_replica eq "FALSE") {
		$frdDNS_provider = $dns_server_ip;
	}
	else {
		my $forest_root = registry::getReg("FORESTROOT");
		$dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$forest_root  +short`;
		$ret = $?;
		chomp($dig_output);
		if ($ret || $dig_output eq "") {
			my $error = $ret >> 8;
			die ("Failed to get the domain provider for domain [$forest_root] [$error]");
		}
		$frdDNS_provider = `echo $dig_output | cut -d " " -f4 | sed 's/\.\$//'`;
		chomp($frdDNS_provider);
	}

	$global_frdDNS_provider = $frdDNS_provider;
	Log(INFO, "Returning Calculated frd provider - $frdDNS_provider");
	return $frdDNS_provider;
}

#FQDN where forward zone of the domain is located
sub get_zone_context() {
	Log(INFO, "Fetching the Zone Context");
	my $tree_administrator = "";
	my $password = "";

	if ($global_zone_context ne " ") {
		Log(INFO, "Returning Cached Zone context - $global_zone_context");
		return $global_zone_context;
	}

	my $server_object = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSSERVER`;
	chomp($server_object);

	my $dns_server_ip = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNS_SERVER_IP`;
	chomp($dns_server_ip);

	$global_tree_administrator = getTreeAdministrator();
	$tree_administrator = $global_tree_administrator;

	my $zone_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
	chomp($zone_name);

	$password = getPassword();

	my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";

	#Read FQDN of the domain zone from the server object,
	#and get its FQDN by removing the entry before first ','
	#TBD Finally change to "-p 636"
	my $ret = exec_ldap_cmd(
		"LDAPTLS_CACERT=$certFile_pem ".
		"/opt/novell/eDirectory/bin/ldapsearch -D '$tree_administrator'".
		" -w '$password' -h '$dns_server_ip' ".
		" -p 389 -b '$server_object' -s base dNIPZoneList -N -LLL -Z ".
		"| sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; '".
		"| grep -i 'dNIPZoneList:'".
		"| grep -iw '$zone_name'".
		"| awk -F 'dNIPZoneList: \+' '{print \$2}'".
		"| perl -pe 's/.*?,//' ");
	if ($ret != 0) {
		Log_Console("Failed to get zone context");
		Log_Array(@global_output);
		Log(ERR, "While executing cmd - $global_cmd");
	}

	$global_zone_context = $global_output[0];
	chomp($global_zone_context);
	Log(INFO, "Zone context - $global_zone_context");

	return $global_zone_context;
}

#For modifying DNS record in parent, Tree Admin credentials are needed
#dns record is used for confirming if the domain has WINS or not
sub create_WINS_SRV_record() {
	Log(INFO, "Creating SRV record for WINS");
	my $domain = get_domain();
	my $hostname = get_host();
	#As a hack even for adc, since we can have WINS configured in server that is not pdc
	my $is_replica = "FALSE";
	my $dns_zone_context = get_zone_context();
	my $ldif_file = "/var/opt/novell/xad/ds/dns/RR/$wins_srv.ldif";
	my $tree_administrator = getTreeAdministrator();
	my $frdDNS_provider = get_frdDNS_provider();
	my $password = getPassword();

	#TODO: Temp code for creating directory, will be removed after Pratap's changes
	`mkdir -p /var/opt/novell/xad/ds/dns/RR`;

	dns::Process_RR("$wins_srv   IN SRV  0 100 389 $hostname.$domain",0,
		"$dns_zone_context","FALSE","add");

	Log(INFO, "Executing...".
		"LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636".
                " -D '$tree_administrator' -w *****".
                " -f '$ldif_file' -x");

	my $ret = exec_ldap_cmd(
		"LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify -H ldaps://$frdDNS_provider:1636".
		" -D '$tree_administrator' -w '$password'".
		" -f '$ldif_file' -x");

	if ($ret != 0) {
			Log_Console("Failed to create WINS SRV record");
			Log_Array(@global_output);
			Log(ERR, "Command - $global_cmd Failed");
	}

	Log(INFO, "Successfully created the SRV record for WINS");
}

#Delete the WINS DNS record
sub remove_WINS_SRV_record() {
	Log(INFO, "Removing existing SRV record for WINS");
	my $domain = get_domain();
	my $hostname = get_host();
	#As a hack even for adc, since we can have WINS configured in server that is not pdc
	my $is_replica = "FALSE";
	my $dns_zone_context = get_zone_context();
	my $ldif_file = "/var/opt/novell/xad/ds/dns/RR/Delete_$wins_srv.ldif";
	my $delete_ldif_file = "/var/opt/novell/xad/ds/dns/RR/unconfigure_$wins_srv";
	my $tree_administrator = getTreeAdministrator();
	my $password = getPassword();
	my $frdDNS_provider = get_frdDNS_provider();

	dns::Process_RR("$wins_srv   IN SRV  0 100 389 $hostname.$domain",0,
		"$dns_zone_context","FALSE","delete");

	#The $ldif_file still has "changetype: add"
	#Create empty or make empty;
	`cat >$delete_ldif_file </dev/null`;
	#Filter empty
	`cat $ldif_file | grep 'dn:' >>$delete_ldif_file`;
	`cat $ldif_file | grep 'changetype:' >>$delete_ldif_file`;
	#change add to delete
	`sed -i 's/changetype: add/changetype: delete/g' $delete_ldif_file`;

	Log(INFO, "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify".
                " -H ldaps://$frdDNS_provider:1636 -D $tree_administrator".
                " -w ***** -f $delete_ldif_file -x");

	my $ret = exec_ldap_cmd("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify".
		" -H 'ldaps://${frdDNS_provider}:1636' -D '$tree_administrator'".
		" -w '$password' -f '$delete_ldif_file' -x");

	if ($ret != 0 ) {
			Log_Console("Failed to remove WINS SRV record");
			Log_Array(@global_output);
			Log(ERR, "Command - $global_cmd Failed");
	}

	Log(INFO, "Successfully removed the SRV record for WINS");
}

sub revert_dns() {
	Log(INFO, "In function revert_dns");
	if ($operation eq CONFIGURE) {
		remove_WINS_SRV_record();
	} else {
		create_WINS_SRV_record();
	}
}

sub update_dns_soa() {
	Log(INFO, "Starting update_dns_soa");

	my $admin_passwd = getPassword();
	my $frdDNS_provider = get_frdDNS_provider();
	my $admin_name = getTreeAdministrator();
	my $dns_zone_context = get_zone_context();
	my $zone_container = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
	chomp($zone_container);
	my $dnsforwardzonefdn = "cn=$zone_container,$dns_zone_context";
	my $_LIB = registry::getReg("_Lib");
#SOA-validation
	my @soa_output=`LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H 'ldaps://${frdDNS_provider}:1636' -x -D '$admin_name' -w '$admin_passwd' -b '$dnsforwardzonefdn' -s base dNIPSOASerial -LLL`;
	if ($? != 0) {
		Log(ERR, "Failed to get the SOA value : [@soa_output]");
	}
	@soa_output=grep(/dNIPSOASerial:/, @soa_output);
	chomp(@soa_output);
	my $soa_serial=`echo -n $soa_output[0] | grep -i "dNIPSOASerial: " | awk -F "dNIPSOASerial: " '{print \$2}'`;
#SOA-validation
	chomp($soa_serial);
	$soa_serial +=10;
	my $dns_soa_serial_ldif = mktemp("/tmp/soaserial.XXXXXXXX");

	open FILE, ">$dns_soa_serial_ldif" or die ("failed to open the file $dns_soa_serial_ldif");
	print FILE "dn: $dnsforwardzonefdn\nchangetype: modify\nreplace: dNIPSOASerial\ndNIPSOASerial: $soa_serial";

	Log(INFO, "Executing... LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify".
                " -H 'ldaps://${frdDNS_provider}:1636' -D '$admin_name'".
                " -w ***** -f '$dns_soa_serial_ldif' -x");

	my $ret = exec_ldap_cmd("LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapmodify".
		" -H 'ldaps://${frdDNS_provider}:1636' -D '$admin_name'".
		" -w '$admin_passwd' -f '$dns_soa_serial_ldif' -x");

	if ($ret != 0) {
		my $failure_cmd = $global_cmd;
		my @failure_array = @global_output;
		#To maintain repeatable state after this failure so that same error happens in retry.
		revert_dns();
		Log_Console("Error while updating zone SOA");
		Log_Array(@failure_array);
		Log(ERR, "Command - $failure_cmd Failed");
	}

	unlink($dns_soa_serial_ldif);
}

sub restart_nmb() {
	Log(INFO, "Restarting nmb service");
	my $nmb_service = "nmb";

	Log(INFO, "Stopping nmb service");
	`/usr/bin/systemctl stop nmb`;

	my $service_nmb_status = $?;
	Log(INFO, "Service nmb stopped with code $service_nmb_status");

	# Reset the LD_LIBRARY_PATH env variable so that following services pick up the
	# system provided kerebos and gssapi libraries.
	my $LD_LIBRARY_PATH = $ENV{'LD_LIBRARY_PATH'};
	$ENV{'LD_LIBRARY_PATH'} = "";

	`/usr/bin/systemctl start nmb`;

	$service_nmb_status = $?;
	Log(INFO, "Service nmb started with code $service_nmb_status");

	$ENV{'LD_LIBRARY_PATH'} = $LD_LIBRARY_PATH;
}

#******************************************************************************
# Subroutine's for configuring WINS on a DSfW server
#******************************************************************************

#Return '0' if WINS entry present
#Return 'non 0' if WINS entry not present
#Does ldapsearch for entry $wins_edir_record in cn=$zone_container,<DNS zone context>
sub search_WINS() {
	Log(INFO, "Searching WINS SRV record in the zone for the domain");
	my $domain_name = get_domain();
	chomp($domain_name);

	my $dns_zone_context = get_zone_context();
	my $certFile_pem = "/etc/opt/novell/certs/SSCert.pem";
	my $ipaddress = registry::getReg("IPADDRESS");
	my $ldapadminname = getTreeAdministrator();
	my $password = getPassword();

	my $zone_container = `/opt/novell/xad/share/dcinit/printConfigKey.pl DNSZONE`;
	chomp($zone_container);

	#Log(INFO, "Executing... LDAPTLS_REQCERT=never ".
	#	"/opt/novell/eDirectory/bin/ldapsearch -h localhost -s sub".
         #       " -b 'cn=$zone_container,$dns_zone_context' dn -N -LLL");

	Log(INFO, "Executing... LDAPTLS_CACERT=$certFile_pem ".
		"/opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' -w $password -s sub".
                " -b 'cn=$zone_container,$dns_zone_context' -Z dn -N -LLL");
	#my $ret = exec_ldap_cmd("LDAPTLS_REQCERT=never ".
	#	"/opt/novell/eDirectory/bin/ldapsearch -h localhost -s sub".
	#	" -b 'cn=$zone_container,$dns_zone_context' dn -N -LLL");
	my $ret = exec_ldap_cmd("LDAPTLS_CACERT=$certFile_pem ".
		"/opt/novell/eDirectory/bin/ldapsearch -D '$ldapadminname' -w $password -s sub".
		" -b 'cn=$zone_container,$dns_zone_context' -Z dn -N -LLL");
	if ($ret != 0) {
		Log_Console("Error while searching DNS zone container");
		Log_Array(@global_output);
		Log(ERR, "While executing cmd - $global_cmd");
	}

	#Output of the command determines whether WINS is configured or not
	Log(INFO, "Executing... LDAPTLS_REQCERT=never ".
		"/opt/novell/eDirectory/bin/ldapsearch -h localhost".
		" -s base -b 'cn=$wins_edir_record,cn=$zone_container,$dns_zone_context' dn".
		" -N -LLL -x 2>/dev/null");

	$ret = exec_ldap_cmd("LDAPTLS_REQCERT=never".
		" /opt/novell/eDirectory/bin/ldapsearch -h localhost".
		" -s base -b 'cn=$wins_edir_record,cn=$zone_container,$dns_zone_context'".
		" dn -N -LLL -x 2>/dev/null");

	return $ret;
}

#Function queries dns SRV records of domain to check if
#WINS is already configured in the domain or not
#Return '0' if WINS entry present
#Return 'non 0' if WINS entry not present
sub query_WINS_using_dns() {
	Log(INFO, "Sending DNS SRV query");
	my $domain_name = `/bin/hostname -d`;
	my $ret = 0;
	chomp($domain_name);

	my $wins_srv_record = $wins_srv.'.'.$domain_name;
	my $wins_cmd_result = `dig -t SRV $wins_srv_record +short`;
	$ret = $?;
	chomp($wins_cmd_result);

	#$ret != 0 for frd
	if ($wins_cmd_result eq "" || $ret != 0) {
		return XAD_DNS_UNREACHABLE;
	}

	return 0;
}

# Log into console if the calling script is the first argument passed(condition)
# Condition can be CONFIGURE or UNCONFIGURE
sub Log_Console_Conditionally {
	my $condition=shift;
	my $string=shift;

	if ($condition == $operation) {
		Log_Console($string);
	} else {
		Log($string);
	}
}

sub display_WINS_server_ip()
{
	my $dig_output;
	my $wins_server_fqdn;
	my $wins_server_ip;
	my $domain_name;

	$domain_name = `hostname -d`;
	chomp($domain_name);

	eval {
		#1.Get WINS server FQDN
		Log(DEBG, "Executing .. /usr/bin/dig -t SRV _netbios._tcp.wins._msdcs.$domain_name  +short");
		$dig_output = `/usr/bin/dig -t SRV _netbios._tcp.wins._msdcs.$domain_name  +short`;
		my $ret = $?;

		chomp($dig_output);
		if ($ret || $dig_output eq "") {
				my $error = $ret >> 8;
				die ("Failed to get the WINS server for domain [$domain_name] [$error]");
		}

		$wins_server_fqdn = `echo $dig_output | cut -d " " -f4 | sed 's/\.\$//'`;
		chomp($wins_server_fqdn);

		#2.Get WINS server IP from FQDN
		Log(DEBG, "Executing .. /usr/bin/dig $wins_server_fqdn +short");
		$wins_server_ip = `/usr/bin/dig $wins_server_fqdn +short`;
		$ret = $?;
		chomp($wins_server_ip);
		if ($ret || $wins_server_ip eq "") {
			my $error = $ret >> 8;
			die ("Failed to get the IP for FQDN [$wins_server_fqdn] [$error]");
		}
	};

	if ($@) {
		Log("Unable to get WINS server IP");
		Log(ERR, $@)
	}

	Log_Console_Conditionally(CONFIGURE, "WINS server IP - $wins_server_ip");
}


#Check if WINS is configured in the domain
#If its called using --isconfigured, then check_WINS_using_dns is used
#Otherwise subroutine check_WINS_using_ldap is used
#LDAP is the default mechanism to fetch informations.
#During the YaST configuration when WINS related 
#information's retrieved then DNS is used.
sub check_WINS() {
	my $result = XAD_WINS_NOT_CONFIGURED;

	if ($is_configured == 1) {
		$result = query_WINS_using_dns();
	} else {
		$result = search_WINS();
	}

	if ($result == XAD_WINS_SUCCESS) {
		Log_Console_Conditionally(CONFIGURE, "WINS already there");
		`grep -i 'wins support = yes' $samba_conf_file`;
		if ($? == 0) {
			Log_Console_Conditionally(CONFIGURE, "WINS configured on same server");
			return XAD_WINS_PRESENT;
		}
		Log_Console_Conditionally(CONFIGURE, "WINS configured on different server");
		display_WINS_server_ip();
		return XAD_WINS_PRESENT_OUTSIDE;
	}

	if ($is_configured == 1 or $is_configured_upgrade == 1) {
		# Unconfigure will never reach this code
		# So no need for conditional console log
		Log_Console("WINS not configured");
		return XAD_WINS_SUCCESS;
	}

	Log(INFO, "WINS not configured");
	return XAD_WINS_NOT_CONFIGURED;
}

sub init() {
	mkdir("$wins_configure_dir",0700);
	Log(INFO, "Initialization Done");
}

#Add WINS parameters to smb.conf file
#wins support = yes
#name resolve order = wins lmhosts host bcast
#state directory = /var/opt/novell/xad/wins
sub update_smb_conf($) {
	my $operation_type = shift;
	Log(INFO, "Updating $samba_conf_file file with WINS configuration");
	my $section = "";
	my $val = "";

	my $cfg = Config::IniFiles->new (-file=> $samba_conf_file, -nocase=>1);

	if (!defined $cfg) {
	       Log(ERR, "Could not open the file $samba_conf_file");
	}

	Log(INFO, "Saving older samba file : stamp - $stamp");
	#Saving old file
	copy ("$samba_conf_file", "$samba_conf_file.$stamp") or
			Log (INFO, "Failed to backup the samba conf file: $!");

	#wins support = yes
	my $var_value = "wins support";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		if ($operation_type == ADD) {
			$cfg->newval ('global', $var_value, 'Yes');
		}

		if ($operation_type == DEL) {
			Log(INFO, "$var_value already deleted");
		}
	} else {
		if ($operation_type == ADD) {
			$cfg->setval ('global', $var_value, 'Yes');
		}

		if ($operation_type == DEL) {
			$cfg->delval ('global', $var_value);
		}
	}

	#name resolve order = wins lmhosts host bcast
	$var_value = "name resolve order";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		if ($operation_type == ADD) {
			$cfg->newval ('global', $var_value, 'wins lmhosts host bcast');
		}

		if ($operation_type == DEL) {
			Log(INFO, "$var_value already deleted");
		}
	} else {
		if ($operation_type == ADD) {
			$cfg->setval ('global', $var_value, 'wins lmhosts host bcast');
		}

		if ($operation_type == DEL) {
			$cfg->delval ('global', $var_value);
		}
	}


	# dns proxy = No
	# This is to ask nmbd not to look for WINS truncted name
	# ending with <20> (server) or <00> (workstations) 
	# with the DNS server.
	$var_value = "dns proxy";
	$val = $cfg->val('global', $var_value);
	if (!defined $val) {
		if ($operation_type == ADD) {
			$cfg->newval ('global', $var_value, 'No');
		}

		if ($operation_type == DEL) {
			Log(INFO, "$var_value already deleted");
		}
	} else {
		if ($operation_type == ADD) {
			$cfg->setval ('global', $var_value, 'No');
		}

		if ($operation_type == DEL) {
			$cfg->delval ('global', $var_value);
		}
	}

	$cfg->RewriteConfig();
	`sed -i 's/^[a-zA-Z]/\t&/g'	$samba_conf_file`;
	`sed -i 's/=/ = /g'	$samba_conf_file`;

	Log (INFO,"Updated $samba_conf_file file with WINS configuration");
	return 0;
}

#dns record is used for confirming if the domain has WINS or not
sub update_dns($) {
	my $operation_type = shift;
	Log(INFO, "Starting Update DNS");

	if ($operation_type == ADD) {
		create_WINS_SRV_record();
	}

	if ($operation_type == DEL) {
		remove_WINS_SRV_record();
	}

	update_dns_soa();

	Log(INFO, "Finished Update DNS");
}

#Need to change path when the path is changed to samba path in smb.conf
sub check_WINS_dat() {
	my $present = 0;

	for (my $i=0;$i<5;$i++) {
		Log(INFO, "Checking files $wins_dat_file and $wins_tdb_file");
		if(!-f $wins_dat_file || !-f $wins_tdb_file)
		{
			Log(INFO, "Files not found. Restarting nmb and rechecking");
			restart_nmb();
			sleep 1;#Wait for second
			next;#Re-execute the loop
		}
		else
		{
			$present = 1;
			Log(INFO, "Files $wins_dat_file and $wins_tdb_file found.");
			last;#Get out of the loop
		}
	}

	if ($present == 0) {
		Log(INFO, "WINS files $wins_tdb_file,$wins_dat_file not created in $wins_dir ");
		return XAD_WINS_FILES_NOT_CREATED;
	}

	Log(INFO, "WINS nmb files checking complete");
	return XAD_WINS_SUCCESS;
}

sub verify_WINS_srv() {
	my $result = -1;
	Log(INFO, "Verifying WINS SRV record creation");

	do {
		$result = search_WINS();
		if ($result != XAD_WINS_SUCCESS) {
			Log(INFO, "Recursively retrying verifying of WINS SRV records".
			       " - $global_srv_verify_retries");
			#The sleep value based on trial and error
			sleep 5;
			$global_srv_verify_retries++;
		} else {
			return $result;
		}
	} while($global_srv_verify_retries < 5);

	#Number of retry may reach 5 in case of WAN
	if ($global_srv_verify_retries == 5) {
		Log(INFO, "WINS configure post check failed while searching WINS dns record");
		#return XAD_UNKNOWN_ERROR;
		return XAD_WINS_SRV_RECORD_NOT_FOUND;
	}
}

sub ConfigureUsage{
	print <<CONFIGUSAGE;
This script configures this DSfW domain controller as WINS server.
However, only one WINS server is permitted per domain.

Options provided are
	--help

	--isconfigured

	--isconfigured-upgrade

	--onlyconfigure

	--postcheck

	--debug

CONFIGUSAGE
	exit 0;
}

#This should be called each time, before calling the
#subroutines ConfigureWINS, checkDuplicateWorkStation.
#This sets the parameters as per the requirement
# of the mentioned subroutines.
sub clear_and_set(@) {
	my %args = @_;

	$wins_common::is_configured = $args{is_cnfg} || 0;
	$wins_common::is_configured_upgrade = $args{is_cnfg_upg} || 0;
	$wins_common::only_configure = $args{cnfg} || 0;
	$wins_common::only_postcheck = $args{postchk} || 0;

}

#Called only for scripts executed from CONSOLE
sub set_logging
{
	if($is_debug == 0) {
		Logger::unset_mask(LOG_CONSOLE);
		Logger::unset_mask(LOG_TIMESTAMP);
	}
	else {
		Logger::unset_mask(LOG_TIMESTAMP);
	}
}

#Following tasks are performed to configure the server as WINS
#1. The DNS entry is created in the corresponding zone object.
#2. The samba file /etc/samba/smb.conf is updated with the variables required
#for samba services to act as WINS server.
#3. The nmb process is restarted.
#4. The post-check operation for the task checks if the DNS entry and data
#files corresponding to WINS are created.
sub ConfigureWINS() {
	#Used in error recovery
	$operation = CONFIGURE;


	if ($environment == XAD_CONSOLE) {
		set_logging();
	}

	init();

	Log(INFO, "isconfigured : [$is_configured]");
	Log(INFO, "isconfigured-upgrade : [$is_configured_upgrade]");
	Log(INFO, "onlyconfigure : [$only_configure]");
	Log(INFO, "postcheck : [$only_postcheck]");

	if ($only_postcheck == 1) {
		Log(INFO, "Only performing post-check for option [postcheck]");
		if (check::post_check_WINS() != XAD_WINS_SUCCESS){
			return XAD_UNKNOWN_ERROR;
		}
		return XAD_WINS_SUCCESS;
	}

	#If $only_configure is '1' dont check if WINS is configured. Directly configure
	if ($only_configure == 0) {
		Log(INFO, "Checking whether WINS is configured in this domain or not?");
		my $return_val = check_WINS();
		if ($return_val != XAD_WINS_NOT_CONFIGURED) {
			return $return_val;
		}
	} else {
		Log(INFO, "Directly configuring without initializing or checking");
	}

	update_dns(ADD);
	update_smb_conf(ADD);
	restart_nmb();

	Log_Console("WINS successfully configured");

	return XAD_WINS_SUCCESS;
}

#******************************************************************************
# Subroutine's to unconfigure WINS on a DSfW server
#******************************************************************************

sub remove_WINS_files() {
	Log(INFO, "Removing WINS files");

	`rm -f $wins_dat_file`;
	`rm -f $wins_tdb_file`;
	Log(INFO, "WINS files removed");
}

sub UnconfigureUsage{
	print <<UNCONFIGUSAGE;
This script unconfigures WINS on this DSfW domain controller.

Options provided are

	--help

	--postcheck

	--debug

UNCONFIGUSAGE
	exit 0;
}

sub UnconfigureWINS() {
	my $returnValue = XAD_UNKNOWN_ERROR;

	if ($environment == XAD_CONSOLE) {
		set_logging();
	}

	Log(INFO, "Starting WINS unconfigure");

	#Used in error recovery
	$operation = UNCONFIGURE;

	#Check if WINS is present!!!
	my $return_val = check_WINS();
	if ($return_val == XAD_WINS_NOT_CONFIGURED) {
		Log(INFO, "WINS is already removed on this server");
		return $return_val;
	}

	update_dns(DEL);
	update_smb_conf(DEL);
	restart_nmb();
	remove_WINS_files();
	$returnValue = check::post_check_WINS();

	Log_Console("WINS successfully unconfigured");

	return($returnValue);
}
1;
