#!/usr/bin/env python3

# (c) Copyright [2020 - 2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (Micro Focus) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

import os
import json
import gettext
import fcntl
from collections import OrderedDict
import subprocess
from datetime import datetime

# Write the health of DSfW service to /var/opt/novell/xad/dsfw.json


def writeJSON(dictionary):
    try:
        with open("/var/opt/novell/xad/dsfw.json", "w") as dsfwstatusfile:
            fcntl.flock(dsfwstatusfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
            json.dump(dictionary, dsfwstatusfile,
                      indent=4, separators=(", ", ": "))
            fcntl.flock(dsfwstatusfile, fcntl.LOCK_UN)
    except IOError:
        raise Exception(
            "Failed to open/write the Json file /var/opt/novell/xad/dsfw.json")


def getExecMainStartTimestamp(service_name):
    try:
        command = ["systemctl", "show", "--value",
                   "--property=ExecMainStartTimestamp", service_name]
        output = subprocess.check_output(command, universal_newlines=True)

        # Extract the date string
        date_string = output.strip()

        # Parse the date string to a datetime object
        date_obj = datetime.strptime(date_string, "%a %Y-%m-%d %H:%M:%S %Z")

        # Calculate the Unix timestamp
        timestamp = int(date_obj.timestamp())

        return timestamp
    except subprocess.CalledProcessError as e:
        raise Exception("Command failed with an error:", e)
        return 0


# To Get DSfW Health status data
def generateHealthStatus():

    jsondsfw_stream = OrderedDict(
        [
            ('serviceName', 'Domain Services for Windows (DSfW)'),
            ('status', ''),
            ('children',
                [OrderedDict(
                    [
                        ('serviceName', 'ndsd'),
                        ('status', ''),
                        ('runningSince', ''),
                        ('AdditionalInfo', ''),
                        ('children', None)
                    ]
                ),
                    OrderedDict(
                    [
                        ('serviceName', 'novell-named'),
                        ('status', ''),
                        ('runningSince', ''),
                        ('AdditionalInfo', ''),
                        ('children', None)
                    ]
                ),
                    OrderedDict(
                    [
                        ('serviceName', 'nscd'),
                        ('status', ''),
                        ('runningSince', ''),
                        ('AdditionalInfo', ''),
                        ('children', None)
                    ]
                ),
                    OrderedDict(
                    [
                        ('serviceName', 'rpcd'),
                        ('status', ''),
                        ('runningSince', ''),
                        ('AdditionalInfo', ''),
                        ('children', None)
                    ]),
                    OrderedDict(
                    [
                        ('serviceName', 'Kerberos'),
                        ('status', ''),
                        ('AdditionalInfo', ''),
                        ('children', [
                            OrderedDict(
                                [
                                    ('serviceName', 'xad-krb5kdc'),
                                    ('status', ''),
                                    ('runningSince', ''),
                                    ('AdditionalInfo', ''),
                                    ('children', None)
                                ]
                            ),
                            OrderedDict(
                                [
                                    ('serviceName', 'xad-kpasswdd'),
                                    ('status', ''),
                                    ('runningSince', ''),
                                    ('AdditionalInfo', ''),
                                    ('children', None)
                                ]
                            )
                        ])
                    ]
                ),
                    OrderedDict(
                    [
                        ('serviceName', 'xadsd'),
                        ('status', ''),
                        ('runningSince', ''),
                        ('AdditionalInfo', ''),
                        ('children', None)
                    ]
                ),
                    OrderedDict(
                    [
                        ('serviceName', 'Samba'),
                        ('status', ''),
                        ('AdditionalInfo', ''),
                        ('children', [
                            OrderedDict(
                                [
                                    ('serviceName', 'nmb'),
                                    ('status', ''),
                                    ('runningSince', ''),
                                    ('AdditionalInfo', ''),
                                    ('children', None)
                                ]
                            ),
                            OrderedDict(
                                [
                                    ('serviceName', 'winbind'),
                                    ('status', ''),
                                    ('runningSince', ''),
                                    ('AdditionalInfo', ''),
                                    ('children', None)
                                ]
                            ),
                            OrderedDict(
                                [
                                    ('serviceName', 'smb'),
                                    ('status', ''),
                                    ('runningSince', ''),
                                    ('AdditionalInfo', ''),
                                    ('children', None)
                                ]
                            )
                        ]
                        )
                    ]
                )
                ]
             )
        ]
    )

    ndsdstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "ndsd"],
                                stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    novell_namedstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active",
                                        "novell-named"],  stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    nscdstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "nscd"],
                                stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    rpcdstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "rpcd"],
                                stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    xad_krb5kdcstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active",
                                       "xad-krb5kdc"],  stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    xad_kpasswddstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active",
                                        "xad-kpasswdd"],  stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    xadsdstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "xadsd"],
                                 stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    nmbstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "nmb"],
                               stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    winbindstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active",
                                   "winbind"],  stdout=subprocess.PIPE).stdout.decode().replace("\n", "")
    smbstatus = subprocess.run(["timeout", "60s", "systemctl", "is-active", "smb"],
                               stdout=subprocess.PIPE).stdout.decode().replace("\n", "")

    if (ndsdstatus == "active"):
        jsondsfw_stream["children"][0]["status"] = ("Healthy")
        jsondsfw_stream["children"][0]["runningSince"] = getExecMainStartTimestamp(
            "ndsd")
    else:
        jsondsfw_stream["children"][0]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][0]["runningSince"] = "0"
        jsondsfw_stream["children"][0]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/eDirectory/log/ndsd.log"]

    if (nscdstatus == "active"):
        jsondsfw_stream["children"][2]["status"] = ("Healthy")
        jsondsfw_stream["children"][2]["runningSince"] = getExecMainStartTimestamp(
            "nscd")
    else:
        jsondsfw_stream["children"][2]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][2]["runningSince"] = "0"
        jsondsfw_stream["children"][2]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages"]

    if (rpcdstatus == "active"):
        jsondsfw_stream["children"][3]["status"] = ("Healthy")
        jsondsfw_stream["children"][3]["runningSince"] = getExecMainStartTimestamp(
            "rpcd")
    else:
        jsondsfw_stream["children"][3]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][3]["runningSince"] = "0"
        jsondsfw_stream["children"][3]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages"]

    if (xad_krb5kdcstatus == "active"):
        jsondsfw_stream["children"][4]["children"][0]["status"] = ("Healthy")
        jsondsfw_stream["children"][4]["children"][0]["runningSince"] = getExecMainStartTimestamp(
            "xad-krb5kdc")
    else:
        jsondsfw_stream["children"][4]["children"][0]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][4]["children"][0]["runningSince"] = "0"
        jsondsfw_stream["children"][4]["children"][0]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/xad/log/kdc.log"]

    if (xad_kpasswddstatus == "active"):
        jsondsfw_stream["children"][4]["children"][1]["status"] = ("Healthy")
        jsondsfw_stream["children"][4]["children"][1]["runningSince"] = getExecMainStartTimestamp(
            "xad-kpasswdd")
    else:
        jsondsfw_stream["children"][4]["children"][1]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][4]["children"][1]["runningSince"] = "0"
        jsondsfw_stream["children"][4]["children"][1]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/xad/log/kpasswdd.log"]

    if (xadsdstatus == "active"):
        jsondsfw_stream["children"][5]["status"] = ("Healthy")
        jsondsfw_stream["children"][5]["runningSince"] = getExecMainStartTimestamp(
            "xadsd")
    else:
        jsondsfw_stream["children"][5]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][5]["runningSince"] = "0"
        jsondsfw_stream["children"][5]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages"]

    if (nmbstatus == "active"):
        jsondsfw_stream["children"][6]["children"][0]["status"] = ("Healthy")
        jsondsfw_stream["children"][6]["children"][0]["runningSince"] = getExecMainStartTimestamp(
            "nmb")
    else:
        jsondsfw_stream["children"][6]["children"][0]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][6]["children"][0]["runningSince"] = "0"
        jsondsfw_stream["children"][6]["children"][0]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.nmbd"]

    if (winbindstatus == "active"):
        jsondsfw_stream["children"][6]["children"][1]["status"] = ("Healthy")
        jsondsfw_stream["children"][6]["children"][1]["runningSince"] = getExecMainStartTimestamp(
            "winbind")
    else:
        jsondsfw_stream["children"][6]["children"][1]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][6]["children"][1]["runningSince"] = "0"
        jsondsfw_stream["children"][6]["children"][1]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.winbindd"]

    if (smbstatus == "active"):
        jsondsfw_stream["children"][6]["children"][2]["status"] = ("Healthy")
        jsondsfw_stream["children"][6]["children"][2]["runningSince"] = getExecMainStartTimestamp(
            "smb")
    else:
        jsondsfw_stream["children"][6]["children"][2]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][6]["children"][2]["runningSince"] = "0"
        jsondsfw_stream["children"][6]["children"][2]["AdditionalInfo"] = [
            "Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.smbd"]
    if ((jsondsfw_stream["children"][4]["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["children"][1]["status"] == ("Healthy"))):
        jsondsfw_stream["children"][4]["status"] = ("Healthy")
    else:
        jsondsfw_stream["children"][4]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][4]["AdditionalInfo"] = [
            "Common_DaemonService1", "Common_DaemonService2"]

    if ((jsondsfw_stream["children"][6]["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["children"][1]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["children"][2]["status"] == ("Healthy"))):
        jsondsfw_stream["children"][6]["status"] = ("Healthy")
    else:
        jsondsfw_stream["children"][6]["status"] = ("UnHealthy")
        jsondsfw_stream["children"][6]["AdditionalInfo"] = [
            "Common_DaemonService1", "Common_DaemonService2"]

    DNS_Master = subprocess.run(["/opt/novell/xad/share/dcinit/printConfigKey.pl", "DNS Master"],
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout.decode().replace("\n", "")
    if (DNS_Master == "TRUE"):
        if (novell_namedstatus == "active"):
            jsondsfw_stream["children"][1]["status"] = ("Healthy")
            jsondsfw_stream["children"][1]["runningSince"] = getExecMainStartTimestamp(
                "novell-named")
        else:
            jsondsfw_stream["children"][1]["status"] = ("UnHealthy")
            jsondsfw_stream["children"][1]["runningSince"] = "0"
            jsondsfw_stream["children"][1]["AdditionalInfo"] = [
                "Common_ReferLogMoreInfo", "/var/log/message", "/var/opt/novell/log/named/named.run"]

        if ((jsondsfw_stream["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][1]["status"] == ("Healthy")) and (jsondsfw_stream["children"][2]["status"] == ("Healthy")) and (jsondsfw_stream["children"][3]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["status"] == ("Healthy")) and (jsondsfw_stream["children"][5]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["status"] == ("Healthy"))):
            jsondsfw_stream["status"] = ("Healthy")
        else:
            jsondsfw_stream["status"] = ("UnHealthy")
    else:
        if ((jsondsfw_stream["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][2]["status"] == ("Healthy")) and (jsondsfw_stream["children"][3]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["status"] == ("Healthy")) and (jsondsfw_stream["children"][5]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["status"] == ("Healthy"))):
            jsondsfw_stream["status"] = ("Healthy")
        else:
            jsondsfw_stream["status"] = ("UnHealthy")

        jsondsfw_stream["children"].remove(jsondsfw_stream["children"][1])

    return jsondsfw_stream


dsfwHealthData = generateHealthStatus()

try:
    writeJSON(dsfwHealthData)
except Exception:
    exit(1)
