#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#	Micro Focus or one of its affiliates.
################################################################################

use warnings;
use strict;
use POSIX qw(strftime setlocale);
use Getopt::Std;
use XAD::registry;
use XAD::dclib;
use XAD::nds;
use Locale::gettext;
use File::Path qw(mkpath);
use File::Copy;
use Socket;
use Term::ReadKey; #ReadMode and ReadLine
use File::Temp;
use Logger qw(:DEFAULT);
use Install::frd_install;
use Install::child_install;
#use Cwd;
use helpers::ndsServerHealthCheck;
use helpers::containerCheck;
use util;
use crosspartitionntp;
use upgrade_dns_data qw(upgrade_dns_data);
use NDS::stage;

my ($ENV_PATH, $_LIB, $LD_LIBRARY_PATH, $SASL_PATH, $TEXTDOMAINDIR, $TEXTDOMAIN); #ENV variable
our  ($id, $PREFIX, $SYSTEM_CONFIG, $DCINIT_CONFIG, $NDS_DCINIT, $require_tgt, 
	$adm_password, $nds_existing_adminname, $nds_existing_adminpasswd, $ReplicationProvider) = ""; #common variable
my ($domain_name, $parent_name, $forest_name, $mapped_container, $mapped_container_ldap_name, $parent_container,
	$netbios_name, $is_replica, $Length_DOMAINNAME, $max_mapped_container_length, $tls_option) = ""; #domain variable
my  ($configure_dns, $dns_locator_obj_context, $dnsdhcp_grp_context, $dnsserver_context, $preferred_dns, $preferred_site, $wins_configured,
	$dns_locator_object, $dnsdhcp_group, $localhostname, $dnsserver, $dns_ncpserver, $IpAddress, $dns_server_ip) = ""; #dns variable
our ($type, $action, $mapped, $domain_guid, $domain_sid, $machine_guid, $Nds, $krb5ccname_domain, $krb5ccname_parent,
	$bootstrap_krb5_config) = "";

my $log_file = "/var/opt/novell/xad/log/ndsdcinit.log";
$DCINIT_CONFIG = "/etc/opt/novell/xad/xad.ini";
my $max_log_size = 1048576;
my $retry = 0;
my $upgrade = 0;
my $sam_account_name = "Administrator";
my $myLocalhost = "127.0.0.1";
my $sedPattern = '\' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; \'';
my $Ldapsearch = "/opt/novell/eDirectory/bin/ldapsearch -x";
my $domain_nc = "";

sub croak { die "$0: @_: $!\n" }

sub Usage {

	print <<USAGE;
	ndsdcinit.pl [retry] domain
	-d		DNS formatted domain name to be installed
	-p		DNS formatted name of the parent domain
	-f		DNS formatted name of the forest root domain
	-n		NetBIOS name of domain being installed
	-s		If domain is to be configured as DNS server
	-m		NDS formatted name of mapped container
	-l		Location of the DNS Locator object
	-g		Location of the DNSDHCP-Group object
	-o		IP address of the preferred DNS server

	ndsdcinit.pl [retry] replica | full-replica
	-d		DNS formatted domain name to be installed
	-p		DNS formatted name of the parent domain
	-f		DNS formatted name of the forest root domain
	-s		Domain is to be configured as DNS server
	-m		NDS formatted name of mapped container
	-l		Location of the DNS Locator object
	-g		Location of the DNSDHCP-Group object
	-o		IP address of the preferred DNS server
USAGE
        exit 0

}

sub prepare_log {

	# Check for existing file
	if (-e $log_file) {

		unlink($log_file) if( $max_log_size > -s $log_file );
	}

	# Create the log file if it does not exist and open
	open LOG, ">$log_file" or croak("Could not open the log file $log_file");

	print LOG POSIX::strftime("%A %B %d  %H:%M:%S %Y", localtime);
	print LOG "\nStart of DSFW installation and Provisioning\n";

}

sub finish_log {

	print LOG POSIX::strftime("%A %B %d  %H:%M:%S %Y", localtime);
	print LOG "End of DSfW installation and provisioning\n";
	close LOG;

}

sub DumpConfigToFile
{

	my $Date = `/bin/date`;
	chomp($Date);

	open(CONFIG,">$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");

	print CONFIG "#\n";
	print CONFIG "# XAD configuration file for domain $domain_name\n";
	print CONFIG "# Generated by dcinit on $Date\n";
	print CONFIG "#\n"; 

	print CONFIG "[DefaultXADConfig]\n";
	print CONFIG "Domain GUID = $domain_guid\n";
	print CONFIG "Domain SID = $domain_sid\n";
	print CONFIG "Netbios Name = $netbios_name\n";
	print CONFIG "Root Domain = $domain_name\n";
	print CONFIG "Forest Root = $forest_name\n";
	print CONFIG "Parent Domain = $parent_name\n";
	print CONFIG "IP Address = $IpAddress\n";
	print CONFIG "Machine GUID = $machine_guid\n";

	print CONFIG "DNS_LOCATOR_OBJECT = $dns_locator_object\n";
	print CONFIG "DNSDHCP_GROUP = $dnsdhcp_group\n";
	print CONFIG "DNSSERVER = $dnsserver\n";
	print CONFIG "DNS_NCPServer = $dns_ncpserver\n";
	print CONFIG "DNSSERVER_CONTEXT = $dnsserver_context\n";
	print CONFIG "DNS_SERVER_IP = $dns_server_ip\n";
	print CONFIG "NDS_DCINIT = $NDS_DCINIT\n";
	print CONFIG "INSTALL_TYPE = $type\n";

	if (defined $ENV{NDSEXISTINGADMINNAME} && $ENV{NDSEXISTINGADMINNAME} ne "") {
		print CONFIG "NDSEXISTINGADMINNAME = $ENV{NDSEXISTINGADMINNAME}\n";
	}

	if (defined $mapped_container_ldap_name && $mapped_container_ldap_name ne "") {
		print CONFIG "Mapped Domain NC = $mapped_container_ldap_name\n";
	}

	if (defined $krb5ccname_domain && $krb5ccname_domain ne "") {
		print CONFIG "KRB5CCNAME_DOMAIN = $krb5ccname_domain\n";
	}

	if (defined $krb5ccname_parent && $krb5ccname_parent ne "") {
		print CONFIG "KRB5CCNAME_PARENT = $krb5ccname_parent\n";
	}

	if ($action eq "full-replica") {
		print CONFIG "CONFIG_SCHEMA_REPLICAS = TRUE\n";
	}elsif ($action eq "replica") {
		print CONFIG "CONFIG_SCHEMA_REPLICAS = FALSE\n";
	}

	print CONFIG "DNS Master = ";
	if ($configure_dns != 0) {
		print CONFIG "TRUE\n";
	} else {
		print CONFIG "FALSE\n";
	}

	if (defined($ReplicationProvider)) {
		print CONFIG "Replication Provider = $ReplicationProvider\n";
	}

	if ($Nds == 0) {
		# for XAD 2.0, default behaviour is to use NT security
		# for XAD 1.0 compat remove or set to FALSE
		print CONFIG "Native Security = TRUE\n";
	} else {
		print CONFIG "NDS = TRUE\n";
	}

	if (defined $ENV{NKDC_REALM} && $ENV{NKDC_REALM} ne "") {
		print CONFIG "NKDC Realm = $ENV{NKDC_REALM}\n";
	}

	print CONFIG "Require TLS = ";
	if ($tls_option == 1) {
		print CONFIG "TRUE\n";
	}else {
		print CONFIG "FALSE\n";
	}

	if (defined $ENV{XADRETAINPOLICIES} and $ENV{XADRETAINPOLICIES} ne "") {
		print CONFIG "XADRETAINPOLICIES = $ENV{XADRETAINPOLICIES}\n";
	}

	if (defined $sam_account_name) {
		if ($type eq "child" && $is_replica eq "false") {
			print CONFIG "SAMACCOUNTNAME_PARENT = $sam_account_name\n";
		}
		elsif ($is_replica eq "true") {
			print CONFIG "SAMACCOUNTNAME_DOMAIN = $sam_account_name\n";
		}
	}

	print CONFIG "Site = $preferred_site\n";
	print CONFIG "WINS = $wins_configured\n";
	print CONFIG "Zone Container = $dnsserver_context\n";
	close(CONFIG);
}

sub ReadDomainName
{
	my $host_name = `/bin/hostname` or croak ("/bin/hostname failed");
	chomp($host_name);

	$domain_name = `/bin/dnsdomainname` || croak ("/bin/dnsdomainname failed\n");
	chomp($domain_name);

	if ($domain_name eq "(none)") { 
		$domain_name = "" ;
	}

	if ($domain_name ne "") {
		$domain_name = $host_name;
	}

	if ($domain_name =~ /\./) {
		$domain_name =~ s/^(\w+)\.//;
	} 
	else {
		$domain_name = "";
	}
}

sub GenerateGuidAndSid
{
	$domain_guid = `uuidgen` || croak("uuidgen failed");
	chomp($domain_guid);

	# Generate the SID subauthorities from the GUID,
	# it is a suitable source of randomness. We do it
	# in reverse order to preserve some semblance of
	# hierarchy; the Ethernet address is last in the
	# GUID.
	my $guid = XAD::packGuid($domain_guid);
	my @subauth = unpack("V4", $guid);
	$domain_sid = "S-1-5-21-$subauth[3]-$subauth[2]-$subauth[1]";

	$machine_guid = `uuidgen` || croak("uuidgen failed");
	chomp($machine_guid);
}

#Bootstrap all the global variable
sub bootstrap_variable {

	#Environment Variables
	$ENV_PATH = "/opt/novell/xad/sbin:/opt/novell/xad/bin:/opt/novell/xad/share/dcinit:/opt/novell/eDirectory/bin:";
	$_LIB = registry::getReg("_Lib");
	$LD_LIBRARY_PATH = "/opt/novell/xad/$_LIB:/opt/novell/xad/$_LIB/nds-modules:/opt/novell/eDirectory/$_LIB:/opt/novell/eDirectory/$_LIB/nds-modules";
	$SASL_PATH = "/opt/novell/xad/$_LIB/sasl2";
	$TEXTDOMAINDIR = "/opt/novell/xad/share/locale";
	$TEXTDOMAIN = "xad";

	$ENV{'PATH'} = $ENV_PATH.$ENV{'PATH'};
	$ENV{'_LIB'} = $_LIB;
	$ENV{'SASL_PATH'} = $SASL_PATH;
	$ENV{'LD_LIBRARY_PATH'} = $LD_LIBRARY_PATH . (defined $ENV{'LD_LIBRARY_PATH'}? $ENV{'LD_LIBRARY_PATH'} : "");

	setlocale(LC_MESSAGES, "");
	bindtextdomain($TEXTDOMAIN, $TEXTDOMAINDIR);
	textdomain($TEXTDOMAIN);

	$PREFIX = "/opt/novell/xad";
	$SYSTEM_CONFIG = "/etc".$PREFIX."/xad.ini";

	if (-e $SYSTEM_CONFIG && $retry == 0) {
		print STDERR gettext("A domain is alreay configured on this machine.");
		print STDERR gettext("To reconfigure this machine, first run the 'dcmake unconfig'");
		exit 1;
	}

	my $fh;
#$$DCINIT_CONFIG = mktemp("/tmp/tmp.XXXXXXXXXX");
	if (-e $DCINIT_CONFIG) {
		`cp $DCINIT_CONFIG $DCINIT_CONFIG.temp`;
		if ($?) {
			print STDERR gettext("Failed to copy the existing xad.ini file.");
			exit 1;
		}

		#Do the grep for machine guid and domain SID 
		#if present donot call the GenerateGuidAndSid()
		#subroutine. and return from here.
		open(XFH, "<$DCINIT_CONFIG") or warn("Could not open file $DCINIT_CONFIG");
		my @file_data=<XFH>;
		close(XFH);

		if(grep("^Domain SID",@file_data) and grep("^Machine GUID",@file_data)){
			$domain_sid=(split(/\s+=\s+/, (grep(/^Domain SID\s+.*/,@file_data))[0], 2))[1];
			$machine_guid=(split(/\s+=\s+/, (grep(/^Machine GUID\s+.*/,@file_data))[0], 2))[1];
			if($domain_sid and $machine_guid){
				chomp($domain_sid);
				chomp($machine_guid);
				$NDS_DCINIT = 1;
				$Nds = 1;
				return 0;
			}
			else{
				#In the worst case if the domain_sid and machine_guid are null
				#delete all the ldif files using the domain sid or machine guid
				#regenerate the ldif files
				print STDERR gettext("Domain SID and Machine GUID are null\n");
				my $tempdir = registry::getReg("TEMPDIR");
				if(-e $tempdir."/stage_domain"){
					my @file_arr=("domain.ldif", "forest.ldif", "domain-object-modify.ldif", "domain-tcb.ldif");
					foreach my $file (@file_arr){
						chomp($file);
						my $tmpfile=registry::getReg("LIVE_LOCALSTATEDIR")."/ds/domain/".$file;
						if(-e $tmpfile){
							`rm -f $tmpfile`;
						}
					}
					unlink "$tempdir/stage_domain";
				}

				if(-e $tempdir."/stage_config"){
					my $tmpfile=registry::getReg("LIVE_LOCALSTATEDIR")."/ds/config/config.ldif";
					if(-e $tmpfile){
						`rm -f $tmpfile`;
					}
					unlink "$tempdir/stage_config";
				}
				unlink "$tempdir/stage_to_live";
			}
		}
	}
	$NDS_DCINIT = 1;
	$Nds = 1;

	GenerateGuidAndSid();
}

# Subroutine to fill the sam account name
sub get_admin_sam_name($) {
	my $admin_dn = shift;
	my $ldapcommand = "";

	if ($upgrade == 1) {
		$ldapcommand = "LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Y EXTERNAL -s base -b '$admin_dn' -LLL samaccountname 2>/dev/null";
	} 
	else {
		my $domain = "";

		if ($type eq "child" && $is_replica eq "false") {
			$domain = $parent_name;
		}elsif ($is_replica eq "true") {
			$domain = $domain_name;
		}
		chomp($domain);
		my @ip_addr_list = `/usr/bin/dig +short $domain`;
		chomp(@ip_addr_list);

		if ($? eq 0 && @ip_addr_list) {
			my $ip_addr = "";
			foreach my $ip (@ip_addr_list) {
				`/opt/novell/eDirectory/bin/ldapsearch -x -h $ip -s base -b "" >/dev/null 2>&1`;
				if ($? eq 0) {
					$ip_addr = $ip;
					last;
				}
				# Last IP in the list, don't go inside and print the message 
                                elsif ($ip ne $ip_addr_list[$#ip_addr_list]) {
                                        print "Could not contact the LDAP server on [$ip], trying next\n";
				}
			}

			if ($ip_addr eq "") {
				die ("Could not contact any LDAP server in the replica ring\n");
			}	

			my $ldapadminname = $ENV{NDSEXISTINGADMINNAME};
			$ldapadminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;

			$ldapcommand = "LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H ldaps://$ip_addr:636 -D '$ldapadminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -s base -b '$admin_dn' -LLL samaccountname ";
		}
		else {
			die("Not able to locate the domain [$domain]");
		}
	}

	my @ldapoutput = `$ldapcommand 2>&1`;
	if ($? ne 0) {
		die ("ldapsearch failed to get the admin [$admin_dn] samAccountName. [@ldapoutput]\n");
	}

	my @samname = grep(/samAccountName/i, @ldapoutput);
	$samname[0] =~ s/samaccountname: //gi;
	chomp($samname[0]);
	if ($samname[0] eq "") {
		die("SamAccountName is not valid for the [$admin_dn]\n");
	}
	
	$sam_account_name = $samname[0];
}

#Validate the container for Name mapped scenario
sub validate_mapped_container{

	$mapped_container_ldap_name = $mapped_container;
	$mapped_container_ldap_name =~  s/\.([a-z,A-Z,0-9]*)=/,$1=/g;

	#Max length of a FQDN name: 255
	#Max eDirectory's DN size (including Typed Tree Name eg. .T=EXAMPLE-TREE (length = 3+12)): 255
	#The size of longest default object in aquila tree: [144 + Size of longest DC Name in the domain]
	#The DC Name need not to be equal to Host Name. Also max length of DC Name can be of 15 characters.
	#Max size of Mapped Container ldap name: [255 - (144 + 15 (Max length of DC Name) + 3 (Typed chars of Tree Name) + 
	#Length of Actual Tree Name)] = [93 - Length of Actual Tree Name]

	my $tree_name = `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`;
	chomp ($tree_name);
	$max_mapped_container_length = 93 - length($tree_name);
	if(length($mapped_container_ldap_name) > $max_mapped_container_length) {
		print STDERR sprintf(gettext("Domain names must be %s characters or less."), $max_mapped_container_length);
		exit 7;
	}

	if (not defined $ENV{NDSEXISTINGADMINNAME}) {
		print gettext("Enter existing tree administrator name in NDS format: ");
		chomp ($nds_existing_adminname = <STDIN>);
		$ENV{NDSEXISTINGADMINNAME} = $nds_existing_adminname;
	}
	croak(gettext("Existing administrator name must be specified.")) if not defined $ENV{NDSEXISTINGADMINNAME};

	if (not defined $ENV{NDSEXISTINGADMINPASSWD}) {
		print gettext("Enter existing tree administrator password: ");
		ReadMode('noecho');
		$nds_existing_adminpasswd = ReadLine(0);
		ReadMode('normal');
		chomp($nds_existing_adminpasswd);
		$ENV{'NDSEXISTINGADMINPASSWD'} = $nds_existing_adminpasswd;
	}
	croak(gettext("Existing administrator password must be specified.")) if not defined $ENV{NDSEXISTINGADMINPASSWD};
	$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};

	if ($domain_name eq $parent_name) {
		`/opt/novell/xad/share/dcinit/provisionTools.sh  partition-query -d $domain_name -a "$ENV{NDSEXISTINGADMINNAME}" -p $myLocalhost -c "$mapped_container"`;
		if ($? != 0) {
			croak("Partition query did not succeed");
		}

	}
	else {
		$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_PARENT};
		my $parent_nc_dn = "dc=$parent_name";
		$parent_nc_dn =~ s/\./,dc=/g;

		my $parent_admin_name = "Administrator";
		if ($ENV{XADPARENTDOMAINADMIN}) {
			my $admin_dn= $ENV{XADPARENTDOMAINADMIN};
			$admin_dn =~ /cn=([^\.]*)/i;
			$parent_admin_name = $1;
		}
		my $parent_admin = "CN=$parent_admin_name,CN=Users,$parent_nc_dn";
		$mapped_container =~ /(.*?)=.*/;
		$ENV{RDN_TYPE} = $1; 
		my $parent_domain_provider=`/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $parent_name`;
		if ($? != 0) {
			my $error = $? >> 8;
			die("Failed to get the domain provider for domain [$parent_name] with error [$error]");
		}

		chomp($parent_domain_provider);
		`/opt/novell/xad/share/dcinit/provisionTools.sh partition-query -d $domain_name -a '$parent_admin' -p $parent_domain_provider -c '$mapped_container'`;
		if ($?) {
			croak("Failed while doing the partition query for child domain.");
		}
	}

}

sub read_pwd_terminal
{
	my $str = shift;
	my $pwd;

	print "\n$str: ";

	ReadMode('noecho');
	$pwd = ReadLine(0);
	ReadMode('normal');

	print "";
	chomp($pwd);
	return $pwd;
}

sub validate_common
{
	my $id = `id -u`;
	chomp($id);

	if ($id != 0) {
		print STDERR gettext("Root permissions are required to install Domain Services for Windows.");
		exit 1;
	}

	# create the mech file
	#mkpath ("/etc$PREFIX/gss") or warn("Could not create gss directory /etc$PREFIX/gss");
	mkpath ("/etc$PREFIX/gss");
	copy("$PREFIX/share/dcinit/templates/mech", "/etc$PREFIX/gss/mech") or
		croak ("Coult not copy the mech file to /etc$PREFIX/gss/mech");


	if ($is_replica eq "false") {
		#Parent Domain Installation.

		#NetBIOS Validation
		if (not defined $netbios_name or $netbios_name eq "") {
			print "Enter the NetBIOS name of the domain to be configured:";
			chomp($netbios_name = <STDIN>);
		}

		croak(gettext("NetBIOS name must be specified.")) if (not defined $netbios_name);
		croak(gettext("NetBIOS name must not have a periods it.")) if ($netbios_name =~ /\./);

		if (length($netbios_name) > 15) {
			croak(gettext("NetBIOS name must be 15 characters or less."));
		}
		
		$netbios_name = uc($netbios_name);

		$require_tgt = 0;

	}
	else {

		#netbios name validation
		if (not defined $netbios_name or $netbios_name eq "") {

			my $adm_principal = "";
			if (defined $ENV{XADDOMAINADMINNAME}) {
				my $admin_dn = $ENV{XADDOMAINADMINNAME};
				$admin_dn =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
				get_admin_sam_name($admin_dn);
				$adm_principal = "'$sam_account_name'\@".uc($domain_name);
			}
			
			

			if (not defined $ENV{NDSEXISTINGADMINNAME} or $ENV{NDSEXISTINGADMINNAME} eq "") {
				#We create the admin name if not exported
				my $domain_nc = "dc=$domain_name";
				$domain_nc =~ s/\./.dc=/g;
				$ENV{NDSEXISTINGADMINNAME}="CN=Administrator.CN=Users.$domain_nc";
			}

			#Check for Existing admin password, required for kinit
			if (not defined $ENV{'ADM_PASSWD'}) {
				$adm_password = read_pwd_terminal("Enter Existing Administrator password");
				$ENV{'ADM_PASSWD'} = $adm_password;
			}
			croak(gettext("Existing Administrator password must be specified.")) if (not defined $ENV{'ADM_PASSWD'});

                	#Create an empty KRB5_CONFIG file so that /etc/krb5.conf is not used.
			my $bootstrap_krb5_config = mktemp("/tmp/tmp.XXXXXXXXXX");
			$ENV{KRB5_CONFIG} = $bootstrap_krb5_config;

			my $rc = system("/opt/novell/xad/bin/kinit -E $adm_principal");
			if ($rc != 0) {
				print STDERR gettext("Could not run kinit with the principal $adm_principal");
				unlink($bootstrap_krb5_config);
				exit(1);
			}

			delete $ENV{KRB5_CONFIG};

			my $ret_netbios_name = `/opt/novell/xad/sbin/provision -q --query "$domain_name" | grep 'NetBIOS Name:' | awk -F ":" '{print \$2}'`;
			
			#The assignment is valid because the code is executed only for additional domain controllers
			my $parent_admin1 = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
			chomp($parent_admin1);
			$parent_admin1 =~ s/cn=//g;
			$parent_admin1 =~ s/dc=//g;
			$parent_admin1 =~ s/,/./g;

			if ( defined $ret_netbios_name && $ret_netbios_name ne "") {
                              $netbios_name = $ret_netbios_name;
                        } else {
                                $netbios_name = `/opt/novell/xad/share/dcinit/provisionTools.sh get-NetBIOS-name -d $domain_name -a $parent_admin1`;
                                chomp($netbios_name);
                                Log("  Netbios name $netbios_name ");
                        }

			if (not defined $netbios_name or $netbios_name eq "") {
				print STDERR gettext("Unable to retrieve NetBIOS name. ".
						"Check if provision --locate-dc $domain_name is working fine.".
						"Check if /opt/novell/xad/sbin/provision -q --query $domain_name is working fine. ".
						"Also, if /opt/novell/xad/share/dcinit/provisionTools.sh get-NetBIOS-name -d $domain_name -a $parent_admin1 is working fine.");
				# Exiting with '3' in case administrator password not set.
				exit(4);
			}

		} #	if (not defined $netbios_name or $netbios_name eq "") 

		#Getting Replication Provider
		$ReplicationProvider = `/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $domain_name`;
		if ($? != 0) {
			my $error = $? >> 8;
			die("Failed to get the replication provider for domain [$domain_name] with error [$error]");
		}
		chomp($ReplicationProvider);


		$require_tgt = 1;

	} #if action is additional dc

	$DCINIT_CONFIG = "config/". lc($netbios_name). ".ini" if $DCINIT_CONFIG eq "";


	if (not defined $ENV{'ADM_PASSWD'}) {
		$adm_password = read_pwd_terminal("Enter Administrator password");
		$ENV{'ADM_PASSWD'} = $adm_password;
		$ENV{'ADM_PASSWD_DOMAIN'} = $adm_password;
	}
	else {
		$ENV{ADM_PASSWD_DOMAIN} = $ENV{ADM_PASSWD};
	}

	croak(gettext("An administrator password must be specified.")) if (not defined $ENV{'ADM_PASSWD'});

	if ($type eq "child" and $is_replica eq "false") {
		if (not defined $ENV{ADM_PASSWD_PARENT} or $ENV{ADM_PASSWD_PARENT} eq "") {
			$ENV{ADM_PASSWD_PARENT} = read_pwd_terminal("Enter administrator password of the parent domain");
		}
		croak(gettext("Parent domain's administrator password must be specified.")) if (not defined $ENV{ADM_PASSWD_PARENT});


		if (not defined $ENV{NDSEXISTINGADMINPASSWD} or $ENV{NDSEXISTINGADMINPASSWD} eq "") {
			$ENV{NDSEXISTINGADMINPASSWD} = read_pwd_terminal("Enter administrator password of the forest root domain");
		}
		croak(gettext("Forest root domain's administrator password must be specified")) if (not defined $ENV{NDSEXISTINGADMINPASSWD});


		$krb5ccname_domain = mktemp("/tmp/tmp.XXXXXXXXXX"); #Write to xad.ini
		$krb5ccname_parent = mktemp("/tmp/tmp.XXXXXXXXXX"); #   ''

		# By default we use KRB5CCNAME_PARENT. When TDO must be created in the
		# current domain, we will use KRB5CCNAME_DOMAIN also.
		$ENV{KRB5CCNAME} = $krb5ccname_parent;

		# Exporting parent domain administrator's password n ADM_PASSWD to get the
		# ticket of the parent domain administrator.
		$ENV{ADM_PASSWD} = $ENV{ADM_PASSWD_PARENT};
		$ENV{ADM_PRINCIPAL} = "Administrator\@".uc($parent_name);

		if (defined $ENV{XADPARENTDOMAINADMIN}) {
			my $admin_dn= $ENV{XADPARENTDOMAINADMIN};
			$admin_dn =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
			get_admin_sam_name($admin_dn);
			$ENV{ADM_PRINCIPAL} = "'$sam_account_name'\@".uc($parent_name);
		}

		my $rc = system("kinit -E $ENV{ADM_PRINCIPAL}");

		if ($rc != 0) {
			print STDERR "Could not get the ticket from the parent domain, Exiting..\n";
			my $count = unlink ($krb5ccname_domain, $krb5ccname_parent);
			exit (1);
		}

		# Exporting current domain administrator's password in ADM_PASSWD so that
		# further operations use current administrator's password.
		$ENV{ADM_PASSWD}=$ENV{ADM_PASSWD_DOMAIN};

		#Use ldapsearch to find if there is a TDO in the parent domain.
		#Set the variable TREE_ROOT if TDO is not found.
		#my $PARENT_NC = nds::ldapDNParent(XAD::domainToDC(lc($domain_name)));

		#$parent_name =~ /(.*?)\.(.*)$/;
		#my $PARENT_RELATIVE_DN= $1;  #`echo $PARENT_NAME | rev | sed 's/.*\.//g' | rev`
		#my $GRANDPARENT_DN=$2;

		#`/usr/bin/ldapsearch -Q -Y GSSAPI -h $PARENTDOMAINPROVIDER -b cn=$GRANDPARENT_DN,cn=system,$PARENT_NC  -LLL >/dev/null 2>&1`;
		#$ENV{TREE_ROOT} = ( $? == 0 ) ? 0:1;

		#We generate a random string and export this with the variable SHAREDSECRET
		#which will be used during the TDO creation.
		$ENV{SHAREDSECRET} =`/opt/novell/xad/sbin/xtrust -q --generate-random-string`;

		if (not defined $ENV{NDSEXISTINGADMINNAME} or $ENV{NDSEXISTINGADMINNAME} eq "") {
			#We create the tree root admin name if not exported
			my $PARENT_NC_DN="dc=$parent_name";
			$PARENT_NC_DN =~ s/\./.dc=/g;
			$ENV{NDSEXISTINGADMINNAME}="CN=Administrator.CN=Users.$PARENT_NC_DN";
		}
	}

	if (defined $mapped_container && $mapped_container ne "") {
		validate_mapped_container();
	}

	validateSITE();

	mkpath("/var/opt/novell/xad/log", 0, 0744);
}

#If the hostname is greater than 15 characters, /etc/hosts file
#should contain an alias name for the server, i.e. exactly
#15 characters in length.
sub UpdateHostName() {
	my $etc_hosts= '/etc/hosts';

	#Is Hostname greater than 15 characters?
	my $HostName = `/bin/hostname -s` or
		croak("/bin/hostname failed to get the short name\n");
	chomp($HostName);
	if (length($HostName) <= 15) {
		Log("Host is having a valid hostname [$HostName]");
		return;
	}

	#Form a hostname of exactly 15 characters
	my $valid_alias_name = substr($HostName, 0, 15);

	#Get the line to be replaced
	my @line_replaced = `grep "^[0-9].* \\+$HostName.*" $etc_hosts`;
	chomp(@line_replaced);

	#If alias list is null and a valid alias name is not present
	#then append a valid alias name
	foreach my $line (@line_replaced) {
		if ((grep { /\A$valid_alias_name\Z/i } split("\\s+", $line)) == 0) {
			util::replace_line_in_file ("$line",
					"$line $valid_alias_name",
					"$etc_hosts");
		}
	}
}

sub ValidateHostNameAndAddress
{
	my $guid;
	my @subauth;
	my @iaddrs=();
	my $iaddr="";
	my ($reversename, $fqdn);
	my $errorinfo = q(# You must ensure that in the YaST network settings for the
# active ethernet controller, the [Assign Hostname to Loopback IP]
# check box is deselected on a DSfW server. In the current server,
# the /etc/hosts file has a loopback IP address assigned with a
# hostname and this entry is commented now.);

	$localhostname = `/bin/hostname` || croak ("/bin/hostname failed\n");
	chomp($localhostname);
	my $HostName = $localhostname;

	$HostName =~ s/\..*//; # strip trailing domain

	@iaddrs = gethostbyname($HostName);
	if (!($#iaddrs+1)) {
		print STDERR sprintf(gettext("Could not resolve IP address for %s."), $HostName), "\n";
		exit 1;
	}

	foreach my $addr (@iaddrs) {
		if ((unpack("C4", $addr))[0] == 127) {
			print STDERR sprintf(gettext("IP address for %s is a loopback address; check /etc/hosts."), $HostName), "\n";
			print STDERR "$errorinfo\n";
			copy("/etc/hosts", "/etc/hosts.bkp") or
				croak("Failed to copy the existing /etc/hosts file.");
			$errorinfo =~ s/\n/\\n/g;
			$errorinfo =~ s/\//\\\//g;
			`sed -i -e 's/^127\\(.*\\)$localhostname\\(.*\\)/$errorinfo\\n#127\\1$localhostname\\2/g' /etc/hosts`;
			if(($?>>8) != 0) {
				print STDERR sprintf(gettext("Unable to disable the loopback address. Check the /etc/hosts file.")), "\n";
				next;
			}
		} else {
			# Found actual IP Address
			$iaddr = $addr;
		}
	}

	# All are loopback IP's or the actual IP Address is commented or invalid
	if (!defined($iaddr) or $iaddr eq "") {
		print STDERR sprintf(gettext("Invalid /etc/hosts file found on %s server."), $HostName), "\n";
		exit 1;
	}

	$IpAddress = inet_ntoa($iaddr); # requires Socket library
	$dns_server_ip = $IpAddress;

	$reversename = gethostbyaddr($iaddr, AF_INET);
	if (!defined($reversename) || !$reversename) {
		print STDERR sprintf(gettext("The IP address %s does not resolve to a name."), $IpAddress), "\n";
		print STDERR sprintf(gettext("Please modify /etc/hosts so that it resolves to %s and retry."), $HostName), "\n";
		exit 1;
	}

	$reversename = lc($reversename); # lower-case for comparison
	$fqdn = $HostName . "." . $domain_name;

	if ($reversename ne lc($fqdn)) {
		print STDERR sprintf(gettext("The IP address %s resolves to %s."),$IpAddress,$reversename), "\n";
		print STDERR sprintf(gettext("Please correct /etc/hosts so that it resolves to %s.%s."),$HostName,$domain_name), "\n";
		exit 1;
	}
}

sub get_dns_server_hostname() {
	my $fhostname = "";
	my $rhostname = "";
	my $value = "";
	my $error1 = "";
	my $error2 = "";
	
	Log ("Entering get_dns_server_hostname");

	my @output = `nslookup $dns_server_ip`;
	chomp(@output);
	Log ("Parsing using forest_name = $forest_name");
	($value) = grep (/$forest_name/i,@output);

	#Searching for error message of the following form
	#** server can't find 32.94.16.172.in-addr.arpa.: NXDOMAIN
	($error1) = grep (/\Q**/,@output);

	#Searching for error message of the following form
	#;; connection timed out; no servers could be reached
	($error2) = grep (/\;\;/,@output);


	if (!defined($value)) {
		if (defined($error1)) {
			print STDERR sprintf(gettext("Reverse lookup failed for server".
						" $dns_server_ip\nError:- %s"),$error1), "\n";
			Log("Reverse lookup failed for server".
						" $dns_server_ip\nError:- $error1");
			croak(gettext("exiting ..."));
			Log("exiting ...");
		} elsif (defined($error2)) {
			print STDERR sprintf(gettext("Error:- %sMake sure that the resolv.conf is ".
						"properly configured, and DNS servers are reachable.\n"),$error2), "\n";
			Log("Error:- %sMake sure that the resolv.conf is ".
						"properly configured, and DNS servers are reachable.$error2\n");
			croak(gettext("exiting ..."));
			Log("exiting ...");
		} else {
			croak(gettext("Please check your configuration.\nVerify that $dns_server_ip ".
						"is a suitable DNS server for the domain $forest_name. exiting ..."));
			Log("Please check your configuration.\nVerify that $dns_server_ip ".
						"is a suitable DNS server for the domain $forest_name. exiting ...");
		}
	} else {
		chomp($value);

		$fhostname = `echo $value | awk -F \"name = \" '{print \$2}'`;
		chomp($fhostname);
		$rhostname = `echo $fhostname | awk -F \".\" '{print \$1}'`;
		chomp($rhostname);
	}
	return ($fhostname, $rhostname);

}

sub validate_dns {

	ValidateHostNameAndAddress();
	Log ("Entering validate_dns");

	if ( not defined $dns_locator_obj_context) {
		print "Enter the context of the DNS Locator object:";
		chomp( $dns_locator_obj_context = <STDIN> );
	}
	croak(gettext("Context of the DNS Locator object must be specified.")) if ( not defined $dns_locator_obj_context);
	$dns_locator_object = "cn=DNS-DHCP,$dns_locator_obj_context";

	if (not defined $dnsdhcp_grp_context) {
		print "Enter the context of the DNSDHCP-Group object:";
		chomp ($dnsdhcp_grp_context = <STDIN>);
	}
	croak(gettext("Context of the DNSDHCP-Group object must be specified.")) if (not defined $dnsdhcp_grp_context);
	$dnsdhcp_group = "cn=DNSDHCP-GROUP,$dnsdhcp_grp_context";



	if ($configure_dns == 1) {

		if (not defined $dnsserver_context) {
			print "Enter the context of the DNS server object:";
			chomp($dnsserver_context = <STDIN>);
		}
		croak(gettext("Context of the DNS server object must be specified.")) if (not defined $dnsserver_context);

		$dnsserver = "cn=DNS_$localhostname,$dnsserver_context";
		$domain_nc =~ s/,/\./g;
		my $LDNS_NCP_Server = `$Ldapsearch -h $dns_server_ip -b "$domain_nc" -s sub "(objectclass=NCP Server)" dn -LLL 2>/dev/null | sed -e $sedPattern -e \'s/st=/s=/gi\' | grep -i "cn=$localhostname" | awk -F "dn: " '{print \$2}'`;
		chomp($LDNS_NCP_Server);
		$dns_ncpserver=$LDNS_NCP_Server;



	} else {
		if (not defined $preferred_dns) {
			print "Enter the IP address of the preferred DNS server:";
			chomp($preferred_dns = <STDIN>);
		}
		croak(gettext("IP address of the preferred DNS server must be specified.")) if (not defined $preferred_dns);

		if ($domain_name ne $forest_name || $is_replica eq "true") 
		{
		# this is not the FRD, and not to be configured as the DNS server
		# check for the preferred DNS server, check if the preferred DNS
		# server is being hosted on a DSfW server or not. If not fall back
		# on the /etc/resolv.conf method
			my $rc = 0;
			my $value = 0;

			$rc = system("/usr/bin/ldapsearch -h $preferred_dns -b \"\" -s base supportedCapabilities -x -LLL | grep '^supportedCapabilities: ' | grep '2.16.840.1.113719.1.513.7.1' >/dev/null 2>&1");
			if ($rc == 0)
			{
				# this is the DSfW server to talk to for DNS related transations
				$dns_server_ip = $preferred_dns;
			}
			else 
			{
				# the server passed as the preferred DNS server is not a DSfW server
				# hence we must look for DNS servers from /etc/resolv.conf, query for
				# the domain name, check if it contains domain name as $FOREST_NAME
				# if so, that $IP_address would point to the correct DSfW server hosting
				# the DNS server.

				# If so, take the IP address and do a rootDSE search to get the mapped
				# or non-named mapped domain NC, then create the DNS server name by
				# prepending the DNS_$machinename,$obtained_nc  

				# calculate the number of nameserver entries
				my $nsentries=`cat /etc/resolv.conf | grep 'nameserver' | awk -F \"nameserver \" '{print \$2}' | wc -l`;
				chomp($nsentries);
				if ($nsentries >= 1) 
				{
					# get each NS entry's IP address and do a nslookp. Look
					# for the FOREST_NAME in the output, if found this is our
					# designated server, keep the hostname and the domain name,
					# and the IP address as well
					my $match_found = 0;
					my @ip_addresses = `cat /etc/resolv.conf | grep 'nameserver' | awk -F \"nameserver \" '{print \$2}'`;
					foreach  my $ip_address (@ip_addresses) 
					{
						if ($match_found == 1) 
						{
							# match found for the FOREST_NAME, break the loop
							last;	
						}
						chomp($ip_address);
						$value =`nslookup "$ip_address" | grep $forest_name`;
						if ( $value ne "")
						{
							# set the MATCH_FOUND flag
							$match_found = 1;
							# set the IP as the DNS server IP
							$dns_server_ip = $ip_address;
							#match found, so break the loop
							last;
						}
					}
				}
			}
			# get the REMOTEHOSTNAME and DOMAINNAME of the server hosting the DNS
			# server
			Log ("Getting hostname details");
			my ($fullhostname, $remotehostname) = get_dns_server_hostname();
			Log ("fullhostname = $fullhostname,remotehostname = $remotehostname");

			# Updating LDAP Server object with extensionInfo for dsearchplugin
			my $default_container = `/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
			if ($? == 0) {
				chomp($default_container);
			}
			else {
				$default_container = "ou=OESSystemObjects";
			}

			# got the REMOTEHOSTNAME, DOMAINNAME and IP_address of the server hosting
			# DNS server of the domain hierarchy. Now use ndsstat -h <$DNS_SERVER_IP>
			# to get the location of NCP server and hence the non name mapped context
			# as DOMAIN_NC
			# check if server is reachable via ndsstat and able to get DOMAIN_NC
			$rc = system("/opt/novell/eDirectory/bin/ndsstat -h $dns_server_ip >/dev/null 2>&1");
			if ($rc != 0)
			{
				croak(gettext ("Cannot contact the preferred DNS server $dns_server_ip. Check if eDirectory is running properly."));
			}
			else
			{
				$domain_nc=`/opt/novell/eDirectory/bin/ndsstat -h $dns_server_ip | grep \"^Server Name:\" | sed -e 's/^Server Name: //' | awk -F \".T=\" '{print \$1}'`;
				$domain_nc =~ s/\.CN=([^\.]*)\.//i;
				$domain_nc =~ s/\.$//;
				$domain_nc =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
				chomp($domain_nc);
			}

			# check if REMOTEHOSTNAME is obtained, otherwise error out
			if ($remotehostname eq "")
			{
				croak(gettext ("Could not contact the preferred DNS server $dns_server_ip. exiting.."));
			}

			# get the DNS server name now
			$dnsserver="cn=DNS_".$remotehostname.",".$domain_nc;

			# get the DNS server context now
			$dnsserver_context=$domain_nc;

			# The DNS container should be below the mapped container. Not anywhere in tree
			my $search_context = `echo $dnsserver_context | awk -F "$default_container," '{print \$2}'`;
			my $RDNS_NCP_Server = `$Ldapsearch -h $dns_server_ip -b "$search_context" -s sub "objectclass=NCP Server" dn -E "2.16.840.1.113719.1.513.4.5" -LLL 2>/dev/null | sed -e $sedPattern -e 's/st=/s=/gi' | grep -i "cn=$remotehostname" | awk -F "dn: " '{print \$2}'`;
			chomp($RDNS_NCP_Server);

			#my $RDNS_Server = `$Ldapsearch -h $dns_server_ip -b "$search_context" -s sub "objectclass=DNIP:DNS Server" dn -N -LLL 2>/dev/null | sed -e $sedPattern -e 's/st=/s=/gi' | grep -i "cn=DNS_$remotehostname" | awk -F "dn: " '{print \$2}'`; 
			#chomp($RDNS_Server);
			#if ($RDNS_Server eq "")
			#{
			#	croak(gettext ("Could not obtain DNS server context from $dns_server_ip. exiting.."));
			#}
			#my $RDNS_Server_Context = `echo $RDNS_Server | awk -F "cn=DNS_$remotehostname," '{print \$2}'`;
			#chomp($RDNS_Server_Context);

			#$dnsserver=$RDNS_Server;
			#$dnsserver_context=$RDNS_Server_Context;
			$dns_ncpserver=$RDNS_NCP_Server;
		}
	}
	if (not defined $dnsserver_context or $dnsserver_context eq "") {
		$domain_nc=`/opt/novell/eDirectory/bin/ndsstat -h $dns_server_ip | grep \"^Server Name:\" | sed -e 's/^Server Name: //' | awk -F \".T=\" '{print \$1}'`;
		Log("dnsserver_context is empty. Retrieving using ndsstat command :");
		Log("/opt/novell/eDirectory/bin/ndsstat -h $dns_server_ip | grep \"^Server Name:\" | sed -e 's/^Server Name: //' | awk -F \".T=\" '{print \$1}'");
		$domain_nc =~ s/\.CN=([^\.]*)\.//i;
		$domain_nc =~ s/\.$//;
		$domain_nc =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
		chomp($domain_nc);

		$dnsserver_context=$domain_nc;
	}
	if (not defined $dnsserver or $dnsserver eq "") {
		$dnsserver = "cn=DNS_$localhostname,$dnsserver_context";
	}
	if ($dnsserver eq "" || $dnsserver_context eq "")
	{
		croak(gettext ("The preferred DNS server $dns_server_ip failed to return the DNS Server context."));
	}

	$dns_locator_object =~ s/st=/s=/gi;
	$dns_locator_obj_context =~ s/st=/s=/g;
	$dnsdhcp_group =~ s/st=/s=/gi;
	$dnsserver =~ s/st=/s=/gi;
	$dns_ncpserver =~ s/st=/s=/gi;
	$dnsserver_context =~ s/st=/s=/gi;
}

sub validate_domain_vars {

	#Validate the domain name
	if (not defined $domain_name) {
		ReadDomainName();
		if ($domain_name eq "") {
			print "Enter the domain name of the domain to be configured:";
			chomp ($domain_name = <STDIN>);
		}
	}
	croak(gettext("Domain name must be specified.")) if $domain_name eq "";
	croak(gettext("The domain name must have a periods it.")) if !($domain_name =~ /\./);

	#As per DNS standard (RFC 882 and RFC 883), maximum length of a FQDN name is 255 chars. 
	#This includes host name which can be of 63 chars max.
	#We store domain name as one object (with '.' replaced by '_') in dns records. Also since max RDN as CN
	#(in our case it is CN) can be of 63 chars. Max length of domain name possible is limited to 63 chars. 
	#This limitation is because of Novell DNS which stores records in this format.

	$Length_DOMAINNAME = 63; #As mentioned in above comment.
	if (length($domain_name) > $Length_DOMAINNAME ) {
		print sprintf(gettext("Domain name must be %s characters or less."), $Length_DOMAINNAME);
		exit 1;
	}


	#Each name in the domain should not contain more than 63 characters
	my @DnsLabels = split (/\./,$domain_name);
	foreach my $Label (@DnsLabels) {
		if (length($Label) > 63) {
			croak(gettext("Domain name Label must be 63 characters or less."));
		}
	}


	# Valid characters are [A-Z][0-9]-\. (case-insensitive)
	if ($domain_name =~ /[^A-Za-z0-9-\.]/) {
		croak(gettext("The domain name contained invalid characters."));
	}

	$domain_name = lc($domain_name);

	if (not defined $parent_name) {
		print "Enter the parent domain name(If its a forest root domain enter the domain name itself):";
		chomp ($parent_name = <STDIN>);
	}
	croak(gettext("Parent Domain name must be mentioned.")) if not defined $parent_name;
	croak(gettext("The parent domain name must have a periods it.")) if !($parent_name =~ /\./);

	if ($parent_name =~ /[^A-Za-z0-9-\.]/) {
		croak(gettext("The parent domain name contained invalid characters."));
	}

	$parent_name = lc($parent_name);

	if (not defined $forest_name) {
		print "Enter the forest root domain name(If its a forest root domain enter the domain name itself):";
		chomp ($forest_name = <STDIN>);
	}

	croak(gettext("Forest Name must be mentioned.")) if not defined $forest_name;
	croak(gettext("The forest root name must have a periods it.")) if !($forest_name =~ /\./);

	if ($forest_name =~ /[^A-Za-z0-9-\.]/) {
		croak(gettext("The forest root domain name contained invalid characters."));
	}

	if (length($forest_name) > 120) {
		croak(gettext("Forest name must be 120 characters or less."));
	}

}


#Validate the containers present in the domain partition
sub validateDomainContainer {
	my ($adminDN, $ipAddress, $error) = 0;
	my $tempdir = registry::getReg("TEMPDIR");

	if ((registry::getReg('MAPPED')) && ($is_replica eq "false") && (! -e $tempdir."/ndsdcinit_container_check")) {

		$adminDN = registry::getReg('NDSADMINNAME'); 

		$ipAddress = registry::getReg("IP Address");

		$error = helpers::topLevelContainersCheck($ipAddress, $adminDN, 636);
		if ($error != 0) {
			Log(ERR, "The domain partition container is not valid");	
		}
		`touch $tempdir/ndsdcinit_container_check`;
	}
}

#Validate Replica Status for Time Synchronization point of view.
sub validateReplica {
	my ($adminDN, $password, $partition, $error, %listOfPartitions);
	$adminDN = registry::getReg("NDSADMINNAME");
	if ($is_replica eq "true") {
		$password = $ENV{ADM_PASSWD};
	} elsif ($type eq "frd") {
		if(registry::getReg("MAPPED")) {
			$password = $ENV{NDSEXISTINGADMINPASSWD};
		} 
		else {
			$password = $ENV{ADM_PASSWD};
		}			
	} elsif ($type eq "child") {
		$password = $ENV{NDSEXISTINGADMINPASSWD};
	}	
	($error, %listOfPartitions) = helpers::partitionList();
	if ($error ne 0) {
		Log(ERR, "Failed to get the list of partitions ");
	}
	foreach my $partition (keys %listOfPartitions) {
		chomp($partition);
		my $error = helpers::replicaStatus($adminDN, $password, $listOfPartitions{$partition}, $partition);
		Log(ERR, "Replica status check failed for partition [$partition]. For more detail see Healthcheck log.") if ($error);
	}
}

sub validateSITE() {
	Log(INFO, "PREFERRED SITE : $preferred_site");

	#SITE name cannot contain characters { | }~[]':;<=>?@!"#$%^&`()+/\,*.
	#SITE Name Validation
	if (not defined $preferred_site or $preferred_site eq "") {
		print "Enter the SITE name of the domain to be configured:";
		chomp($preferred_site = <STDIN>);
	}

	croak(gettext("SITE name must be specified.")) if (not defined $preferred_site);
	croak(gettext("SITE name must not have a tilde in it.")) if ($preferred_site =~ /~/);
	croak(gettext("SITE name must not have a number sign in it.")) if ($preferred_site =~ /#/);
	croak(gettext("SITE name must not have a percent in it.")) if ($preferred_site =~ /%/);
	croak(gettext("SITE name must not have an apmersand in it.")) if ($preferred_site =~ /&/);
	croak(gettext("SITE name must not have an asterisk in it.")) if ($preferred_site =~ /\*/);
	croak(gettext("SITE name must not have a block begin brace in it.")) if ($preferred_site =~ /\{/);
	croak(gettext("SITE name must not have a block end brace in it.")) if ($preferred_site =~ /\}/);
	croak(gettext("SITE name must not have a back slash in it.")) if ($preferred_site =~ /\\/);
	croak(gettext("SITE name must not have a colon in it.")) if ($preferred_site =~ /:/);
	croak(gettext("SITE name must not have a begin angle bracket in it.")) if ($preferred_site =~ /</);
	croak(gettext("SITE name must not have an end angle bracket in it.")) if ($preferred_site =~ />/);
	croak(gettext("SITE name must not have a question mark in it.")) if ($preferred_site =~ /\?/);
	croak(gettext("SITE name must not have a slash in it.")) if ($preferred_site =~ /\//);
	croak(gettext("SITE name must not have a plus sign in it.")) if ($preferred_site =~ /\+/);
	croak(gettext("SITE name must not have a pipe in it.")) if ($preferred_site =~ /\|/);
	croak(gettext("SITE name must not have a quotation mark in it.")) if ($preferred_site =~ /"/);
	croak(gettext("SITE name must not have an underscore at the begining.")) if ($preferred_site =~ /^_/);
	croak(gettext("SITE name must not have a period at the begining.")) if ($preferred_site =~ /^\./);
	croak(gettext("SITE name must not have a period at the end.")) if ($preferred_site =~ /\.$/);
	croak(gettext("SITE name must not have an opening bracket in it.")) if ($preferred_site =~ /\[/);
	croak(gettext("SITE name must not have a closing bracket in it.")) if ($preferred_site =~ /]/);
	croak(gettext("SITE name must not have a single quote in it.")) if ($preferred_site =~ /'/);
	croak(gettext("SITE name must not have a semicolon in it.")) if ($preferred_site =~ /;/);
	croak(gettext("SITE name must not have an equal in it.")) if ($preferred_site =~ /=/);
	croak(gettext("SITE name must not have an at in it.")) if ($preferred_site =~ /@/);
	croak(gettext("SITE name must not have an exclamation in it.")) if ($preferred_site =~ /!/);
	croak(gettext("SITE name must not have a dollar in it.")) if ($preferred_site =~ /\$/);
	croak(gettext("SITE name must not have a caret in it.")) if ($preferred_site =~ /\^/);
	croak(gettext("SITE name must not have a backtick in it.")) if ($preferred_site =~ /`/);
	croak(gettext("SITE name must not have an opening paranthesis in it.")) if ($preferred_site =~ /\(/);
	croak(gettext("SITE name must not have a closing paranthesis in it.")) if ($preferred_site =~ /\)/);
	croak(gettext("SITE name must not have a comma in it.")) if ($preferred_site =~ /,/);
	croak(gettext("SITE name must not have a space in it.")) if ($preferred_site =~ / /);
}

# Update xad.ini for some Administrator DNs.
sub update_xad_conf() {
	#By this time we have 'Mapped Domain NC' registry created in installation scenarios.
	$mapped_container = registry::getReg("Mapped Domain NC");
	$mapped_container =~ s/,/\./g;
	$mapped_container =~ s/st=/s=/gi;
	my ($ldapsearch, $parent_domain_provider, $forest_domain_provider, $actual_forest_nc, $parent_domain_nc);

	open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");
	
	my $nds_domain_admin = registry::getReg("Mapped Administrator DN");
	$nds_domain_admin =~ s/,/\./g;
	$nds_domain_admin =~ s/st=/s=/gi;
	print CONFIG "NDSDOMAINADMIN = $nds_domain_admin\n";

	if ($type eq "child" && $is_replica eq "false") {
		my $admin_name = "Administrator";
		if (defined $ENV{XADPARENTDOMAINADMIN}) {
			my $parent_admin_dn = $ENV{XADPARENTDOMAINADMIN};
			$parent_admin_dn =~ /cn=([^\.]*)/i;
			$admin_name = $1;	
		}
	
		if ($mapped eq "true") {
			# Fetch the Actual Forest NC
			$forest_domain_provider=`/opt/novell/xad/sbin/provision -q -q -l 60 --locate-dc $forest_name`;
			if ($? != 0) {
				my $error = $? >> 8;
				die("Failed to get the domain provider for domain [$forest_name] with error [$error]");
			}
			chomp($forest_domain_provider);

			$ldapsearch = "/opt/novell/eDirectory/bin/ldapsearch -x";

			$actual_forest_nc=`$ldapsearch -h $forest_domain_provider -b "" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | grep -i "^namingContexts: cn=Configuration" | awk -F "namingContexts: cn=Configuration," '{print \$2}'`;
			chomp($actual_forest_nc);

			# Fetch the Parent Domain NC
			my $ldapadminname = $ENV{NDSEXISTINGADMINNAME};
			$ldapadminname =~ s/\.([a-z,A-Z,0-9]*)=/,$1=/g;
			chomp($actual_forest_nc);
			$parent_domain_nc=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $ldapsearch -h '$forest_domain_provider' -D '$ldapadminname' -w '$ENV{NDSEXISTINGADMINPASSWD}' -b 'cn=partitions,cn=configuration,$actual_forest_nc' '(&(objectclass=crossref)(dnsRoot=$parent_name)(systemflags=3))' nCName -N -Z -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //; }; ' | sed 's/st=/s=/gi' | grep -i "^nCName: " | awk -F "^nCName: " '{print \$2}'`;
			chomp($parent_domain_nc);

			if ($parent_domain_nc ne "") {
				$parent_domain_nc =~ s/,/\./g;
				$parent_domain_nc =~ s/st=/s=/gi;
				print CONFIG "NDSPARENTDOMAINADMIN = cn=$admin_name.cn=Users.$parent_domain_nc\n";
			} else {
				Log(WARN, "Could not fetch the Parent Domain NC. It is used during provisioning authentication");
			}

		} elsif ($mapped eq "false") {
			$parent_domain_nc = "dc=$parent_name";
			$parent_domain_nc =~ s/\./\.dc=/g;
			print CONFIG "NDSPARENTDOMAINADMIN = cn=$admin_name.cn=Users.$parent_domain_nc\n";
		}	
	}
	close(CONFIG);
}

sub log_variable {
print <<VAR;

	ENV PATH = $ENV_PATH
	_LIB = $_LIB
	LD LIBRARY PATH = $LD_LIBRARY_PATH
	SASL PATH = $SASL_PATH
	DCINIT CONFIG: $DCINIT_CONFIG
	DOMAIN NAME: $domain_name
	PARENT NAME: $parent_name
	FOREST NAME: $forest_name
	NETBIOS NAME: $netbios_name

	DOMAIN GUID: $domain_guid
	DOMAIN SID: $domain_sid
	MACHINE GUID: $machine_guid

	DNS LOCATOR OBJECT: $dns_locator_object
	DNS DHCP GROUP:  $dnsdhcp_group
	DNS SERVER: $dnsserver
	DNS NCP SERVER: $dns_ncpserver
	DNS SERVER CONTEXT: $dnsserver_context
	DNS LOCATOR CONTEXT: $dns_locator_obj_context
VAR
	
	#Push some of the variable to the registry so that they can be accisible globally
	$ENV{'DCINIT_CONFIG'} = $DCINIT_CONFIG;
	if ($Nds == 1) {
		$ENV{'NDS'} = "TRUE";
	}
	else {
		$ENV{'NDS'} = "FALSE";
	}

	$ENV{'DOMAIN_NAME'} = $domain_name;
	$ENV{'FOREST_NAME'} = $forest_name;
	$ENV{'PARENT_NAME'} = $parent_name;

	$ENV{'DNS_LOCATOR_OBJECT'} = $dns_locator_object;
	$ENV{'DNSDHCP_GROUP'} = $dnsdhcp_group;
	$ENV{'DNSSERVER'} = $dnsserver;
	$ENV{'DNS_NCPServer'} = $dns_ncpserver;
	$ENV{'DNSSERVER_CONTEXT'} = $dnsserver_context;
	$ENV{'DNS_SERVER_IP'} = $dns_server_ip;
	$ENV{'NDS_DCINIT'} = $NDS_DCINIT;
	$ENV{'NAME_MAPPED_FRD'} = 0;

# 	if ($type ne "adc") { registry::setReg("IS_REPLICA", "FALSE"); }
# 	else { registry::setReg("IS_REPLICA" , "TRUE"); }

#	$ENV{'NDS'} = 1;
}

sub decide_type {
	my $action = shift;

	if ($action eq "replica" or $action eq "full-replica") {
		$is_replica = "true";
	}
	else {
		$is_replica = "false";
	}
	if ($domain_name eq $forest_name) {
		$type = "frd";
	}
	else {
		$type = "child";
	}
	if (defined $mapped_container && $mapped_container ne "") {
		$mapped = "true";
	}
	else {
		$mapped = "false";
	}
}

# Upgrade the DSfW 
sub upgrade_dc
{
	my $inst = "";

	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");

	#fips disable
	#Stop the ndsd service
	#util::stop_ndsd();

	#`pgrep ndsd`;
	#if ($? == 0) {
	#	`pkill -9 ndsd`;
	#}

	#Replace the exact environment variable
	#`/usr/bin/grep -i "^n4u.server.fips_tls=" /etc/opt/novell/eDirectory/conf/nds.conf`;
	#if($? == 0){
	#	`/usr/bin/sed -i -e 's/^n4u.server.fips_tls=[0-9]/n4u.server.fips_tls=0/g' /etc/opt/novell/eDirectory/conf/nds.conf`;

	#	if(($?>>8) != 0) {
	#		Log(WARN, "Unable to disable the fips mode. Check the /etc/opt/novell/eDirectory/conf/nds.conf file.");
	#	}
	#}

	#Start the ndsd service
#	util::start_ndsd();

	if (registry::getReg("DNS Master") eq "TRUE") {
		util::novell_named_restart();
		`/usr/bin/systemctl status novell-named.service`;

		if ($?) {
			Log(ERR, "Failed to start novell-named");
		}
	}

	util::load_dsearch_plugin();

	nds::wait_for_samspm();

	util::refresh_nldap_server();

	if ($ENV{ADM_PASSWD} ne "") {
		if (not defined $domain_name or $domain_name eq "") {
			$domain_name = `/bin/dnsdomainname` || croak ("/bin/dnsdomainname failed\n");
			chomp($domain_name);
		}
		if(not defined $forest_name or $forest_name eq "") {
			Log("forest_name is empty, retrieving it from registry variable ...");
			$forest_name = registry::getReg("Forest Root");
		}

		my $admin_principal = "Administrator\@".uc($domain_name);
		my $admin_dn = `/opt/novell/xad/share/dcinit/provisionTools.sh get-admin-name`;
		chomp($admin_dn);

		if ($admin_dn ne "") {
			get_admin_sam_name($admin_dn);
			$admin_principal = "'$sam_account_name'\@".uc($domain_name);
		}
		else {
			Log ("Couldn't get the administrator DN, using [$sam_account_name]");
		}


#Added for OES2018 upgrade
		my $templatedir = registry::getReg("TEMPLATEDIR");
		my $stagedir = registry::getReg("STAGEDIR");
		my $xadroot = registry::getReg("XADROOT");

		mkpath( $stagedir."/usr/lib/systemd/system" );
		mkpath( $stagedir."/opt/novell/xad/bin" );
                mkpath( $stagedir."/opt/novell/xad/sbin" );

		Log("   Creating krb5 systemd service in main");
		if(-e "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service"){
                	Log("   Staging krb5 systemd again, in main");
			`/usr/bin/rm $stagedir/usr/lib/systemd/system/xad-krb5kdc.service`;
		}
		stage::reg_substitute("$templatedir/xad-krb5kdc.service", "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service");

		if (util::chkfor_reg("$stagedir/usr/lib/systemd/system/xad-krb5kdc.service") == 0) {
			die("Not all values are replaced in the $stagedir/usr/lib/systemd/system/xad-krb5kdc.service");
		}

                Log("   Creating krb5 start script in main");
		if(-e "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc"){
                	Log("   Staging krb5 start script again, in main");
			`/usr/bin/rm $stagedir/opt/novell/xad/sbin/start-xad-krb5kdc`;

                }
		
                stage::reg_substitute("$templatedir/start-xad-krb5kdc", "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc");

	        if (util::chkfor_reg("$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc") == 0) {
        	          die("Not all values are replaced in the $stagedir/opt/novell/xad/sbin/start-xad-krb5kdc");
		}

		`ln -sf /usr/sbin/service $xadroot/sbin/rcxad-krb5kdc`;
		chmod 0644, "$stagedir/usr/lib/systemd/system/xad-krb5kdc.service";
                chmod 0755, "$stagedir/opt/novell/xad/sbin/start-xad-krb5kdc";
		`cp $stagedir/usr/lib/systemd/system/xad-krb5kdc.service /usr/lib/systemd/system/xad-krb5kdc.service`;
		`cp $stagedir/opt/novell/xad/sbin/start-xad-krb5kdc /opt/novell/xad/sbin/start-xad-krb5kdc`;
		`/usr/bin/systemctl daemon-reload`;
#end of 2018
		my $rc = 0;
		my $count = 5; 

		do {
			`pgrep krb5kdc`;
			if ($?) {
				Log ("The xad-krb5kdc is not running, going to start it.");
				`/usr/bin/systemctl restart xad-krb5kdc.service`;
				sleep 2;
			}

			Log ("Executing .. /opt/novell/xad/bin/kinit -E $admin_principal");

			$rc = system("/opt/novell/xad/bin/kinit -E $admin_principal");

			#Return value 2 for wrong password
			#Return value 3 for expired password
			#Return value 1 for general kinit error
			if($rc == 2 || $rc == 3) {
				#No point iterating, 
				#if password is wrong or expired
				last;
			}

			if ($rc != 0) {
				Log ("Could not get the ticket now, retrying");
				$count--;
				util::refresh_nldap_server();
			}

			
		}while (($count >= 0) && ($rc != 0)) ;

		#$rc exit inherited from kinit return value
		if ($rc != 0) {
			my $kinit_error = `/opt/novell/xad/bin/kinit -E $admin_principal`;
			chomp($kinit_error);

			Log("kinit error : $kinit_error");

			#Exit value 2 for wrong password
			#Exit value 3 for expired password
			#Exit value 1 for general kinit error
			exit $rc;
		}
	} 
	else {
		Log("Domain Administrator password is not available.");
		exit 2;
	}

	if (-e $DCINIT_CONFIG) {
		copy($DCINIT_CONFIG, "$DCINIT_CONFIG.temp") or die("Failed to copy the existing xad.ini file.");
	}
	else {
		die("Could not find xad.ini file.");
	}

	$localhostname = `/bin/hostname` || croak ("/bin/hostname failed\n");
	chomp($localhostname);

	#Finds the domain[controller] type
	decide_type($action);
	Log ("Validating DNS");
	validate_dns();
	my ($upgrade_from , $upgrade_to) = util::get_upgrade_from_to();
	open (CONFIG, "<$DCINIT_CONFIG") or die ("Could not open the file $DCINIT_CONFIG");
	if (grep (/^UPGRADE.*/,<CONFIG>)) {
		close CONFIG;
		util::replace_line_in_file("UPGRADE_FROM ",
			"UPGRADE_FROM = $upgrade_from", $DCINIT_CONFIG);
		util::replace_line_in_file("UPGRADE_TO ", "UPGRADE_TO = $upgrade_to", $DCINIT_CONFIG);
		open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");
	}
	else {
		open(CONFIG,">>$DCINIT_CONFIG") || croak("could not open $DCINIT_CONFIG\n");
		print CONFIG "UPGRADE = YES\n";
		print CONFIG "UPGRADE_FROM = $upgrade_from\n";
		print CONFIG "UPGRADE_TO = $upgrade_to\n";
	}

	if ($upgrade_from eq "sp1") {
		print CONFIG "DNS_LOCATOR_OBJECT = $dns_locator_object\n";
		print CONFIG "DNSDHCP_GROUP = $dnsdhcp_group\n";
		print CONFIG "DNSSERVER = $dnsserver\n";
		print CONFIG "DNS_NCPServer = $dns_ncpserver\n";
		print CONFIG "DNSSERVER_CONTEXT = $dnsserver_context\n";
		print CONFIG "DNS_SERVER_IP = $dns_server_ip\n";
		print CONFIG "INSTALL_TYPE = $type\n";

		if (defined $ENV{NKDC_REALM} && $ENV{NKDC_REALM} ne "") {
			print CONFIG "NKDC Realm = $ENV{NKDC_REALM}\n";
		}

		if ($type eq "full-replica") {
			print CONFIG "CONFIG_SCHEMA_REPLICAS = TRUE\n";
		}
		close(CONFIG);

		# Update xad.ini for Administrator DNs.
		$parent_name = registry::getReg("Parent Domain");
		update_xad_conf();

	}
	else {
		close(CONFIG);
	}

	#Write new xadsd location in a cross partition NTP setup
	util::write_newxadsd_location();


	if ($retry == 0) {
		stage::stage_clean();
	}

	Log("Start of DSfW upgrade\n");


	if ($is_replica eq "true") {
		require Install::adc_install;
		$inst = adc_install->new();
	}
	else {
		if ($type eq "frd") {
			chdir("/opt/novell/xad/share/dcinit");
			$inst = frd_install->new();
		}
		else {
			$inst = child_install->new();
		}
	}

	# STAGING
	$inst->stage_dir();

	$inst->stage_rc(); #Generates rc script file
	$inst->stage_schema_prepare(); #generate schema
	$inst->schema_aggregate();
	$inst->generate_upgrade_ldifs(); #Generating upgrade related ldif files
	$inst->stage_to_live();

	#Creates a link for keytab. Samba expects the keytab in default location
	`ln -sf /var/opt/novell/xad/ds/krb5kdc/krb5.keytab /etc/`;

	# SCHEMA EXTENSIONS
	registry::setReg("NDSADMINNAME", $ENV{NDSEXISTINGADMINNAME});
	if (not defined $ENV{NDSTREENAME} or $ENV{NDSTREENAME} eq "") {
		registry::setReg("NDSTREENAME", `/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`);
		Log("Registry entry for NDSTREENAME is being populated. Command used is:");
		Log("/opt/novell/xad/share/dcinit/ndsConfigGet.sh \"n4u.base.tree-name\"");
	}

	if (defined $ENV{NDSEXISTINGADMINPASSWD} && $ENV{NDSEXISTINGADMINPASSWD} ne "") {
		$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
	}

        `/opt/novell/eDirectory/bin/ldapsearch -s base -b cn=schema attributetypes=uniquedomainid -LLL >&/dev/null`;
	if ($?) {
		Log(MESG, "Schema not synched to the local server, extending it locally");
		$inst->extend_nam_schema();
		$inst->extend_schema();
	}
	# ldapsearch checks for an attribute which is unique to AD schema level v0319
	`/opt/novell/eDirectory/bin/ldapsearch -s base -b cn=schema attributetypes=msDS-preferredDataLocation -LLL >&/dev/null`;
	if ($?) {
		Log(MESG, "Schema 2019 not synched to the local server, extending it locally.");
		$inst->extend_schema();
	}else{
		Log(MESG, "Schema 2019 already synched to the local server.");
	}

	#Configure nmas Methods
	if($type eq "frd" && $is_replica eq "false") {
		#Configure EXTERNAL NMAS method
		nds::configure_nmas_ipc_method();
		#Configure NEGOTIATE NMAS method
		nds::configure_nmas_spnego_method();
		#Configure KERBEROS NMAS method 
		nds::configure_nmas_mskrb_method();
	}
	nds::trigger_backlinker();

	$inst->stage_clean();

	#We need to fix the dns data before we proceed
	upgrade_dns_data();
}

sub main {

	my %options=();
	$action = shift @ARGV;
	my $inst;
	Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");

	if ($action eq "retry") {
		$retry = 1;
		main (@ARGV);
	}elsif (lc($action) eq "upgrade") {
		$upgrade = 1;
		main (@ARGV);
	}
	else {

		if (not ($action eq "replica" or $action eq "full-replica" or $action eq "domain")) {
			#The action is nither replica, full-replica nor domain. Exit.
			Usage();
		}

		getopts("d:p:f:m:n:l:g:e:z:swto:", \%options) or croak("Unexpected arguments");

		$domain_name = $options{d} if defined $options{d};
		$parent_name = $options{p} if defined $options{p};
		$forest_name = $options{f} if defined $options{f};
		$mapped_container = $options{m} if defined $options{m};
		$netbios_name = $options{n} if defined $options{n};
		$configure_dns = defined $options{s}? 1:0;
		$dns_locator_obj_context = $options{l} if defined $options{l};
		$dnsdhcp_grp_context = $options{g} if defined $options{g};
		$dnsserver_context = $options{e} if defined $options{e};
		$preferred_dns = $options{o} if defined $options{o};
		$preferred_site = $options{z} if defined $options{z};
		$wins_configured = defined $options{w}? "YES" : "NO";
		$tls_option = $options{t} if defined $options{t};		
		Usage if $ARGV[0]; # Un expected arguments

		if ($upgrade == 1) {
			upgrade_dc();
			exit 0;
		}

		validate_domain_vars();
		bootstrap_variable();
		decide_type($action);
		validate_dns();
		validate_common();
		UpdateHostName();
		DumpConfigToFile();
		log_variable; #This also exports the required variable

		Logger::set_filename("/var/opt/novell/xad/log/ndsdcinit.log");

		if ($retry == 0) {
			stage::stage_clean();
		}

		Log("Start of DSfW installation and provisioning\n");

		if ($is_replica eq "true") {
			require Install::adc_install;
			$inst = adc_install->new();
		}
		else {
			if ($type eq "frd") {

				#`touch /var/opt/novell/xad/log/ndsdcinit.log`;
				chdir("/opt/novell/xad/share/dcinit");

				$inst = frd_install->new();
			}
			else {
				$inst = child_install->new();
			}
		}

		#Write new xadsd location in a cross partition NTP setup
		util::write_newxadsd_location();

		# STAGING
		$inst->stage_dir();
		$inst->stage_default_config();

		# Update xad.ini for Administrator DNs.
		update_xad_conf();

		$inst->stage_domainguid();
		if ($is_replica eq "true") {
			$inst->stage_domainsid();
		}
		$inst->stage_libldap();
		$inst->stage_rc();
		$inst->stage_linkengine();
		$inst->stage_schema_prepare();
		$inst->schema_aggregate();

		$inst->stage_config();

		$inst->stage_domain();
		$inst->stage_ndsd();
		$inst->stage_krb5();
		$inst->stage_samba();
		$inst->stage_xadsd();

		$inst->stage_site();

		# Validate Replica Status for Time Synchronization point of view.
		validateReplica();

		# Validating the domain partition for DSfW specific containers.
		validateDomainContainer();

		$inst->stage_to_live();
		$inst->load_samspm();

		if ($is_replica eq "false") {
			# SCHEMA EXTENSIONS
			if (defined $ENV{NDSEXISTINGADMINPASSWD}) {
				$ENV{ADM_PASSWD} = $ENV{NDSEXISTINGADMINPASSWD};
			}
			$inst->extend_nam_schema();
			$inst->extend_schema();

			# eDir CONFIGURATION
			if($type eq "frd") {
				$inst->configure_nmas_methods();
			}
		}

		$inst->configure_nldap_settings();

		if ($is_replica eq "false") {
			if (not defined $ENV{ADM_PASSWD_DOMAIN} or $ENV{ADM_PASSWD_DOMAIN} eq "") {
				die("Missing credential for current domain administrator, existing..");
			}

			# Exiting with '3' in case administrator password not set.
			$inst->create_administrator();
		}
	}


}

main



