#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows nsimAssignments Check Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2012
#       Novell, Inc.  All rights reserved.
################################################################################

use Getopt::Std;
use File::Path;
use File::Copy;
use File::Temp;
use Cwd;
use warnings;
use strict;

my $err = 0;
my @nspmPasswordPolicies = (); # All the Password Policies
my %ppAssignments = (); # Hash for Password Policies' Assignments
my %ppAssociations = (); # Hash for Password Policies' Associations
my $debug = 0;
my $dir = getcwd;
my $ldif_file = "$dir/correct-nsimAssignments.ldif";
my $oid_control="2.16.840.1.113719.1.513.4.5";

sub writeLDIF {
	my $assignment = shift;
	my $policyDN = shift;

	open FH, ">>$ldif_file" or die "File open error";
	printf FH "dn: $policyDN\n";
	printf FH "changetype: modify\n";
	printf FH "delete: nsimAssignments\n";
	printf FH "nsimAssignments: $assignment\n";
	printf FH "\n";

	close FH;
}


sub getPasswordPoliciesList {
	my $rc = 0;

	# Get a list of nspmPasswordPolicies
	@nspmPasswordPolicies = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -LLL -E $oid_control -Y EXTERNAL -b "" "(objectclass=nspmpasswordpolicy)" -s sub dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; '  | sed '/^\$/d' | awk -F ':' '{print \$2}' | sed 's/^\\s\\+//g'`;

	chomp(@nspmPasswordPolicies);

	if ($? != 0) {
		$err = $? >> 8;
		printf "getPasswordPoliciesList failed ($err) : [@nspmPasswordPolicies]. Aborting...\n";
		exit ($err);

	}
}


sub getPasswordPoliciesAssignments {

	foreach my $nspmPasswordPolicy (@nspmPasswordPolicies)
	{
		my @nsimAssignments = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -E $oid_control -Q -LLL -Y EXTERNAL -b "$nspmPasswordPolicy" "(objectclass=nspmpasswordpolicy)" -s base nsimAssignments | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; ' | grep -i '^nsimassignments: ' | awk -F':' '{print \$2} ' | sed 's/^\\s\\+//g'`;

		chomp(@nsimAssignments);

		if ($? != 0) {
			$err = $? >> 8; 
			printf "getPasswordPoliciesAssigments for $nspmPasswordPolicy failed ($err) : [@nsimAssignments]. Aborting...\n";
			exit ($err);
		}

		if ($#nsimAssignments >= 0) {
			$ppAssignments{$nspmPasswordPolicy} = \@nsimAssignments; # Assignments on a password policy. Hash it.
		}
	}

	printAssignments();
}


sub getPasswordPoliciesAssociations {

	foreach my $nspmPasswordPolicy (@nspmPasswordPolicies)
	{
		my @nspmAssociations = `LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Q -LLL -E $oid_control -Y EXTERNAL -b "" "(nspmPasswordPolicyDN=$nspmPasswordPolicy)" -s sub dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\\n //g; }; ' | grep -i '^dn: ' | sed '/^\$/d' | awk -F':' '{print \$2} ' | sed 's/^\\s\\+//g'`;  
			chomp(@nspmAssociations);

			if ($? != 0) {
				$err = $?;
				printf "getPasswordPoliciesAssociations for $nspmPasswordPolicy failed ($err) : [@nspmAssociations]. Aborting...\n";
				exit ($err);
			}

		if ($#nspmAssociations >= 0) {
			$ppAssociations{$nspmPasswordPolicy} = \@nspmAssociations; # Associations for a password policy	
		}
	}

	printAssociations();
}


sub getInvalidAssignments {
	foreach my $policyDN (keys %ppAssignments) {

		if (!defined($ppAssignments{$policyDN})) {
			next;
		}

		if (!defined($ppAssociations{$policyDN})) {
			foreach my $assignment (@{$ppAssignments{$policyDN}}) {
				if ( $debug == 1 ) {
					printf "\t $assignment assignment on $policyDN is INVALID \n";
				}
				writeLDIF($assignment, $policyDN);
			}
			next;
		}

		printf "Checking INVALID assignments for $policyDN\n";

		my $invalidAssignment = 1;
		my @associations = @{$ppAssociations{$policyDN}};
		my @assignments = @{$ppAssignments{$policyDN}};

		if ( $debug == 1 ) {
			printf "POLICYDN : $policyDN. Assignments : ...\n";
		}

		foreach my $assignment (@assignments) {
			$invalidAssignment = 1;

			my @out = grep( /$assignment/, @associations);
			if (@out != 0) {
				# Association found
				$invalidAssignment = 0;
			}

			if ($invalidAssignment == 1) {
				if ( $debug == 1 ) {
					printf "\t $assignment assignment on $policyDN is INVALID \n";
				}
				writeLDIF($assignment, $policyDN);
			} else {
				if ( $debug == 1 ) {
					printf "\t $assignment assignment on $policyDN is OK \n";
				}
			}
		}
		if ( $debug == 1 ) {
			printf "\n";
		}
	}
}

sub printAssignments {

	if ( $debug == 0 ) {
		return 0;
	}

	foreach my $assignment (keys %ppAssignments) {
		printf "The assignments on \'$assignment\' policy are \n";
		foreach (@{$ppAssignments{$assignment}}) {
			print "\t$_\n";
		}
	}

	printf "\n\n";
}

sub printAssociations {

	if ( $debug == 0) {
		return 0;
	}

	foreach my $assocation (keys %ppAssociations) {
		printf "The associations for \'$assocation\' policy are \n";
		foreach (@{$ppAssociations{$assocation}}) {
			printf "\t$_\n";
		}
	}

	printf "\n\n";
}

sub Usage {

	print <<USAGE;
	Usage:/usr/bin/perl nsimAssignments_check_latest.pl [options]

      	options:
	      -d	Enable debug option
	      -h	Help
USAGE
      exit 0
}

sub main {
	my %options=();

	getopts("dh", \%options) or die("Unexpected arguments");
	$debug = 1 if defined $options{d};
	Usage if defined $options{h};

	unlink("$ldif_file");

	getPasswordPoliciesList();

	getPasswordPoliciesAssignments();

	getPasswordPoliciesAssociations();		

	getInvalidAssignments();

	if ( -f "$ldif_file" ) {
		printf "INVALID password policy assignments found.\n";
		printf "Check $ldif_file for the INVALID Assignments.\n";
		printf "If the ldif looks OK, in order to cleanup, run the below command ...\n";
		printf "\t LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapmodify -Y EXTERNAL -f $ldif_file\n";
	} else {
		printf "All the password policy assignments are VALID \n";
	}
}
main;
