#!/usr/bin/env python3
################################################################################
# Domain Services for Windows Sites and Subnets Configuration Utility
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2010-2012
#   Novell, Inc.  All rights reserved.
################################################################################

import ldap
import ldap.sasl
import socket
import base64
import sys
import re
import inspect
import logging
import struct
import os, fnmatch, pwd
import subprocess
#import commands
from subprocess import Popen, PIPE
import getpass
import time
from tempfile import NamedTemporaryFile
import getopt
import fcntl
import errno
import glob
import shutil

#Logging Operation Constants
DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL
NOTSET = logging.NOTSET

#Authermtication mechanisms
AUTH_GSSAPI = 'GSSAPI'
AUTH_EXTERNAL = 'EXTERNAL'

log_file_path = '/var/opt/novell/xad/log/'
rec_add_count = 0
rec_del_count = 0
enable_logging = 0

#delete all the temporary files with size zero on an existing DSfW server
def deleteFilesWithZeroSize():
        #only match with file name like tmpXXXXXX
        file_list=glob.glob("/tmp/tmp??????")

        if len(file_list) > 0:
                #traverse through all files and delete file only with size zero
                for f_name in file_list:
                        if os.path.exists(f_name) and int(os.path.getsize(f_name)) == 0:
                                os.remove(f_name)

def updateTheCrontab():
        croncommand="/usr/bin/crontab"
        cronlogpath='/var/opt/novell/xad/log'
        crontempfile=NamedTemporaryFile(delete=False)

        #get the counts for sites_dnsupdate.py entry
        p1 = subprocess.Popen([croncommand, '-l'], stdout=subprocess.PIPE)
        p2 = subprocess.Popen(['grep', '-i', 'sites_dnsupdate.py'], stdin=p1.stdout, stdout=subprocess.PIPE)
        p3 = subprocess.Popen(['wc', '-l'], stdin=p2.stdout, stdout=subprocess.PIPE)
        count = p3.communicate()[0]

        #If entry is greater than one then only modify the crontab
        if int(count)>1:
                #get the details from crontab file and write the details to a temporary file
                with open(crontempfile.name, "w") as tempfile:
                        subprocess.call([croncommand, '-l'], stdout=tempfile)

                #delete all the entries with /opt/novell/xad/sbin/sites_dnsupdate.py in temporary file
                subprocess.call(['sed','-i','/^.*\/opt\/novell\/xad\/sbin\/sites_dnsupdate.py.*/d', crontempfile.name])

                #make sure only one entry is available in the temporary file
                details='* * * * * /usr/bin/python3 /opt/novell/xad/sbin/sites_dnsupdate.py 1>/dev/null 2>>'+cronlogpath+'/sitesandsubnets.log'
                with open(crontempfile.name, "a") as outfile:
                        subprocess.call(['echo', details], stdout=outfile)

                #update the crontab file with the temporary file
                subprocess.call([croncommand, crontempfile.name])

                #delete the temporary file
                subprocess.call(['rm', '-f', crontempfile.name])

#call this before any other processing via sites_dnsupdate.py
deleteFilesWithZeroSize()
updateTheCrontab()

# bug #815934
# Need to store the Kerberos credentials in local temp file, else the credentials
# would be stored in the default /tmp/krb5cc_0 file and replace ticket taken outside
# by user. This interfares with some admin opeartions like << $domaincntrl --add >>
#

tmp_krb_file = NamedTemporaryFile(delete=False)

#Class for Logging
class LogOperation:
        file_name = ""
        def __init__(self, f_name):
                self.file_name = f_name

        def setupLog(self):
                #create logger
                logger = logging.getLogger("SNS")
                logger.setLevel(logging.DEBUG)

                #create console handler and set level to debug
                fh = logging.FileHandler(self.file_name)
                fh.setLevel(logging.DEBUG)

                #create formatter
                formatter = logging.Formatter("%(asctime)s-%(name)s-%(levelname)s-%(message)s")

                #add formatter to ch
                fh.setFormatter(formatter)

                #add ch to logger
                logger.addHandler(fh)

                return logger

#Global Log Handler creation
log = LogOperation(log_file_path+"sitesandsubnets.log")
lh = log.setupLog()

#Global Log function
def Log(type, message):
        # For DEBUG, INFO, WARNING to be printed to sitesandsubnets.log
        # -l or --logging has to be provided to the script.
        global enable_logging
        if(type == DEBUG and enable_logging):
                lh.debug(message)
        if(type == INFO and enable_logging):
                lh.info(message)
        if(type == WARNING and enable_logging):
                lh.warning(message)
        if(type == ERROR):
                lh.error(message)
        if(type == CRITICAL):
                lh.critical(message)

def LogNScreen(type, message):
        Log(type, message)
        print ('%s' %(message))

def LogException(msg, excp):
        Log(ERROR, "%s [%s]\n" %(msg, excp))
        logging.exception(excp)

#Class for LDAP operations
class LDAPOperation:
        operation_type=0
        dn_val = ""
        attr_val = ""
        mechanism = ""

        def __init__(self, mech):
                Log(INFO,inspect.stack()[1][3])
                self.mechanism = mech

        def bind_to_ldap(self):
                Log(INFO,inspect.stack()[1][3])
                handle = 0
                try:
                        #Bind to the server
                        for ldapuri,sasl_mech,sasl_cb_value_dict in [
                        (
                                SitesAndSubnetsScript.ldap_uri,
                                self.mechanism,
                                {
                                        ldap.sasl.CB_AUTHNAME        : '',
                                        ldap.sasl.CB_PASS        : '',
                                }
                        ),
                        ]:
                                if sasl_mech == AUTH_GSSAPI:
                                        sasl_auth = ldap.sasl.sasl(sasl_cb_value_dict,sasl_mech)
                                else:
                                        sasl_auth = ldap.sasl.external()
                                handle = ldap.initialize(SitesAndSubnetsScript.ldap_uri, trace_level=0)
                                handle.protocol_version = ldap.VERSION3
                                handle.sasl_interactive_bind_s('', sasl_auth)
                except ldap.LDAPError as error:
                        handle.unbind_s()
                        Log(ERROR,'Unable to bind to ldap [%s]' %(error))
                return handle

        def unbind_ldap(self, ldap_handle):
                Log(INFO,inspect.stack()[1][3])
                try:
                        ldap_handle.unbind_s()
                except ldap.LDAPError as error:
                        Log(ERROR,'Unable to unbind to ldap [%s]' %(error))
                return

        def ldap_search(self, ldap_handle, base, scope, search_filter, retrieve_attributes, result_set):
                Log(INFO,inspect.stack()[1][3])
                try:
                        Log(INFO,"----LDAP SEARCH----")
                        Log(INFO,"base : %s" %(base))
                        Log(INFO,"scope : %s" %(scope))
                        Log(INFO,"filter : %s" %(search_filter))
                        Log(INFO,"attributes : %s" %(retrieve_attributes))
                        #Prepare for the search
                        result_id=ldap_handle.search(base, scope, search_filter, retrieve_attributes)
                        timeout=0

                        #Iterate through the search result
                        while 1:
                                result_type, result_data = ldap_handle.result(result_id, timeout)
                                if (result_data == []):
                                        break
                                else:
                                        if result_type == ldap.RES_SEARCH_ENTRY:
                                                result_set.append(result_data)

                        #Check for empty result set
                        if len(result_set) == 0:
                                Log(INFO,"No Result")
                                return 0

                except ldap.LDAPError as error:
                        self.unbind_ldap(ldap_handle)
                        raise

        def getattr(self, name):
                Log(INFO,inspect.stack()[1][3])
                try:
                        data = self.attr_val[name]
                except KeyError:
                        raise AttributeError("object %s has no attribute %s" % (self, name))
                else:
                        # all LDAP attribs are multivalued by default,
                        # even when the schema says they are monovalued
                        if len(data) == 1:
                                return data[0]
                        else:
                                return data[:]

class Kerberos:
        def __init__(self):
                os.environ['KRB5CCNAME'] = tmp_krb_file.name
                kinit = '/opt/novell/xad/bin/kinit'
                kinit_args = [ kinit, '-f', '-k', '-t', '/etc/krb5.keytab', '%s$' % (socket.gethostname()) ]
                kinit = Popen(kinit_args, stdin=PIPE, stdout=PIPE, stderr=PIPE)
                kinit.wait()
                if kinit.wait() != 0:
                        print ("Error:- %s" %(kinit.stderr.read()))
                        Log(ERROR,'Error [%s]' %(kinit.stderr.read()))

        def kdestroy(self):
                kdestroy = '/opt/novell/xad/bin/kdestroy'
                kdestroy_args = [ kdestroy ]
                kdestroy = Popen(kdestroy_args , stdin=PIPE, stdout=PIPE, stderr=PIPE)
                if kdestroy.wait() != 0:
                        print ("Error:- %s" %(kdestroy.stderr.read()))
                        Log(ERROR,'Error [%s]' %(kdestroy.stderr.read()))

        def __del__(self):
                class_name = self.__class__.__name__
                del os.environ['KRB5CCNAME']

class rootDSE:
        result_rootDSE = ""
        list_of_Sites = []
        def __init__(self):
                try:
                        #Bind to the server
                        ldap_operation = LDAPOperation(AUTH_EXTERNAL)
                        ldap_handle = ldap_operation.bind_to_ldap()
                        #Prepare for the search
                        base=""
                        scope=ldap.SCOPE_BASE
                        search_filter="(objectclass=*)"
                        results = ldap_handle.search_s(base, scope, search_filter)
                        for dn, entry in results:
                                self.result_rootDSE = entry
                        base = "cn=Sites," + self.result_rootDSE['configurationNamingContext'][0].decode()
                        scope=ldap.SCOPE_ONELEVEL
                        search_filter="(objectclass=site)"
                        results = ldap_handle.search_s(base, scope, search_filter)
                        self.list_of_Sites= []
                        for dn, entry in results:
                                self.list_of_Sites.append(ldap.dn.explode_dn(dn, 1)[0])

                except ldap.LDAPError as error:
                        ldap_operation.unbind_ldap(ldap_handle)
                        raise

        def getRootDSE(self):
                return self.result_rootDSE

class domainConfiguration:
        rootDSEEntry = 0
        rDSE = 0

        def __init__(self):
                self.rDSE = rootDSE()
                self.rootDSEEntry = self.rDSE.getRootDSE()

        def getDomainName(self):
                return self.rootDSEEntry['dnsDomain'][0]

        def getDomainNamingContext(self):
                return self.rootDSEEntry['defaultNamingContext'][0]

        def getConfigurationNamingContext(self):
                return self.rootDSEEntry['configurationNamingContext'][0]

        def getADServerObjectDN(self):
                return self.rootDSEEntry['serverName'][0]

        def getSiteName(self):
                return ldap.dn.explode_dn(self.rootDSEEntry['serverName'][0], notypes=1)[2]

        def getSites(self):
                return self.rDSE.list_of_Sites

        def getValueFromXADINI(self, key):
                fh=open('/etc/opt/novell/xad/xad.ini', 'r')
                file_values = fh.read()
                fh.close()
                pattern = key+'.*'
                ret_val = re.search(pattern, file_values)
                ret_val = ret_val.group(0)
                ret_val = (ret_val.split("= ",1))[1]
                return ret_val

class CheckReplicas:
        def __init__(self):
                self.check_status_of_replicas()

        def check_status_of_replicas(self):
                not_ready = 1
                count = 0
                try:
                        #status_check_cmd = "/opt/novell/eDirectory/bin/ndsstat -r -d | grep \"^\.\""
                        status_check_cmd1 = subprocess.Popen(["/opt/novell/eDirectory/bin/ndsstat", "-r", "-d"],  stdout=subprocess.PIPE)
                        status_check_cmd2 = subprocess.Popen(["grep", "^\."], stdin=status_check_cmd1.stdout, stdout=subprocess.PIPE)
                        status_check_cmd3 = status_check_cmd2.communicate()[0]
                        status_check_cmd3 = status_check_cmd3.decode()
                        while not_ready:
                                #c_stat, c_text = commands.getstatusoutput(status_check_cmd)
                                c_stat = status_check_cmd2.returncode
                                c_text = status_check_cmd3
                                if c_stat != 0 and count <5:
                                        count = count + 1
                                        continue

                                if c_stat !=0 and count == 5:
                                        print ('Status: x%04x (%d)' % (c_stat, (c_stat % 255)))
                                        print ('Output: '+c_text)
                                        break

                                formatted_text = re.sub(' +', " ", c_text)
                                list_text = []
                                list_text = formatted_text.rstrip("\n").split('\n')
                                for text in list_text:
                                        # |.Replica Name.T=TREE-NAME. |Replica Type |Replica State |Rest...
                                        replica_name = re.search('^\..*?\.T=.*?\.', text).group(0)
                                        replica_state = (text[len(replica_name)+1:].split(' '))[1]
                                        Log(INFO, "%10s - %s" %(replica_state, replica_name))
                                        if replica_state.lower() == 'on':
                                                not_ready = 0
                                        else:
                                                not_ready = 1
                                                break
                                time.sleep(1)
                                sys.stdout.write('.')
                                sys.stdout.flush()
                        #print

                        #ndsstat command failed
                        if c_stat != 0:
                                Log(ERROR, 'ndsstat command failed. Please check sitesandsubnets.log file for more detail.')
                                sys.exit(-1)

                        #Some or all replicas are not in proper state
                        if not_ready == 1:
                                Log(ERROR, 'All the servers that are part of the replica ring are not active')
                                sys.exit(-1)

                except Exception as error:
                        LogException('Replica check failed with error : ', error)


#Class to Process the Sites and Subnet related operations
class SitesAndSubnetsScript:
        DEF_PAD = 10
        SiteList = []
        selected_site = ""
        from_site = ""
        ldap_uri = "ldapi://%2fvar%2fopt%2fnovell%2fxad%2frun%2fldapi/"
        file_path = '/var/opt/novell/xad/ds/dns/RR/'
        ldiffile_data = ""
        SRVRecordList = ['_ldap._tcp.New-Site-Name._sites',
                '_ldap._tcp.New-Site-Name._sites.gc._msdcs',
                '_ldap._tcp.New-Site-Name._sites.dc._msdcs',
                '_kerberos._tcp.New-Site-Name._sites',
                '_kerberos._tcp.New-Site-Name._sites.dc._msdcs',
                '_gc._tcp.New-Site-Name._sites']

        def __init__(self):
                Log(INFO,inspect.stack()[1][3])
                self.read_zone_file_data()
                self.check_for_the_site_change()
                self.is_site_renamed()
                self.update_xad_ini_file()

        def locate_file(self, pat, path):
                Log(INFO,inspect.stack()[1][3])
                result_list=[]
                files = os.listdir(os.path.abspath(path))
                for filename in fnmatch.filter(files, pat):
                        result_list.append(filename)
                return path+filename

        def get_formatted_domain_name(self, pattern):
                Log(INFO,inspect.stack()[1][3])
                host=socket.getfqdn()
                server_name = socket.gethostname()
                if pattern == '.':
                        formatted_name = host.replace(server_name+".", "")
                        return formatted_name
                formatted_name = host.replace('.', pattern)
                formatted_name = formatted_name.replace(server_name, "")
                if pattern == ',dc=':
                        formatted_name = formatted_name.replace(',', "", 1)
                if pattern == '_':
                        formatted_name = formatted_name.replace(pattern, "", 1)
                return formatted_name

        def read_zone_file_data(self):
                file_pattern='[a-zA-Z]*.zone'
                zone_file_path = '/var/opt/novell/xad/ds/dns/RR/'
                try:
                        zone_file = self.locate_file(file_pattern, zone_file_path)
                        fh = open(zone_file, 'r')
                        SitesAndSubnetsScript.ldiffile_data = fh.read()
                        fh.close()

                except Exception as error:
                        LogException('Reading zone file data failed with error : ', error)

        def process_the_query(self, oper, SRVrrName, m_site, target_server_fqdn):
                global rec_add_count
                global rec_del_count
                zone_record = ""

                host = self.get_formatted_domain_name(".")
                m_fqdn = socket.getfqdn()
                RRString = SRVrrName.replace('New-Site-Name', '(.*?)')
                pattern = RRString+'\s+.*'+m_fqdn
                matched_data = re.search( pattern, SitesAndSubnetsScript.ldiffile_data, re.I)

                if matched_data:
                        zone_record = matched_data.group(0)
                        zone_record = re.sub("\s+", ' ', zone_record)
                        zone_record = zone_record.replace('\n', '')

                RRString_array_temp = zone_record.split("IN");
                RRString_array_temp[1] = RRString_array_temp[1].strip()
                RRString_array = RRString_array_temp[1].split(' ')

                RR_Time_To_Live = "0"
                RR_Data_Value = {
                        "Priority"        : "0",
                        "Weight"        : "0",
                        "Port"        : "0",
                        "Data"        : "0"
                }

                # fill the value of priority, weight and data on the basis of
                # RR type
                if RRString_array[0] == "SRV":
                        RR_Data_Value["Priority"] = RRString_array[1]
                        RR_Data_Value["Weight"] = RRString_array[2]
                        RR_Data_Value["Port"] = RRString_array[3]
                        RR_Data_Value["Data"] = RRString_array[4]
                formatted_name = SRVrrName.replace("New-Site-Name", m_site)
                Log(INFO,"Executing ... /opt/novell/xad/sbin/DNSRRUpdate.pl -o "+oper+" -D '"+host+"'"+\
                        " -r '"+formatted_name+"'"+\
                        " -p "+RR_Data_Value["Priority"]+\
                        " -w "+RR_Data_Value["Weight"]+\
                        " -P "+RR_Data_Value["Port"]+\
                        " -t '"+target_server_fqdn+"'"+\
                        " -T SRV"+\
                        " -L "+RR_Time_To_Live)

                command = "/usr/bin/perl /opt/novell/xad/sbin/DNSRRUpdate.pl -o "+oper+" -D '"+host+"'"+\
                        " -r '"+formatted_name+"'"+\
                        " -p "+RR_Data_Value["Priority"]+\
                        " -w "+RR_Data_Value["Weight"]+\
                        " -P "+RR_Data_Value["Port"]+\
                        " -t '"+target_server_fqdn+"'"+\
                        " -s "+m_site+\
                        " -T SRV"+\
                        " -L "+RR_Time_To_Live+" 2>/dev/null"

                RR_Data_Value_Priority = RR_Data_Value["Priority"]
                RR_Data_Value_weight = RR_Data_Value["Weight"]
                RR_Data_Value_Port = RR_Data_Value["Port"]

                command1 = subprocess.Popen(["/usr/bin/perl", "/opt/novell/xad/sbin/DNSRRUpdate.pl", "-o", oper, "-D", "'"+host+"'", "-r", formatted_name, "-p", RR_Data_Value_Priority, "-w", RR_Data_Value_weight, "-P", RR_Data_Value_Port, "-t", "'"+target_server_fqdn+"'", "-s", m_site, "-T", "SRV", "-L", RR_Time_To_Live], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                command1.wait(); 
                c_stat = command1.returncode;
                c_text = command1.communicate()[0];
                c_text = c_text.decode().rstrip("\n")
                #c_stat, c_text = commands.getstatusoutput(command)
                exit_code = c_stat >> 8 # high byte
                signal_num = c_stat % 256 # low byte
                Log(INFO, 'Status: x%04x (%d)' % (c_stat, (c_stat % 255)))
                Log(INFO, 'Signal: x%02x (%d)' % (signal_num, signal_num))
                Log(INFO, 'Exit  : x%02x (%d)' % (exit_code, exit_code))
                Log(INFO, 'Core? : %s' % bool(signal_num & (1 << 8))) # high bit
                Log(INFO, 'Output: '+c_text)
                if oper == 'add':
                        if c_stat == 0:
                                rec_add_count = rec_add_count + 1
                if oper == 'del':
                        if c_stat == 0:
                                rec_del_count = rec_del_count + 1
                sys.stdout.write('.')
                sys.stdout.flush()
                print
                #if re.search("No credential cache found", c_text, re.I):
                if (256 - (c_stat % 255)) == 29 or (256 - (c_stat % 255)) == 30:
                        LogNScreen(ERROR, "...No credentials cache found. Ensure that Kerberos server is up and running.")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 33:
                        LogNScreen(ERROR, "...Invalid argument supplied for nsupdate")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 30:
                        LogNScreen(ERROR, "...No /etc/resolve.conf file found")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 31:
                        LogNScreen(ERROR, "Usage : /usr/bin/perl /opt/novell/xad/sbin/DNSRRUpdate.pl")
                        LogNScreen(ERROR, " -o operation (add/del)")
                        LogNScreen(ERROR, " -D domain name")
                        LogNScreen(ERROR, " -r record name")
                        LogNScreen(ERROR, " -p priority")
                        LogNScreen(ERROR, " -w weight")
                        LogNScreen(ERROR, " -P port")
                        LogNScreen(ERROR, " -t target")
                        LogNScreen(ERROR, " -T record type")
                        LogNScreen(ERROR, " -L time to live")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 15:
                        LogNScreen(ERROR, "...DNS RR packet handle creation failed")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 16:
                        LogNScreen(ERROR, "...Error in sending TKEY packet")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 18:
                        LogNScreen(ERROR, "...No reply to dynamic update")
                        LogNScreen(ERROR, "...Could not complete site movement operation")
                        sys.exit(0)

                if (256 - (c_stat % 255)) == 32:
                        LogNScreen(INFO, "...Dynamic DNS Update failed")
                        LogNScreen(INFO, "...Could not complete site movement operation")
                        return 32

        # Add DNS RR's to the new site
        def add_dnsrr_to_new_site(self, server_name, site_name):
                Log(INFO,"Add DNS RR's to the New Site")
                try:
                        host=socket.getfqdn()
                        #server_name = socket.gethostname()
                        domain_name = self.get_formatted_domain_name(".")
                        formatted_name = host.replace(server_name+".", "")
                        target_server_fqdn = server_name + "."+ domain_name
                        target_server_fqdn = target_server_fqdn.lower()

                        for SRVrrName in SitesAndSubnetsScript.SRVRecordList:
                                #nslookup_cmd = "/usr/bin/nslookup -querytype=srv "+SRVrrName+"."+domain_name+" | grep -i "+server_name
                                #nslookup_cmd = nslookup_cmd.replace('New-Site-Name', site_name)
                                #c_stat, c_text = commands.getstatusoutput(nslookup_cmd)
                                SRVrrName1 = SRVrrName
                                SRVrrName1 = SRVrrName1.replace('New-Site-Name', site_name)
                                nslookup_cmd_string = "/usr/bin/nslookup -querytype=srv "+SRVrrName1+"."+domain_name+" | grep -i "+server_name
                                nslookup1 = subprocess.Popen(["/usr/bin/nslookup", "-querytype=srv", ""+SRVrrName1+"."+domain_name+""], stdout=subprocess.PIPE)
                                nslookup2 = subprocess.Popen(["grep", "-i", server_name], stdin=nslookup1.stdout, stdout=subprocess.PIPE)
                                nslookup3 = nslookup2.communicate()[0]
                                nslookup3 = nslookup3.decode().rstrip("\n")
                                c_stat = nslookup2.returncode;
                                c_text = nslookup3;
                                Log(INFO, "%s with %s" %(SRVrrName1, nslookup_cmd_string))
                                Log(INFO, 'status : x%04x' %(c_stat))
                                Log(INFO, 'output : '+c_text)
                                if c_stat != 0:
                                        rc = self.process_the_query("add", SRVrrName, site_name, target_server_fqdn)
                                        if rc == 32:
                                                rc = self.process_the_query("mod", SRVrrName, site_name, target_server_fqdn)

                except Exception as error:
                        LogException('DNS RR addition failed with error : ', error)

                Log(INFO,"Addition of DNS RR's to the New Site Completed")

        # To Deleting DNS RR's from the old or other sites except the actual site of DC.
        # This function will be called after adding RR's to the new site.
        # This function takes arguments as server name and their site name.
        def delete_dnsrr_from_old_site(self, server_name, site_name):
                Log(INFO,"Delete DNS RR's from the Old Site")
                try:
                        host=socket.getfqdn()
                        #server_name = socket.gethostname()
                        domain_name = self.get_formatted_domain_name(".")
                        formatted_name = host.replace(server_name+".", "")
                        target_server_fqdn = server_name + "."+ domain_name
                        target_server_fqdn = target_server_fqdn.lower()
                        #Search through all sites for duplicate entry
                        for site in SitesAndSubnetsScript.SiteList:
                                #Delete records in other sites
                                if site != site_name:
                                        #Records to delete in all other sites
                                        for SRVrrName in SitesAndSubnetsScript.SRVRecordList:
                                                #nslookup_cmd = "/usr/bin/nslookup -querytype=srv "+SRVrrName+"."+domain_name+" | grep -i "+server_name
                                                #nslookup_cmd = nslookup_cmd.replace('New-Site-Name', site)
                                                #c_stat, c_text = commands.getstatusoutput(nslookup_cmd)
                                                SRVrrName1 = SRVrrName
                                                SRVrrName1 = SRVrrName1.replace('New-Site-Name', site)
                                                nslookup_cmd_string = "/usr/bin/nslookup -querytype=srv "+SRVrrName1+"."+domain_name+" | grep -i "+server_name
                                                nslookup_cmd1 = subprocess.Popen(["/usr/bin/nslookup", "-querytype=srv", ""+SRVrrName1+"."+domain_name+""], stdout=subprocess.PIPE)
                                                nslookup_cmd2 = subprocess.Popen(["grep", "-i", server_name], stdin=nslookup_cmd1.stdout, stdout=subprocess.PIPE)
                                                nslookup_cmd3 = nslookup_cmd2.communicate()[0]
                                                nslookup_cmd3 = nslookup_cmd3.decode().rstrip("\n")
                                                c_stat = nslookup_cmd2.returncode;
                                                c_text = nslookup_cmd3;

                                                Log(INFO, "%s with %s" %(SRVrrName1, nslookup_cmd_string))
                                                Log(INFO, 'status : x%04x' %(c_stat))
                                                Log(INFO, 'output : '+c_text)
                                                if c_stat == 0:
                                                        #SitesAndSubnetsScript.from_site = site
                                                        # 'target_server_fqdn' is a FQDN of the server on which the RR's will get deleted.
                                                        self.process_the_query("del", SRVrrName, site, target_server_fqdn)

                except Exception as error:
                        LogException('DNS RR deletion failed with error : ', error)

                Log(INFO,"Deletion of DNS RR's from the Old Site Completed")

        def is_site_renamed(self):
                sites_from_dns=[]
                sites_from_edir=[]
                list_of_sites_in_zone = []
                try:
                        host=socket.getfqdn()
                        server_name = socket.gethostname()
                        formatted_name = host.replace(server_name+".", "")

                        #Bind to the server
                        ldap_operation = LDAPOperation(AUTH_GSSAPI)
                        ldap_handle = ldap_operation.bind_to_ldap()

                        #Prepare for the search
                        dom_cnfg = domainConfiguration()
                        sites_from_edir = dom_cnfg.getSites()
                        host_us = self.get_formatted_domain_name("_")
                        base = "cn="+host_us+","+dom_cnfg.getValueFromXADINI('Zone Container')
                        scope=ldap.SCOPE_ONELEVEL
                        search_filter="(&(ObjectClass=DNIP:DNS RRSet)(cn=*gc*_#sites*))"
                        retrieve_attributes=['dNIPDNSDomainName']

                        results=ldap_operation.ldap_search(ldap_handle, base, scope, search_filter, retrieve_attributes, sites_from_dns)
                        Log(INFO, "Number of sites collected from the zone %s is %d" %(host_us, len(sites_from_dns)))
                        Log(INFO, "Number of valid sites present in the zone %s is %d" %(host_us, len(sites_from_edir)))
                        if len(sites_from_dns) != len(sites_from_edir):
                                for i in range(len(sites_from_dns)):
                                        for entry in sites_from_dns[i]:
                                                site_name= entry[1]['dNIPDNSDomainName'][0];
                                                site_name = site_name.decode()
                                                site_name=site_name.replace('_gc._tcp.', '')
                                                site_name=re.sub('._sites.*', '', site_name)
                                                list_of_sites_in_zone.append(site_name)
                        Log(INFO, "SRV records found for sites %s in zone %s" %(list_of_sites_in_zone, host_us))
                        Log(INFO, "Valid SRV records found for sites %s in zone %s " %(sites_from_edir, host_us))
                        extra_sites_to_delete = list(set(list_of_sites_in_zone) - set(sites_from_edir))
                        Log(INFO, "Invalid SRV records found for sites %s in zone %s" %(extra_sites_to_delete, host_us))
                        #Search through the invalid sites for duplicate entry
                        for site in extra_sites_to_delete:
                                #Records to delete in invalid sites
                                for SRVrrName in SitesAndSubnetsScript.SRVRecordList:
                                        nslookup_cmd = "/usr/bin/nslookup -querytype=srv "+SRVrrName+"."+formatted_name+" | grep -i "+server_name
                                        nslookup_cmd = nslookup_cmd.replace('New-Site-Name', site)
                                        c_stat, c_text = commands.getstatusoutput(nslookup_cmd)
                                        srvRecord = SRVrrName.replace('New-Site-Name', site)
                                        Log(INFO, "%s with %s" %(srvRecord, nslookup_cmd))
                                        Log(INFO, 'status : x%04x' %(c_stat))
                                        Log(INFO, 'output : '+c_text)
                                        if c_stat == 0:
                                                self.process_the_query("del", SRVrrName, site, "")

                except Exception as error:
                        LogException('Check for renaming of site failed with error : ', error)

        def check_for_the_site_change(self):
                Log(INFO,inspect.stack()[1][3])
                global rec_add_count
                global rec_del_count

                try:
                        #Find the Sever object
                        server_name = socket.gethostname()
                        curent_server_name = socket.gethostname()
                        dom_cnfg = domainConfiguration()
                        SitesAndSubnetsScript.SiteList = dom_cnfg.getSites()
                        #Search in the Site List for DC
                        SitesAndSubnetsScript.selected_site = dom_cnfg.getSiteName();
                        #sys.stdout.write('Processing ')
                        #sys.stdout.flush()
                        out1 = subprocess.Popen(['/usr/bin/ldapsearch', '-x', '-b', '', '-s', 'base', 'namingcontexts_edir', '|'],  stdout=subprocess.PIPE)
                        out2 = subprocess.Popen(['grep', 'cn=Servers'], stdin=out1.stdout, stdout=subprocess.PIPE)
                        out3 = out2.communicate()[0]
                        out3 = out3.decode()
                        serv_array = out3.rstrip("\n").split('\n')
                        for s_name in serv_array:
                                st = s_name.split(',')
                                server_name = st[0].split('=')[1]
                                site_name = st[2].split('=')[1]
                                Log(INFO,"server_name : %s, site_name : %s"%(server_name, site_name))
                                self.add_dnsrr_to_new_site(server_name, site_name)
                                self.delete_dnsrr_from_old_site(server_name,site_name)

                        if rec_add_count == 0 and rec_del_count == 0:
                                sys.stdout.write(' skipped')
                        else:
                                sys.stdout.write(' done')
                        sys.stdout.flush()
                        print

                        print ("\nServer \'%s\' Details:" %(curent_server_name))
                        Log(INFO,"\nServer \'%s\' Details:" %(curent_server_name))

                        if rec_add_count != 0:
                                print ("%-13s : %-32s" %("Old Site", SitesAndSubnetsScript.from_site))
                                Log(INFO,"%-13s : %-32s" %("Old Site", SitesAndSubnetsScript.from_site))
                        else:
                                print ("%-13s : %-32s" %("Old Site", SitesAndSubnetsScript.selected_site))
                                Log(INFO,"%-13s : %-32s" %("Old Site", SitesAndSubnetsScript.selected_site))

                        print ("%-13s : %-32s" %("Current Site", SitesAndSubnetsScript.selected_site))
                        Log(INFO,"%-13s : %-32s" %("Current Site", SitesAndSubnetsScript.selected_site))
                        if rec_add_count != 0:
                                print ("Movement detected. Updated the DNS records.")
                                Log(INFO, "Movement detected. Updated the DNS records.")
                        else:
                                print ("No movement detected.")
                                Log(INFO, "No movement detected.")
                        print

                except Exception as error:
                        LogException('Check for site movement failed with error : ', error)

        def update_xad_ini_file(self):
                try:
                        dom_cnfg = domainConfiguration()
                        site_in_xad_ini = dom_cnfg.getValueFromXADINI('Site')
                        if site_in_xad_ini != SitesAndSubnetsScript.selected_site:
                                with open('/etc/opt/novell/xad/xad.ini') as f:
                                        newText=f.read().replace('Site = '+site_in_xad_ini, 'Site = '+SitesAndSubnetsScript.selected_site)

                                shutil.copy('/etc/opt/novell/xad/xad.ini', '/etc/opt/novell/xad/xad.ini-backup')

                                with open('/etc/opt/novell/xad/xad.ini', "w") as f:
                                        f.write(newText)

                except Exception as error:
                        LogException('Update site name in xad.ini file failed with error : ', error)


class Utility:
        def __init__(self):
                Log(INFO,inspect.stack()[1][3])

        def parse_arguments(self, argp):
                global enable_logging
                try:
                        opts, args = getopt.getopt(argp,"hl",["help", "logging"])
                except getopt.GetoptError:
                        print ('sites_dnsupdate.py [-h|--help] [-l|--logging]')

                for opt, arg in opts:
                        if opt in ('-h', '--help'):
                                print ('sites_dnsupdate.py [-h|--help] [-l|--logging]')
                                sys.exit()
                        elif opt in ('-l', '--logging'):
                                enable_logging = 1
                        else:
                                assert False, "Unhandled option"

class SingleProcess:
        # The constructor creates a lock file with EXCLUSIVE
        # and NON BLOCK flag.
        def __init__(self):
                self.initialized = False
                temp_file_path = "/var/opt/novell/xad"
                basename = os.path.basename(os.path.splitext(sys.argv[0])[0]) + '.lock'
                self.lockfile = os.path.normpath(temp_file_path + '/.' + basename)
                self.fp = open(self.lockfile, 'w')

                try:
                        # Acquire an exclusive lock, avoid blocking on lock acquisition
                        fcntl.lockf(self.fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
                except IOError:
                        Log(WARNING, "Another instance is already running, quitting.")
                        print ("Another instance is already running, quitting.")
                        sys.exit(-1)
                self.initialized = True

        # This destructor is must to take care interrupted situations.
        # The destructor takes care of deleting the lock file under
        # abnormal termination, or exit.
        def __del__(self):
                if not self.initialized:
                        return
                try:
                        # Unlock the file
                        fcntl.lockf(self.fp, fcntl.LOCK_UN)
                        if os.path.isfile(self.lockfile):
                                os.unlink(self.lockfile)
                except Exception as e:
                        LogException('Object destruction error', e)
                        sys.exit(-1)

def main():
        Log(INFO,inspect.stack()[1][3])
        return

if __name__ == "__main__":
        # Make sure that only one instance of the process
        # is available in the environment
        singleprocess = SingleProcess()

        # Sites and Subnet specific operations
        sasutil = Utility()
        sasutil.parse_arguments(sys.argv[1:])
        sys.stdout.write('\nProcessing ')
        sys.stdout.flush()
        #os.environ['SASL_PATH'] = "/opt/novell/xad/lib64/sasl2"
        CheckReplicas()
        auth = Kerberos()
        sas = SitesAndSubnetsScript()
        auth.kdestroy()
        main()
