#!/usr/bin/perl -I. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Domain Controller DNS configuration tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2007
#	Novell, Inc.  All rights reserved.
################################################################################

use Getopt::Long;
use Term::ReadKey;
use POSIX;     # for setlocale()
use Locale::gettext;
use File::Copy;

### Variables ###
# To hold the values of the variables to be substituted in the template.
my(%Vars)=();
$Debug=0 ;   # Set to non zero to get some debug information.
$ProgramName=$0 ;
my($IP_Address, $Domain_Name, $DnsDomainName, $Hostname, $DomainGUID, $MachineGUID, $Master_IP_Address, $Secondary_IP_Address) = "";
my ($Additional_dc, $Child_Domain, $action, $Stub_zone, $Verfication_input, $Secondary_Server_Conf, $Notify) = "";
my @Domainname_array = "";
my $Ldapsearch = "/usr/bin/ldapsearch";
my $Ldapi = "ldapi://%2fvar%2fopt%2fnovell%2fxad%2frun%2fldapi";
my $Notify_Flg = "no";
my $Allow_Notify = "none";
my $Allow_Transfer = "none";
my $Ldapsearch_filter = ' | sed " /^ / {; H; d; }; /^ /! {; x; s/\n //; }; "';
my $myLocalhost = "127.0.0.1";


# Setting the Locale
setlocale(LC_MESSAGES, "");
bindtextdomain("xad", "/opt/novell/xad/share/locale");
textdomain("xad");

if(!(($ARGV[0] eq "--help") || ($ARGV[0] eq "-h")))
{
	$action = shift(@ARGV);
}

if($action eq "add")
{
	GetOptions(
		"domain-name|d=s"	=> \$Domain_Name,
		"ip-address|p=s"	=> \$IP_Address,
		"hostname|s=s"		=> \$Hostname,
		"notify|n"		=> \$Notify,
		"secondary-dns-server-ip|e=s"	=> \$Secondary_IP_Address,
		"additional-dc|a"	=> \$Additional_dc,
		"child-domain|c"	=> \$Child_Domain,
		"Stub-zone|z"		=> \$Stub_zone,
		"config-as-secondary-dns-server|l"	=> \$Secondary_Server_Conf,
		"master-dns-server-ip|m=s"	=> \$Master_IP_Address,
		"help|h"		=> \&HELP_MESSAGE,
		) or die HELP_MESSAGE();
}
elsif($action eq "remove")
{
	GetOptions(
		"ip-address|p=s"        => \$IP_Address,
		"additional-dc|a"       => \$Additional_dc,
		"child-domain|c"        => \$Child_Domain,
		) or die HELP_MESSAGE();
}


### Functions ###

sub HELP_MESSAGE()
{
	my $help = <<"EOH";
-------------------------------------------------------------
Domain Services for Windows Server DNS configuration tool
Copyright (c) 2001-2007 Novell, Inc. All rights reserved.
Usage:
$0:	add
	-c 		DNS configuration for child domain.
	-a		DNS configuration for additional
			domain controller.
	-z		DNS configuration for stub zone creation.
	-n		Set NOTIFY flag to 'yes'.
	-l              Secondary DNS server configuration for domain.
	-p ServIP	IP Address of the Domain Controller added.
	-s Hostname	Hostname of the Domain Controller added.
	-d DomainDN	DN of the domain.
	-e ServIP	IP address of the secondary DNS server.
	-m ServIP	IP Address of the primary DNS server.

$0:	remove
	-c              DNS configuration for child domain.
	-a              DNS configuration for additional
			domain controller.
	-p ServIP	IP Address of the Domain Controller to be
			removed.
EOH
	print STDERR sprintf(gettext(" %s."), $help), "\n";
	exit();
}
sub exitOnFailure
{
	if($_[0] != 0)
	{
		print STDERR sprintf(gettext (" %s. "), $_[1]), "\n";
		exit(1);
	}
}
sub SubstituteVars {
	my($Line) = @_ ;
	my(@Tokens) ;
	my($i,$Time) ;

	@Tokens = split(/[\[\]]/,$Line) ;
	for ($i=1;$i<=scalar(@Tokens);$i=$i+2) {
	@Tokens[$i]=$Vars{@Tokens[$i]} ;
	}
	return join("",@Tokens) ;
}

sub ProcessTemplateFile 
{
	my($TemplateFile)=@_ ;
	my($ReadLine, $Line) ;
	open($TemplateFile, $TemplateFile) || die "Cannot open $TemplateFile" ;
	printf STDERR "Reading from $TemplateFile\n" if $Debug ;
	open(OLDOUT, ">&STDOUT");
	while ($ReadLine = <$TemplateFile>) {
		chomp $ReadLine ;
		$Line = SubstituteVars($ReadLine) ;
		if ( $Line =~ /^#/) {

			ProcessCommand($TemplateFile,$Line) ;
		} else {
			printf "$Line\n" ;
		}
	}
	close($TemplateFile) ;
	close(STDOUT);
	open(STDOUT, ">&OLDOUT");
	printf STDERR "End reading from $TemplateFile\n" if $Debug ;
}

sub ProcessCommand {
	my($Input,$Line) = @_ ;
	my(@Tokens, $Time) ;
	@Tokens = split(/\s+/,$Line) ;
	if (@Tokens[0] eq "#write") {
		die "Invalid syntax in $Line" if scalar(@Tokens) != 2;
		open STDOUT, ">@Tokens[1]" ;
	} elsif (@Tokens[0] eq "#append") {
		die "Invalid syntax in $Line" if scalar(@Tokens) != 2;
		open STDOUT, ">>@Tokens[1]" ;
	} elsif (@Tokens[0] eq "#dsfw") {
		# don't do anything, just print the line
		print $Line, "\n";
	} else {
		die "Invalid command starting with @Tokens[0]\n" ;
	}
}

sub reverseIPAddress
{
	my ($ip) = $_[0];
	my (@ip, $rev, @rev);
	
	@ip = split /\./, $ip;
	@rev = reverse @ip;
	$rev = join ('.', @rev, "in-addr.arpa.");
                                                                                                                                                             
	return $rev;
}

sub Validate_IP
{
	my $IP = shift;
	return -1 unless defined($IP);
	if ($IP =~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/)
	{
		# IP address is in valid format
		if($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255)
		{
			# Range check passed
			return 0;
		}
		else
		{
			# Range check failed
			return -2;
		}
	}
	else
	{
		#IP address is not in valid format
		return -3;
	}
}

sub CheckCredentials
{
	my ($Admin_Name, $Admin_passwd) = @_;
	my ($NdsLogin_ret, $Status, $Error_Code) = "";

	# Check for occurance of ndslogin utility
	if(!(-e "/opt/novell/eDirectory/bin/ndslogin"))
	{
		print STDERR gettext(" Missing file '/opt/novell/eDirectory/bin/ndslogin'. This file is required for DNS configuration, exiting.."), "\n";
		exit();
	}
	$NdsLogin_ret = `/opt/novell/eDirectory/bin/ndslogin -h $myLocalhost "$Admin_Name" -p "$Admin_passwd" 2>&1`;
	chomp $NdsLogin_ret;

	$Status = `echo "$NdsLogin_ret" | awk -F ":" '{print \$2}'`;
	chomp $Status;

	if($Status =~ m/failed/)
	{
		# A failure was found, check for the error code
		$Error_code = `echo "$Status" | awk -F "(" '{print \$2}'`;
		chomp $Error_code;
		# remove the trailing brace
		chop $Error_code;

		# Invalid Password or User Name
		if ($Error_code == "-669" || $Error_code == "-601")
		{
			print STDERR gettext(" Invalid credentials. Exiting."), "\n";
			exit();
		}
	}
	# eDiretory is down
	elsif($NdsLogin_ret =~ m/login cannot be done/)
	{
		print STDERR gettext(" eDirectory login cannot be done. Check whether NetIQ eDirectory server is running on the machine . Exiting"), "\n";
		exit();
	}
	else
	{
		return;
	}
}

sub Remove_Appended_Conf
{

	my ($DNS_file1,$DnsDomainName) = @_;
	my $temp = "";
	my $dev_null = "/dev/null";
	my $temp_file = "/tmp/temp.$DnsDomainName.$IP_Address";
	my $delete_flg = 0;

	open(dev_null_fh, "> $dev_null") or die "dev null cannot be opened\n";
	open(NEW, "> $temp_file") or die "temp file cannot be opened.\n";
	open(OLD, "< $DNS_file1") or die "DNS file cannot be opened.\n";

	while (<OLD>)
	{
		$temp = $_;
		chomp $temp;
		
		# for all files except domain-zone.conf
		if($temp eq ";dsfw start:$IP_Address:$DnsDomainName:$Hostname")
		{
			#set a check flg
			$delete_flg = 1;
		}
	
		# for domain-zone.conf
		if($temp eq "#dsfw start:$IP_Address:$DnsDomainName:$Hostname")
		{
			#set a check flg
			$delete_flg = 1;
		}
		
		# for all files except domain-zone.conf
		if($temp eq ";dsfw end:$IP_Address:$DnsDomainName:$Hostname")
		{
			#set the flg to allow deletion of last line
			$delete_flg = 2;
		}
		
		# for domain-zone.conf
		if($temp eq "#dsfw end:$IP_Address:$DnsDomainName:$Hostname")
		{
			#set the flg to allow deletion of last line
			$delete_flg = 2;
		}
		#throw all lines to /dev/null if the delete_flg is set
		if($delete_flg == 1)
		{
			print dev_null_fh $temp;
		}
		elsif($delete_flg == 2)
		{
			#the last comment "#/;dsfw:end:$IP_Address:$DnsDomainName:$Hostname" is
			#encountered
			print dev_null_fh $temp;
			
			#reset the delete_flg
			$delete_flg = 0;
		}
		else
		{
			print NEW "$temp\n";
		}
	}
	
	# Closing the opened file descriptors
	close(dev_null_fh);
	close(NEW);
	close(OLD);

	#now write the temp file back to the DNS_file and delete the 
	#temp file
	copy($temp_file, $DNS_file1) or die "File cannot be copied: $!\n";
	unlink($temp_file) or die "Can't delete $temp_file: $!\n";
}

sub Compute_File_deletion
{
	# We trim the content of file and compute the length
	# if length is greater than 0, file has content so don't
	# delete it. Parsing the enite file till EOF so that we don't
	# get misleaded by intial blank line(s). Intial notion is that
	# file would be empty
	my $DNS_File = $_[0];
	my $temp = "";
	my $delete_flg = 1;
	my $length;

	open(FILE_HNDL, $DNS_File) or die "Can't open the file '$DNS_File' : $!\n";
	while(<FILE_HNDL>)
	{
		$temp = $_;

		# removing multiple whitespaces
		$temp =~ s/\s+//g;
		# removing  leading tabs
		$temp =~ s/^\t+//g;
		# removing leading whitespaces
		$temp =~ s/^\s+//g;
		# removing trailing tabs
		$temp =~ s/\t+$//g;
		# removing trailing whitespaces
		$temp =~ s/\s+$//g;
		# removing tabs
		$temp +~ s/\t//g;

		$length = length($temp);

		if($length == 0)
		{
			$delete_flg = 1;
		}
		else
		{
			$delete_flg = 0;
		}
	}

	close(FILE_HNDL) or die "cannot close file '$DNS_File' : $!\n";
	return $delete_flg;
}
		
sub Additional_Dc_Configuration
{
	# Additional Domain Controller DNS configuration.
	# The content of "/var/lib/named/dyn/<domain_name>.zone" file 
	# of Additional Domain Controller should be appended 
	# to the domain zone file of the same name in the DNS server
                                                                                                                                                             
	# Validating the IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf (gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP_Address), "\n";
		exit();
	}

	# Stripping out the domainname part from <hostname>.<domainname>
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];
	
	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;
	
	# Forming a Domain Name in dc format
	my $LdapDomainName = $DnsDomainName;
	$LdapDomainName =~ s/\./.dc=/g;
	$LdapDomainName ="dc=".$LdapDomainName;

	# Check if DNS server is configured already for the additonal
	# Domain Controller.
	my $check = `cat /var/lib/named/dyn/"$DnsDomainName".zone | grep -i "$IP_Address" | tail -1`;
	$check =~ s/\s+//g;
	$check =~ s/IN([a-z,A-Z,0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)//g;
	chomp $check;

	if($check eq $Hostname)
	{
		print  STDERR sprintf(gettext("DNS server already has the DNS configuration for the %s(IP:%s) as an additonal Domain Controller for domain '%s'."), $Hostname, $IP_Address, $DnsDomainName),"\n";
		exit();
	}

	# Getting the domain GUID and Machine GUID

	# Domain GUID
	$LdapDomainName =~ s/\.([a-z,A-Z,0-9]+)=/,$1=/g;
	my $Command = $Ldapsearch.' -H '.$Ldapi.' -s sub -Q -Y EXTERNAL -b '.$LdapDomainName.' objectguid  -LLL '.$Ldapsearch_filter.'| grep -i "objectGUID::" 2> /dev/null';
	my $DomainSid = `$Command`;
	exitOnFailure($?,"Could not contact ldap server or could not do ldap search on the given domain.");
	

	$DomainSid =~ s/objectGUID:: //g;
	print ">> Generating DomainGUID...\n";
	
	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/base64ToGUID.pl"))
	{
		print STDERR gettext(" Missing file '/opt/novell/xad/share/dcinit/base64ToGUID.pl', this files is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}
	
	$DomainGUID = `/opt/novell/xad/share/dcinit/base64ToGUID.pl "$DomainSid"`;
	chomp $DomainGUID;
                                                                                                                                                             
	# MachineGUID
	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/printConfigKey.pl"))
	{
		print STDERR gettext(" Missing file '/opt/novell/xad/share/dcinit/printConfigKey.pl', this files is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	my $LdapForestRootDomainName = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC"`;
	chomp $LdapForestRootDomainName;

	my $MachineNTDSSettingsDN = "cn=NTDS Settings,cn=$Hostname,cn=Servers,cn=Default-First-Site-Name,cn=Sites,cn=Configuration,$LdapForestRootDomainName";
	$Command = $Ldapsearch.' -H '.$Ldapi.' -s sub -Q -Y EXTERNAL -b "'.$MachineNTDSSettingsDN.'" objectGUID  -LLL '.$Ldapsearch_filter.' | grep -i "objectGUID::" 2> /dev/null';
	my $MachineSid = `$Command`;
	exitOnFailure($?,"Could not contact ldap server or could not do ldap search on the given domain.");
	$MachineSid =~ s/objectGUID:: //g;
	print STDERR ">>", gettext("Generating MachineGUID..."), "\n";

	#Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/base64ToGUID.pl"))
	{
		print STDERR gettext(" Missing file '/opt/novell/xad/share/dcinit/base64ToGUID.pl', this files is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	$MachineGUID = `/opt/novell/xad/share/dcinit/base64ToGUID.pl "$MachineSid"`;
	chomp $MachineGUID;

	if($DomainGUID eq "")
	{
		print STDERR gettext("Error while generating DomainGUID, exiting..."), "\n";
		exit();
	}
	print STDERR ">>", gettext("Setting DomainGUID..."), "\n";

	if($MachineGUID eq "")
	{
		print STDERR gettext("Error while generating MachineGUID, exiting..."), "\n";
		exit();
	}
	print STDERR ">>",gettext("Setting MachineGUID..."), "\n";

	if($Notify)
	{
		# Set Notify_Flg to "yes"
		$Notify_Flg = "yes";
	}

	my @Allowed_IP = split(";", $Secondary_IP_Address);
	my $Allowed_IP_String = "";
	my $IP_Check = "";
	foreach my $IP (@Allowed_IP)
	{
		$IP_check = Validate_IP($IP);
		if($IP_check != 0)
		{
			print STDERR sprintf (gettext("The secondary server IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP), "\n";
			exit();
		}
		$Allowed_IP_String .= "$Allowed_IP_String$IP;";
	}
	
	#Setting up the Variables to be filled in the <domain_name>.zone file
	$Vars{"DNS Root"} = "$DnsDomainName";
	$Vars{"DNS Host Address"} = &reverseIPAddress("$IP_Address");
	$Vars{"IP Address"} = "$IP_Address";
	$Vars{"DNS Unqualified Host Name"} = "$Hostname";
	$Vars{"DNS SRV Priority"} = 0;
	$Vars{"DNS SRV Weight"} = 100;
	$Vars{"Site"} = "Default-First-Site-Name";
	$Vars{"Machine GUID"} = "$MachineGUID";
	$Vars{"Domain GUID"} = "$DomainGUID";
	$Vars{"DNS Host Name"} = "$Hostname". "."."$DnsDomainName";
	$Vars{"Notify"} = "$Notify_Flg";
	$Vars{"Allowed Notify"} = "$Allowed_IP_String";
	$Vars{"Allowed Transfer"} = "$Allowed_IP_String";
	
	$template_file = "/opt/novell/xad/share/dcinit/templates/dns-zone-add-dc.conf";
	
	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s, this file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to generate/append the required DNS file(s)
	print STDERR ">>", sprintf(gettext("Updating the %s.zone file..."), $DnsDomainName), "\n";
	ProcessTemplateFile($template_file);

	# Defining the template file to be used for generation of <IP Address>.zone file
	$template_file = "/opt/novell/xad/share/dcinit/templates/dc-reverse-zone-dns.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s, this file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to create the <ipaddr>.zone file for additional domain configuration
	print STDERR ">>", sprintf(gettext("Updating the %s.zone file..."), $IP_Address), "\n";
	ProcessTemplateFile($template_file);

	# Defining the template file to be used for generation of domain‑zone.conf file
	$template_file = "/opt/novell/xad/share/dcinit/templates/domain-zone-dns-reverseip.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s, this file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append domain‑zone.conf for additional domain controller's configuration
	print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
	ProcessTemplateFile($template_file);

	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	# Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;

	# post-configuration validation
	print STDERR ">>", gettext("Validating DNS setting done..."), "\n";
	#my $Post_configuration_IP_check = `/usr/bin/host "$DnsDomainName" | awk -F " " '{print \$4}'`;
	#chomp $Post_configuration_IP_check;

	my $Post_configuration_hostname_check = `/usr/bin/host "$IP_Address" | awk -F " " '{print \$5}' | awk -F "." '{print \$1}'`;
	chomp $Post_configuration_hostname_check;

	#if(($Post_configuration_IP_check ne $IP_Address) || ($Post_configuration_hostname_check ne $Hostname))
	if($Post_configuration_hostname_check ne $Hostname)
	{
		print STDERR sprintf(gettext("%n: Failed while performing the post-configuration validation checks for DNS configuration. The DNS configuration might not have been done properly. Please perform the DNS configuration once again with proper paramaters. If the problem persists, contact the Administrator"), $?),"\n";
		exit();
	}
}

sub Child_Domain_Configuration
{
	# Child Domain DNS configuration.
	# The DNS server configuration files "/etc/named.d/domain‑zone.conf", "/var/lib/named/<ipaddr>.zone"
	# and "/var/lib/named/dyn/<domainname>.zone" will be updated for Child Domain
	# DNS configuration.

	# Validating IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf(gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP_Address), "\n";
		exit();
	}

	# Stripping out the domainname part from <hostname>.<domainname>
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];
	
	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;
	
	# Forming a Domain Name in dc format
	my $LdapDomainName = $DnsDomainName;
	$LdapDomainName =~ s/\./.dc=/g;
	$LdapDomainName ="dc=".$LdapDomainName;


	# Check if DNS server is configured already for the Child Domain.
	if((-f "/var/lib/named/dyn/$DnsDomainName.zone") || (-f "/var/lib/named/$IP_Address.zone"))
	{
		print STDERR sprintf(gettext("DNS server already has the DNS configuration for the domain %s."), $DnsDomainName), "\n";
		exit();
	}

	# Refreshing the ldap server
	my ($Admin_name, $Admin_passwd) = "";
	
	print STDERR gettext("Enter the admin DN of the tree administrator:"), "\n";
	$Admin_name = <STDIN>;
	chomp $Admin_name;

	print STDERR gettext("Enter the password for the tree administrator:"), "\n";
	ReadMode 'noecho';
	$Admin_passwd = <STDIN>;
	chomp $Admin_passwd;
	ReadMode 'normal';

	# verify the obtained credentails
	CheckCredentials($Admin_name, $Admin_passwd);

	# refersh ldap server with ldapconfig
	if(!(-e "/opt/novell/eDirectory/bin/ldapconfig"))
	{
		print STDERR gettext("Missing file '/opt/novell/eDirectory/bin/ldapconfig'.  This file is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	my $ldapconfig_rc = `/opt/novell/eDirectory/bin/ldapconfig -R -a "$Admin_name" -w "$Admin_passwd"`;

	# Getting the domain GUID and Machine GUID

	# MachineGUID
	$LdapDomainName =~ s/\.([a-z,A-Z,0-9]+)=/,$1=/g;
	
	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/printConfigKey.pl"))
	{
		print STDERR gettext("Missing file '/opt/novell/xad/share/dcinit/printConfigKey.pl'. This file is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	my $LdapForestRootDomainName = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC"`;
	chomp $LdapForestRootDomainName;
	my $SearchCriteria = "(&(dNSHostName=$Hostname.$DnsDomainName)(objectClass=server))";
	my $Command = $Ldapsearch.' -H '.$Ldapi.' -s sub -Q -Y EXTERNAL -b cn=Configuration,'.$LdapForestRootDomainName.' "'.$SearchCriteria.'" dn -LLL '.$Ldapsearch_filter.'  2> /dev/null';
	my $MachineDN = `$Command`;
	exitOnFailure("$?","Could not contact ldap server or could not do ldap search on the given domain.");
	$MachineDN =~ s/\n//g;
	my @temp = split('#', $MachineDN);
	$MachineDN = $temp[0];
	$MachineDN =~ s/dn: //g;
	# Removing the extra white spaces
	$MachineDN =~ s/\s+//g;

	$Command = $Ldapsearch.' -H '.$Ldapi.' -s sub -Q -Y EXTERNAL -b '.$MachineDN.' objectclass=ntdsdsa objectGUID -LLL 2> /dev/null '.$Ldapsearch_filter.'|  grep -i "objectGUID::" 2> /dev/null';

	my $MachineSid = `$Command`;

	#my $MachineSid = `$Ldapsearch -H $Ldapi -s sub -Q -Y EXTERNAL -b "$MachineDN" objectclass=ntdsdsa objectGUID -LLL 2> /dev/null |sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' |  grep -i "objectGUID::" 2> /dev/null`;

	exitOnFailure("$?","Could not contact ldap server or could not do ldap search on the given domain.");
	print "\n proceeding to nect search";
	$MachineSid =~ s/objectGUID:: //g;
	print STDERR ">>", gettext("Generating MachineGUID..."), "\n";

	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/base64ToGUID.pl"))
	{
		print STDERR gettext("Missing file '/opt/novell/xad/share/dcinit/base64ToGUID.pl'. This file is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	$MachineGUID = `/opt/novell/xad/share/dcinit/base64ToGUID.pl "$MachineSid"`;
	chomp $MachineGUID;

	if($MachineGUID eq "")
	{
		print STDERR gettext("Error while generating MachineGUID, exiting.."), "\n";
		exit();
	}

	print STDERR ">>",gettext("Setting MachineGUID..."), "\n";

	if($Notify)
	{
		# Set Notify_Flg to "yes"
		$Notify_Flg = "yes";
	}

	my @Allowed_IP = split(";", $Secondary_IP_Address);
	my $Allowed_IP_String = "";
	my $IP_Check = "";
	foreach my $IP (@Allowed_IP)
	{
		$IP_check = Validate_IP($IP);
		if($IP_check != 0)
		{
			print STDERR sprintf (gettext("The secondary server IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP), "\n";
			exit();
		}
		$Allowed_IP_String .= "$Allowed_IP_String$IP;";
	}

	# Setting up the Variables to be in <ipaddr>.zone, <domain_name>.zone and domain-zone.conf.default files
	$Vars{"DNS Root"} = "$DnsDomainName";
	$Vars{"DNS Host Address"} = &reverseIPAddress("$IP_Address");
	$Vars{"IP Address"} = "$IP_Address";
	$Vars{"DNS Unqualified Host Name"} = "$Hostname";
	$Vars{"DNS SRV Priority"} = 0;
	$Vars{"DNS SRV Weight"} = 100;
	$Vars{"Site"} = "Default-First-Site-Name";
	$Vars{"Machine GUID"} = "$MachineGUID";
	# Do not specify the Domain GUID value for child domain in
	# first run
	$Vars{"Domain GUID"} = "[Domain GUID]";
	$Vars{"DNS Host Name"} = "$Hostname". "."."$DnsDomainName";
	$Vars{"Notify"} = "$Notify_Flg";
	$Vars{"Allowed Notify"} = "$Allowed_IP_String";
	$Vars{"Allowed Transfer"} = "$Allowed_IP_String";

	print STDERR ">>", gettext("Creating Zone Configuration files.."), "\n";
	# Defining the template file to be used for generation of <Ipaddr>.zone file
	$template_file = "/opt/novell/xad/share/dcinit/templates/dc-reverse-zone-dns.conf";
	
	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to generate <ippaddr>.zone file for child domain's configuration
	print STDERR ">>", sprintf(gettext("Generating %s.zone file..."), $IP_Address), "\n";
	ProcessTemplateFile($template_file);
	
	
	# Defining the template file to be used for generation of <domain_name>.zone file
	$template_file = "/opt/novell/xad/share/dcinit/templates/dns-zone.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettect("Missing file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to generate <domain_name>.zone for child domain's configuration
	print STDERR ">>", sprintf(gettext("Generating %s.zone file..."), $DnsDomainName), "\n";
	ProcessTemplateFile($template_file);

	# Defining the template file to be used for generation of domain‑zone.conf file
	$template_file = "/opt/novell/xad/share/dcinit/templates/domain-zone-dns.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append domain‑zone.conf for child domain's configuration
	print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
	ProcessTemplateFile($template_file);
	
	# Defining the template file to be used for generation of domain‑zone.conf file
	$template_file = "/opt/novell/xad/share/dcinit/templates/domain-zone-dns-reverseip.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append domain‑zone.conf for child domain's configuration
	print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
	ProcessTemplateFile($template_file);
	
	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	# Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;
	

	# Doing kinit to get the Kerberos tickets for the current domain Administrator.
	# Checking whether ADM_PASSWD is exported. If not prompt the user to enter.
	my ($ADM_PASSWD, $SASL_PATH, $lib) = "";

	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/printConfigKey.pl"))
	{
		print STDERR gettext("Missing file '/opt/novell/xad/share/dcinit/printConfigKey.pl'. This file is required for proper DNS configuration, exiting.."), "\n";
	exit();
	}

	$lib =`/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`;
	chomp $lib;
	$SASL_PATH = "/opt/novell/xad/$lib/sasl2/";
	

	if(!(defined $ENV{'ADM_PASSWD'}))
	{
		
		print STDERR gettext("Enter Domain Controller Administrator's password of the domain holding the DNS server:"), "\n";
		ReadMode 'noecho';
		$password = ReadLine 0;
		chomp $password;
		ReadMode 'normal';
		print "\n";
	
		my $DomainNC=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC" | sed 's/,/\./g' | tr "[:upper:]" "[:lower:]"`;
		chomp $DomainNC;
		my $Domain_admin = "cn=administrator.cn=users.$DomainNC";
		CheckCredentials($Domain_admin, $password);
		$ENV{'ADM_PASSWD'} = "$password";
	}
	
	if(!(defined $ENV{'SASL_PATH'}))
	{
		$ENV{'SASL_PATH'} = "$SASL_PATH";
	}

	# Check for file existence
	if(!(-e "/opt/novell/xad/bin/kinit"))
	{
		print STDERR gettext("Missing file '/opt/novell/xad/bin/kinit'. This file is required for proper DNS configuration, exiting.."), "\n";
		exit();
	}

	my $kinit_rc = `/opt/novell/xad/bin/kinit -N Administrator  2> /dev/null`;
	exitOnFailure("$?","Error while generating Kerberos credentials");		

	$Command = $Ldapsearch.' -H '.$Ldapi.' -s sub -Q -Y GSSAPI -b '.$LdapDomainName.' objectGUID -LLL '.$Ldapsearch_filter.' | grep -i "objectGUID::" 2> /dev/null';

	my $DomainSid = `$Command`;
	exitOnFailure("$?","Could not contact ldap server or could not do ldap search on the given domain.");	
	
	$DomainSid =~ s/objectGUID:: //g;
	chomp $DomainSid;

	# Generating the DomainGUID from DomainSid obtained
	print STDERR ">>", gettext("Generating DomainGUID..."), "\n";

	# Check for file existence
	if(!(-e "/opt/novell/xad/share/dcinit/base64ToGUID.pl"))
	{
		print STDERR gettext("Missing file '/opt/novell/xad/share/dcinit/base64ToGUID.pl'. This file is required for proper DNS Configuration, exiting.."), "\n";
		exit();
	}

	$DomainGUID = `/opt/novell/xad/share/dcinit/base64ToGUID.pl "$DomainSid"`;
	chomp $DomainGUID;
	
	if($DomainGUID eq "")
	{
		print STDERR gettext("Error while generating DomainGUID, exiting.."), "\n";
		exit();
	}

	# Setting the DomainGUID in hash and regenerating the <domainname>.zone
	# file
	print STDERR ">>", gettext("Setting DomainGUID..."), "\n";
	$Vars{"Domain GUID"} = "$DomainGUID";

	$template_file = "/opt/novell/xad/share/dcinit/templates/dns-zone.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append domain‑zone.conf for Child domain configuration.
	print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
	ProcessTemplateFile($template_file);

	# Restarting named services.
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	# post-configuration validation
	print STDERR ">>", gettext("Validating DNS setting done..."), "\n";
	my $Post_configuration_IP_check = `/usr/bin/host "$DnsDomainName" | awk -F " " '{print \$4}'`;
	chomp $Post_configuration_IP_check;

	my $Post_configuration_hostname_check = `/usr/bin/host "$IP_Address" | awk -F " " '{print \$5}' | awk -F "." '{print \$1}'`;
	chomp $Post_configuration_hostname_check;

	if(($Post_configuration_IP_check ne $IP_Address) || ($Post_configuration_hostname_check ne $Hostname))
	{
		print STDERR sprintf(gettext("%d: Failed while performing the post-configuration validation checks for DNS configuration. The DNS configuration might not have been done properly. Please perform the DNS configuration once again with proper paramaters. If the problem persists, contact the Administrator"), $?), "\n";
		exit();
	}

}

sub Stub_Zone_Configuration
{
	# Stub zone DNS configuration
	# The DNS server configuration file "/etc/named.d/domain‑zone.conf" needs to be
	# appended with proper stub entries to enable stub zones.

	# Validating IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf(gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP_Address), "\n";
		exit();
	}

	# Stripping out the domainname part from <hostname>.<domainname>
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];

	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;

	# Check if DNS server is configured already for the stub zone entries
	my $check = `cat /etc/named.d/domain-zone.conf | grep -i "$DnsDomainName" | awk -F " " '{print \$2}'`;
	$check =~ s/"//g;
	chomp $check;

	if("$check" eq "$DnsDomainName")
	{
		print STDERR sprintf(gettext("DNS server already has the DNS configuration for the stub entry of domain '%s'(IP: %s)"), $DnsDomainName, $IP_Address), "\n";
		exit();
	}

	# Setting up the Variables to be in domain-zone.conf file
	$Vars{"DNS Root"} = "$DnsDomainName";
	$Vars{"DNS Host Address"} = &reverseIPAddress("$IP_Address");
	$Vars{"IP Address"} = "$IP_Address";
	
	$template_file = "/opt/novell/xad/share/dcinit/templates/domain-zone-dns-stubentry.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append the stub entries in domain-zone.conf file
	print STDERR ">>", gettext("Updating domain-zone.conf file.."), "\n";
	ProcessTemplateFile($template_file);

	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	# Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;

}

sub Additional_Dc_Removal
{
	# Removal of DNS configuration done by Dnsconfig.pl
	# for additional Domain controller.
	# We need to remove <ipaddr>.zone file for additional 
	# domain configuration, remove the appended contents from
	# <domainname>.zone file and domain-zone.conf file
	
	my $rc = 0;
	# Validating the IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf (gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."),$IP_Address), "\n";
		exit();
	}

	# Stripping out the domainname part from <hostname>.<domainname>
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];

	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;

	# Check if DNS server is configured already for the additonal
	# Domain Controller.
	my $check = `cat /var/lib/named/dyn/"$DnsDomainName".zone | grep -i "$IP_Address" | grep -v -i ";"`;
	$check =~ s/\s+//g;
	$check =~ s/IN([a-z,A-Z,0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)//g;
	chomp $check;
	if($check ne $Hostname)
	{
		print  STDERR sprintf(gettext("DNS server is not configured for the %s(IP:%s) as an additonal Domain Controller for domain '%s'."), $Hostname, $IP_Address, $DnsDomainName),"\n";
		exit();
	}

	# Check the existence of <ip_addr>.zone file
	if(!(-e "/var/lib/named/$IP_Address.zone"))
	{
		print STDERR sprintf(gettext("Missing %s.zone file. The DNS server does not holds full configuration for domain '%s' (IP: %s), exiting.."), $IP_Address, $DnsDomainName, $IP_Address), "\n";
		exit();
	}
	else
	{
		# remove the file <ip_addr>.zone
		print STDERR ">>", sprintf(gettext("Removing %s.zone file..."), $IP_Address), "\n";
		unlink("/var/lib/named/$IP_Address.zone") or die "Cannot delete $IP_Address.zone file :$!\n";
	}

	# Check the existence of <domainname>.zone file
	if(!(-e "/var/lib/named/dyn/$DnsDomainName.zone"))
	{
		print STDERR sprintf(gettext("Missing %s.zone file. The DNS server does not holds full configuration for domain '%s' (IP: %s), exiting.."), $DnsDomainName, $DnsDomainName, $IP_Address), "\n";
		exit();
	}
	else
	{
		#remove the appended text for the additional DC configured
		print STDERR ">>", sprintf(gettext("Updating %s.zone file..."), $DnsDomainName), "\n";
		Remove_Appended_Conf("/var/lib/named/dyn/$DnsDomainName.zone", $DnsDomainName);
		
	}

	# Check the existence of domain-zone.conf file
	if(!(-e "/etc/named.d/domain-zone.conf"))
	{
		print STDERR sprintf(gettext("Missing domain-zone.conf. This server is not configured as a default DNS server, exiting...")), "\n";
		exit();
	}
	else
	{
		# remove the appended text for the additional DC configured
		print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
		Remove_Appended_Conf("/etc/named.d/domain-zone.conf", $DnsDomainName);
	}

	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	#Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;
}

sub Child_Domain_Removal
{
	# Removal of DNS configuration done by Dnsconfig.pl
	# for child domain's domain controller
	# We need to remove <ipaddr>.zone file for child domain
	# configuration, remove the appended contents from
	# <domainname>.zone file and domain-zone.conf file

	my $rc = 0;

	# Validating the IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf (gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP_Address), "\n";
		exit();
	}

	# Stripping out the domainname part from <hostname>.<domainname>
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];

	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;

	# Check if DNS server is configured already for the child domain
	if((!(-f "/var/lib/named/dyn/$DnsDomainName.zone")) || (!(-f "/var/lib/named/$IP_Address.zone")))
	{
		print STDERR sprintf(gettext("DNS server is not configured for the domain %s."), $DnsDomainName), "\n";
		exit();
	}

	# Check for the existence of <ip_addr>.zone file
	if(!(-e "/var/lib/named/$IP_Address.zone"))
	{
		print STDERR sprintf(gettext("Missing %s.zone file. The DNS server does not holds full configuration for domain '%s' (IP: %s), exiting.."), $IP_Address, $DnsDomainName, $IP_Address), "\n";
		exit();
	}
	else
	{
		#remove the <ip_addr>.zone file
		print STDERR ">>", sprintf(gettext("Removing %s.zone file..."), $IP_Address), "\n";
		unlink("/var/lib/named/$IP_Address.zone") or die "Cannot delete $IP_Address.zone file :$!\n";
	}

	# Check the existence of <domainname>.zone file
	if(!(-e "/var/lib/named/dyn/$DnsDomainName.zone"))
	{
		print STDERR sprintf(gettext("Missing %s.zone file. The DNS server does not holds full configuration for domain '%s' (IP: %s), exiting.."), $DnsDomainName, $DnsDomainName, $IP_Address), "\n";
		exit();
	}
	else
	{
		#remove the appended text for the child domain configured
		print STDERR ">>", sprintf(gettext("Updating %s.zone file..."), $DnsDomainName), "\n";
		Remove_Appended_Conf("/var/lib/named/dyn/$DnsDomainName.zone", $DnsDomainName);
		$rc = Compute_File_deletion("/var/lib/named/dyn/$DnsDomainName.zone");
		if($rc)
		{
			# file is empty, delete it
			unlink("/var/lib/named/dyn/$DnsDomainName.zone") or die "Can't delete file '/var/lib/named/dyn/$DnsDomainName.zone' :$!\n";
		}
	}

	# Check the existence of domain-zone.conf file
	if(!(-e "/etc/named.d/domain-zone.conf"))
	{
		print STDERR sprintf(gettext("Missing domain-zone.conf. This server is not configured as a default DNS server, exiting...")), "\n";
		exit();
	}
	else
	{
		#remove the appended text for the child domain configured
		print STDERR ">>", gettext("Updating domain-zone.conf file..."), "\n";
		Remove_Appended_Conf("/etc/named.d/domain-zone.conf", $DnsDomainName);
	}

	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	#Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;
}

sub Secondary_DNS_Server_Configuration
{
	# Secondary DNS server configuration for a specific zone
	# The DNS server configuration file "/etc/named.d/domain‑zone.conf" needs to be
	# appended with proper zone entries to enable the DNS server  to act as a slave/
	# secondary server for that particular zone.

	# Validating IP address passed
	my $IP_check = Validate_IP($IP_Address);
	if($IP_check != 0)
	{
		print STDERR sprintf(gettext("The IP address entered, %s, is not valid. Enter a valid IP address, exiting.."), $IP_Address), "\n";
		exit();
	}

	# Validating the IP address of the master DNS server passed
	@Master_IPs = split(';', $Master_IP_Address);
	$IP_Count = @Master_IPs;
	if($IP_Count != 0)
	{
		foreach my $IP (@Master_IPs)
		{
			$IP_check = Validate_IP($IP);
			if($IP_check != 0)
			{
				print STDERR sprintf(gettext("The IP address entered for Master DNS server, %s, is not valid. Enter a valid IP address, exiting.."), $IP), "\n";
				exit();
			}
		}
	}

	# Stripping out the domainname part from <hostname>.<domainname>		
	my @Hostname_check = split('\.', $Hostname);
	$Hostname = $Hostname_check[0];

	# Converting the 'dc' formatted domainname to DNS formatted name
	$DnsDomainName = $Domain_Name;
	$DnsDomainName =~ s/dc=//g;

	# Check if DNS server is configured already for the zone entry
	my $check = `cat /etc/named.d/domain-zone.conf | grep -i "$DnsDomainName" | head -1 | awk -F " " '{print \$2}'`;
	$check =~ s/"//g;
	chomp $check;


	if("$check" eq "$DnsDomainName")
	{
		print STDERR sprintf(gettext("DNS server already has the DNS configuration for the domain '%s'(IP: %s)"), $DnsDomainName, $IP_Address), "\n";
		exit();
	}

	# Computing the Master DNS server(s) IP Address string
	my $Master_IP_string = "";
	foreach my $IP (@Master_IPs)
	{
		$Master_IP_String .= "$Master_IP_string$IP;";
	}

	# Setting up the Variables to be in domain-zone.conf file
	$Vars{"DNS Root"} = "$DnsDomainName";
	$Vars{"DNS Host Address"} = &reverseIPAddress("$IP_Address");
	$Vars{"IP Address"} = "$IP_Address";
	$Vars{"Master Server IP"} = "$Master_IP_String";

	$template_file = "/opt/novell/xad/share/dcinit/templates/domain-zone-dns-slave-zoneentry.conf";

	# Check for template file existence
	if(!(-e $template_file))
	{
		print STDERR sprintf(gettext("Missing template file %s. This file is required for proper DNS configuration, exiting.."), $template_file), "\n";
		exit();
	}

	# Calling the function to append the stub entries in domain-zone.conf file
	print STDERR ">>", gettext("Updating domain-zone.conf file.."), "\n";
	ProcessTemplateFile($template_file);

	# Restart named services
	print STDERR ">>", gettext("Restarting Name services.."), "\n";
	$rc = `/etc/init.d/named restart >/dev/null 2>&1`;

	# Restart nscd services
	print STDERR ">>", gettext("Restarting nscd services.."), "\n";
	$rc = `/etc/init.d/nscd restart >/dev/null 2>&1`;
		
}	

if(($action eq "add"))
{
	if(($IP_Address eq " ") || ($IP_Address eq "-d") || ($IP_Address eq "-s"))
	{
		print STDERR gettext("Specify the IP Address of the Domain Controller added."), "\n";
		exit();
	}
	if(($Domain_Name eq " ") || ($Domain_Name eq "-s") || ($Domain_Name eq "-p"))
	{
		print STDERR gettext("Specify the Domain name."), "\n";
		exit();
	}
	if(($Hostname eq " ") || ($Hostname eq "-d") || ($Hostname eq "-p"))
	{
		print STDERR gettext("Specify the Hostname of the Domain Controller added."), "\n";
		exit();
	}
	if($Additional_dc)
	{
		Additional_Dc_Configuration();
		print STDERR sprintf(gettext("DNS Configuration for the Domain Controller '%s' is done."), $Hostname), "\n";
	}
	if($Child_Domain)
	{
		Child_Domain_Configuration();
		print STDERR sprintf(gettext("DNS Configuration for the Domain Controller '%s' is done"), $Hostname), "\n";
	}
	if($Stub_zone)
	{
		Stub_Zone_Configuration();
		print STDERR sprintf(gettext("DNS Configuration for stub zone entry of '%s' is done"), $Domain_Name), "\n";
	}

	if($Secondary_Server_Conf)
	{
		if(($Master_IP_Address eq " ") || ($Master_IP_Address eq "-d") || ($Master_IP_Address eq "-s") || ($Master_IP_Address eq "-p"))
		{
			print STDERR gettext("Specify the IP Address(es) for Master/Primary DNS Server."), "\n";
			exit();
		} 
		Secondary_DNS_Server_Configuration();
		print STDERR sprintf(gettext("Secondary DNS configuration for '%s' is done"), $Domain_Name), "\n";
	}
}
elsif(($action eq "remove"))
{
	if($IP_Address eq " ")
	{
		print STDERR gettext("Specify the IP Address of the Domain Controller added."), "\n";
		exit();
	}
	else
	{
		# detecting the hostname associated with IP address
		$Hostname = `/usr/bin/host $IP_Address | awk -F " " '{print \$5}' | awk -F "." '{print \$1}'`;
		chomp $Hostname;

		# getting the domain name associated with IP address
		$Domain_Name =`/usr/bin/host $IP_Address | awk -F " " '{print \$5}'`;
		@Domainname_array = split /\./, $Domain_Name;
		shift @Domainname_array;
		$Domain_Name = join('.', @Domainname_array);
		chomp $Domain_Name;
		chop $Domain_Name;

		#Display the determined Hostname and Domainname to user
		# asking for confirmation
		print STDERR "\n", sprintf(gettext(" Host '%s' and domain '%s' were found to be associated with IP '%s'.Enter 'yes' to continue with the detected host and domain name or 'no' to enter different host name and domain name:"), $Hostname, $Domain_Name, $IP_Address);
		$Verfication_input = <STDIN>;
		chomp $Verfication_input;

		if($Verfication_input eq "no")
		{
			# Get the hostname and domain name input from user
			print STDERR "\n", gettext("Enter the host name:");
			$Hostname = <STDIN>;
			chomp $Hostname;

			print STDERR "\n", gettext("Enter the domain name:");
			$Domain_Name = <STDIN>;
			chomp $Domain_Name;
		}
	}
	if($Additional_dc)
	{
		Additional_Dc_Removal();
		print STDERR sprintf(gettext("DNS Configuration for the Domain Controller '%s' is successfully removed."),$Hostname), "\n";
	}
	if($Child_Domain)
	{
		Child_Domain_Removal();
		print STDERR sprintf(gettext("DNS Configuration for the Domain Controller '%s' is successfully removed."),$Hostname), "\n";
	}
}
else
{
	HELP_MESSAGE();
}
