#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl
################################################################################
# Domain Services for Windows Provisioning Tool
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2001-2020
#       Micro Focus or one of its affiliates.
################################################################################


use strict;
use warnings;
use Getopt::Long;
use helpers::remoteStatus;
use helpers::utility;
use helpers::containerCheck;
use helpers::ndsServerHealthCheck;
use XAD::registry;

my $launch_config_wizard = registry::getReg("LAUNCH_CONFIG_WIZARD");

sub usage {

	print <<USAGE;
	healthCheck.pl
	-a  : Health check for all modules
	-r  : Remote Server Health check
	-n  : DNS Server Health check
	-b  : Check for Bad Address Cache
	-c  : Check for Purger Status
	-t  : Top level containers check
	-e  : Health check for remote eDirectory servers
	-F  : Log file name
	-L  : Log level (1-5)
	-I  : IP Address (Edir server IP for NM FRD, Parent Server IP for NM/NNM ADC/CHILD, optional for NNM FRD)
	-A  : The LDAP formatted FQDN of admin user( eg- NM --> cn=admin,ou=novell, NNM --> cn=administrator,cn=users,dc=dsfw,dc=com)
	-S  : SSL Port Number 
	-h or --help : Help on the command line options

USAGE
   	exit 0
}

# Getting the health check options 
sub getHlthChkOptions { 
    my %hlthChk = ();
    my ($logFileName, $ipAddress, $adminDN, $logLevel, $sslPort, $help);
               
    $logFileName =0;
    $ipAddress = ""; 
    $adminDN = 0;
    $logLevel = 1;
    $sslPort = 0;
    $help = 0;

    Getopt::Long::Configure("no_ignore_case");

    my $error = GetOptions(\%hlthChk,'r','a','n','b','c','t','e',
               'F=s' => \$logFileName,
               'L=i' => \$logLevel,
               'I=s' => \$ipAddress,
               'A=s' => \$adminDN,
               'S=i' => \$sslPort,
               'h|help' => \$help) or eval 
                    { 
                        usage(); 
                        exit (-1); 
                    };
    if($ARGV[0]) { 
	    print "The health check done with an invalid option [$ARGV[0]]"; 
	    usage(); 
	    exit (-1); 
    }

    if ($help) { 
        usage(); 
        exit(0);
    }

    return (\%hlthChk, $logFileName, $logLevel, $ipAddress, $adminDN, $sslPort);
}

package main;
{
	my ($path, $error, $val, $opt, $instType, $optCount) = 0;

	my $hlthChkOpt = 0; 
	my ($logFileName, $logLevel, $adminDN, $installationType, $domainType, $sslPort) = 0;
	my $treeIPAddress = "";
	my $hostName = "";

	$sslPort = 0;

	# Get the command line options and put in local data structures. 
	($hlthChkOpt, $logFileName, $logLevel, $treeIPAddress, $adminDN, $sslPort) =  getHlthChkOptions();


	$optCount = keys %$hlthChkOpt; 
	if (!(keys %$hlthChkOpt)) {
		usage();
		$error = -1;
		goto ERROR;
	}

	helpers::logAttrsSet($logFileName, $logLevel);

	helpers::logWrite(0, "Starting the Health Check");

	($installationType, $domainType, $instType) = helpers::getInstallationType();
	helpers::logWrite(5, " installationType [$installationType], domainType[$domainType], inst [$instType]");

	my $xaddir = "/var/opt/novell/xad/";
	my $fpartition = `df -kh $xaddir | grep -v "^Filesystem" | awk -F" " '{print \$1}'`;
	chomp($fpartition);

	my $type = `df -T $fpartition | tail -1 | awk '{print \$2}'`;
	chomp($type);

	my $logMessage = "The partition [$fpartition] mounted on [$xaddir] is not supporting";
	my $logMessage2 = "Please rectify the problem before continuing.";

	my $retval = helpers::isAclSupported($fpartition);
	if ($retval == 1) {
		helpers::logWrite(0, "$logMessage acls. $logMessage2");
		$error = -1;
		goto ERROR;
	} elsif ($retval == 2) {
		# Failed to create a temporary file.
		logWrite(0, "Failed to determine whether [$fpartition] supports ACL or not.");
		$error = -1;
		goto ERROR;
	}
	helpers::logWrite(5, "The partition [$fpartition] of filesystem type [$type] supports ACL");

	$retval = helpers::isXattrSupported($fpartition);
	if ($retval == 1) {
		helpers::logWrite(0, "$logMessage Extended Atrributes. $logMessage2");
		$error = -1;
		goto ERROR;
	} elsif ($retval == 2) {
		# Failed to create a temporary file.
		logWrite(0, "Failed to determine whether [$fpartition] supports extended attributes or not.");
		$error = -1;
		goto ERROR;
	}
	helpers::logWrite(5, "The partition [$fpartition] of filesystem type [$type] supports extended attributes");

	if ($treeIPAddress eq "" || not defined $treeIPAddress) {
		my $is_replica = registry::getReg("IS_REPLICA");
		my $domain_name = "";
		if ($is_replica eq "TRUE") {
			$domain_name = registry::getReg("DNS Root");
			my $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$domain_name +short`;
			chomp($dig_output);
			my @split_out = split(' ', $dig_output);
			my $dns_hostname = $split_out[3];
			$treeIPAddress = `/usr/bin/dig $dns_hostname +short`;
			chomp($treeIPAddress);
			if ($treeIPAddress eq "") {
				helpers::logWrite(0,"The IP Address of the domain PDC is not obtained");
				$error = -1;
				goto ERROR;
			}
		} elsif ($domainType eq "child") {
			$domain_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`;
			chomp($domain_name);
			my $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$domain_name +short`;
			chomp($dig_output);
			my @split_out = split(' ', $dig_output);
			my $dns_hostname = $split_out[3];
			$treeIPAddress = `/usr/bin/dig $dns_hostname +short`;
			chomp($treeIPAddress);

			#my @nsout;
			#chomp($parent_domain);
			#@nsout = `nslookup $parent_domain `;
			#chomp($nsout[4]);
			#$treeIPAddress = `echo $nsout[4] | awk -F \"Address: \" '{print \$2}'`;
			if ($treeIPAddress eq "") {
				helpers::logWrite(0,"The IP Address of the parent is not obtained");
				$error = -1;
				goto ERROR;
			}
		}
	}

	if ($hostName eq "" || not defined $hostName) {
		my $is_replica = registry::getReg("IS_REPLICA");
		my $domain_name = "";
		if ($is_replica eq "TRUE") {
			$domain_name = registry::getReg("DNS Root");
			chomp($domain_name);
			my $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$domain_name +short`;
			chomp($dig_output);
			my @split_out = split(' ', $dig_output);
			my $dns_hostname = $split_out[3];
			$dns_hostname =~ s/.$//; # replace . at the end of the string with empty string
			chomp($dns_hostname);
			$hostName = $dns_hostname;
			if ($hostName eq "") {
				helpers::logWrite(0,"The host name of the domain PDC is not obtained");
				$error = -1;
				goto ERROR;
			}
		} elsif ($domainType eq "child") {
			$domain_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`;
			chomp($domain_name);
			my $dig_output = `/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$domain_name +short`;
			chomp($dig_output);
			my @split_out = split(' ', $dig_output);
			my $dns_hostname = $split_out[3];
			$dns_hostname =~ s/.$//; # replace . at the end of the string with empty string
			chomp($dns_hostname);
			$hostName = $dns_hostname;
			if ($hostName eq "") {
				helpers::logWrite(0,"The host name of the parent is not obtained");
				$error = -1;
				goto ERROR;
			}
		} elsif ($domainType eq "frd") {
			$hostName = `/bin/hostname -f`;
			chomp($hostName);
			if ($hostName eq "") {
				helpers::logWrite(0,"The host name of the root domain PDC is not obtained");
				$error = -1;
				goto ERROR;
			}
		}
	}

	# Validate - IP, Admin DN. The password is deduced based on the installation type.
	$error = helpers::validateParams($adminDN, $treeIPAddress, $installationType, $domainType);
	if ($error) {
		helpers::logWrite(0, "The parameters passed to the health check utility are not proper. <$error>");
		exit ($error);
	}

	# This is for the future Edir support of custom SSL port.
	if (!$sslPort) {
		helpers::logWrite(3, "The ssl Port is not supplied, taking the default one (636)");
		$sslPort = 636; 
	}

	while (($opt, $val) = each (%{$hlthChkOpt})) {
		SWITCH : {
			 if ($opt eq "r" && $val == 1) {
				 ($error) = remoteStatusCheck($treeIPAddress, $hostName);
				 if ($error == 101) {
					 helpers::logWrite(0, "The remote server health check is not done"); 
					 $error = 0;
					 last SWITCH;
				 }
				 elsif ($error){
					 helpers::logWrite(0, "The remote server health check failed <$error>");
					 goto ERROR;
				 }
				 last SWITCH;
			 }

			# DNS query change
			 if ($opt eq "n" && $val == 1) {

				 if ($domainType eq "frd") {
					 helpers::logWrite(0, "The DNS server check need not be done for the FRD installation");
					 $error = 0;
				 }
				 else {
					 if ($domainType eq "child") {
						 $error = check_dns_on_frd_parent();
						 if ($error) {
							 helpers::logWrite(1, "The DNS is not properly configured for the domain installation");	
							 goto ERROR;
						 }
					 }
					 my $dnsMaster = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Master"`;
					 chomp($dnsMaster);
					 if ($dnsMaster eq "TRUE") {
						 helpers::logWrite(0,"The DNS is getting configured on locally, so DNS check is not required.");
						 $error = 0;
					 }
					 else {
						 ($error) = dnsServerStatusCheck();
						 if ($error) {
							 helpers::logWrite(0, "The DNS server status health check failed <$error>");
							 goto ERROR;    
						 }
					 }
				 }
				 last SWITCH;
			 }                

			# Bad Address Cache reset
			 if ($opt eq "b" && $val == 1) {
				 ($error) = badAddressCacheRemove();
				 if ($error != 0) {
					 helpers::logWrite(0, "The bad address cache health check failed <$error>");
					 goto ERROR;    
				 }
				 last SWITCH;
			 }

			# Purger process execution
			 if ($opt eq "c" && $val == 1) {
				 if (!$instType) {
					 helpers::logWrite(0, "Failed to run the purger process"); 
					 $error = 131;
					 goto ERROR;
				 }
				 if ($instType eq "nnmfrd") {
					 helpers::logWrite(0, "The background process need not run on non-name mapped frd");
					 $error = 0;
				 }
				 else {
					 ($error) = runBackgroundProcess($instType, $adminDN, $treeIPAddress, $sslPort);
					 if ($error != 0) {
						 helpers::logWrite(0, "Failed to do the background process health check. <$error>");
						 goto ERROR;    
					 }
				 }
				 last SWITCH;
			 }

			# Top level containers check 
			 if ($opt eq "t" && $val == 1) {
				 if (!$installationType) {
					 helpers::logWrite(0, "The installation type [$installationType] or domain type [$domainType] not supplied");
					 $error = 141;
					 goto ERROR;
				 }

				 if ($installationType eq "nm") {
					 ($error) = helpers::topLevelContainersCheck($treeIPAddress, $adminDN, $sslPort);
					 if ($error) {
						 helpers::logWrite(0, "The top level containers check failed. <$error>"); 
						 goto ERROR;
					 }
				 }
				 else
				 {
					 helpers::logWrite(0, "The top level container is applicable for only name mapped setup");
					 $error = 0;
				 }

				 last SWITCH;
			 }

			# Health check of the NDS servers
			 if ($opt eq "e" && $val == 1) {
				 ($error) = helpers::ndsServerStatusCheck($adminDN, $installationType, $domainType, $treeIPAddress, $sslPort);
				 if ($error) {
					 helpers::logWrite(0, "The nds server health check failed. <$error>"); 
					 if($error == 157){
						 helpers::logWrite(0,"The DSfW Schema is at OES2SP2 level. To utilize features introduced in OES2SP3 or later releases, DSfW schema must be upgraded to the latest");
					 }
					 else{
						 goto ERROR;
					 }
				 }
				 last SWITCH;
			 }

			#All the checks
			 if ($opt eq "a" && $val == 1) {
				 { # Remote server status;
					 ($error) = remoteStatusCheck($treeIPAddress, $hostName);
					 if ($error && $error != 101) {
						 helpers::logWrite(0, "The remote server health check failed. <$error>");
						 goto ERROR;
					 }
				 }

				 { # DNS server status;
					 if ($domainType eq "frd") {
						 helpers::logWrite(0, "The DNS server check need not be done for the FRD installation");
						 $error = 0;
					 }
					 else {
						 if ($domainType eq "child") {
							 $error = check_dns_on_frd_parent();
							 if ($error) {
								 helpers::logWrite(1, "The DNS is not properly configured for the domain installation");	
								 goto ERROR;
							 }
						 }
						 my $dnsMaster = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Master"`;
						 chomp($dnsMaster);
						 if ($dnsMaster eq "TRUE") {
							 helpers::logWrite(0,"The DNS is getting configured on locally, so DNS check is not required.");
							 $error = 0;
						 }
						 else {
							 ($error) = dnsServerStatusCheck();
							 if ($error) {
								 helpers::logWrite(0, "The dns server health check failed. <$error> ");  
								 goto ERROR;
							 }     
						 }	
					 }
				 }

				 { # Bad Address Cache Reset;
					 ($error) = badAddressCacheRemove();
					 if ($error) {
						 helpers::logWrite(0, "The bad address cache health check failed. <$error>"); 
						 goto ERROR;
					 }
				 }

				 { # Run background process ;
					 if (!$instType ) { 
						 helpers::logWrite(0, "Failed to run the background process without knowing the installation type."); 
						 $error = 131;
						 goto ERROR;
					 }

					 if ($instType eq "nnmfrd") {
						 helpers::logWrite(0, "The purger process need not be run on the non-name mapped frd");
						 $error = 0;
					 }
					 else {
						 ($error) = runBackgroundProcess($instType, $adminDN, $treeIPAddress, $sslPort);
						 if ($error) {
							 helpers::logWrite(0, "Failed to do the background process health check. <$error>");
							 goto ERROR;
						 }
					 }
				 }

				 { # Top level container validation               
					 if (!$installationType) {
						 helpers::logWrite(0, "The installation type [$installationType] or domain type [$domainType] not supplied");
						 $error = 141;
						 goto ERROR;
					 }

					 if ($installationType eq "nm") {
						 ($error) = helpers::topLevelContainersCheck($treeIPAddress, $adminDN, $sslPort);
						 if ($error) {
							 helpers::logWrite(0, "The top level containers check failed. <$error>"); 
							 goto ERROR;
						 }
					 }
					 else {
						 helpers::logWrite(0, "The top level container check applicable for only name mapped setup"); 
						 $error = 0;
					 }
				 }

				 {# The NDS Server status check
					 ($error) = helpers::ndsServerStatusCheck($adminDN, $installationType, $domainType, $treeIPAddress, $sslPort);
					 if ($error) {
						 helpers::logWrite(0, "The nds server health check failed. <$error>"); 
						 goto ERROR;
					 }
				 }

				 last SWITCH;
			 }

			# Incase of an invalid option being supplied.
			 {
				 helpers::logWrite(0, "Invalid option supplied $opt");
				 usage();
				 $error = -1;
				 goto ERROR;
			 }
		 }
	}
	helpers::logWrite(0, "The health check completed successfully ");
	exit(0);

ERROR:
	helpers::logWrite(0, "The health check failed. <$error>"); 
	exit($error);
}



sub remoteStatusCheck {
    my ($ipAddress, $hostName) = @_;
    my ($domainNC, $forestNC, $parentDomain, $currentDomain, $isReplica);
    my ($message, $error);

    $domainNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Domain NC"`; 
    $forestNC = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest NC"`; 
    $parentDomain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`; 
    $currentDomain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`; 
    $isReplica = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Is Replica"`;

    chomp($domainNC, $forestNC, $parentDomain, $currentDomain, $isReplica);

    if ($domainNC eq $forestNC && $isReplica eq "FALSE") {
        helpers::logWrite(1, "This is the first domain controller (FRD), so remote health check is not needed. ");
        return (101);
    }

    helpers::logWrite(3, "Starting the remote server health check ");

    #The remote server check is based on the assumption that two enviornments are set:
    #   1. ADM_PASSWD_PARENT - The password of the remote server which is being queried.

    if (!$ENV{'ADM_PASSWD_PARENT'}) {
        helpers::logWrite(0, "The administrator password for parent domain not set ");
        return (102);
    }
        
    helpers::logWrite(5, "The Parent Domain name: $parentDomain");
    helpers::logWrite(5, "The Current Domain name: $currentDomain");

# the order should be 
#1.remote server's DNS resolution (anyhow if its an child or ADC installation resolv.conf will point to the proper DNS server.
#2. ldap and eDir check
#3. kerberos
#4. rpc & smb
#5. nmb
    
    ($message, $error) = helpers::ldapStatus($parentDomain);
    if ($error) {
        helpers::logWrite (1, "LDAP server health check failed. : $message : $error ");
        return (105);
    }

    my $principal_name = "";
    my $samacc_type = "";
    my $domain_name = "";
    my $admin_password = "";
    if ($isReplica eq "TRUE") {
	$samacc_type = "SAMACCOUNTNAME_DOMAIN";
	$domain_name = $currentDomain;
	$admin_password = $ENV{'ADM_PASSWD'};
    }
    else {
	$samacc_type = "SAMACCOUNTNAME_PARENT";
	$domain_name = $parentDomain;
	$admin_password = $ENV{'ADM_PASSWD_PARENT'};
    }

    my $admin_name = `/opt/novell/xad/share/dcinit/printConfigKey.pl $samacc_type`;

    # This section is added for health check in configuration wizard.
    # Before to DSfW upgrade, anyone changes the domain administrator
    # SAMAccountname then the administrator name deduced through the 
    # printConfigKey.pl will be incorrect.
    # To fix this issue, obtaining latest value of SAMAccountName
    # for parent domain administrator in case of child DC and local
    # domain administrator in case of ADC.

    if (uc($launch_config_wizard) eq "YES") {
	$admin_name = `/opt/novell/xad/share/dcinit/provisionTools.sh \\
			get-admin-sam-name -d $domain_name`;
    }
    chomp($admin_name);
    $principal_name = "'$admin_name'\@".uc($domain_name);
    ($message, $error) = helpers::kerberosStatus($principal_name, $admin_password);

    if ($error) {
        helpers::logWrite (1, "Kerberos Service health check failed : $message: $error ");
        return (103);
    }
    if ($isReplica eq "TRUE") {
        ($message, $error) = helpers::rpcNSambaStatus($principal_name, $ENV{'ADM_PASSWD'}, $ipAddress, $hostName);
    }
    else {
        ($message, $error) = helpers::rpcNSambaStatus($principal_name, $ENV{'ADM_PASSWD_PARENT'}, $ipAddress, $hostName);
    }
    if ($error) {
        helpers::logWrite (1, "RPC and Samba Service health check failed : $message: $error ");
        return (104);
    }

    ($message, $error) = helpers::nmbStatus($ipAddress);
    if ($error) {
        helpers::logWrite (1, " NMB status health check failed. : $message : $error ");
        return (105);
    }

    helpers::logWrite (0, "Health checked with the remote domain [$parentDomain] successfull. "); 
    return (0);
}


# The subroutine to check the DNS server status;
# Assumption 
# 1. The DNS_SERVER_IP is exported as env variable. This could be computed
#    similar to what is done in ndsdcinit script.

sub dnsServerStatusCheck {
    my ($ipAddress, $hostName, @nsOut, $error) = 0;

    helpers::logWrite(1, "Starting the DNS server health check ");
    
    # With old install, this isn't present. With new perl implementation this is present.
    $ipAddress = `/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS_SERVER_IP"`; 
    if ($? != 0) {
        $ipAddress = $ENV{'DNS_SERVER_IP'};
        if (!$ipAddress) {
            helpers::logWrite(1, "The ip address of the DNS is not yet available. ");
            return (111);
        }
    }
    chomp($ipAddress); 
    helpers::logWrite(4, "The DNS server IP is [$ipAddress]");

# Forward lookup
    @nsOut = `nslookup $ipAddress`;  

    chomp($nsOut[0]);

# Changing the $nsOut array index from 3 to 0 because of "nslookup" command output has changed in SLES12SP5.
    if ($nsOut[0] =~ /(\S*)(\s*)name =(\s*)(\S*)./) {
        $hostName = $4;
        helpers::logWrite(1, "Successfully obtained the hostname $hostName for IP $ipAddress ");
    }
    else {
        $error = $? >> 8;
        helpers::logWrite(1, "The IP address lookup from the DNS server failed. <$error>"); 
        return (112);
    }

# Reverse lookup
    @nsOut = 0;
    @nsOut = `nslookup $hostName`; 
    
    chomp ($nsOut[3]);
   
    if ($nsOut[3] =~ /Name:(\s*)(\S*)/) {
        my ($hostOut, $ipOut) = 0;
        $hostOut = $2;
        if ($hostOut eq $hostName) {
            chomp($nsOut[4]);
            if ($nsOut[4] =~ /Address:(\s*)(\S*)/) {
                $ipOut = $2;
                if ($ipOut eq $ipAddress) {
                    helpers::logWrite(1, "The ip address from reverse lookup is similar to input");
                } 
                else {
                    helpers::logWrite(1, "The ip address [$ipOut] from the reverse \
                                    lookup of [$hostName] doesn't match input [$ipAddress");
                    return (113);
                }
            }
            else {
                helpers::logWrite(1, "Failed to get the ip adddress in from the host name $hostName");
                return (114);
            }
        }
        else {
            helpers::logWrite(1, "The host name output <$hostOut> is not same as supplied $hostName");
            return (115);
        }
    }
    else {
        helpers::logWrite(1, "Failed to get the IP Address for the host name $hostName: $nsOut[3]");
        return (116);
    }
                
    helpers::logWrite(0, "Health checked for the DNS server [$ipAddress] is successfull");
    return (0);
}


# The subroutine to remove the local bad address cache.
sub badAddressCacheRemove {
    my ($output, $pid, $error, $data);
    my ($binDir) = "/opt/novell/eDirectory/bin";
    my ($confDir) = "/etc/opt/novell/eDirectory/conf";


    helpers::logWrite(1, "Resetting the Bad Address cache ");
# Load the dstrace
    system("$binDir/ndsstat --config-file $confDir/nds.conf -c \"load dstrace\">/dev/null 2>&1");
    if ($?) {
        $error = $? >> 8;
        helpers::logWrite(1, "Failed to load the dstrace. <$error> ");
        return (121);
    }

# Remove the bad address cache.
    system("$binDir/ndsstat --config-file $confDir/nds.conf -c \"set ndstrace=*u\"");
    if ($?) {
        $error = $? >> 8;
        helpers::logWrite(1, "Failed to remove the bad address cache. <$error>");
        return (122);
    }

# Unload the dstrace 
    system("$binDir/ndsstat --config-file $confDir/nds.conf -c \"unload dstrace\">/dev/null 2>&1");
    if ($?) {
        $error = $? >> 8;
        helpers::logWrite(1, "Failed to unload the dstrace. <$error>");
        return (123);
    }

    helpers::logWrite(0, "Health check for the bad address cache is successfull ");
    return (0);
}


# The subroutine to start the background processes :
# 1. Purger     : For cleaning the marked for deletion objects.

sub runBackgroundProcess {
    my ($instType, $adminDN, $treeIPAddress, $sslPort) = @_;
    my ($parentDomain, $curDomain, @parentNC, $password, $error, $output) = 0;                                                                                                       
    helpers::logWrite(1, "Running the background process [PURGER]");

    if (!$adminDN) {
        helpers::logWrite(1, "The admin DN is not supplied");
        return 132;
    }
    if (lc($adminDN) !~ /^cn=/) { 
        helpers::logWrite(1, "The admin DN is not in LDAP format while running the background process");
        return 133;
    }

    if ($instType eq "nmfrd") {
        $password = $ENV {'NDSEXISTINGADMINPASSWD'};
    }
    else {
        $password = $ENV {'ADM_PASSWD_PARENT'};
    }

    if (!$password) {
        helpers::logWrite(1, "For the setup [$instType], the password for parent is not supplied");
        return 134;
    }

    $parentDomain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`; 
    $curDomain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Root Domain"`;

    chomp($parentDomain);
    chomp($curDomain);

	# Get the SSL certificate
    $error = helpers::getCert($adminDN, $password);
    if ($error) {
        helpers::logWrite(1, "Failed to get the certificate for the admin DN [$adminDN]");
        return 135;
    }

    helpers::logWrite(5, "The Prnt Dom[$parentDomain], Current Dom[$curDomain] AdmnDN [$adminDN], port [$sslPort], TreeIP [$treeIPAddress]"); 
	my $purger_ret = 0;
	# Running purger on tree server (nmfrd) or on its parent (child)
	if ($instType =~ /adc$/) {
		my $str = `dig $curDomain +short`;
		foreach(split(/\n/,$str)) {
			my $ipAddr = $_;
			$output = `/opt/novell/xad/sbin/purger $ipAddr $sslPort '$adminDN' '$password' /etc/opt/novell/certs/SSCert.der 2>&1`; 
			$purger_ret = $?;
			if($purger_ret == 0) {
				last;
			}
		}
	}
	elsif ($instType eq "nmfrd") {
		$output = `/opt/novell/xad/sbin/purger $treeIPAddress $sslPort '$adminDN' '$password' /etc/opt/novell/certs/SSCert.der 2>&1`; 
		$purger_ret = $?;
	}else {
		my $str = `dig $parentDomain +short`;
		foreach(split(/\n/,$str)) {
	                my $ipAddr = $_;
			$output = `/opt/novell/xad/sbin/purger $ipAddr $sslPort '$adminDN' '$password' /etc/opt/novell/certs/SSCert.der 2>&1`; 
			$purger_ret = $?;
			if($purger_ret == 0) {
                                last;
                        }
		}
	}

    if ($purger_ret) {
	$error = $purger_ret >> 8;
	chomp($output);
        helpers::logWrite(1, "Failed to run the purger process. [$output]. <$error>");
        return (136);
    }
    
    helpers::logWrite(0, "Health check for purger execution successfull.");
    return (0);
}


# Subroutine to check the resolution is happening for parent and the frd.
sub check_dns_on_frd_parent {
	my $parent_domain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Parent Domain"`;
	chomp($parent_domain);

	`/opt/novell/xad/share/dcinit/provisionTools.sh check-domain-resolution -d $parent_domain`;
	if ($?) {
		helpers::logWrite(1, "Failed to contact the parent domain [$parent_domain]");	
		return $?;	
	}

	my $forest_domain = `/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest Root"`;
	chomp($forest_domain);
	`/opt/novell/xad/share/dcinit/provisionTools.sh check-domain-resolution -d $forest_domain`;
	if ($?) {
		helpers::logWrite(1, "Failed to contact the forest root domain [$forest_domain]");	
		return $?;
	}

	helpers::logWrite(5, "Successfully validated the DNS configuration for parent and frd");
	return 0;
}
