#!/bin/sh
################################################################################
# Domain Services for Windows tool for obtaining ndsconfig Server Context
# Verify the supported server versions in the tree root during YaST install
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2006-2020
#	Micro Focus or one of its affiliates.
################################################################################

# Return Values:
#---------------
# 0   The given server meets the minimum version requirement for Domain
# 1   Server is not reachable using the given IP address or is not a server in
#     the Aquila Forest.
# 2   Either of the four parameters DomainName,AdminName,ExistingServerIP or
#     ForestName are not given.	

ADD_DC="false"
NM_CHILD="false"
if [ "$1" = "--nm-child" ]
then
	NM_CHILD="true"
	shift
fi

if [ "$1" = "--add-dc" ]
then 
	ADD_DC="true"
	shift
fi

if [ "$1" = "--ldap-command" ]
then 
	LDAP_COMMAND_PRINT="true"
	shift
fi


# Common Locations and Files
LDAP_COMMAND=/opt/novell/eDirectory/bin/ldapsearch
GETCERT_COMMAND=/opt/novell/oes-install/util/getSSCert
LDAP_PORT=389
LDAPS_PORT=636
SERVER_CERT=/etc/opt/novell/certs/SSCert.der
SERVER_PEM=/etc/opt/novell/certs/SSCert.pem
THIS_PRG=$0

# Input parsing.
DomainName=$1
NdsAdminName=$2
ExistingServerIP=`echo "$3" |cut -f 1 -d ':'`  # if the port is different, YaST should pass the address as xx.xx.xx.xx:xx
ExistingServerPort=`echo "$3" | cut -s -f 2 -d ':'`
test -z "$ExistingServerPort"  && ExistingServerPort=0
RequireSecureChannel=1
#ForestName=$

#
# Log messages to stderr.
#
LogToErr()
{
	echo 1>&2 "$1"
	test $2 -ne 0 && exit $2
}

#
# Print the given arguments for debugging
#
PrintArgs()
{
	LogToErr "LDAP Based utility [ndsConfigServerContext.sh] to retrieve server context for YaST" 0
	LogToErr "        DomainName        : $DomainName" 0
	LogToErr "        NdsAdminName      : $NdsAdminName" 0
	LogToErr "        ExistingServerIP  : $ExistingServerIP" 0
	test -n "$ExistingServerPort" && LogToErr "        ExistingServerPort: $ExistingServerPort" 0
	test "$ADD_DC" = "true" && LogToErr "        Add_DC            : true" 0
}

#
# This sub is used to check if LDAP server is reachable. Script will 
# exit from the same sub if none of the options work.
#
GetLDAPCommandString()
{
	if [ -z "$TREE_NAME" ]
	then
		TREE_NAME=`/opt/novell/xad/share/dcinit/ndsConfigGet.sh "n4u.base.tree-name"`
	fi

	if [ -z "$TREE_NAME" ]
	then
		LogToErr "Could not determine the tree name..." 0
	fi

	#Test if the LDAP server is reachable and we are able to do ldapsearch.  Attempt the secure port if none is given.
#	LD_LIBRARY_PATH=/opt/novell/eDirectory/lib64:/opt/novell/eDirectory/lib64/nds-modules:$LD_LIBRARY_PATH "${GETCERT_COMMAND}" -a $ExistingServerIP -t ${TREE_NAME} -u "${NdsAdminName}" -x "${ADM_PASSWD}" > /dev/null 2>&1
	`/opt/novell/xad/share/dcinit/getsscert_script.sh $ExistingServerIP`;
	if [ ! -f "${SERVER_CERT}" ]; then
		LogToErr "Unable to retrieve the CA certificate. Secure LDAP will not be attempted..." 0
		RequireSecureChannel=0
	fi

	openssl x509 -in $SERVER_CERT -inform DER > $SERVER_PEM

	# first try the given port. We will not attempt any other port if this fails.
	if [ $ExistingServerPort -ne 0 ]; then
		if [  $RequireSecureChannel -eq 1 ]; then
			LDAPCommand="/opt/novell/eDirectory/bin/ldapsearch -LLL -E 2.16.840.1.113719.1.513.4.5 -H ldaps://$ExistingServerIP:$ExistingServerPort"
		else
			LDAPCommand="/opt/novell/eDirectory/bin/ldapsearch -x -LLL -H ldaps://$ExistingServerIP:$ExistingServerPort -ZZ"
		fi
		LDAPTLS_CACERT=$SERVER_PEM $LDAPCommand -D "$LdapAdminName" -w "$ADM_PASSWD" -s base > /dev/null 2>&1
		test $? -eq 0 && return 
	else
		# First attempt Secure port if certificate is available.
		if [ $RequireSecureChannel -eq 1 ]; then
			LDAPCommand="/opt/novell/eDirectory/bin/ldapsearch -LLL -H ldaps://$ExistingServerIP:$LDAPS_PORT"
			LDAPTLS_CACERT=$SERVER_PEM $LDAPCommand -b '' -D "$LdapAdminName" -w "$ADM_PASSWD" -s base >/dev/null 2>&1
			test $? -eq 0 && return
		fi

		LogToErr "Unable to reach the server in secure port.  Attempting clear...." 0
		LDAPCommand="/opt/novell/eDirectory/bin/ldapsearch -LLL -h $ExistingServerIP -p $LDAP_PORT -x  -ZZ"
		LDAPTLS_CACERT=$SERVER_PEM $LDAPCommand -D "$LdapAdminName" -w "$ADM_PASSWD" -b "" -s base > /dev/null 2>&1
		test $? -eq 0 && return
		
	fi
	LogToErr "Unable to contact or authenticate to the given server. Exiting..."  1
}


test -z "$DomainName"  && LogToErr "Missing argument: DomainName"  2
test -z "$NdsAdminName"  && LogToErr "Missing argument: NdsAdminName"  2
test -z "$ExistingServerIP"  && LogToErr "Missing argument: ExistingServerIP"  2
#test -z "$ForestName"  && LogToErr "Missing argument: ForestName" 2

LdapAdminName=`echo $NdsAdminName | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/g'`
DOMAIN_RELATIVE_DN=`echo $DomainName | rev | sed 's/.*\.//g' | rev`
ParentName=`echo $DomainName | sed 's/^'$DOMAIN_RELATIVE_DN'.//g'`
PARENT_NC_DN="dc=`echo $ParentName | sed 's/\./,dc=/g'`"
#FOREST_NC_DN="dc=`echo $ForestName | sed 's/\./,dc=/g'`"

PrintArgs

GetLDAPCommandString

test -z "${LDAPCommand}" && exit 1

if [ "$LDAP_COMMAND_PRINT" = "true" ]
then
	echo "${LDAPCommand}"
	exit 0
fi

if [ "$ADD_DC" = "true" ]
then
	LdapServerContext=`LDAPTLS_CACERT=$SERVER_PEM $LDAPCommand -D "$LdapAdminName" -w "$ADM_PASSWD" -E "2.16.840.1.113719.1.513.4.5" -b "" "(&(objectclass=crossRef)(!(cn=Enterprise*))(dnsRoot=$DomainName))" ncName | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; '| grep ncName: | awk -F"ncName: " '{print $2}'`
	test -z "$LdapServerContext" && exit 1

	defaultContainer=`/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container`;
	if [ $? != 0 ]
	then
		$defaultContainer="ou=OESSystemObjects";
	fi

	LdapServerContext="$defaultContainer,$LdapServerContext"
else
	ParentLdapServerContext=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCommand -D "$LdapAdminName" -w "$ADM_PASSWD" -N -b "" "(&(objectclass=crossRef)(!(cn=Enterprise*))(dnsRoot=$ParentName))" ncName | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' 2>/dev/null | grep ncName: | awk -F"ncName: " '{print $2}'`
	
	if [ -z "$ParentLdapServerContext" ]
	then
		ParentLdapServerContext="$PARENT_NC_DN"
	fi
	if [ "$NM_CHILD" = "true" ]
	then
		LdapServerContext="$RDN_TYPE=$DOMAIN_RELATIVE_DN,$ParentLdapServerContext"
		unset RDN_TYPE
	else
		LdapServerContext="ou=OESSystemObjects,dc=$DOMAIN_RELATIVE_DN,$ParentLdapServerContext"
	fi
fi

NdsServerContext=`echo $LdapServerContext | sed -e 's/,/./g' -e 's/st=/s=/gi' `
test -z "$NdsServerContext" && LogToErr "Unable to retrieve server context for YaST" 2
LogToErr "Returning server context->$NdsServerContext" 0
echo "$NdsServerContext"
exit 0
