#!/bin/sh
################################################################################
# Domain services for Windows tool for provisioning operations
# Verify the supported server versions in the tree root during YaST install
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2006-2008
#	Novell, Inc.  All rights reserved.
################################################################################

# This script is a general tool to which we can add more functions
# for doing various provisioning related operations.

PATH=/opt/novell/xad/sbin:/opt/novell/xad/bin:/opt/novell/xad/share/dcinit:/opt/novell/eDirectory/bin:$PATH
export PATH

_LIB=`/opt/novell/xad/share/dcinit/printConfigKey.pl "_Lib"`
export _LIB

LD_LIBRARY_PATH=/opt/novell/xad/$_LIB:/opt/novell/xad/$_LIB/nds-modules:/opt/novell/eDirectory/$_LIB:/opt/novell/eDirectory/$_LIB/nds-modules:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH

SASL_PATH=/opt/novell/xad/$_LIB/sasl2
export SASL_PATH

TEXTDOMAINDIR=/opt/novell/xad/share/locale
TEXTDOMAIN=xad
export TEXTDOMAINDIR TEXTDOMAIN

PROVISION_TIMEOUT=60

# Define variables for various error codes returned
SUCCESS=0
ERR_MISSING_PARAMETER=1
ERR_SERVER_UNREACHABLE=2
ERR_CONTAINER_NOT_PARTITIONED=3
ERR_FOREST_EXISTS=4
ERR_NETBIOS_NAME_EXISTS_IN_TREE=5
ERR_SCH_VERSION_UNSUPPORTED=6
ERR_NO_RW_REPLICA=7
ERR_KINIT_FAILED=8
ERR_LDAPSEARCH_FAILED=9
ERR_DOMAIN_EXISTS=10
ERR_PROVISION_FAILED=11
ERR_RPCCLIENT_FAILED=12
ERR_NETBIOS_NAME_EXISTS_IN_SUBNET=13
ERR_NM_INVALID_CONTAINER=14
ERR_INVALID_TPO=15
ERR_CONTEXT_NAME=16
ERR_CERTIFICATE_NOT_PRESENT=17
ERR_DOMAIN_RESOLUTION_FAILED=18
ERR_NETBIOS_NAME_INVALID=19
ERR_ADC_NAMESERVER_ENTRY=20
ERR_PARENT_DOMAIN_NOT_AVAILABLE=21
ERR_INCONSISTENT_MAPPING=22
ERR_LAN_CONFIGURATION_INCORRECT=23
ERR_INVALID_PARAMETER=24
ERR_INVALID_SITE_NAME=25
ERR_SITE_NAME_HAS_UNDERSCORE=26
#Dont use error exit value '25', as it is returned to yast code if Administrator password is not created

#Define variables for various messages printed
MSG_SERVER_UNREACHABLE="Could not contact LDAP server."
LDAP_SERVER_NOMAPPING_OID="2.16.840.1.113719.1.513.4.7"

# Define vairables to hold the command-line parameters
TEMPLATE_NDS_CONF=""
STAGE_NDS_CONF=""
DOMAIN_NAME=""
ADMIN_NAME=""
IP_ADDRESS=""
NDS_CONTAINER_NAME=""
TEMPLATE_NTP_FILE=""
NETBIOS_NAME=""
FORESTROOTDOMAIN=""
FILENAME=""
TRUSTPOSIXOFFSET=""
DNSMASTER=""
DOMAIN_TYPE=""
INFORMATION_TYPE=""
FOREST_DOMAIN_NAME=""

CRONTAB_LOGFILE_PATH="/var/opt/novell/xad/log"

ExitOnFailure() {
        FirstParameter=$1
	if [ "$FirstParameter" = "--message" ]
	then
		message="$2"
		RetVal="$3"
		RetValFromScript="$4"
		shift
		shift
	else
		RetVal="$1"
		RetValFromScript="$2"
		shift
		shift
	fi

        TmpFiles="$*"
        if [ $RetVal -ne $SUCCESS ]; then
                rm -f "$TmpFiles"
		gettext -s "$message"
                exit $RetValFromScript
        fi

	message=""
}

ndsToLdap() {

	NDS_NAME="$1"

	test -z "$NDS_NAME" && ExitOnFailure --message "NDS name is empty." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	LDAP_NAME=`echo $NDS_NAME | sed 's/\.\([a-z,A-Z,0-9]*\)=/,\1=/g'`

}

ndsConfigSet() {

	test -z "$TEMPLATE_NDS_CONF" && ExitOnFailure $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$STAGE_NDS_CONF" && ExitOnFailure $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	test -r "$TEMPLATE_NDS_CONF" || ExitOnFailure $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
 	test -r "$STAGE_NDS_CONF" ||  ExitOnFailure $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	NDS_CONF_TMP1=`/bin/mktemp`
	NDS_CONF_TMP2=`/bin/mktemp`

	/opt/novell/xad/share/dcinit/regSubstitute.pl < $TEMPLATE_NDS_CONF > $NDS_CONF_TMP1;

	while read line
	do
		param=`echo "$line"| cut -d "=" -f1`;
		grep "$param" $NDS_CONF_TMP1 > /dev/null;
		if [ $? != 0 ]
		then
			echo $line >> $NDS_CONF_TMP2
		fi
	done < $STAGE_NDS_CONF

	cat $NDS_CONF_TMP2 >$STAGE_NDS_CONF
	cat $NDS_CONF_TMP1 >> $STAGE_NDS_CONF

	ExitOnFailure $SUCCESS $SUCCESS $NDS_CONF_TMP1 $NDS_CONF_TMP2

}

ntpConfigSet() {

	test -z "$TEMPLATE_NTP_FILE" && ExitOnFailure $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	NTP_CONF_TMP1=`/bin/mktemp`
	NTP_CONF_TMP2=`/bin/mktemp`
	DEFAULT_NTP_CONF="/etc/chrony.conf"

	/opt/novell/xad/share/dcinit/regSubstitute.pl < $TEMPLATE_NTP_FILE > $NTP_CONF_TMP1;
	# Need to remove the iburst in case of local IP
	grep -q "^server\s*127.127.1.0" $DEFAULT_NTP_CONF
	if [ $? == 0 ]
	then
		# Remove the entry in our template
		sed -i '/^server\s*127.*/d' $NTP_CONF_TMP1;
		echo "Skipping addition of server 127.127.1.0 entry to NTP Configuration as it is already present";
	else
		sed -i -e '/^server\s*127.*/{s/ iburst//g}' $NTP_CONF_TMP1;
		echo "Adding server 127.127.1.0 entry to NTP Configuration";
	fi

	# Check for existence of the default chrony.conf file
	if [ -w "$DEFAULT_NTP_CONF" ]
	then
		# The file exists, we look for same values as in NTP_CONF_TMP1 and
		# append only the non-existing value
		while read line
		do
			grep "$line" $NTP_CONF_TMP1 > /dev/null;
			if [ $? != 0 ] || [ -z "$line" ];
			then
				#entry not found in NTP_CONF_TMP1
				echo "$line" >> $NTP_CONF_TMP2
			fi
		done < $DEFAULT_NTP_CONF
	fi

	cat $NTP_CONF_TMP2 > $DEFAULT_NTP_CONF
	cat $NTP_CONF_TMP1 >> $DEFAULT_NTP_CONF

	ExitOnFailure 1 $SUCCESS $NTP_CONF_TMP1 $NTP_CONF_TMP2
}


#checkContainerPartitionStatus()
# Return Values:
#---------------
# 0   The given server meets is reachable and the given container is partitioned.
# 1   If required parameters are missing.
# 2   If the server is not reachable using the given IP address or could not perform base LDAP search.
# 3   If the given container is not partitioned.

checkContainerPartitionStatus() {

	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$NDS_CONTAINER_NAME" && ExitOnFailure --message "Missing container name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ndsToLdap "$NDS_CONTAINER_NAME"
	LDAP_CONTAINER_NAME="$LDAP_NAME"

	LDAPCOMMAND=`LD_LIBRARY_PATH=" " /opt/novell/xad/share/dcinit/ndsConfigServerContext.sh --ldap-command $DOMAIN_NAME "$ADMIN_NAME" $IP_ADDRESS`

	test -z "$LDAPCOMMAND" && ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	ndsToLdap "$ADMIN_NAME"
	ADMIN_NAME="$LDAP_NAME"

	LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "$LDAP_CONTAINER_NAME" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL >/dev/null 2>&1
	ExitOnFailure --message "Could not perform LDAP base search on the given container." $? $ERR_SERVER_UNREACHABLE

	LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "$LDAP_CONTAINER_NAME" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL objectClass=Partition | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -i Partition >/dev/null 2>&1
	ExitOnFailure --message "The container to be mapped should already be partitioned." $? $ERR_CONTAINER_NOT_PARTITIONED

}

#GetSOAValue()
# Return Values:
#---------------
# <SOAValue>	The current value of the dNIPSOASerial attribute for the DNS zone + 10.
# 1		If required parameters are missing.
# 2		If the server is not reachable using the given IP address or could not perform base LDAP search.
#17		If tree certificates are not present on the server.

GetSOAValue() {

	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$NDS_CONTAINER_NAME" && ExitOnFailure --message "Missing container name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ndsToLdap "$NDS_CONTAINER_NAME"
	LDAP_CONTAINER_NAME="$LDAP_NAME"

	# check if the certificates are present or not
	if [ ! -f "/etc/opt/novell/certs/SSCert.der" ]
	then
		echo "Failed to get certificates. exiting";
		exit $ERR_CERTIFICATE_NOT_PRESENT
	fi

	SOAVALUE=`LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /opt/novell/eDirectory/bin/ldapsearch -H "ldaps://$IP_ADDRESS:636" -x -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "$LDAP_CONTAINER_NAME" -s base dNIPSOASerial -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep "dNIPSOASerial: " | awk -F "dNIPSOASerial: " '{print $2}'`
	SOAVALUE=`expr $SOAVALUE + 10`
	echo $SOAVALUE
}

#getDNSMaster()
# Return Values:
#---------------
# 1		If required parameters are missing.
# 2		If the server is not reachable using the given IP address or could not perform base LDAP search.
#17		If tree certificates are not present on the server
#<DNSMaster>	hostname of the DC acting as the DNS Master

getDNSMaster() {

	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$NDS_CONTAINER_NAME" && ExitOnFailure --message "Missing container name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ndsToLdap "$NDS_CONTAINER_NAME"
	LDAP_CONTAINER_NAME="$LDAP_NAME"

	# check if the certificates are present or not
	if [ ! -f "/etc/opt/novell/certs/SSCert.der" ]
	then
		echo "Failed to get certificates. exiting";
		exit $ERR_CERTIFICATE_NOT_PRESENT
	fi

	# get the DNS Master
	DNSMASTER=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 /opt/novell/eDirectory/bin/ldapsearch -H ldaps://"$IP_ADDRESS":636 -x -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "$LDAP_CONTAINER_NAME" -s base dNIPServerDNSNames -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep "dNIPServerDNSNames" | awk -F "dNIPServerDNSNames: " '{print $2}'`
	echo $DNSMASTER
}



#getDomainGUID()
# Return Values:
#---------------
# <DomainGUID>	DomainGUID for the domain being installed
# 1		If required parameters are missing.
# 2		If the server is not reachable using the given IP address or could not perform base LDAP search.

getDomainGUID() {

	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$NDS_CONTAINER_NAME" && ExitOnFailure --message "Missing container name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ndsToLdap "$NDS_CONTAINER_NAME"
	LDAP_CONTAINER_NAME="$LDAP_NAME"

	GUID=`/usr/bin/ldapsearch -h $IP_ADDRESS -b "$LDAP_CONTAINER_NAME" -s base -x -LLL GUID | grep 'GUID:: ' | awk -F 'GUID:: ' '{print $2}'`
	DOMAINGUID=`/opt/novell/xad/share/dcinit/base64ToGUID.pl $GUID`
	echo -n $DOMAINGUID
}

#getDomainSID()
# Return Values:
#---------------
# <DomainSID>   DomainSID for the domain being installed
getDomainSID(){
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

#	IP_ADDRESS_LIST=`/usr/bin/dig $DOMAIN_NAME +short +time=15 2>/dev/null`
	IP_ADDRESS_LIST=`/usr/bin/dig -t SRV _ldap._tcp.$DOMAIN_NAME +short +time=15 | cut -d' ' -f4 | sed 's/.$//g' 2>/dev/null`
	ExitOnFailure --message "Cannot retrive the provider for this domain." $? $ERR_DOMAIN_RESOLUTION_FAILED
	found=0
	for CHECK_IP in $IP_ADDRESS_LIST
	do
		CHECK_IP=`/usr/bin/dig $CHECK_IP +short`
		rc=`/usr/bin/ldapsearch -h $CHECK_IP -s base -b "" -x`
		if [ $? -ne 0 ]
		then
			continue;
		else
			found=1
			break
		fi
	done

	if [ $found -eq 0 ]
	then
		ExitOnFailure --message "Could not reach the IP Address $IP_ADDRESS_LIST for domain $DOMAIN_NAME" $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE
	fi

	#Obtain the domain IP
	IP_ADDRESS=$CHECK_IP

	DOMAIN_NC=`/usr/bin/ldapsearch -h $IP_ADDRESS "" -s base defaultnamingcontext -x -LLL | grep -i "^defaultNamingContext:" |sed 's/defaultNamingContext: //g'`;

	DOMAIN_SID=`/usr/bin/ldapsearch -x -h "$IP_ADDRESS" -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "$DOMAIN_NC" -s base objectsid -LLL -Z 2>/dev/null | grep "objectsid" -i | awk -F ":: " '{print $2}'`

	DOMAIN_SID=`/opt/novell/xad/share/dcinit/base64ToSid.pl $DOMAIN_SID`

	echo -n $DOMAIN_SID
}

#updateTrustPosixOffset()
# Return Values:
#----------------
# 0	File is updated with the proper UIDs.
# 1	If required parameters are missing.

updateTrustPosixOffset() {

	test -z "$FILENAME" && ExitOnFailure --message "Missing filename." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$TRUSTPOSIXOFFSET" && ExitOnFailure --message "Missing trustposixoffset." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	UID_ADMINISTRATOR=`expr $TRUSTPOSIXOFFSET + 500`
	UID_GUEST=`expr $TRUSTPOSIXOFFSET + 501`
	UID_KRBTGT=`expr $TRUSTPOSIXOFFSET + 502`

	# Correcting the GIDs
	GID_CERT_PUBLISHERS=`expr $TRUSTPOSIXOFFSET + 517`
	GID_DHCP_ADMINISTRATORS=`expr $TRUSTPOSIXOFFSET + 960`
	GID_DHCP_USERS=`expr $TRUSTPOSIXOFFSET + 944`
	GID_DNSADMINS=`expr $TRUSTPOSIXOFFSET + 961`
	GID_DNSUPDATEPROXY=`expr $TRUSTPOSIXOFFSET + 962`
	GID_DOMAIN_ADMINS=`expr $TRUSTPOSIXOFFSET + 512`
	GID_DOMAIN_COMPUTERS=`expr $TRUSTPOSIXOFFSET + 515`
	GID_DOMAIN_USERS=`expr $TRUSTPOSIXOFFSET + 513`
	GID_DOMAIN_CONTROLLERS=`expr $TRUSTPOSIXOFFSET + 516`
	GID_DOMAIN_GUESTS=`expr $TRUSTPOSIXOFFSET + 514`
	GID_GP_CREATOR_OWNERS=`expr $TRUSTPOSIXOFFSET + 520`
	GID_HELP_SERVICES_GRP=`expr $TRUSTPOSIXOFFSET + 896`
	GID_RAS_IAS=`expr $TRUSTPOSIXOFFSET + 553`
	GID_SCHEMA_ADMINS=`expr $TRUSTPOSIXOFFSET + 518`
	GID_TELNET_CLIENTS=`expr $TRUSTPOSIXOFFSET + 928`


	TEMP_FILE1=`/bin/mktemp`
	TEMP_FILE2=`/bin/mktemp`
	TEMP_FILE3=`/bin/mktemp`

	# Update the correct UID of administrator
	sed "s/^uidNumber: 1049076/uidNumber: $UID_ADMINISTRATOR/g" $FILENAME > $TEMP_FILE1

	# Update the correct UID of guest
	sed "s/^uidNumber: 65534/uidNumber: $UID_GUEST/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct UID of krbtgt
	sed "s/^uidNumber: 1049078/uidNumber: $UID_KRBTGT/g" $TEMP_FILE2 > $TEMP_FILE3

	# Update the correct GID of 'cert publsihers'
	sed "s/^gidNumber: 1049093/gidNumber: $GID_CERT_PUBLISHERS/g" $TEMP_FILE3 > $TEMP_FILE1


	# Update the correct GID of 'DHCP administrators'
	sed "s/^gidNumber: 1049536/gidNumber: $GID_DHCP_ADMINISTRATORS/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct GID of 'DHCP users'
	sed "s/^gidNumber: 1049520/gidNumber: $GID_DHCP_USERS/g" $TEMP_FILE2 > $TEMP_FILE3

	# Update the correct GID of 'DnsAdmins'
	sed "s/^gidNumber: 1049537/gidNumber: $GID_DNSADMINS/g" $TEMP_FILE3 > $TEMP_FILE1

	# Update the correct GID of 'DnsUpdateProxy'
	sed "s/^gidNumber: 1049538/gidNumber: $GID_DNSUPDATEPROXY/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct GID of 'Domain Admins'
	sed "s/^gidNumber: 1049088/gidNumber: $GID_DOMAIN_ADMINS/g" $TEMP_FILE2 > $TEMP_FILE3

	# Update the correct GID of 'Domain Computers'
	sed "s/^gidNumber: 1049091/gidNumber: $GID_DOMAIN_COMPUTERS/g" $TEMP_FILE3 > $TEMP_FILE1

	# Update the correct GID of 'Domain Controllers'
	sed "s/^gidNumber: 1049092/gidNumber: $GID_DOMAIN_CONTROLLERS/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct GID of 'Domain Users'
	sed "s/^gidNumber: 1049089/gidNumber: $GID_DOMAIN_USERS/g" $TEMP_FILE2 > $TEMP_FILE3

	# Update the correct GID of 'Domain Guests'
	sed "s/^gidNumber: 65533/gidNumber: $GID_DOMAIN_GUESTS/g" $TEMP_FILE3 > $TEMP_FILE1

	# Update the correct GID of 'Group Policy Creator owners'
	sed "s/^gidNumber: 1049096/gidNumber: $GID_GP_CREATOR_OWNERS/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct GID of 'HelpServicesGroup'
	sed "s/^gidNumber: 1049472/gidNumber: $GID_HELP_SERVICES_GRP/g" $TEMP_FILE2 > $TEMP_FILE3

	# Update the correct GID of 'RAS and IAS'
	sed "s/^gidNumber: 1049129/gidNumber: $GID_RAS_IAS/g" $TEMP_FILE3 > $TEMP_FILE1

	# Update the correct GID of 'Schema admins'
	sed "s/^gidNumber: 1049094/gidNumber: $GID_SCHEMA_ADMINS/g" $TEMP_FILE1 > $TEMP_FILE2

	# Update the correct GID of 'Telnet Clients'
	sed "s/^gidNumber: 1049504/gidNumber: $GID_TELNET_CLIENTS/g" $TEMP_FILE2 > $FILENAME

	# clean up the temp files
	rm -f $TEMP_FILE1
	rm -f $TEMP_FILE2
	rm -f $TEMP_FILE3

}

#validateTrustPosixOffset()
# Return Values:
#---------------
# 1048576	Default minimum value, in case tpo passed is 0.
# 1		If required parameters are missing.
# 15		If passed tpo value is less than 1048576.

validateTrustPosixOffset() {

	test -z "$TRUSTPOSIXOFFSET" && ExitOnFailure --message "Missing trustposixoffset." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

		if [ $TRUSTPOSIXOFFSET -eq 0 ]
		then
			TRUSTPOSIXOFFSET=1048576
			echo $TRUSTPOSIXOFFSET
		fi
		if [ $TRUSTPOSIXOFFSET -lt 1048576 ]
		then
			ExitOnFailure --message "Trustposixoffset must be greater than 1048576." 1 $ERR_INVALID_TPO
		fi
}

#CheckForestExistence()
# Return Values:
#---------------
# 0   The given server is reachable and there exists no DSfW domain in the tree.
# 1   If required parameters are missing.
# 2   Server is not reachable using the given IP address.
# 4   If there exists a forest in the eDirectory tree hosted at given IP address.

CheckForestExistence() {
	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	LDAPCOMMAND=`LD_LIBRARY_PATH=" " /opt/novell/xad/share/dcinit/ndsConfigServerContext.sh --ldap-command $DOMAIN_NAME "$ADMIN_NAME" $IP_ADDRESS`

	test -z "$LDAPCOMMAND" && ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	ndsToLdap "$ADMIN_NAME"
	ADMIN_NAME="$LDAP_NAME"

	base=`hostname -d | sed 's/\./,dc=/g' | sed 's/^/dc=/g' | xargs echo -n`

	LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -b "" -s base  -E "2.16.840.1.113719.1.513.4.5" -LLL >/dev/null 2>&1
	ExitOnFailure --message "Could not perform LDAP base search on the base container." $? $ERR_SERVER_UNREACHABLE

	NcName=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND  -D "$ADMIN_NAME" -w "$ADM_PASSWD" -E "2.16.840.1.113719.1.513.4.5" -b "" -LLL objectclass=crossref dn | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -I cn=Configuration | head -1 | awk -F "cn=Configuration," '{print $2}'`

	if [ "X$NcName" != "X" ]
	then
			DNSROOT=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -E "2.16.840.1.113719.1.513.4.5" -b "" '(&(objectclass=crossref)(ncName='"$NcName"'))' dnsRoot -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | head -3 | tail -1`
			FORESTNAME=`echo "$DNSROOT" | awk -F "dnsRoot: " '{print $2}'`

			ExitOnFailure --message "$FORESTNAME" 1 $ERR_FOREST_EXISTS
	fi
}

#CheckDomainExistence()
# Return Values:
# 0   The given server is reachable and there exists no DSfW domain matching the domain name passed.
# 1   If required parameters are missing.
# 2   Server is not reachable using the given IP address.
# 10  If there exists a domain with the same name as that of the domain passed as parameter in the eDirectory tree hosted at given IP address.

CheckDomainExistence() {
	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	LDAPCOMMAND=`LD_LIBRARY_PATH=" " /opt/novell/xad/share/dcinit/ndsConfigServerContext.sh --ldap-command $DOMAIN_NAME "$ADMIN_NAME" $IP_ADDRESS`

	test -z "$LDAPCOMMAND" && ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	ndsToLdap "$ADMIN_NAME"
	ADMIN_NAME="$LDAP_NAME"

	LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -s base  -E "2.16.840.1.113719.1.513.4.5" -LLL >/dev/null 2>&1
	ExitOnFailure --message "Could not perform LDAP base search on the base container." $? $ERR_SERVER_UNREACHABLE

	LD_PRELOAD=/opt/novell/xad/$_LIB/libadmpasswd.so.2 LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$ADMIN_NAME" -w "$ADM_PASSWD" -E "2.16.840.1.113719.1.513.4.5" -b '' -LLL objectclass=crossref dnsRoot | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep dnsRoot | grep -iw "$DOMAIN_NAME" >/dev/null 2>&1
	if [ $? -eq 0 ]
	then
		# Domain with the same name exists
		ExitOnFailure --message "A domain with same name already exists." 1 $ERR_DOMAIN_EXISTS
	fi
}

#getActualForestNamingContext()
# Return Values:
#---------------
# 0   The given server is reachable and there exists a forest. Also prints the Forest root domain found.
# 1   If required parameters are missing.
# 2   Server is not reachable using the given IP address.

getActualForestNamingContext() {

	FORESTROOT=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Forest Root"`
	FORESTROOTDOMAINPROVIDER=`/opt/novell/xad/sbin/provision -q -q -l $PROVISION_TIMEOUT --locate-dc $FORESTROOT`
	test -z "$FORESTROOTDOMAINPROVIDER" && ExitOnFailure --message "Could not reach forest root domain controller." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	ACTUALFORESTNC=`/opt/novell/eDirectory/bin/ldapsearch -x -E "2.16.840.1.113719.1.513.4.5" -b "" -s base -h $FORESTROOTDOMAINPROVIDER | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; '| sed 's/st=/s=/gi' | grep -i "^namingContexts: cn=Configuration" | awk -F"namingContexts: cn=Configuration," '{print $2}'`
	test -z "$ACTUALFORESTNC" && ExitOnFailure --message "Could not determine actual forest naming context." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	echo $ACTUALFORESTNC

}

#getForestNamingContextforLUM()
# Return Values:
# ----------------
# 0	The given server is reachable and there exists a forest. Also prints the Forest root domain found.
# 1	If required parameters are missing.
# 2	Server is not reachable using the given IP address.

getForestNamingContextforLUM() {

	test -z "$FORESTROOTDOMAIN" && ExitOnFailure --message "Missing forest root domain." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	FORESTROOTDOMAINPROVIDER=`/opt/novell/xad/sbin/provision -q -q -l $PROVISION_TIMEOUT --locate-dc $FORESTROOTDOMAIN`
	test -z "$FORESTROOTDOMAINPROVIDER" && ExitOnFailure --message "Could not reach forest root domain controller." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	ACTUALFORESTNC=`/opt/novell/eDirectory/bin/ldapsearch -x -E "2.16.840.1.113719.1.513.4.5" -b "" -s base -h $FORESTROOTDOMAINPROVIDER | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi' | grep -i "^namingContexts: cn=Configuration" | awk -F"namingContexts: cn=Configuration," '{print $2}'`
	test -z "$ACTUALFORESTNC" && ExitOnFailure --message "Could not determine actual forest naming context." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE

	defaultContainer=`/opt/novell/xad/share/dcinit/provisionTools.sh get-default-container -d $FORESTROOTDOMAIN`
	if [ $? -ne 0 ]
	then
		defaultContainer="ou=OESSystemObjects";
	fi
	echo "$defaultContainer,$ACTUALFORESTNC"

}

updateCronEntryGPO2NMAS() {

	CRONCOMMAND="/usr/bin/crontab"
	CRONLISTCOMMAND="/usr/bin/crontab -l"
	CRONTEMPFILE=`/bin/mktemp`

	$CRONLISTCOMMAND > $CRONTEMPFILE

	cat $CRONTEMPFILE | grep -i "gposync"  >/dev/null 2>&1
	if [ $? -ne 0 ]
	then
		# An entry needs to be appended to the existing cron entries
		echo "*/30 * * * * /opt/novell/xad/sbin/gposync.sh 1>/dev/null"\
		"2>>$CRONTAB_LOGFILE_PATH/gposync.log">>$CRONTEMPFILE
		$CRONCOMMAND $CRONTEMPFILE
	fi

	# Delete the CRONTEMPFILE file
	rm -f "$CRONTEMPFILE"
}

updateCronEntryFGP2NMAS() {

	CRONCOMMAND="/usr/bin/crontab"
	CRONLISTCOMMAND="/usr/bin/crontab -l"
	CRONTEMPFILE=`/bin/mktemp`

	$CRONLISTCOMMAND > $CRONTEMPFILE

	cat $CRONTEMPFILE | grep -i "fgpsync"  >/dev/null 2>&1
	if [ $? -ne 0 ]
	then
		# An entry needs to be appended to the existing cron entries
		echo "*/30 * * * * /opt/novell/xad/sbin/fgpsync.sh 1>/dev/null"\
		"2>>$CRONTAB_LOGFILE_PATH/fgpsync.log">>$CRONTEMPFILE
		$CRONCOMMAND $CRONTEMPFILE
	fi

	# Delete the CRONTEMPFILE file
	rm -f "$CRONTEMPFILE"
}

updateCronEntryKDC() {

        CRONCOMMAND="/usr/bin/crontab"
        CRONLISTCOMMAND="/usr/bin/crontab -l"
        CRONTEMPFILE=`/bin/mktemp`

        $CRONLISTCOMMAND > $CRONTEMPFILE
        cat $CRONTEMPFILE | grep -i "/opt/novell/xad/sbin/krb5kdc"  >/dev/null 2>&1
        if [ $? -ne 0 ]
        then
                # An entry needs to be appended to the existing cron entries
                echo "*/30 * * * * /sbin/killproc -HUP /opt/novell/xad/sbin/krb5kdc">>$CRONTEMPFILE
                $CRONCOMMAND $CRONTEMPFILE
        fi

        # Delete the CRONTEMPFILE file
        rm -f "$CRONTEMPFILE"
}

updateCronEntrySYSVOLSYNC() {

	CRONCOMMAND="/usr/bin/crontab"
	CRONLISTCOMMAND="/usr/bin/crontab -l"
	CRONTEMPFILE=`/bin/mktemp`

	$CRONLISTCOMMAND > $CRONTEMPFILE
	cat $CRONTEMPFILE | grep -i "sysvolsync" >/dev/null 2>&1
	if [ $? -ne 0 ]
	then
		# An entry needs to be appended to the existing cron entries
		echo "*/30 * * * * /opt/novell/xad/sbin/sysvolsync 1>/dev/null"\
		"2>>$CRONTAB_LOGFILE_PATH/sysvolsync.log">>$CRONTEMPFILE
		$CRONCOMMAND $CRONTEMPFILE
	fi

	# Delete the CRONTEMPFILE file
	rm -f "$CRONTEMPFILE"
}

updateCronEntrySITESANDSUBNETS() {

        CRONCOMMAND="/usr/bin/crontab"
        CRONLISTCOMMAND="/usr/bin/crontab -l"
        CRONTEMPFILE=`/bin/mktemp`

        $CRONLISTCOMMAND > $CRONTEMPFILE
        cat $CRONTEMPFILE | grep -i "/opt/novell/xad/sbin/sites_dnsupdate.py" | grep -iv "^#" >/dev/null 2>&1
        if [ $? -ne 0 ]
        then
                # An entry needs to be appended to the existing cron entries
                echo "* * * * * /usr/bin/python3 /opt/novell/xad/sbin/sites_dnsupdate.py"\
		"1>/dev/null 2>>$CRONTAB_LOGFILE_PATH/sitesandsubnets.log">>$CRONTEMPFILE
                $CRONCOMMAND $CRONTEMPFILE
        fi

        # Delete the CRONTEMPFILE file
        rm -f "$CRONTEMPFILE"
}

checkNetbiosName() {

	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
       test -z "$NETBIOS_NAME" && ExitOnFailure --message "Missing NetBIOS name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	valdiateNetBIOSName
	checkDuplicateNetbiosNameInTree
	checkDuplicateNetbiosNameInSubNet

}

checkValidNetBIOSNameQuery() {
        test -z "$NETBIOS_NAME" && ExitOnFailure --message "Missing NetBIOS name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

        valdiateNetBIOSName
        checkDuplicateNetbiosNameInSubNet
}

CheckNumericNetBIOS() {

rc=`echo "$NETBIOS_NAME" | awk '$0 ~/[^0-9]/ { print "nonNumeric" }'`;

if [ "X$rc" != "XnonNumeric" ]
then
        ExitOnFailure --message "The NetBIOS name $NETBIOS_NAME is all numeric. This is not supported." $ERR_NETBIOS_NAME_INVALID $ERR_NETBIOS_NAME_INVALID
fi

}

#Checking that the NetBIOS name is not having following names (Bug#336613).
#SECURITY; SCHEMA; LINKENGINE; ADMINISTRATOR; NDSSCHEMA; NDSCONTAINER
#Also, Checking that the NetBIOS name is not having all numeric name
#It's a limitation of eDirectory schema that it can not accept these names.
valdiateNetBIOSName() {

        CheckNumericNetBIOS

	if [ "$NETBIOS_NAME" = "SECURITY" -o "$NETBIOS_NAME" = "SCHEMA" -o "$NETBIOS_NAME" = "LINKENGINE" -o "$NETBIOS_NAME" = "ADMINISTRATOR" -o "$NETBIOS_NAME" = "NDSSCHEMA" -o "$NETBIOS_NAME" = "NDSCONTAINER" ]
	then
		#Unsupported NetBIOS name.
		ExitOnFailure --message "The NetBIOS name $NETBIOS_NAME is not supported." $ERR_NETBIOS_NAME_INVALID $ERR_NETBIOS_NAME_INVALID
	else
		#Valid NetBIOS name.
		return $SUCCESS
	fi
}

checkDuplicateNetbiosNameInTree() {
	LDAPCOMMAND="/opt/novell/eDirectory/bin/ldapsearch -x -h $IP_ADDRESS"

	$LDAPCOMMAND -s base -b "" dn -LLL >/dev/null 2>&1
	ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" $? $ERR_SERVER_UNREACHABLE

	$LDAPCOMMAND -b "" -s one -E "2.16.840.1.113719.1.513.4.5" -LLL >/dev/null 2>&1
	ExitOnFailure --message "Could not perform LDAP base search on the given container." $? $ERR_SERVER_UNREACHABLE
	$LDAPCOMMAND -b "" objectclass=configuration dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -i "configuration" >/dev/null 2>&1
	if [ $? -eq 0 ]
	then
		#a forest exists, hence looking for the NetBIOS name in tree as well
		$LDAPCOMMAND -b "" objectclass=crossref dn -E "2.16.840.1.113719.1.513.4.5" -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -w "$NETBIOS_NAME" >/dev/null 2>&1
		if [ $? -ne 0 ]
		then
			# Duplicate NetBIOS name does not occur in tree
			return $SUCCESS
		else
			# Duplicate NetBIOS name exists in tree
			ExitOnFailure --message "NetBIOS name already exists." 1 $ERR_NETBIOS_NAME_EXISTS_IN_TREE
		fi
	fi
}

checkDuplicateNetbiosNameInSubNet() {
	NMBLOOKUPCMD="/usr/bin/nmblookup $NETBIOS_NAME"

	# Duplicate NetBIOS name does not occurs in tree, look in subnet
	$NMBLOOKUPCMD | tail -1 | grep '<00>' >/dev/null 2>&1
	if [ $? -eq 0 ]
	then
		# Duplicate NetBIOS name exists on subnet
		ExitOnFailure --message "NetBIOS name already exists." 1 $ERR_NETBIOS_NAME_EXISTS_IN_SUBNET
	fi
}

#ndsSchVersionCheck()
# Return Values:
#---------------
# 0   The given server meets the minimum version requirement for Domain
#     Services for Windows.
# 2   Server is not reachable using the given IP address.
# 6   Patch required for 64 bit syntax is not applied in the given server.
# 7   The given server does not have a read/write or master replica of
#     the root partition.

ndsSchVersionCheck () {

	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$TREE_NAME" && ExitOnFailure --message "Missing tree name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ROOT_SERVER="$IP_ADDRESS"
	treeName=`echo $TREE_NAME | tr "[:lower:]" "[:upper:]"`
	NcpServerDN=`/usr/bin/ldapsearch -h $ROOT_SERVER -b "" -s base -LLL -x dsaName | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep dsaName |awk -F"dsaName: " '{print $2}'`
	if [ "$NcpServerDN" = "" ]
	then
		ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" 1 $ERR_SERVER_UNREACHABLE
	fi

	NcpServerName=`echo $NcpServerDN | awk -F"," '{print $1}' | awk -F"cn=" '{print $2}'`
	ReplicaRing=`/opt/novell/eDirectory/bin/ndsstat -h $ROOT_SERVER -p ".T=$treeName." -d | grep -i "^.cn=$NcpServerName"`

	test -z "$ReplicaRing" && ExitOnFailure --message "The given server does not contain R/W replica of the root partition." 1 $ERR_NO_RW_REPLICA

	ReplicaType=`echo "$ReplicaRing" | awk '{print $2}'`

	test -z "$ReplicaType"  && ExitOnFailure --message "The given server does not contain R/W replica of the root partition." 1 $ERR_NO_RW_REPLICA

	ServerMajorVersion=0
	ServerMajorVersion=`echo "$ReplicaRing" | awk -F"DS]" '{print $2}' | awk -F"." '{print $1}'`
	ServerMinorVersion=`echo "$ReplicaRing" | awk -F"DS]" '{print $2}' | awk -F"." '{print $2}'`

	if [ "$ReplicaType" = "Master" -o "$ReplicaType" = "Read/Write" ]
	then
		if [ -n $ServerMajorVersion -a -n $ServerMinorVersion ]
	        then
	                if [ $ServerMajorVersion -lt 20000 ]
			then
	                        if [ $ServerMajorVersion -ge 10553 -a $ServerMinorVersion -ge 85 ]
	                        then
	                                exit $SUCCESS
	                        fi
	                else
	                        test $ServerMajorVersion -ge 20214 && exit $SUCCESS
	                fi
	                ExitOnFailure --message "The given server does not have the patch for 64 bit integer syntax." 1 $ERR_SCH_VERSION_UNSUPPORTED
	        else
			ExitOnFailure --message "The given server does not contain R/W replica of the root partition." 1 $ERR_NO_RW_REPLICA
	        fi
	else
		ExitOnFailure --message "The given server does not contain R/W replica of the root partition." 1 $ERR_NO_RW_REPLICA
	fi
}

kinitAdministrator() {

	test -z "$ADM_PASSWD" && ExitOnFailure --message "Admin password is not exported." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	kinit -E Administrator@$DOMAIN_NAME
	if [ $? -ne 0 ]
	then
		echo ""
		ExitOnFailure --message "kinit failed." 1 $ERR_KINIT_FAILED
	fi

}
getNetBIOSName(){

		test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
		test -z "$ACTUALFORESTNC" && ACTUALFORESTNC=`/usr/bin/ldapsearch -H ldap://$DOMAIN_NAME -b "" -s base  "rootDomainNamingContext" -x -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi' | awk -F ":" '{print $2}'| sed 's/ *$//g' | sed 's/^ *//g'`
		test -z "$ACTUALFORESTNC" && ExitOnFailure --message "" $ERR_LDAPSEARCH_FAILED
		NETBIOS_NAME=`/usr/bin/ldapsearch -H ldap://$DOMAIN_NAME -b "cn=Partitions,cn=Configuration,$ACTUALFORESTNC" "(&(objectclass=crossref)(dnsroot=$DOMAIN_NAME))" -LLL -x | grep -i '^nETBIOSName: ' | awk -F":" '{print $2}' | sed 's/ *$//g' | sed 's/^ *//g'`
		echo $NETBIOS_NAME
}

ProvisionLocateDc() {

	PROVISION_CMD="/opt/novell/xad/sbin/provision"

	$PROVISION_CMD -l $PROVISION_TIMEOUT --locate-dc $DOMAIN_NAME
	if [ $? -ne 0 ]
	then
		echo ""
		ExitOnFailure --message "provision --locate-dc failed." 1 $ERR_PROVISION_FAILED
	fi
}

RpcClientCheck() {

	## get UID for the current user.
	uid=$(id -u)
	KRB5CCNAME=/tmp/krb5cc_$uid rpcclient -k `/bin/hostname -f` -c dsroledominfo
	if [ $? -ne 0 ]
	then
		echo ""
		ExitOnFailure --message "rpcclient failed." 1 $ERR_RPCCLIENT_FAILED
	else
		echo ""
		echo "All Domain Services for Windows services are up and running."
		exit
	fi
}

CheckServices() {

	# Call the individual functions to check the status of various services

	# Get the installed domain name from printConfigKey.pl
	DOMAIN_NAME=`/opt/novell/xad/share/dcinit/printConfigKey.pl "DNS Root"`

	# Check functionality of kinit
	kinitAdministrator
	echo ""

	# Check functionality of provision --locate-dc
	ProvisionLocateDc
	echo ""

	# Check functionality of rpcclient
	RpcClientCheck
}

# Check the partition mapping based on the domain partition list attribute value
# which is already present in the tree.
checkPartitionMapping () {
	domain_partition_list=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$LDAP_ADMIN_DN" -w "$ADM_PASSWD" -b "cn=partitions,cn=configuration,$ACTUALFORESTNC" domainpartitionlist -LLL -E "2.16.840.1.113719.1.513.4.5" | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi'| grep -i domainpartitionlist | awk -F ": " '{print $2}'`

	# The container being mapped is exactly matching the existing entry of domainpartitionlist
	`echo -n "$domain_partition_list" | grep -ixq "$LDAP_CONTAINER_NAME"`
	if [ $? -eq 0 ]
	then
		ExitOnFailure --message "The container DN being mapped is already part of other domain. Mapping this container to a new DSfW domain will lead to inconsistency in the tree." $ERR_NM_INVALID_CONTAINER $ERR_NM_INVALID_CONTAINER
	fi

	# Container being mapped is lying between the already existing domain boundary
	`echo -n "$domain_partition_list" | grep -iwq "$LDAP_CONTAINER_NAME"`
	if [ $? -eq 0 ]
	then
		ExitOnFailure --message "Partition being mapped is falling between already installed domain boundary." $ERR_INCONSISTENT_MAPPING $ERR_INCONSISTENT_MAPPING
	fi
}

# Validate the new domain and the resulting hierarchy.
validateNewNMDomain() {
	test -z "$IP_ADDRESS" && ExitOnFailure --message "Missing IP Address." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	test -z "$NDS_CONTAINER_NAME" && ExitOnFailure --message "Missing Container name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	ndsToLdap "$NDS_CONTAINER_NAME"
        LDAP_CONTAINER_NAME="$LDAP_NAME"

	test -z "$NDS_ADMIN_DN" && ExitOnFailure --message "Missing Administrator DN." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	ndsToLdap "$NDS_ADMIN_DN"
	LDAP_ADMIN_DN="$LDAP_NAME"

	test -z "$ADM_PASSWD" && ExitOnFailure --message "Administrator password is not exported." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	DOMAIN_NAME=`/bin/dnsdomainname`
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Could not fetch domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	# Get the certificate if not present.
	if [ ! -f "/etc/opt/novell/certs/SSCert.der" ]
	then
		LD_LIBRARY_PATH=/opt/novell/eDirectory/lib:/opt/novell/eDirectory/lib/nds-module/:$LD_LIBRARY_PATH /opt/novell/oes-install/util/getSSCert -t $TREE_NAME -u $NDS_ADMIN_DN -a $IP_ADDRESS -x $ADM_PASSWD >/dev/null 2>&1
		ExitOnFailure --message "Failed to get certificates. exiting" $? $ERR_CERTIFICATE_NOT_PRESENT
	fi

	LDAPCOMMAND="/opt/novell/eDirectory/bin/ldapsearch -x -H ldaps://$IP_ADDRESS:636 "
        $LDAPCOMMAND -D "$LDAP_ADMIN_DN" -w "$ADM_PASSWD" -b "$LDAP_CONTAINER_NAME" -s base dn -LLL >/dev/null 2>&1
	rc=$?
        (test $rc -eq 32) && ExitOnFailure --message "Invalid container DN." $rc $ERR_NM_INVALID_CONTAINER
        ExitOnFailure --message "$MSG_SERVER_UNREACHABLE" $rc $ERR_SERVER_UNREACHABLE

	ACTUALFORESTNC=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -b "" -s base -E "2.16.840.1.113719.1.513.4.5" -LLL| sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi' | grep -i "^namingContexts: cn=Configuration" | awk -F "namingContexts: cn=Configuration," '{print $2}'`

	# Let us exit here as the Parent Domain has not returned naming contexts.
	test -z "$ACTUALFORESTNC" && ExitOnFailure --message "Could not fetch the Actual Forest NC[ldapsearch failed]." $ERR_LDAPSEARCH_FAILED $ERR_LDAPSEARCH_FAILED

	list=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$LDAP_ADMIN_DN" -w "$ADM_PASSWD" -b "cn=partitions,cn=configuration,$ACTUALFORESTNC" nCName -LLL -E "2.16.840.1.113719.1.513.4.5" | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi' | grep "nCName: " | awk -F "nCName: " '{print $2}'`

	# Check if any container lying below this container in hierarchy is mapped to a DSfW domain.
	# We do not support such a mapping as it will lead to mess up the existing tree.
	echo "$list" | grep -vqi "$LDAP_CONTAINER_NAME"
	ExitOnFailure --message "A container under this container DN has already mapped. Mapping this container to a DSfW domain will lead to inconsistency in the tree." $? $ERR_NM_INVALID_CONTAINER

	# Check if the parent domain exists.
	context_domain=`echo $DOMAIN_NAME | cut -d "." -f2-`
	parent_domain_partition=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$LDAP_ADMIN_DN" -w "$ADM_PASSWD" -b "cn=partitions,cn=configuration,$ACTUALFORESTNC" "(&(dnsRoot=$context_domain)(systemflags=3))" -LLL -E "2.16.840.1.113719.1.513.4.5" | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi'`
	test -z "$parent_domain_partition" && ExitOnFailure --message "Parent Domain [$context_domain] does not exists." $ERR_PARENT_DOMAIN_NOT_AVAILABLE $ERR_PARENT_DOMAIN_NOT_AVAILABLE

	# We do not support none of the following 2 cases.
	# Check if this container mapping is under its sibling container in domain hierarchy.
	# Check if this container mapping is under the domain hierarchy of the container
	# which is not lying in this container's hierarchy.
	context_container=$LDAP_CONTAINER_NAME
	while [ "$context_container" != "" ]
	do
		context_container=`echo $context_container | cut -d "," -f2-`
		echo "$list" | grep -xiq "$context_container"
		if [ $? -eq 0 ]
		then
			upper_NM_con=`echo "$list" | grep -xi "$context_container"`
			dns_root=`LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem $LDAPCOMMAND -D "$LDAP_ADMIN_DN" -w "$ADM_PASSWD" -b "cn=partitions,cn=configuration,$ACTUALFORESTNC" nCName="$upper_NM_con" dnsRoot -LLL -E "2.16.840.1.113719.1.513.4.5" | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | sed 's/st=/s=/gi'`

			echo "$dns_root" | grep -ixq "dnsRoot: $context_domain"
			if [ $? -eq 0 ]
			then
				checkPartitionMapping
				exit 0
			else
				ExitOnFailure --message "Container to domain mapping is not consistent with respect to hierarchy of eDirectory Tree." $ERR_INCONSISTENT_MAPPING $ERR_INCONSISTENT_MAPPING
			fi
		fi

		# Exit here if this container is not even under the top container.
		echo "$context_container" | grep -qe ","
		ExitOnFailure --message "Invalid container DN." $? $ERR_NM_INVALID_CONTAINER
	done
}

checkDomainResolution () {
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing Domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	ipAddress=`/usr/bin/dig $DOMAIN_NAME +short`
	if [ "$ipAddress" = "" ]
	then
		ExitOnFailure --message "Couldn't resolve the domain, $DOMAIN_NAME" $ERR_DOMAIN_RESOLUTION_FAILED $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	domain=`/usr/bin/dig -x $ipAddress +short | grep -qi "$DOMAIN_NAME"`
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "Couldn't resolve the domain, $DOMAIN_NAME" $ERR_DOMAIN_RESOLUTION_FAILED $ERR_DOMAIN_RESOLUTION_FAILED
	fi
}

getUIDRange() {

	LDAP_ADMIN_DN=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Administrator DN"`
	INSTALL_TYPE=`/opt/novell/xad/share/dcinit/printConfigKey.pl "INSTALL_TYPE"`

	test -z "$LDAP_ADMIN_DN" && ExitOnFailure --message "Missing Admin DN Name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$INSTALL_TYPE" && ExitOnFailure --message "Missing Install Type." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	LDAP_OUTPUT=`/opt/novell/eDirectory/bin/ldapsearch -x -p 389 -b "$LDAP_ADMIN_DN" -s base uidNumber -LLL`
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "Could not determine uid number for this domain." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE
	fi

	if [ -n "$LDAP_OUTPUT" ]
	then
		UID_NUMBER=`echo $LDAP_OUTPUT | awk -F "uidNumber: " '{print $2}'`
		if [ $INSTALL_TYPE = "frd" ]
		then
			DOMAIN_UID_MIN=$UID_NUMBER
			DOMAIN_UID_MAX=6291455
		else
			DOMAIN_UID_MIN=`expr $UID_NUMBER - 1`
			DOMAIN_UID_MAX=`expr $DOMAIN_UID_MIN + 5242879`
		fi

		echo "UID range for this domain is [$DOMAIN_UID_MIN - $DOMAIN_UID_MAX]"
	fi
}

#The below is a internal call, and used only for ADC installation
getDefaultContainer() {

	defaultContainerName="ou=OESSystemObjects"

	test -z $DOMAIN_NAME
	if [ $? -eq 0 ]
	then
		DOMAIN_NAME=`/bin/dnsdomainname`
	fi

	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Could not get domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	mappedDomain=`echo $DOMAIN_NAME | sed -e 's/\./,dc=/g' -e 's/^/dc=/'`
	containerName="ou=novell,$mappedDomain";

	hostAddressList=`dig $DOMAIN_NAME +short +time=30 2>/dev/null`
	test -z "$hostAddressList" && ExitOnFailure --message "Could not resolve the domain." $ERR_DOMAIN_RESOLUTION_FAILED $ERR_DOMAIN_RESOLUTION_FAILED

	for hostAddress in $hostAddressList
	do
		serverhostname=`dig -x $hostAddress +short +time=30 | awk -F '.' '{print $1}' 2>/dev/null`
		result=`/usr/bin/ldapsearch -h $hostAddress -x -s sub -b "$mappedDomain" objectclass=ncpserver dn -LLL `
		rc1=`echo "$result"| grep -ic "$containerName"`
		rc2=`echo "$result"| grep -ic "$defaultContainerName,$mappedDomain"`
		# If both the container are present and both contain valid ncp server object then we go with default.
		if [ $rc1 -gt 0 ] && [ $rc2 -gt 0 ]
		then
			defaultContainerName="ou=OESSystemObjects"
			break;
		fi
		if [ $rc1 -gt 0 ] && [ $rc2 -eq 0 ]
		then
			defaultContainerName="ou=novell"
			break;
		fi

	done

	echo "$defaultContainerName"

}

getLocalDomainName() {

	DOMAIN_NC=`/opt/novell/xad/share/dcinit/printConfigKey.pl "Mapped Domain NC" | sed 's/s=/st=/gi'`;

	DOMAIN_SID=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -b "$DOMAIN_NC" -s base objectsid -Y EXTERNAL -LLL 2>/dev/null | grep "objectsid" -i | awk -F ":: " '{print $2}'`

	DOMAIN_SID=`/opt/novell/xad/share/dcinit/base64ToSid.pl $DOMAIN_SID`

	ADMINISTRATOR_SID=`/opt/novell/xad/share/dcinit/sidToBase64.pl $DOMAIN_SID-500`

	ADMIN_NAME=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -b "cn=users,$DOMAIN_NC" -s one objectsid -Y EXTERNAL -LLL 2>/dev/null| sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep "$ADMINISTRATOR_SID" -B2 | grep "dn: " | awk -F "dn: " '{print $2}' | sed 's/st=/s=/gi'`; 
}

getRemoteAdminName() {

	DOMAIN_NC=`echo $DOMAIN_NAME | sed -e 's/\./,dc=/g' -e 's/^/dc=/'`

	OUTPUT=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -b "$DOMAIN_NC" -s base objectsid -Y EXTERNAL -LLL 2>/dev/null `
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "Failed to get the Domain SID of domain '$DOMAIN_NAME'" $ERR_LDAPSEARCH_FAILED  $ERR_LDAPSEARCH_FAILED
	fi

	DOMAIN_SID=`echo "$OUTPUT" | grep "^objectsid" -i | awk -F ":: " '{print $2}'`

	DOMAIN_SID=`/opt/novell/xad/share/dcinit/base64ToSid.pl $DOMAIN_SID`

	ADMINISTRATOR_SID=`/opt/novell/xad/share/dcinit/sidToBase64.pl $DOMAIN_SID-500`

	OUTPUT=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -E $LDAP_SERVER_NOMAPPING_OID -b "cn=users,$DOMAIN_NC" -s one objectsid -Y EXTERNAL -LLL 2>/dev/null | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' `

	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "Failed to get the domain admin name for '$DOMAIN_NAME'" $ERR_LDAPSEARCH_FAILED  $ERR_LDAPSEARCH_FAILED
	fi

	DOMAIN_ADMIN_NAME=`echo "$OUTPUT" | grep "$ADMINISTRATOR_SID" -B1 | grep -i "^dn: " | awk -F "dn: " '{print $2}' | sed 's/st=/s=/gi'`
}

getAdminName() {

	test -z $DOMAIN_NAME && test -z $ADMIN_NAME && getLocalDomainName && echo $ADMIN_NAME && exit 0

	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && getRemoteAdminName && echo $DOMAIN_ADMIN_NAME && exit 0

	IP_ADDRESS_LIST=`/usr/bin/dig $DOMAIN_NAME +short +time=15 2>/dev/null`

	ExitOnFailure --message "Cannot retrive the provider for this domain." $? $ERR_DOMAIN_RESOLUTION_FAILED

	found=0

	for CHECK_IP in $IP_ADDRESS_LIST
	do
		rc=`/usr/bin/ldapsearch -h $CHECK_IP -s base -b "" -x`
		if [ $? -ne 0 ]
		then
			continue;
		else
		found=1
	    	break
		fi
	done

	if [ $found -eq 0 ]
	then
	    ExitOnFailure --message "Could not reach the IP Address $IP_ADDRESS_LIST for domain $DOMAIN_NAME" $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE
	fi

	#Obtain the domain IP
	IP_ADDRESS=$CHECK_IP

	ndsToLdap "$ADMIN_NAME"
	ADMIN_NAME="$LDAP_NAME"

	DOMAIN_PARTITION=`echo $DOMAIN_NAME | sed -e 's/\./,dc=/g' -e 's/^/dc=/'`
	OBJECT_SID=`/usr/bin/ldapsearch -h $IP_ADDRESS -D "$ADMIN_NAME" -w "$ADM_PASSWD" -s base -b "$DOMAIN_PARTITION" -x -Z -LLL dn objectsid | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep -i "dn: $DOMAIN_PARTITION" -A2 | grep -i objectsid | awk -F ":: " '{print $2}'`

	DOMAIN_SID=`/opt/novell/xad/share/dcinit/base64ToSid.pl $OBJECT_SID`

	ADMINISTRATOR_SID=`/opt/novell/xad/share/dcinit/sidToBase64.pl $DOMAIN_SID-500`
	PARENT_DOMAIN_ADMIN=`/usr/bin/ldapsearch -E $LDAP_SERVER_NOMAPPING_OID -h $IP_ADDRESS -D "$ADMIN_NAME" -w "$ADM_PASSWD" -s one -b "cn=users,$DOMAIN_PARTITION" -x -Z -LLL dn objectsid | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //; }; ' | grep "$ADMINISTRATOR_SID" -B 1 | grep dn: | awk -F "dn:" '{print $2}' | sed 's/st=/s=/gi'`
	echo $PARENT_DOMAIN_ADMIN
}

#Get the administrator sam name
getAdminSamName() {
	if [ "X$DOMAIN_NAME" != "X" ]
	then
		getRemoteAdminName
		ADMIN_NAME=$DOMAIN_ADMIN_NAME
	else
		getLocalDomainName
	fi

	ADMIN_SAM_NAME=`LDAPCONF=/etc/opt/novell/xad/openldap/ldap.conf /usr/bin/ldapsearch -Y EXTERNAL -s base -b "$ADMIN_NAME" -LLL samaccountname 2> /dev/null`
	if [ $? -ne 0 ]
	then
		echo "Failed to get the admin sam name"
		exit 1
	fi
	ADMIN_SAM_NAME=`echo -n $ADMIN_SAM_NAME | awk -F "samaccountname: " '{print $2}'`
	echo $ADMIN_SAM_NAME
}

validateADCNameServerEntry() {
	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Could not get domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	NS_LIST=`/usr/bin/dig $DOMAIN_NAME +authority NS +short 2>/dev/null`
	test -z "$NS_LIST" && ExitOnFailure --message "Could not fetch Name Server entry for [$DOMAIN_NAME]" $ERR_ADC_NAMESERVER_ENTRY $ERR_ADC_NAMESERVER_ENTRY

	NAMESERVER_IP_LIST=`cat /etc/resolv.conf | grep -i -m 3 nameserver | cut -d ' ' -f2`

	rc=$ERR_ADC_NAMESERVER_ENTRY

	for NAMESERVER_IP in $NAMESERVER_IP_LIST
	do
		for NS_NAME in $NS_LIST
		do
			NS_IP=`/usr/bin/dig $NS_NAME +short`
			if [ "$NS_IP" == "$NAMESERVER_IP" ]
			then
				exit 0
			fi
		done
	done
	ExitOnFailure --message "Server not pointing to the right DNS Server" $rc $ERR_ADC_NAMESERVER_ENTRY
}




getDomainType() {

	test -z "$DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$ADMIN_NAME" && ExitOnFailure --message "Missing admin name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	IP_ADDRESS_LIST=`/usr/bin/dig $DOMAIN_NAME +short +time=15 2>/dev/null`

	ExitOnFailure --message "Can not retrive the provider for this domain." $? $ERR_DOMAIN_RESOLUTION_FAILED

	ndsToLdap "$ADMIN_NAME"
	ADMIN_NAME="$LDAP_NAME"

	DOMAIN_PARTITION=`echo $DOMAIN_NAME | sed -e 's/\./,dc=/g' -e 's/^/dc=/'`
	found=0

	for IP_ADDRESS in $IP_ADDRESS_LIST
	do
		OUT=`/usr/bin/ldapsearch -h $IP_ADDRESS -D "$ADMIN_NAME" -w "$ADM_PASSWD" -s base -b "$DOMAIN_PARTITION" -x -Z -LLL xad-domain-flag 2>/dev/null | grep -i xad-domain-flag`
		if [ $? -ne 0 ]; then continue; fi

		XAD_DOMAIN_FLAG=`echo $OUT | awk -F "xad-domain-flag: " '{print $2}'`
		export XAD_DOMAIN_FLAG
		echo `/usr/bin/perl -e 'print $ENV{XAD_DOMAIN_FLAG}&1'`

		found=1
		break
	done

	if [ $found -eq 0 ]
	then
	    ExitOnFailure --message "Could not retrive xad-domain-flag from the domain container." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE
	fi
}

getSiteList() {

	test -z "$FOREST_DOMAIN_NAME" && ExitOnFailure --message "Missing domain name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	IP_ADDRESS_LIST=`/usr/bin/dig $FOREST_DOMAIN_NAME +short +time=15 2>/dev/null`

	ExitOnFailure --message "Can not retrive the provider for this domain." $? $ERR_DOMAIN_RESOLUTION_FAILED

	DOMAIN_PARTITION=`echo $FOREST_DOMAIN_NAME | sed -e 's/\./,dc=/g' -e 's/^/dc=/'`
	found=0

	#Obtain a valid IP address
	for IP_ADDRESS in $IP_ADDRESS_LIST
	do
		OUT=`LDAPTLS_REQCERT=never /usr/bin/ldapsearch -h $IP_ADDRESS -s base -b "$DOMAIN_PARTITION" -x -Z -LLL 2>/dev/null`
		if [ $? -ne 0 ]; then continue; fi
		found=1
		break
	done

	if [ $found -eq 0 ]
	then
	    ExitOnFailure --message "Could not retrive Site's from the domain controller." $ERR_SERVER_UNREACHABLE $ERR_SERVER_UNREACHABLE
	fi

	#Getting the list of sites
	SITES_CONTAINER="cn=Sites,cn=Configuration,${DOMAIN_PARTITION}"
	SEARCH_FILTER="(objectclass=site)"
	SITE_RETRIVE_COMMAND="/usr/bin/ldapsearch -s one -b $SITES_CONTAINER -h $IP_ADDRESS -x $SEARCH_FILTER dn -LLL 2>&1"
	SITES=$(LDAPTLS_REQCERT=never $SITE_RETRIVE_COMMAND)

	#Validating the ldapsearch command result
        if [ $? -ne 0 ]
        then
                ExitOnFailure --message "ldap Error: Failure Command -  'LDAPTLS_REQCERT=never $SITE_RETRIVE_COMMAND'" $ERR_LDAPSEARCH_FAILED  $ERR_LDAPSEARCH_FAILED
        elif [ "$SITES" = "" ]
        then
                ExitOnFailure --message  "Site list not found : Failure Command -  'LDAPTLS_REQCERT=never $SITE_RETRIVE_COMMAND'" $ERR_LDAPSEARCH_FAILED $ERR_LDAPSEARCH_FAILED
        fi

	#Returning the list of sites
	SITES_LIST=$(echo "$SITES" | grep -i '^dn: \+' | sed 's/^dn: \+cn=//gi' | sed 's/\(.*\),cn=Sites.*/\1/gi' | sort -u | tr "\n$" " " | sed 's/ $//g')

	echo "$SITES_LIST"
}

validateSite(){
	test -z "$PREFERRED_SITE" && ExitOnFailure --message "Missing site name." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	#SITE name cannot contain characters { | }~[]':;<=>?@!"#$%^&`()+/\,*.
	#SITE Name Validation
	echo "$PREFERRED_SITE" | grep -q '[.{|}~:;<=>?@!"#$%^&`()+/\,*[]'
	rc=$?
        if [ $rc -eq 0 ]
        then
		ExitOnFailure --message "A site name cannot contain the space or any of the following characters: .{|}~[]':;<=>?@!\"#$%^&\`()+/\,*." 1 $ERR_INVALID_SITE_NAME
	fi

	#For '
	echo "$PREFERRED_SITE" | grep -q "'"
        rc=$?
	if [ $rc -eq 0 ]
        then
                ExitOnFailure --message "A site name cannot contain the single quote or any of the following characters: .{|}~[]':;<=>?@!\"#$%^&\`()+/\,*." 1 $ERR_INVALID_SITE_NAME
        fi

	#For space
	echo "$PREFERRED_SITE" | grep -q '^ \+.*' || echo "$PREFERRED_SITE" | grep -q '.* \+$' || echo "$PREFERRED_SITE" | grep -q '.* \+.*'
        rc=$?
	if [ $rc -eq 0 ]
        then
                ExitOnFailure --message "A site name cannot contain the space or any of the following characters: .{|}~[]':;<=>?@!\"#$%^&\`()+/\,*." 1 $ERR_INVALID_SITE_NAME
        fi

	#Special case _
	echo "$PREFERRED_SITE" | grep -q '_'
        rc=$?
	if [ $rc -eq 0 ]
        then
		#This line needs to be as written below, to take care of a new line
                ExitOnFailure --message "The site name with underscore(_) does not confirm to Internet standards for naming.
Using this name might affect your ability to interoperate with networks using DNS servers." 1 $ERR_SITE_NAME_HAS_UNDERSCORE
        fi
}

getExistingTreeName() {

	PARENT_DOMAIN_IP=$(getPrimaryDcIP);
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "$PARENT_DOMAIN_IP" $ERR_DOMAIN_RESOLUTION_FAILED  $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	tree_name_cmd1="/opt/novell/eDirectory/bin/ldapsearch -x -b \"\" -s base -h $PARENT_DOMAIN_IP";

	TREE_NAME_STRING=`/opt/novell/eDirectory/bin/ldapsearch -x -b "" -s base -h $PARENT_DOMAIN_IP 2>/dev/null| grep directoryTreeName | awk -F ': ' '{print $2}'`
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "ldap Error: Failure Command -  '$tree_name_cmd1'" $ERR_LDAPSEARCH_FAILED  $ERR_LDAPSEARCH_FAILED
	elif [ "$TREE_NAME_STRING" = "" ]
	then
		ExitOnFailure --message  "Tree not found : Failure Command -  '$tree_name_cmd1'" $ERR_LDAPSEARCH_FAILED $ERR_LDAPSEARCH_FAILED
	fi

	echo $TREE_NAME_STRING;
}


getDomainName() {

	domain_name_cmd="/bin/hostname -d ";
	if [ "$DOMAIN_TYPE" == "child" ]
	then
		PARENT_DOMAIN_NAME=`$domain_name_cmd 2>/dev/null | perl -pe 's/.*?\.//'`;
	else
		PARENT_DOMAIN_NAME=`$domain_name_cmd 2>/dev/null`;
	fi
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message  "Lan Configuration Error: Failure:  Domain Name not Found $PARENT_DOMAIN_NAME" $ERR_LAN_CONFIGURATION_INCORRECT $ERR_LAN_CONFIGURATION_INCORRECT
	elif [ "$PARENT_DOMAIN_NAME" = "" ]
	then
		ExitOnFailure --message  "Lan Configuration Error: Failure:  Domain Name not Found" $ERR_LAN_CONFIGURATION_INCORRECT  $ERR_LAN_CONFIGURATION_INCORRECT
	fi

	echo $PARENT_DOMAIN_NAME;
}

getPrimaryDcIP() {

	PARENT_DOMAIN_NAME=$(getDomainName)
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "$PARENT_DOMAIN_NAME" $ERR_DOMAIN_RESOLUTION_FAILED  $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	parent_domain_ip_cmd1="/usr/bin/dig -t SRV _ldap._tcp.pdc._msdcs.$PARENT_DOMAIN_NAME +short";
	cmd1_output=`$parent_domain_ip_cmd1`;
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message  "DNS Error: Failure Command -  '$parent_domain_ip_cmd1'.(The output was $cmd1_output.)" $ERR_DOMAIN_RESOLUTION_FAILED  $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	PARENT_DOMAIN_PDC=`echo $cmd1_output | awk '{print \$4}' | sed 's/.$//'`;
	if [ "$PARENT_DOMAIN_PDC" = "" ]
	then
		ExitOnFailure --message "Unable to locate PDC for $PARENT_DOMAIN_NAME: Failure Command  - '$parent_domain_ip_cmd1'.(The Domain name returned for \"Existing Server PDC\" was NULL.)" $ERR_DOMAIN_RESOLUTION_FAILED $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	parent_domain_ip_cmd2="/usr/bin/dig $PARENT_DOMAIN_PDC +short";
	cmd2_output=`$parent_domain_ip_cmd2`;
	if [ $? -ne 0 ]
        then
                ExitOnFailure --message  "DNS Error: Failure Command -  '$parent_domain_ip_cmd2'.(Error returned is $cmd2_output.)" $ERR_DOMAIN_RESOLUTION_FAILED  $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	PARENT_DOMAIN_IP=`echo $cmd2_output`;
        if [ "$PARENT_DOMAIN_IP" = "" ]
        then
                ExitOnFailure --message "Unable to resolve the server $PARENT_DOMAIN_PDC: Failure Command  - '$parent_domain_ip_cmd2'" $ERR_DOMAIN_RESOLUTION_FAILED $ERR_DOMAIN_RESOLUTION_FAILED
        fi

	IP_Found=0
	for dc_ip in $PARENT_DOMAIN_IP ;
	do
	 	`/opt/novell/eDirectory/bin/ldapsearch -h $dc_ip -b "" -s base -LLL -x dn > /dev/null 2>&1`
		if [ $? -eq 0 ]
		then
			IP_Found=1
			break
		fi
	done
	if [ $IP_Found -eq 0 ]
	then
		ExitOnFailure --message "Unable to reach any Domain Controllers for Domain $PARENT_DOMAIN_NAME" $ERR_SERVER_UNREACHABLE	 $ERR_SERVER_UNREACHABLE
	fi
	echo $dc_ip;
}

getForestRootDomain() {
	PARENT_DOMAIN_IP=$(getPrimaryDcIP);
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message "$PARENT_DOMAIN_IP" $ERR_DOMAIN_RESOLUTION_FAILED  $ERR_DOMAIN_RESOLUTION_FAILED
	fi

	frd_name_cmd1="/opt/novell/eDirectory/bin/ldapsearch -x -b \"\" -s base -h $PARENT_DOMAIN_IP";

	FRD_NAME_STRING=`/opt/novell/eDirectory/bin/ldapsearch -x -b "" -s base -h $PARENT_DOMAIN_IP | grep rootDomainNamingContext | awk -F ': ' '{print $2}' | sed s/dc=//ig | sed s/\,/\./g 2>/dev/null`;
	if [ $? -ne 0 ]
	then
		ExitOnFailure --message  "ldap Error: Failure Command -  '$frd_name_cmd1'" $ERR_LDAPSEARCH_FAILED $ERR_LDAPSEARCH_FAILED
	elif [ "$FRD_NAME_STRING" = "" ]
	then
		ExitOnFailure --message "Unable to deduce the Forest Root Domain Information: Failure Command -  '$frd_name_cmd1'" $ERR_LDAPSEARCH_FAILED $ERR_LDAPSEARCH_FAILED
	fi

	echo $FRD_NAME_STRING;
}

#
# Call to Fetch the tree IP and name based on /etc/hosts and /etc/resolv.conf entries
#

getExistingTreeInfo() {

	test -z "$DOMAIN_TYPE" && ExitOnFailure --message "Missing domain type." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$INFORMATION_TYPE" && ExitOnFailure --message "Missing information type." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	if [ "$DOMAIN_TYPE" != "adc" -a "$DOMAIN_TYPE" != "child" ]
	then
		#Unsupported Domain Type
		ExitOnFailure --message "The Domain type $DOMAIN_TYPE is not supported." $ERR_INVALID_PARAMETER $ERR_INVALID_PARAMETER
	elif [ "$INFORMATION_TYPE" != "treeName" -a "$INFORMATION_TYPE" != "serverIpAddress" ]
	then
		#Unsupported Information Type
		ExitOnFailure --message "The Information type $INFORMATION_TYPE is not supported." $ERR_INVALID_PARAMETER $ERR_INVALID_PARAMETER
	fi

	if [ "$INFORMATION_TYPE" = "treeName" ]
	then
		getExistingTreeName
	elif [ "$INFORMATION_TYPE" = "serverIpAddress"  ]
	then
		getPrimaryDcIP
	fi

	return $SUCCESS;
}


#
# Call to fetch the DC information based on /etc/hosts and /etc/resolv.conf entries
# It expects a Domain Type and Information Type as inputs and returns Forest Domain, Parent Domain, Parent IP,
#
getExistingDomainInfo() {

	test -z "$DOMAIN_TYPE" && ExitOnFailure --message "Missing domain type." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER
	test -z "$INFORMATION_TYPE" && ExitOnFailure --message "Missing information type." $ERR_MISSING_PARAMETER $ERR_MISSING_PARAMETER

	if [ "$DOMAIN_TYPE" != "adc" -a "$DOMAIN_TYPE" != "child" ]
	then
		#Unsupported Domain Type
		ExitOnFailure --message "The Domain type $DOMAIN_TYPE is not supported." $ERR_INVALID_PARAMETER $ERR_INVALID_PARAMETER
	elif [ "$INFORMATION_TYPE" != "forestDomain" -a "$INFORMATION_TYPE" != "parentDomain"  -a "$INFORMATION_TYPE" != "parentIpAddress" ]
	then
		#Unsupported Information Type
		ExitOnFailure --message "The Information type $INFORMATION_TYPE is not supported." $ERR_INVALID_PARAMETER $ERR_INVALID_PARAMETER
	fi

	if [ "$DOMAIN_TYPE" = "child" ]
	then
		if [ "$INFORMATION_TYPE" = "forestDomain" ]
		then
			getForestRootDomain
		elif [ "$INFORMATION_TYPE" = "parentDomain"  ]
		then
			getDomainName
		elif [ "$INFORMATION_TYPE" = "parentIpAddress"  ]
		then
			getPrimaryDcIP
		fi
	elif [ "$DOMAIN_TYPE" = "adc" ]
	then
		if [ "$INFORMATION_TYPE" = "forestDomain" ]
		then
			getForestRootDomain
		fi
	fi
}

main() {
	action=$1
	shift
	case $action in
		ndsconf-set) while getopts t:s: OPTION;
				do
					case $OPTION in
						t) TEMPLATE_NDS_CONF=$OPTARG ;;
						s) STAGE_NDS_CONF=$OPTARG ;;
						*) gettext -s "Invalid option" ;;
					esac
				done
				ndsConfigSet
				;;
		partition-query) while getopts d:a:p:c: OPTION;
				do
					case $OPTION in
						d) DOMAIN_NAME=$OPTARG ;;
						a) ADMIN_NAME=$OPTARG ;;
						p) IP_ADDRESS=$OPTARG ;;
						c) NDS_CONTAINER_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				checkContainerPartitionStatus
				;;
		ntpconfig) while getopts t: OPTION;
				do
					case $OPTION in
						t) TEMPLATE_NTP_FILE=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				ntpConfigSet
				;;
		netbios-query) while getopts p:n: OPTION;
				do
					case $OPTION in
						p) IP_ADDRESS=$OPTARG ;;
						n) NETBIOS_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";
					esac
				done
				checkNetbiosName
				;;
		netbios-valid-query) while getopts n: OPTION;
                                do
                                        case $OPTION in
                                                n) NETBIOS_NAME=$OPTARG ;;
                                                *) gettext -s "Invalid option";
                                        esac
                                done
                                checkValidNetBIOSNameQuery
                                ;;
		netbios-query-in-subnet) while getopts n: OPTION;
				do
					case $OPTION in
						n) NETBIOS_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";
					esac
				done
				checkDuplicateNetbiosNameInSubNet
				;;
		actual-forest-nc) getActualForestNamingContext ;;
		update-crontab-GPO2NMAS) updateCronEntryGPO2NMAS ;;
		update-crontab-FGP2NMAS) updateCronEntryFGP2NMAS ;;
		update-crontab-KDC) updateCronEntryKDC ;;
		update-crontab-SYSVOLSYNC) updateCronEntrySYSVOLSYNC ;;
		update-crontab-SITESANDSUBNETS) updateCronEntrySITESANDSUBNETS;;
		forest-query) while getopts d:a:p: OPTION;
				do
					case $OPTION in
						d) DOMAIN_NAME=$OPTARG ;;
						a) ADMIN_NAME=$OPTARG ;;
						p) IP_ADDRESS=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				CheckForestExistence
				;;
		domain-query) while getopts d:a:p: OPTION;
				do
					 case $OPTION in
						d) DOMAIN_NAME=$OPTARG ;;
						a) ADMIN_NAME=$OPTARG ;;
						p) IP_ADDRESS=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				CheckDomainExistence
				;;
		sch-version-validate) while getopts p:t: OPTION;
				do
					case $OPTION in
						p) IP_ADDRESS=$OPTARG ;;
						t) TREE_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				ndsSchVersionCheck
				;;
		check-services) CheckServices
				;;
		validate-new-nm-domain) while getopts p:c:a: OPTION;
				do
					case $OPTION in
						p) IP_ADDRESS=$OPTARG ;;
						c) NDS_CONTAINER_NAME=$OPTARG ;;
						a) NDS_ADMIN_DN=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				validateNewNMDomain
				;;
		soa-query) while getopts p:a:c: OPTION;
			do
				case $OPTION in
					p) IP_ADDRESS=$OPTARG ;;
					a) ADMIN_NAME=$OPTARG ;;
					c) NDS_CONTAINER_NAME=$OPTARG ;;
					*) gettext -s "Invalid option";;
				esac
			done
			GetSOAValue
			;;
		get-nc-for-lum) while getopts f: OPTION;
				do
					case $OPTION in
						f) FORESTROOTDOMAIN=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getForestNamingContextforLUM
				;;
		get-domain-guid) while getopts p:c: OPTION;
				do
					case $OPTION in
						p) IP_ADDRESS=$OPTARG ;;
						c) NDS_CONTAINER_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getDomainGUID
				;;
		get-domain-sid)	while getopts a:d: OPTION;
				do
					case $OPTION in
						a) ADMIN_NAME=$OPTARG ;;
						d) DOMAIN_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getDomainSID
				;;
		update-tpo) while getopts f:t: OPTION;
			do
				case $OPTION in
					f) FILENAME=$OPTARG ;;
					t) TRUSTPOSIXOFFSET=$OPTARG ;;
					*) gettext -s "Invalid option";;
				esac
			done
			updateTrustPosixOffset
			;;
		validate-tpo) while getopts t: OPTION;
			do
				case $OPTION in
					t) TRUSTPOSIXOFFSET=$OPTARG ;;
					*) gettext -s "Invalid option";;
				esac
			done
			validateTrustPosixOffset
			;;
		get-dns-master) while getopts p:a:c: OPTION;
				do
					case $OPTION in
						p) IP_ADDRESS=$OPTARG ;;
						a) ADMIN_NAME=$OPTARG ;;
						c) NDS_CONTAINER_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getDNSMaster
				;;
		 get-NetBIOS-name) while getopts d:a:p: OPTION;
	                        do
                                        case $OPTION in
                                                d) DOMAIN_NAME=$OPTARG ;;
                                                a) ADMIN_NAME=$OPTARG ;;
                                                *) gettext -s "Invalid option";;
                                        esac
                                done
                                getNetBIOSName
                                ;;
		 check-domain-resolution) while getopts d: OPTION;
				do
					case $OPTION in
						d) DOMAIN_NAME=$OPTARG;;
						*) gettext -s "Invalid option";;
					esac
				done
				checkDomainResolution
				;;
		  get-uid-range)
				getUIDRange;;
		  get-default-container) while getopts d: OPTION;
				do
					case $OPTION in
						d) DOMAIN_NAME=$OPTARG;;
						*) gettext -s "Invalid option";;
					esac
				done
				getDefaultContainer;;
		  get-admin-name) while getopts a:d: OPTION;
				do
					case $OPTION in
						a) ADMIN_NAME=$OPTARG ;;
						d) DOMAIN_NAME=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getAdminName;;
		  get-domain-type) while getopts a:d: OPTION;
				do
					case $OPTION in
					    	a) ADMIN_NAME=$OPTARG ;;
					    	d) DOMAIN_NAME=$OPTARG ;;
					    	*) gettext -s "Invalid option";;
					esac
				done
				getDomainType;;
		validate-adc-nameserver-entry) while getopts d: OPTION;
			do
				case $OPTION in
					d) DOMAIN_NAME=$OPTARG;;
					*) gettext -s "Invalid option";;
				esac
			done
			validateADCNameServerEntry;;
		  get-admin-sam-name) while getopts d: OPTION;
			do
				case $OPTION in
					d) DOMAIN_NAME=$OPTARG ;;
					*) gettext -s "Invalid option";;
				esac
			done
			getAdminSamName;;
		  get-existing-tree)while getopts d:t: OPTION;
				do
					case $OPTION in
						d) DOMAIN_TYPE=$OPTARG ;;
						t) INFORMATION_TYPE=$OPTARG ;;
					esac
				done
				getExistingTreeInfo;;
		  get-domain) while getopts d:t: OPTION;
				do
					case $OPTION in
						d) DOMAIN_TYPE=$OPTARG ;;
						t) INFORMATION_TYPE=$OPTARG ;;
						*) gettext -s "Invalid option";;
					esac
				done
				getExistingDomainInfo;;
		  get-site-list) while getopts d: OPTION;
				do
					case $OPTION in
					    	d) FOREST_DOMAIN_NAME=$OPTARG ;;
					    	*) gettext -s "Invalid option";;
					esac
				done
				getSiteList;;
		  validate-site-name) while getopts z: OPTION;
				do
					case $OPTION in
						z) PREFERRED_SITE=$OPTARG ;;
					    	*) gettext -s "Invalid option";;
					esac
				done
				validateSite;;

		*) gettext -s "Invalid option" ;;

	esac
}

main "$@"

