#!/usr/bin/perl -I. -I.. -I/opt/novell/xad/lib64/perl -I/opt/novell/xad/lib/perl

################################################################################
# The tool to validate the rights for a given trustee. This can be done for the 
# rights on containers (mapped container and security), [ROOT] partition and the
# DNS related containers and the proxy user.
#
# See the file LICENSE for redistribution information.
#
# Copyright (c) 2011-2020
#	Micro Focus or one of its affiliates.
################################################################################
#
#   NAME:
#	  validateInput.pl
#
#   FACILITY:
#	  Utility to validate the container admin rights.
#
#   ABSTRACT:
#
#   REVISION: $Id$
#
#################################################################################

use warnings;
use strict;
use Getopt::Std;
use Locale::gettext;

use XAD::secure;
use Net::LDAP;

my $debug = 0;
my $LDAP_DS_ENTRY_SUPERVISOR = 16 ; # 0x00000010L from ldap_ds_constants.h
my $LDAP_DS_ATTR_SUPERVISOR = 32 ;  # 0x00000020L from ldap_ds_constants.h
my $LDAP_DS_ENTRY_ADD   = 2;        # 0x00000002L frpm ldap_ds_constants.h
my $LDAP_NO_SUCH_OBJECT =  32;      # Net LDAP Error code for NO Such Object

sub Usage {

	print <<USAGE;

	validateInput.pl 
	-M		To check rights on mapped and security container 
	-D		To check rights on DNS context
	-T		To check rights on tree root partition (for schema extension)
	----------------
	-a		The admin DN to be validate for rights (with -M, -D, -T)
	-w		The admin password (with -M, -D, -T)
	-i		The IP address of the server holding the partition (with -M, -D, -T)
	-t 		Tree Name (with -M, -D,-T)
	-m		The container to be mapped (with -M )
	-g		DNS group context suppiled in YaST (with -D)
	-l 		DNS locator context supplied in YaST (with -D)
	-r		DNS root server info context in YaST (with -D)
	-u		DNS proxy user context (with -D)

USAGE
        exit 1

}

# Generic routine which would check the rights
sub checkRights ($$$$$) {
	my ($ldapAdmin, $remoteServerIP, $container, $rights_type, $value_expected) = @_;

	print "Executing .. /opt/novell/xad/sbin/get_rights -h $remoteServerIP -p 636 -D $ldapAdmin -w XXX -c /etc/opt/novell/certs/SSCert.der -o $container -t $ldapAdmin -a \"$rights_type\" \n" if $debug;

	my $output = `/opt/novell/xad/sbin/get_rights -h $remoteServerIP -p 636 -D '$ldapAdmin' -w '$ENV{ADM_PASSWD}' -c /etc/opt/novell/certs/SSCert.der -o '$container' -t '$ldapAdmin' -a \"$rights_type\"`;
		
	if ($output =~ /ldapssl_add_trusted_cert error/) {
		return (1, "Failed due to invalid certificate\n");
	}

	my $returnVal = $? >> 8;
	if ($returnVal == 255) {
		return (1, "The $rights_type could't be located");
	}
	
	my $rights = $returnVal;
	if (($rights & $value_expected) == 0) {
		return (1, "The admin [$ldapAdmin] doesn't have supervisor $rights_type over [$container].");
	}
	return (0, "Successfully obtained $rights_type over [$container]");
}

# Subroutine to check the rights of the user on the given container
sub verifySupervisoryRights {

	my ($adminUser, $remoteServerIP, $container) = @_;

	# Check that given admin has appropriate rights over the given container container
	my $ldapAdmin = $adminUser;
	$ldapAdmin =~ s/\./,/g;
	my ($rc, $msg) = checkRights ($adminUser, $remoteServerIP, $container, 
				"[Entry Rights]", $LDAP_DS_ENTRY_SUPERVISOR);
	if ($rc != 0) {
		return ($rc, $msg);
	}

	($rc, $msg) = checkRights ($adminUser, $remoteServerIP, $container, 
				"[All Attributes Rights]", $LDAP_DS_ATTR_SUPERVISOR);
	if ($rc != 0) {
		return ($rc, $msg);
	}

	return (0, "Successfully completed the verification of rights of $adminUser on $container");
}


# This function would be used to verify the rights 
# for the mapped container and security container.
sub adminRightsCheck {
	
	my ($adminUser, $remoteServerIP, $mappedContainer, $treeName) = @_;
	my $count = 0;

	# Check rights of user on the mapped partition.
RETRY:
	my ($rc, $msg) = verifySupervisoryRights($adminUser, $remoteServerIP, $mappedContainer);
	if ($rc != 0) {
		if ($rc == 2) {
			unlink("/etc/opt/novell/certs/SSCert.der");
			print "Executing getCerts : $remoteServerIP, $treeName, $adminUser\n" if $debug;

			($rc, $msg) = getCertificate($remoteServerIP, $treeName, $adminUser);
			if ($rc != 0) {
				return ($rc, "Could not get the eDirrectory certificate: $msg");
			}
			
			$count++;

			if ($count <= 5) {
				goto RETRY;	
			}
		}
		else {
			return ($rc, $msg);
		}
	}
	
	# All the servers are part of parition where container admin is present
# 	$mappedContainer =~ s/,/\./g;
# 	my $partitionName = ".".$mappedContainer.".T=".$treeName.".";
# 
# 	my @serverList = `/opt/novell/eDirectory/bin/ndsstat -n -d -h $remoteServerIP -p $partitionName | grep -i '\.CN'`;
#         
# 	if ($? != 0) {
# 		return (1, "The partition information is not available");
# 	}
# 
# 	foreach my $server (@serverList) {
# 		$server =~ s/\.T(.*)//g;
# 		$server =~ s/^\.//g;
# 		$server =~ s/\./,/g;
# 		chomp($server);
# 		($rc, $msg) = verifySupervisoryRights($adminUser, $remoteServerIP, $server);
# 		if ($rc != 0) {
# 			return ($rc, $msg)
# 		}
# 	}

	# Check the rights on the security container.
	($rc, $msg) = verifySupervisoryRights($adminUser, $remoteServerIP, "cn=security");
	if ($rc != 0) {
		return ($rc, $msg); 
	}

	return (0, "The admin [$adminUser] has appropriate rights on [$mappedContainer] and security container");
}

# checks the hierarchy by checking the last existing container and
# then checking create right on that.
sub checkHierarchy ($$$) {
	my ($admin_user, $remote_edir_ip, $container) = @_;
	my $retcode = 0;

	# perform a search
	my $ldap = Net::LDAP->new($remote_edir_ip, scheme=> "ldaps", port=>636);
	if (!$ldap) {
		return (-1, "Couldn't create the connection to Edir server on [$remote_edir_ip]");
	}

        # bind to a directory with dn and password
	my $mesg = $ldap->bind ($admin_user,
				password => $ENV{ADM_PASSWD});
	if ($mesg->code != 0) {
		return (-1, "Failed to bind to remote edir ");
	}

        # perform a search
	my $cont = $container;
	while ($cont ne "") {

		print "Doing the operation for $cont\n" if $debug;
		$mesg = $ldap->search(base=> $cont, scope =>'base', filter=> "objectclass=*");

		$retcode = $mesg->code;

		if ($retcode == $LDAP_NO_SUCH_OBJECT) {
			$cont =~ s/^.*?\,//g;
			print "After regex sub: $cont\n" if $debug;
		}
		elsif ($retcode != 0) {
			return ($retcode, "Failed to search the container [$cont]");
			last;
		}elsif ($retcode == 0) {
			print "Success\n" if $debug;
			return checkRights ($admin_user, $remote_edir_ip, $cont, 
						"[Entry Rights]", $LDAP_DS_ENTRY_ADD);
			last;
		}
	}

	if ($cont eq "") {
		return (-1, "Failed to find the container.");
	}

	$mesg = $ldap->unbind;
}

#user check the user context;
sub checkUserContext ($$$) {
	my ($admin_user, $remote_edir_ip, $userContext) = @_;
	my $retcode = 0;

	# perform a search
	my $ldap = Net::LDAP->new($remote_edir_ip, scheme=> "ldaps", port=>636);
	if (!$ldap) {
		return (-1, "Couldn't create the connection to Edir server on [$remote_edir_ip]");;
	}

        # bind to a directory with dn and password
	my $mesg = $ldap->bind ($admin_user,
				password => $ENV{ADM_PASSWD});
	if ($mesg->code != 0) {
		return ($mesg->code,"Failed to bind to remote edir ".$mesg->error);
	}

        # perform a search
	$mesg = $ldap->search(base=> $userContext, scope =>'base', filter=> "objectclass=*");
	$retcode = $mesg->code;
	$ldap->unbind;

	if ($retcode == $LDAP_NO_SUCH_OBJECT) {
		return (-1, "The dns proxy user context [$userContext] does not exit");	
	}
	elsif ($retcode != 0) {
		return (-1, "For dns proxy user, failed to locate the container [$userContext]");
	}elsif ($retcode == 0) {
		print "Checking rights on the user context [$userContext]\n" if $debug;
		($retcode, $mesg) = checkRights ($admin_user, $remote_edir_ip, $userContext, 
			"[Entry Rights]", $LDAP_DS_ENTRY_ADD);
		if ($retcode != 0) {
			return ($retcode, "For dns proxy user, the admin [$admin_user] doesn't have proper rights on [$userContext]");
		}
	}

	return (0, "Successfully verified the container context");
}

# The admin has rights over the context in which dns objects are getting created
sub dnsContainersCheck ($$$$) {
	my ($adminUser, $remoteServerIP, $dnsProxyUser, $dnsContainers) = @_;
	my ($key , $container) = 0;	
	my ($rc, $msg) = 0;

	while (($key,$container) = each (%$dnsContainers)) {
		#my $container = $dnsContainers{$key};
		$container =~ s/\./,/g;
		chomp($container);

		($rc, $msg) = checkHierarchy($adminUser, $remoteServerIP, $container);
		if ($rc != 0) {
			return ($rc, $msg);
		}
	}

	if ($dnsProxyUser) {
		my $userContext = $dnsProxyUser;
		$userContext =~ s/\./,/g;
		$userContext =~ s/^.*?,//;
		print "Checking the user context [$userContext]\n" if $debug;

		($rc, $msg) = checkUserContext($adminUser, $remoteServerIP, $userContext);
		if ($rc != 0) {
			return ($rc, $msg);
		}
	}

	return (0, "Successfully validated the DNS related containers");
}

# Get the Edir certificate
sub getCertificate ($$$) {
	my ($tree_ip, $tree_name, $admin_dn) = @_;
#	my $lib = "ADM_PWD='$ENV{ADM_PASSWD}' LD_LIBRARY_PATH=/opt/novell/eDirectory/lib64:/opt/novell/eDirectory/lib64/nds-modules:$ENV{LD_LIBRARY_PATH}";
#	my $command = "$lib /opt/novell/oes-install/util/getSSCert ";
	chomp($tree_ip, $tree_name, $admin_dn);

#	$admin_dn =~ s/,/\./g;
#	$command .= "-a $tree_ip -t $tree_name";
#	print "Executing .. $command -u $admin_dn\n" if $debug;

#	`$command -u $admin_dn`;
	`/opt/novell/xad/share/dcinit/getsscert_script.sh $tree_ip`;
	if ($?) {
		return (-1, "Failed to get the certificate");
	}
}

# Main subroutine
sub main {

	my %inputs = ();
	my ($adminDN, $adminPswd, $mappedContainer, $remoteServerIP, $treeName) = ("", "", "", "", "");
	my ($dnsLocatorContext, $dnsRootServerInfoContext, $dnsGroupContext,
	       	$dnsProxyUser) = ("", "", "", "");
	my ($validateCont, $validateDNS,$validateTreeRoot) = (0,0,0);
	my %dnsContainers = ();

	getopts("a:m:i:g:l:r:t:u:w:TDMd", \%inputs) or die("Unexpected arguments\n");

	$validateCont = defined $inputs{M} ? 1: 0;
	$validateDNS = defined $inputs{D} ? 1 : 0;
	$validateTreeRoot = defined $inputs{T} ? 1 : 0;

	$adminDN = defined $inputs{a} ? $inputs{a} : "";
	$adminPswd = defined $inputs{w} ? $inputs{w} : "";
	$mappedContainer = defined $inputs{m} ? $inputs{m} : "";
	$remoteServerIP = defined $inputs{i} ? $inputs{i} : "";
	$treeName = defined $inputs{t} ? $inputs{t} : "";
	$dnsContainers {g} = defined $inputs{g} ? $inputs{g} : "";
	$dnsContainers {r} = defined $inputs{r} ? $inputs{r} : "";
	$dnsContainers {l} = defined $inputs{l} ? $inputs{l} : "";
	$dnsProxyUser = defined $inputs{u} ? $inputs{u} : "";
	$debug = defined $inputs{d} ? $inputs{d}: 0;
	
	if ($validateCont == 0 and 
	    $validateDNS == 0 and 
	    $validateTreeRoot == 0) {
		print STDERR gettext ("Insufficient argument, the validation type is not supplied\n");
		Usage();
	}

        my $option = $validateTreeRoot + $validateCont + $validateDNS;
        if ($option != 1) {
		print STDERR gettext ("There should be only one validate option supplied\n");
		Usage();
        }

	if (not defined $ENV{ADM_PASSWD} and $adminPswd eq "") {
		print STDERR gettext ("The eDirectory admin password not supplied\n");
		exit (-1);
	}

	if (defined $ENV{ADM_PASSWD} and $adminPswd ne "") { 
		if ($ENV {ADM_PASSWD} ne $adminPswd) {
			print STDERR gettext ("The admin password supplied twice and both are different\n");
			exit (-1);
		}
	}

	if (not defined $ENV{ADM_PASSWD}) {
		$ENV{ADM_PASSWD} = $adminPswd;
	}

	$adminDN =~ s/\./,/g;

	if (!(-e "/etc/opt/novell/certs/SSCert.der")) {
		print "Getting the certificate \n" if $debug;
		if ($adminDN ne "" and $remoteServerIP ne "" and $treeName ne "") {
			my ($rc, $msg) = getCertificate($remoteServerIP, $treeName, $adminDN);
			if ($rc != 0) {
				print STDERR gettext ("Cannot retrieve eDirectory certificate: command failed\n");
				exit (-2);
			}
		}else {
			print STDERR gettext ("Cannot retrieve eDirectory certificate: Insufficient arguments\n");
			exit (-2);
		}
	} 

	if ($validateTreeRoot) {
		if ($adminDN eq "" or
		    $remoteServerIP eq "" or
                    $treeName eq "") {
			print STDERR gettext("The arguments supplied are inappropriate.\n");
			Usage();
		}

		$adminDN =~ s/\./,/g;
		print "Going to tree root rights check : $adminDN, $remoteServerIP, $treeName\n" if $debug;

		my ($rc, $msg) = checkRights ($adminDN, $remoteServerIP, "T=$treeName", 
					"[Entry Rights]", $LDAP_DS_ENTRY_SUPERVISOR);
		if ($rc != 0) {
			print STDERR gettext("The admin [$adminDN] do not have supervisor rights on tree root\n");
			exit (-3);
		}
		print "The admin [$adminDN] has supervisor rights on tree root partition.\n" if $debug;
	}
	elsif ($validateCont) {
		if ($adminDN eq "" or
		    $remoteServerIP eq "" or
		    $mappedContainer eq "" or 
                    $treeName eq "") {
			print STDERR gettext("The arguments supplied are inappropriate.\n");
			Usage();
		}

		$adminDN =~ s/\./,/g;
		print "Going to adminRights check : $adminDN, $remoteServerIP, $mappedContainer, $treeName\n" if $debug;
		my ($rc, $msg) = adminRightsCheck($adminDN, $remoteServerIP, $mappedContainer, $treeName);

		if ($rc != 0) {
			print STDERR gettext($msg."\n");
			exit (-4);
		}
		
		print gettext ("$msg\n");
	}
	elsif ($validateDNS) {
		# validate DNS rights on the context specified.
		print "Executing .. $adminDN, $remoteServerIP , ".$dnsContainers{g}." , ".$dnsContainers{l}." , ".$dnsContainers{r}." \n" if $debug;
		if ($adminDN eq "" or
		    $remoteServerIP eq "" or
		    $dnsContainers{g} eq "" or
		    $dnsContainers{l} eq "" or
		    $dnsContainers{r} eq "") {
			print STDERR gettext("The arguments supplied are inappropriate.\n");
			Usage();
		}

		my ($rc, $msg) = dnsContainersCheck($adminDN, $remoteServerIP,
					$dnsProxyUser, \%dnsContainers);
		if ($rc != 0) {
			print STDERR gettext("$msg\n");
			exit (-5);
		}
		print gettext ("$msg\n");
	}else {
            print STDERR gettext("Invalid option, cannot handle this.")
        }
}

main;

